#include "stdafx.h"
#include "MsgList.h"
#include "resource.h"
#include <Themis/AppFrameBase.h>
#include <Themis/StringScanner.h>
#include <Themis/Allocator.h>
#include <Themis/Window.h>
#include <Themis/DialogResizer.h>
#include <Themis/Config2.h>
#include <Themis/WindowSnapper.h>
#include <Themis/Windows/Mailslot.h>
#include <Themis/Windows/System.h>
#include <Themis/Windows/Environment.h>
#include <Themis/Windows/ListViewUtility.h>
#include <Themis/Thread/XTime.h>

namespace Themis {
extern const char* g_pszLogWindowPath;
}

struct SMsgData {
    size_t nSize;
    int nPID;
    int nNum;
    wchar_t szMsg[1];
};

class CAppWindow : private IMsgHooker {
    auto_ptr<CWindow> m_pWindow;
    auto_ptr<CDialogResizer> m_pResizer;
    auto_ptr<CWindowSnapper> m_pSnapper;
    CConfig2 config;
    CConfig2 data;
    CMsgList m_oMsgList;
    int m_nLastPID;

    bool threadValid;
    auto_ptr<boost::thread> thread;
    auto_ptr<boost::mutex> mutex;
    auto_ptr<boost::condition> condition;

    // ŏ̃bZ[W
    bool m_bMsg;
    int m_nPID;
    int m_nNum;
    string firstMsg;
public:
    CAppWindow() : m_bMsg(false)  { Init(); }
    CAppWindow(int nPID, int nNum, const string& msg) : m_bMsg(true), m_nPID(nPID), m_nNum(nNum), firstMsg(msg) { Init(); }
private:
    void Init() {
        m_nLastPID = -1;
        m_pSnapper.reset(new CWindowSnapper);
        m_pSnapper->SetSnapWorkArea(true);
        m_pSnapper->SetSnapDist(15);
        m_pWindow.reset(new CWindow);
        m_pWindow->GetHook().Add(this);
        data.Point(L"WindowPos") << CPoint(128, 256);
        data.Size(L"WindowSize") << CSize(240, 320);
        data.Num(L"Column0") << 30;
        data.Num(L"Column1") << 0;
        data.GetStorage().Load(CFileNameModuleDir(L"LogWindow.dat"));
        data.GetStorage().Save(CFileNameModuleDir(L"LogWindow.dat"));
        config.Bool(L"ClearOnAnotherPID") << true;
        config.GetStorage().Load(CFileNameModuleDir(L"LogWindow.cfg"));
        config.GetStorage().Save(CFileNameModuleDir(L"LogWindow.cfg"));

        condition.reset(new boost::condition);
        mutex.reset(new boost::mutex);
        threadValid = true;
        thread.reset(new boost::thread(std::tr1::bind(&CAppWindow::OnThread, this)));
    }
public:
    ~CAppWindow() {
        {
            boost::mutex::scoped_lock lock(*mutex);
            threadValid = false;
            condition->notify_all();
        }
        thread->join();
    }

    int Run(CAppFrameBase& rAppFrame) {
        CModalDialogParameter oPrm(IDD_DIALOG1);
        m_pWindow->Create(oPrm);
        return 0;
    }

private:
    bool HookMsg(CMsgHookPrms& rPrms) {
        if (rPrms.GetMsg() == WM_COMMAND) {
            CListview oList1(m_pWindow->GetCtrl(IDC_LIST1));
            switch (rPrms.GetWPrmLoWord()) {
            case ID_40001: // I
                m_pWindow->Close();
                break;
            case ID_40002: // 폜
                m_oMsgList.Delete(oList1.GetSelected());
                oList1.SetItemCount(m_oMsgList.GetCount());
                oList1.UpdateItems();
                oList1.ScrollToSelect();
                break;
            case ID_40003: // Rs[
                if (m_oMsgList.IsValidIndex(oList1.GetSelected())) {
                    CSystem::SetClipboard(m_oMsgList.GetMsg(oList1.GetSelected()));
                }
                break;
            case ID_40005: // Sč폜
                m_oMsgList.Clear();
                oList1.SetItemCount(m_oMsgList.GetCount());
                oList1.UpdateItems();
                oList1.ScrollToSelect();
                break;
            default:
                return false;
            }
            return true;
        } else if (rPrms.GetMsg() == WM_NOTIFY) {
            NMHDR* hdr = (NMHDR*)rPrms.GetLPrm();
            if (hdr->idFrom == IDC_LIST1) {
                if (hdr->code == LVN_GETDISPINFOA || 
                    hdr->code == LVN_GETDISPINFOW ) { // \̂̔zuɂĂ͊낤̂c
                    LV_DISPINFO* di = (LV_DISPINFO*)hdr;
                    if (0 <= di->item.iItem && di->item.iItem < m_oMsgList.GetCount()) {
                        if (di->item.mask & LVIF_TEXT) {
                            switch (di->item.iSubItem) {
                            case 0:
                                ListViewUtility::SetText(*di, ItoA(m_oMsgList.GetPID(di->item.iItem)));
                                return true;
                            case 1:
                                ListViewUtility::SetText(*di, ItoA(m_oMsgList.GetNum(di->item.iItem)));
                                return true;
                            case 2:
                                ListViewUtility::SetText(*di, m_oMsgList.GetMsg(di->item.iItem));
                                return true;
                            }
                        }
                    }
                } else if (hdr->code == NM_CUSTOMDRAW) { // J̃TCYςɂĂ΂̂ŁAŐFX
                    AdjustColmnWidth();
                    return true;
                }
            }
            return false;
        } else if (rPrms.GetMsg() == WM_SIZE) {
            if (m_pResizer.get() != NULL) {
                m_pResizer->OnSize(rPrms);
            }
            AdjustColmnWidth();
            return true;
        } else if (rPrms.GetMsg() == WM_ENTERSIZEMOVE) {
            return m_pSnapper->OnEnterSizeMove(rPrms);
        } else if (rPrms.GetMsg() == WM_EXITSIZEMOVE) {
            return m_pSnapper->OnExitSizeMove(rPrms);
        } else if (rPrms.GetMsg() == WM_MOVING) {
            return m_pSnapper->OnMoving(rPrms);
        } else if (rPrms.GetMsg() == WM_INITDIALOG) {
            // TCY
            m_pResizer.reset(new CDialogResizer(m_pWindow->GetHandle()));
            m_pResizer->Add(IDC_LIST1, 0, 0, 1, 1);
            // 肷Ƃт
            CListview oList1(m_pWindow->GetCtrl(IDC_LIST1));
            oList1.SetExListStyle(LVS_EX_FULLROWSELECT); //LVS_EX_TWOCLICKACTIVATE
            int anCol[] = {data.Num(L"Column0").Get(), data.Num(L"Column1").Get()};
            oList1.AddColumn(0, L"PID", anCol[0], 2);
            oList1.AddColumn(1, L"", anCol[1], 2);
            oList1.AddColumn(2, L"bZ[W", oList1.GetValidClientSize().cx - anCol[0] - anCol[1]);
            // ǂʒuƂ
            m_pWindow->SetRectClip(CRect(data.Point(L"WindowPos").Get(), data.Size(L"WindowSize").Get()));
            // x
            m_pWindow->PostMsg(WM_APP, 0, 0);
            return true;
        } else if (rPrms.GetMsg() == WM_APP) {
            // bZ[W
            if (m_bMsg) {
                AddMsg(m_nPID, m_nNum, firstMsg);
            }
            return true;
        } else if (rPrms.GetMsg() == WM_COPYDATA) {
            COPYDATASTRUCT* cd = (COPYDATASTRUCT*)rPrms.GetLPrm();
            SMsgData* p = reinterpret_cast<SMsgData*>(cd->lpData);
            AddMsg(p->nPID, p->nNum, p->szMsg);
            rPrms.SetResult(TRUE);
            return true;
        } else if (rPrms.GetMsg() == WM_CLOSE) {
            if (m_pWindow->IsShowNormal()) {
                data.Point(L"WindowPos") << m_pWindow->GetPos();
                data.Size(L"WindowSize") << m_pWindow->GetSize();
            }
            CListview oList1(m_pWindow->GetCtrl(IDC_LIST1));
            data.Num(L"Column0") << oList1.GetColumnWidth(0);
            data.Num(L"Column1") << oList1.GetColumnWidth(1);
            data.GetStorage().Save(CFileNameModuleDir(L"LogWindow.dat"));
            m_pWindow->Delete();
            return true;
        } else {
            return false;
        }
    }
    void AdjustColmnWidth() {
        // J𒲐
        CListview oList1(m_pWindow->GetCtrl(IDC_LIST1));
        int anCol[] = {oList1.GetColumnWidth(0), oList1.GetColumnWidth(1)};
        oList1.SetColumnWidth(2, oList1.GetValidClientSize().cx - anCol[0] - anCol[1]);
    }
    void AddMsg(int nPID, int nNum, const string& msg) {
        CListview oList1(m_pWindow->GetCtrl(IDC_LIST1));

        if (m_nLastPID != nPID && config.Bool(L"ClearOnAnotherPID").Get()) {
            // PIDȂNAȏB
            m_oMsgList.Clear();
        }
        m_nLastPID = nPID;

        int nIndex = m_oMsgList.Add(nPID, nNum, msg);
        oList1.SetItemCount(m_oMsgList.GetCount()); // f[^̌ݒ
        oList1.UpdateItems();
    /*
        oList1.SetSelected(nIndex);
    /*/
        oList1.SetSelected(oList1.GetCount() - 1);
    //*/
        oList1.ScrollToSelect();
    }
    void OnThread() {
        CMailslotReceiver receiver;
        vector<byte> vctBuffer;
        while (threadValid) {
            try {
                size_t nSize;
                // f[^܂ő҂
                {
                    boost::mutex::scoped_lock lock(*mutex);
                    if (!threadValid) return;

                    while (true) {
                        nSize = receiver.GetSize();
                        if (0 < nSize) break;
                        // f[^܂ő҂B
                        if (!condition->timed_wait(lock, XTime::AfterMilliSeconds(10))) {
                            // ҂҂B
                        }
                        if (!threadValid) return;
                    }
                }
                // f[^̎M
                vctBuffer.clear();
                vctBuffer.resize(nSize);
                receiver.Receive(&vctBuffer[0], nSize);
                if (sizeof(size_t) < vctBuffer.size()) {
                    if (reinterpret_cast<SMsgData*>(&vctBuffer[0])->nSize != vctBuffer.size()) {
                        throw RuntimeException(L"f[^`ۂłB");
                    } else {
                        SMsgData* p = reinterpret_cast<SMsgData*>(&vctBuffer[0]);
                        AddMsg(p->nPID, p->nNum, p->szMsg);
                    }
                }
            } catch (std::exception& e) {
                // Ƃ肠B
                Themis_DebugWarning(Encoding::GetDefault().GetString(e.what())); e;
            }
        }
    }
};

class CAppFrame : public CSingleAppFrameBase {
    int InnerRun() {
        CStringScanner scanner(Environment::GetCommandLine());
        if (scanner.IsMatch(L'"')) {
            while (scanner && !scanner.IsMatch(L'"')) {
                scanner.ToNext();
            }
            scanner.IsMatch(L' ');
        } else {
            while (scanner && !scanner.IsMatch(L' ')) {
                scanner.ToNext();
            }
        }
        const int nPID = scanner.GetDigit();
        const int nNum = scanner.GetDigit();
        if (scanner.GetPos() < scanner.GetLength()) {
            const string msg = scanner.Substring(scanner.GetPos());
            // Ƃꍇ
            // EBhE쐬 or I֑M
            if (IsFirstRun()) {
                return CAppWindow(nPID, nNum, msg).Run(*this);
            }

            size_t msgStrSize = (msg.GetLength() + 1) * sizeof(wchar_t); // '\0' ܂߂Ă܂
            size_t nAlloc = sizeof(size_t) + sizeof(int) * 2 + msgStrSize;
            SMsgData* p = static_cast<SMsgData*>(operator new(nAlloc));
            p->nSize = nAlloc;
            p->nPID = nPID;
            p->nNum = nNum;
            memcpy(p->szMsg, msg.ToCharPtr(), msgStrSize);
            for (int i = 0; i < 4; i++) { // sĂS񂾂gC
                try {
                    CMailslotSender oSender;
                    oSender.Send(p, nAlloc);
                    // 烋[v
                    break;
                } catch (std::exception& e) {
                    // Ƃ肠B
                    Themis_DebugWarning(Encoding::GetDefault().GetString(e.what())); e;
                    ::Sleep(1000);
                }
            }
            operator delete(p);
        } else {
            // Ƃ̏ꍇ
            if (IsFirstRun()) {
                return CAppWindow().Run(*this);
            }
        }
        return 0;
    }
};

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow) {
    g_pszLogWindowPath = "..\\..\\LogWindow\\LogWindowDummy.bat";

    CAppFrame theApp;
    theApp.Initialize(hInstance, hPrevInstance, lpCmdLine, nCmdShow);
    return theApp.Run();
}
/*
namespace {
 #if 0
    #define ALLOC_LOG   , CAllocatorLog
 #else
    #define ALLOC_LOG
 #endif
    typedef CAllocator<0x100, 0x100 ALLOC_LOG> TAllocator;
    TAllocator& GetAllocator() { static TAllocator o; return o; }
}
void* ::operator new(size_t n) {
    return GetAllocator().Alloc(n);
}
void ::operator delete(void* p) {
    GetAllocator().Free(p);
}
//*/
