#include "stdafx.h"
#include "../TempFile.h"
#include "ArchiveLoader.h"
namespace Themis {

CArchiveLoader::CArchiveLoader(const ArchiveOwner& owner) : owner(owner) {
}

CArchiveLoader::~CArchiveLoader() {
    Clear();
    // e|t@C͉̔qɍ폜Ɏs邩񂪁Aǁ[Ȃ̂ŖَEB
    try {
        m_vctTempDir.clear();
    } catch (std::exception& e) {
        Themis_DebugLog(e.what()); e;
    }
}

void CArchiveLoader::SetTempDir(const CFileName& o) {
    m_vctTempDir.push_back(new CTempDir(o));
}

void CArchiveLoader::Clear() {
    // NA
    m_vctArchives.clear();
}

bool CArchiveLoader::Add(const CFileName& oFileName, vector<CArchive::SFileInfo>& vctFiles) {
    if (!CArchive::IsSupportedFileName(oFileName)) {
        // T|[g
        return false;
    }
    try {
        std::tr1::shared_ptr<CArchive> p(new CArchive(GetTempDir(), owner));
        if (!p->Open(oFileName)) {
            // ۂɊJĂ݂疢T|[gꍇ
            return false;
        }
        // t@CXg擾AɃXg֒ǉ
        p->GetFileList(vctFiles);
        m_vctArchives.push_back(p);
        return true;
    } catch (std::exception&) {
        // 玸sۂꍇBʓ|Ȃ̂őSfalseB
        return false;
    }
}

/// ǂݍ߂鏑ɂȂtrue
bool CArchiveLoader::IsReadable(const CFileName& oFileName) {
    if (!CArchive::IsSupportedFileName(oFileName)) {
        // T|[g
        return false;
    }
    try {
        std::tr1::shared_ptr<CArchive> p(new CArchive(GetTempDir(), owner));
        if (!p->Open(oFileName)) {
            // ۂɊJĂ݂疢T|[gꍇ
            return false;
        }
        return true;
    } catch (std::exception&) {
        // 玸sۂꍇBʓ|Ȃ̂őSfalseB
        return false;
    }
}

/// ɉ𓀃tÕZbg
void CArchiveLoader::ResetExtractFlags() {
    for (size_t i = 0, n = m_vctArchives.size(); i < n; i++) {
        m_vctArchives[i]->ResetExtractFlags();
    }
}

pair<std::tr1::weak_ptr<CArchive>, int> CArchiveLoader::InnerFind(const string& strViewName) const {
    for (size_t i = 0, n = m_vctArchives.size(); i < n; i++) {
        pair<std::tr1::weak_ptr<CArchive>, int> p = m_vctArchives[i]->Find(strViewName);
        if (p.first.lock().get() != NULL) {
            return p;
        }
    }
    return pair<std::tr1::weak_ptr<CArchive>, int>(std::tr1::weak_ptr<CArchive>(), -1);
}

void CArchiveLoader::OnSerialize(ISerializer& r) const {
    r << static_cast<int>(m_vctArchives.size());
    for (size_t i = 0, n = m_vctArchives.size(); i < n; i++) {
        r << CFileName::RemoveDirSeparator(m_vctArchives[i]->GetViewName());
    }
}

void CArchiveLoader::OnRestore(IRestorer& r) {
    Themis_DebugAssert(m_vctArchives.empty());
    int nArchive;
    r << nArchive;
    for (int i = 0; i < nArchive; i++) {
        string strArchiveFile;
        r << strArchiveFile;
        try {
            std::tr1::shared_ptr<CArchive> p(new CArchive(GetTempDir(), owner));
            if (!p->Open(CFileName(strArchiveFile))) {
                throw RuntimeException(L"ɂ̓ǂݍ݂Ɏs܂");
            }
            vector<CArchive::SFileInfo> vct;
            p->GetFileList(vct);
            m_vctArchives.push_back(p);
        } catch (std::exception&) {
            Themis_DebugLog(L"CArchiveLoader::OnRestore()ɏ '" + strArchiveFile + L"' ̓ǂݍ݂Ɏs");
        }
    }
}

string CArchiveLoader::GetFileName(int i) {
    return CFileName::RemoveDirSeparator(m_vctArchives[i]->GetViewName());
}

void CArchiveLoader::AddFileName(const CFileName& fileName) {
    std::tr1::shared_ptr<CArchive> p(new CArchive(GetTempDir(), owner));
    if (!p->Open(fileName)) {
        throw RuntimeException(L" '" + fileName.Get() + L"' ̓ǂݍ݂Ɏs܂");
    }
    vector<CArchive::SFileInfo> vct;
    p->GetFileList(vct);
    m_vctArchives.push_back(p);
}

const CFileName& CArchiveLoader::GetTempDir() {
    return m_vctTempDir[m_vctTempDir.size() - 1].GetName();
}

}
