#include "stdafx.h"
#include "CommDlg.h"
#include "File.h"
#include "Char.h"
#include "Windows/COM.h"
#include "Windows/Environment.h"
#include "Windows/System.h"
#include "String/Encoding.h"
#include "Utility/Algorithm.h"
#include "Utility/StringUtility.h"
#include <commdlg.h>
#include <shlobj.h>
#pragma comment(lib, "comdlg32.lib")
namespace Themis {

namespace {
#ifdef OPENFILENAME_SIZE_VERSION_400
    struct OPENFILENAMEv5 : public OPENFILENAME {};
#else
    #define OPENFILENAME_SIZE_VERSION_400   ((unsigned)sizeof(OPENFILENAME))
    struct OPENFILENAMEv5 : public OPENFILENAME {
        void*   pvReserved;
        DWORD   dwReserved;
        DWORD   FlagsEx;
    };
#endif
}

struct ChooseFileData {
    OPENFILENAMEv5 ofn;
    CFileName oCurDir;
    string strTitle;
    string strExt;
    string strDefalt;
    vector<CFileName> vctResults;
    vector<char> szNameFilter;
};

CChooseFile::CChooseFile() {
    memset(&data->ofn, 0x00, sizeof(data->ofn));
    data->ofn.lStructSize = CSystem::Is2kLater() ?
        sizeof(data->ofn) :
        OPENFILENAME_SIZE_VERSION_400;
    data->ofn.Flags = OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST |
        OFN_HIDEREADONLY | OFN_LONGNAMES | OFN_ENABLESIZING | OFN_OVERWRITEPROMPT;
    // tO 
 // data->ofn.FlagsEx = 0x01000000;
    // Hook
 // data->ofn.Flags |= OFN_ENABLEHOOK;
 // data->ofn.lpfnHook = &CChooseFile::HookProc;
 // data->ofn.lCustData = reinterpret_cast<LPARAM>(this);
    ClearNameFilter();
}

CChooseFile::~CChooseFile() {
}

void CChooseFile::SetCurrentDir(const CFileName& oDir) {
    data->oCurDir = CFile::GetExistDirName(oDir);
    // ݂Ƃ܂ŏɏオĂ
}
CFileName CChooseFile::GetCurrentDir() const {
    return data->oCurDir;
}

void CChooseFile::SetFileMustExist(bool b) {
    if (b) {
        data->ofn.Flags |= OFN_FILEMUSTEXIST;
    } else  {
        data->ofn.Flags &= ~OFN_FILEMUSTEXIST;
    }
}
bool CChooseFile::IsFileMustExist() const {
    return (data->ofn.Flags & OFN_FILEMUSTEXIST) != 0;
}

void CChooseFile::SetDefault(const string& strPureName) {
    data->strDefalt = strPureName;
}

void CChooseFile::SetOwner(HWND hWnd) {
    data->ofn.hwndOwner = hWnd;
}
void CChooseFile::SetTitle(const string& strTitle) {
    data->strTitle = strTitle;
}

void CChooseFile::ClearNameFilter() {
    data->szNameFilter.clear();
    data->szNameFilter.push_back('\0'); //ɂP͏I[LĂ
}

void CChooseFile::AddNameFilter(const string& explanation, const string& fileNameFilter) {
    std::string strExplanation = Encoding::GetDefault().GetBytes(explanation);
    std::string strFileNameFilter = Encoding::GetDefault().GetBytes(fileNameFilter);
    data->szNameFilter.pop_back();
    data->szNameFilter.insert(data->szNameFilter.end(), strExplanation.begin(), strExplanation.end());
    data->szNameFilter.push_back(' ');
    data->szNameFilter.push_back('(');
    data->szNameFilter.insert(data->szNameFilter.end(), strFileNameFilter.begin(), strFileNameFilter.end());
    data->szNameFilter.push_back(')');
    data->szNameFilter.push_back('\0');
    data->szNameFilter.insert(data->szNameFilter.end(), strFileNameFilter.begin(), strFileNameFilter.end());
    data->szNameFilter.push_back('\0');
    data->szNameFilter.push_back('\0');
}
void CChooseFile::SetDefaultExt(const string& strExt) {
    if (strExt.StartsWith(L".")) {
        data->strExt = strExt.Substring(1);
    } else {
        data->strExt = strExt;
    }
}

bool CChooseFile::ShowOpen() {
    return InnerShow(&::GetOpenFileName, false);
}

bool CChooseFile::ShowOpenMulti() {
    return InnerShow(&::GetOpenFileName, true);
}

bool CChooseFile::ShowSave() {
    return InnerShow(&::GetSaveFileName, false);
}

bool CChooseFile::InnerShow(void* pfn, bool bMulti) {
    string currentDir = Environment::GetCurrentDirectory();
    if (!data->oCurDir.IsEmpty()) {
        Environment::SetCurrentDirectory(data->oCurDir.RemoveDirSeparator());
    }
    BOOL bRet;
    try {
        bRet = InnerShow2(pfn, bMulti);
    } catch (std::exception&) {
        Environment::SetCurrentDirectory(currentDir);
        throw;
    }
    Environment::SetCurrentDirectory(currentDir);
    return (bRet != FALSE);
}

BOOL CChooseFile::InnerShow2(void* pfn, bool bMulti) {
    char szBuf[0x2000] = {0};
    if (!string::IsNullOrEmpty(data->strDefalt)) {
        std::string str = Encoding::GetDefault().GetBytes(data->strDefalt);
        memcpy(szBuf, str.c_str(), str.length() + 1);
        data->strDefalt = string::Empty;
    }

    if (bMulti) {
        data->ofn.Flags |= (OFN_ALLOWMULTISELECT | OFN_EXPLORER);
    } else {
        data->ofn.Flags &= ~(OFN_ALLOWMULTISELECT | OFN_EXPLORER);
    }
    std::string curDirStr = Encoding::GetDefault().GetBytes(data->oCurDir.Get());
    std::string titleStr = Encoding::GetDefault().GetBytes(data->strTitle);
    std::string extStr = Encoding::GetDefault().GetBytes(data->strExt);
    data->ofn.lpstrFile = szBuf;
    data->ofn.nMaxFile = lengthof(szBuf);
    data->ofn.lpstrFilter = data->szNameFilter.size() <= 1 ? NULL : &data->szNameFilter[0];
    data->ofn.lpstrInitialDir = curDirStr.empty() ? NULL : curDirStr.c_str();
    data->ofn.lpstrTitle = titleStr.empty() ? NULL : titleStr.c_str();
    data->ofn.lpstrDefExt = extStr.empty() ? NULL : extStr.c_str();

    bool bRetry = false;
LRetry:
    ::ShowCursor(TRUE); ::ShowCursor(TRUE);
    typedef BOOL (WINAPI* FN)(OPENFILENAME*);
    BOOL bRet = static_cast<FN>(pfn)(&data->ofn);
    ::ShowCursor(FALSE); ::ShowCursor(FALSE);

    if (bRet == FALSE) {
        DWORD dwErr = ::CommDlgExtendedError();
        if (!bRetry && dwErr == 0x3002) { // FNERR_INVALIDFILENAME
            // ȃt@CG[ȏꍇAKɒuăgCĂ݂
            ReplaceInvalidChar(data->ofn.lpstrFile);
            bRetry = true;
            goto LRetry;
        } else if (dwErr != 0) {
            throw RuntimeException(L"Ȃ񂩃G[ۂłBG[R[hF" + ItoA(dwErr));
            //<cderr.h>Q
        }
    } else {
        data->vctResults.clear();
        char* p = szBuf;
        size_t n = strlen(p);
        if (!bMulti || p[n + 1] == L'\0') {
            //t@C̏ꍇ
            data->vctResults.push_back(CFileName(Encoding::GetDefault().GetString(p)));
            data->oCurDir = data->vctResults[0].GetDir();
        } else {
            Themis_DebugAssert(bMulti);
            //Ȃŏ̓fBNgĂ
            string strDir = Encoding::GetDefault().GetString(p);
            if (!strDir.EndsWith(L"\\")) {
                strDir += L"\\";
            }
            data->oCurDir = strDir;
            p += n + 1;
            while (true) {
                size_t n = strlen(p);
                if (n <= 0) break;
                data->vctResults.push_back(CFileName(strDir +
                    Encoding::GetDefault().GetString(std::string(p, p + n))));
                p += n + 1;
            }
            //ԋtɂȂ肪Ȃ̂ŕёւƂBc]vȂbH
         // std::reverse(data->vctResults.begin(), data->vctResults.end());
        }
    }
    return bRet;
}

void CChooseFile::ReplaceInvalidChar(char* psz) {
    int i = 0;
    while (psz[i] != L'\0') {
        if (psz[i] == L'\\' || psz[i] == L'/' || (psz[i] == L':' && 2 < i) || psz[i] == L'*' ||
            psz[i] == L'"' || psz[i] == L'?' || psz[i] == L'<' || psz[i] == L'>' || psz[i] == L'|') {
            psz[i] = L'-';
        }
        i += CChar::GetByteCount(psz[i]);
    }
}

/// t@C̎擾
int CChooseFile::GetResultCount() const { return static_cast<int>(data->vctResults.size()); }
/// t@C̎擾
const CFileName& CChooseFile::GetResult(int n) const { return data->vctResults[n]; }
/// ʂ̎擾
const vector<CFileName>& CChooseFile::GetResults() const { return data->vctResults; }

/// OPENFILENAMElpfnHookȁB
UINT CALLBACK CChooseFile::HookProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam) {
    return 0;
}

////////////////////////////////////////////////////////////////////////////////

CChooseDir::CChooseDir(): m_pBrwsInfo(NULL) {
    CCOM::GetInstance().Initialize();
    m_pBrwsInfo = operator new(sizeof(BROWSEINFO));
    memset(m_pBrwsInfo, 0x00, sizeof(BROWSEINFO));
 // static_cast<BROWSEINFO*>(m_pBrwsInfo)->ulFlags = BIF_EDITBOX;
    static_cast<BROWSEINFO*>(m_pBrwsInfo)->ulFlags = BIF_USENEWUI;
        // BIF_RETURNONLYFSDIRS: fBNĝ
        // BIF_USENEWUI == BIF_EDITBOX + BIF_NEWDIALOGSTYLE
 /*
    if (CSystem::Is2kLater()) {
        static_cast<BROWSEINFO*>(m_pBrwsInfo)->ulFlags |= BIF_SHAREABLE;
    }
 //*/
    IMalloc* pMalloc;
    ::SHGetMalloc(&pMalloc);
    m_pMalloc = pMalloc;
}

CChooseDir::~CChooseDir() {
    if (m_pMalloc != NULL)
        static_cast<IMalloc*>(m_pMalloc)->Release();
    operator delete(m_pBrwsInfo);
    CCOM::GetInstance().Terminate();
}

void CChooseDir::SetOwner(HWND hwndOwner) {
    static_cast<BROWSEINFO*>(m_pBrwsInfo)->hwndOwner = hwndOwner;
}

void CChooseDir::SetTitle(const string& szTitle) {
    m_strTitle = szTitle;
}

void CChooseDir::SetDefault(const CFileName& oDir) {
    m_oResult = oDir;
}

bool CChooseDir::Show() {
    m_oResult = CFile::GetExistDirName(m_oResult);
    // ݂Ƃ܂ŏɏオĂ

    //tH_I_CAO̕\
    std::string title = Encoding::GetDefault().GetBytes(m_strTitle);
    static_cast<BROWSEINFO*>(m_pBrwsInfo)->lpszTitle = title.c_str();
    static_cast<BROWSEINFO*>(m_pBrwsInfo)->lpfn = BrowseCallbackProc;
    static_cast<BROWSEINFO*>(m_pBrwsInfo)->lParam = reinterpret_cast<LPARAM>(this);
    ::ShowCursor(TRUE); ::ShowCursor(TRUE);
    LPITEMIDLIST pIML = ::SHBrowseForFolder(static_cast<BROWSEINFO*>(m_pBrwsInfo));
    ::ShowCursor(FALSE); ::ShowCursor(FALSE);
    if (pIML == NULL) {
        return false;
    }
    //t@CVXẽpXɕϊ
    char szBuf[_MAX_PATH];
    if (::SHGetPathFromIDList(pIML, szBuf) == FALSE) {
        return false;
    }
    m_oResult = CFileName::AddDirSeparator(Encoding::GetDefault().GetString(szBuf));
    static_cast<IMalloc*>(m_pMalloc)->Free(pIML);
    return true;
}

int CALLBACK CChooseDir::BrowseCallbackProc(HWND hWnd, UINT uMsg, LPARAM lParam, LPARAM lpData) {
    switch(uMsg) {
    case BFFM_INITIALIZED: //
        {
            CChooseDir* pDlg = reinterpret_cast<CChooseDir*>(lpData);
            //tH_I
            if (!pDlg->m_oResult.IsEmpty()) {
                string str = pDlg->m_oResult.RemoveDirSeparator();
                ::SendMessage(hWnd, BFFM_SETSELECTION, TRUE, reinterpret_cast<LPARAM>(Encoding::GetDefault().GetBytes(str).c_str()));
            }
        }
        break;
    case BFFM_SELCHANGED: //tH_Iꂽ
        try {
            string path = CSystem::GetPath((ITEMIDLIST*)lParam);
            bool enable = !string::IsNullOrEmpty(path) && CFile(CFileName(path)).ExistsDir();
            ::SendMessage(hWnd, BFFM_ENABLEOK, 0, enable ? 1 : 0);
        } catch (RuntimeException& e) {
            Themis_DebugLog(e.what()); e;
            ::SendMessage(hWnd, BFFM_ENABLEOK, 0, 0); // ɁB
        }
        break;
    case BFFM_VALIDATEFAILED:   // ȃACe͂ꂽ
        break;
    }
    return 0;
}

////////////////////////////////////////////////////////////////////////////////

CChooseColor::CChooseColor(): m_cc(NULL), m_rgbResult(RGB(0, 0, 0)) {
    m_cc = operator new(sizeof(CHOOSECOLOR));
    memset(m_cc, 0x00, sizeof(CHOOSECOLOR));
    static_cast<CHOOSECOLOR*>(m_cc)->lStructSize  = sizeof(CHOOSECOLOR);
    static_cast<CHOOSECOLOR*>(m_cc)->lpCustColors = m_argbCustColor;
    memset(m_argbCustColor, 0x00, sizeof(COLORREF) * 0x10);
}

CChooseColor::~CChooseColor() {
    operator delete(m_cc);
}

void CChooseColor::SetOwner(HWND hWnd) {
    static_cast<CHOOSECOLOR*>(m_cc)->hwndOwner = hWnd;
}

void CChooseColor::SetCustColors(COLORREF argbColor[0x10]) {
    memcpy(m_argbCustColor, argbColor, sizeof(COLORREF)*0x10);
}

string CChooseColor::SaveToString() const {
    string str;
    int r = GetRValue(m_rgbResult);
    int g = GetGValue(m_rgbResult);
    int b = GetBValue(m_rgbResult);
    str += string::Format(L"%d,%d,%d ", r, g, b);
    for (int i = 0; i < 0x10; i++) {
        int r = GetRValue(m_argbCustColor[i]);
        int g = GetGValue(m_argbCustColor[i]);
        int b = GetBValue(m_argbCustColor[i]);
        str += string::Format(L"%d,%d,%d ", r, g, b);
    }
    return str;
}

void CChooseColor::LoadFromString(const string& str) {
    vector<string> vct = str.Split(Array(L' '));
    if (vct.size() == 1) { // Âۂꍇ
        vct = str.Split(Array(L'-'));
        if (!vct.empty()) {
            int r = 0, g = 0, b = 0;
            sscanf(Encoding::GetDefault().GetBytes(vct[0]).c_str(), "%d:%d:%d", &r, &g, &b);
            m_rgbResult = RGB(r, g, b);
            for (size_t i = 1, n = Min(static_cast<int>(vct.size()), 0x10 + 1); i < n; i++) {
                int r = 0, g = 0, b = 0;
                sscanf(Encoding::GetDefault().GetBytes(vct[i]).c_str(), "%d:%d:%d", &r, &g, &b);
                m_argbCustColor[i - 1] = RGB(r, g, b);
            }
        }
    }
    if (!vct.empty()) {
        int r = 0, g = 0, b = 0;
        sscanf(Encoding::GetDefault().GetBytes(vct[0]).c_str(), "%d,%d,%d", &r, &g, &b);
        m_rgbResult = RGB(r, g, b);
        for (size_t i = 1, n = Min(static_cast<int>(vct.size()), 0x10 + 1); i < n; i++) {
            int r = 0, g = 0, b = 0;
            sscanf(Encoding::GetDefault().GetBytes(vct[i]).c_str(), "%d,%d,%d", &r, &g, &b);
            // ꌋ\_TCC邯ǁA܂Â郂ł̂Łc
            m_argbCustColor[i - 1] = RGB(r, g, b);
        }
    }
}

bool CChooseColor::Show() {
    static_cast<CHOOSECOLOR*>(m_cc)->Flags = CC_RGBINIT;
    static_cast<CHOOSECOLOR*>(m_cc)->rgbResult = m_rgbResult;
    ::ShowCursor(TRUE); ::ShowCursor(TRUE);
    BOOL bRet = ::ChooseColor(static_cast<CHOOSECOLOR*>(m_cc));
    ::ShowCursor(FALSE); ::ShowCursor(FALSE);
    if (bRet != FALSE) {
        m_rgbResult = static_cast<CHOOSECOLOR*>(m_cc)->rgbResult;
        return true;
    }
    return false;
}

////////////////////////////////////////////////////////////////////////////////

CChooseFont::CChooseFont(): m_cf(NULL) {
    m_cf = operator new(sizeof(CHOOSEFONT));
    memset(m_cf, 0x00, sizeof(CHOOSEFONT));
    static_cast<CHOOSEFONT*>(m_cf)->lStructSize = sizeof(CHOOSEFONT);
    static_cast<CHOOSEFONT*>(m_cf)->Flags = CF_SCREENFONTS | CF_EFFECTS;
    static_cast<CHOOSEFONT*>(m_cf)->nFontType = SCREEN_FONTTYPE;
}

CChooseFont::~CChooseFont() {
    operator delete(m_cf);
}

void CChooseFont::SetOwner(HWND hWnd) {
    static_cast<CHOOSEFONT*>(m_cf)->hwndOwner = hWnd;
}

void CChooseFont::SetColor(COLORREF rgb) {
    static_cast<CHOOSEFONT*>(m_cf)->rgbColors = rgb;
}

bool CChooseFont::Show(LOGFONT& rLogFont, COLORREF& rgbColor) {
    LOGFONT lf = {0};
    static_cast<CHOOSEFONT*>(m_cf)->lpLogFont = &lf;
    ::ShowCursor(TRUE); ::ShowCursor(TRUE);
    BOOL ret = ::ChooseFont(static_cast<CHOOSEFONT*>(m_cf));
    ::ShowCursor(FALSE); ::ShowCursor(FALSE);
    if (ret != FALSE) {
        rLogFont = lf;
        rgbColor = static_cast<CHOOSEFONT*>(m_cf)->rgbColors;
        return true;
    }
    return false;
}

}
