#include "stdafx.h"
#include "Graphics.h"
#include "Window.h"
#include "DebugWindow.h"
#include "String/Encoding.h"
namespace Themis {

CImageWindow& CImageWindow::Get() {
    static CImageWindow oWindow;
    return oWindow;
}

COutputWindow& COutputWindow::Get() {
    static COutputWindow oWindow;
    return oWindow;
}

CLogWindow& CLogWindow::Get() {
    static CLogWindow oWindow;
    return oWindow;
}

//////////////////////////////////////////////////////////////////////

CImageWindow::CImageWindow(const string& strWindowTitle) {
    m_pWindow.reset(new CWindow);
    m_pWindow->GetHook().Add(this);
    m_pWindow->Create(CWindowParameter());
    m_pWindow->SetText(strWindowTitle);
 // m_pWindow->Show();
}

CImageWindow::~CImageWindow() {
    if (m_pWindow.get() != NULL) {
        m_pWindow->PostClose();
        m_pWindow.reset();
    }
}

void CImageWindow::DrawImage(const CPlane& rSrcPlane, const CRect& rc) {
    if (m_pWindow.get() != NULL && m_pWindow->Exists()) {
        CSize szSrc;
        {
            boost::mutex::scoped_lock lock(mutex);
            //sNZtH[}bg𑵂
            if (m_pPlane.get()==NULL || m_pPlane->GetBpp() != rSrcPlane.GetBpp()) {
                m_pPlane.reset(new CDib(rSrcPlane.GetBpp()));
            }
            //傫ĕ`[
            szSrc = rc.GetIntersect(rSrcPlane.GetSize().ToRect()).GetSize();
            m_pPlane->SetSize(szSrc);
            m_pPlane->Blt(rSrcPlane, CPoint(), CRect(rc.GetPoint(), szSrc));
        }

        HWND hWnd = ::GetForegroundWindow();
        m_pWindow->SetSize(szSrc);
        m_pWindow->Show();
        m_pWindow->PostUpdate();
        ::SetForegroundWindow(hWnd);
    }
}

void CImageWindow::Close() {
    m_pWindow->Close();
}

bool CImageWindow::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_PAINT) {
        if (m_pPlane.get() != NULL && m_pPlane->IsCreated()) {
            std::tr1::shared_ptr<PAINTSTRUCT> ps = m_pWindow->BeginPaint();
            CDC(ps->hdc).Blt(CPlane::CGdiLock(*m_pPlane), CRect(ps->rcPaint));
            return false;
        }
    } else if (rPrms.GetMsg() == WM_CLOSE) {
        m_pWindow->Delete();
    }
    return false;
}

//////////////////////////////////////////////////////////////////////

COutputWindow::COutputWindow(const string& strWindowTitle) : m_pPlane(new CDib(16)) {
    m_pFont.reset(new CFont(CFontStyle()));
    m_pPlane->SetSize(CSize(240, 180));
    m_pPlane->Fill(RGB(0x00, 0x00, 0x00));
    m_pWindow.reset(new CWindow);
    m_pWindow->GetHook().Add(this);
    m_pWindow->Create(CWindowParameter());
    m_pWindow->SetClientSize(m_pPlane->GetSize());
    m_pWindow->SetText(strWindowTitle);
    m_pWindow->Show();
}

COutputWindow::~COutputWindow() {
    if (m_pWindow.get() != NULL) {
        m_pWindow->WaitForQuit();
        m_pWindow.reset();
    }
}

void COutputWindow::Out(int nLine, const string& str) {
    if (m_pWindow.get() != NULL && m_pWindow->Exists()) {
        int nLineInterval = m_pFont->GetSize(L"|").cy;   //Ȏ擾@ǖʓ|Ȃ̂ŁBB
        CPoint ptDst(0, nLineInterval*nLine);
        m_pPlane->Fill(RGB(0x00, 0x00, 0x00), CRect(ptDst, CSize(m_pWindow->GetClientSize().cx, m_pFont->GetSize(str).cy)));
        m_pFont->DrawText(CPlane::CGdiLock(*m_pPlane), ptDst, str);
        m_pWindow->Update();
    }
}

void COutputWindow::Close() {
    m_pWindow->Close();
}

bool COutputWindow::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_PAINT) {
        if (m_pPlane->IsCreated()) {
            std::tr1::shared_ptr<PAINTSTRUCT> ps = m_pWindow->BeginPaint();
            CDC(ps->hdc).Blt(CPlane::CGdiLock(*m_pPlane), CRect(ps->rcPaint));
            return false;
        }
    } else if (rPrms.GetMsg() == WM_CLOSE) {
        m_pWindow->Delete();
        return true;
    }
    return false;
}

//////////////////////////////////////////////////////////////////////

CLogWindow::CLogWindow(const string& strWindowTitle) {
    CWindowParameter oOpt;
    oOpt.SetClassName(L"LISTBOX");
    oOpt.GetStyle() = LBS_NOINTEGRALHEIGHT |
        WS_CAPTION | WS_SYSMENU | WS_MAXIMIZEBOX | WS_MINIMIZEBOX |
        WS_VSCROLL | WS_HSCROLL | WS_SIZEBOX | WS_BORDER;
    oOpt.SetActiveOnCreate(false);
    m_pWindow.reset(new CWindow);
    m_pWindow->GetHook().Add(this);
    m_pWindow->Create(oOpt);
    m_pWindow->SetClientSize(CSize(640, 640));
    m_pWindow->SetText(strWindowTitle);
    m_pWindow->Show(SW_SHOWNOACTIVATE);
}

CLogWindow::~CLogWindow() {
    if (m_pWindow.get() != NULL) {
        m_pWindow->WaitForQuit();
        m_pWindow.reset();
    }
}

void CLogWindow::Out(const string& str) {
    if (m_pWindow.get() != NULL && m_pWindow->Exists()) {
        m_pWindow->SendMsg(LB_ADDSTRING, 0, reinterpret_cast<LPARAM>(Encoding::GetDefault().GetBytes(str).c_str()));
        UINT n = static_cast<UINT>(m_pWindow->SendMsg(LB_GETCOUNT, 0, 0));
        if (n != LB_ERR) {
            m_pWindow->SendMsg(LB_SETCURSEL, n-1, 0);
        }
    }
}

void CLogWindow::Clear() {
    if (m_pWindow.get() != NULL && m_pWindow->Exists()) {
        m_pWindow->SendMsg(LB_RESETCONTENT, 0, 0);
    }
}

void CLogWindow::SetClientSize(const CSize& sz) {
    if (m_pWindow.get() != NULL && m_pWindow->Exists()) {
        m_pWindow->SetClientSize(sz);
    }
}

void CLogWindow::Close() {
    m_pWindow->Close();
}

bool CLogWindow::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_CLOSE) {
        m_pWindow->Delete();
        return true;
    }
    return false;
}

}
