#pragma once
#include "Windows/PushErrorMode.h"
#include "Windows/NativeString.h"
#include "String/Encoding.h"
namespace Themis {

/// t@C֌WŃG[NƂpO
class IOException : public RuntimeException {
protected:
    IOException() {}
public:
    IOException(const string& str) : RuntimeException(str) {}
};

/**
    <stdio.h>̓ǂݍ݂ƃ̓ǂݍ݂
    C^[tF[Xōsׂ̊NXB

    I}PFILE*̎擾@\(ł̓e|t@Cg)
    ڎ擾(FILE*ł͌Ă΂ꂽǂݍłQxڈȍ~̓LbV)B
*/
class IFile : private boost::noncopyable {
    CFileName m_oFileName;
    IFile(const IFile&);
    IFile& operator=(const IFile&);
public:
    IFile(const CFileName& oFileName) : m_oFileName(oFileName) {}
    virtual ~IFile() {}

    /// RTTIB0:t@CA1:stdioȃt@CA2:WinAPIȃt@C
    virtual int GetType() const = 0;

    //@{ stdioۂC^[tF[X{GetSize()
    virtual void Seek(long offset, int origin) = 0;
    virtual long Tell() const = 0;
    virtual size_t GetSize() const = 0;
    virtual void Read(void* buffer, size_t size) = 0;
    virtual void Write(const void* buffer, size_t size) = 0;
    virtual void Reserve(size_t size) = 0;
    virtual void Flush() const = 0;
    //@}

    /// t@Cnh̎擾
    virtual FILE* GetHandle() = 0;

    /// t@C̎擾BɃG[\pB
    const CFileName& GetName() const { return m_oFileName; }
};

/**
 * IFilestdio
 *
 * stdiõt@C색bp[ƂB
 */
class CStdFile : public IFile {
    FILE* fp;
public:
    CStdFile(const CFileName& oFileName, const string& mode) : IFile(oFileName), fp(NULL) {
        Themis_DebugAssert(!GetName().IsRelativePath());
     // fp = _wfopen((L"\\\\?\\" + GetName().Get()).ToCharPtr(), mode.ToCharPtr());
        fp = _wfopen(GetName().Get().ToCharPtr(), mode.ToCharPtr());
        if (fp == NULL) {
            throw IOException(GetName().Get() + L" J܂ł");
        }
    }
    ~CStdFile() {
        if (fp != NULL) fclose(fp);
    }

    int GetType() const { return 1; }
    void Seek(long offset, int origin) {
        if (fseek(fp, offset, origin) != 0) {
            throw IOException(GetName().Get() + L" ̃t@C|C^̐ݒɎs܂");
        }
    }
    long Tell() const { return ftell(fp); }
    size_t GetSize() const {
        long n = ftell(fp);
        if (fseek(fp, 0, SEEK_END) != 0) {
            throw IOException(GetName().Get() + L" ̃t@C|C^̐ݒɎs܂");
        }
        size_t nSize = ftell(fp);
        if (fseek(fp, n, SEEK_SET) != 0) {
            throw IOException(GetName().Get() + L" ̃t@C|C^̐ݒɎs܂");
        }
        return nSize;
    }
    void Read(void* buffer, size_t size) {
        if (0 < size && fread(buffer, size, 1, fp) != 1) {
            throw IOException(GetName().Get() + L" ̓ǂݍ݂Ɏs܂");
        }
    }
    void Write(const void* buffer, size_t size) {
        if (0 < size && fwrite(buffer, size, 1, fp) != 1) {
            throw IOException(GetName().Get() + L" ւ݂̏Ɏs܂");
        }
    }
    void Reserve(size_t size) {
        if (1024 < size) { // KBӖȂtʂ̋CȂȂB
            long n = ftell(fp);
            fseek(fp, SEEK_SET, static_cast<int>(size - 1));
            size_t nRet = fwrite(L"", 1, 1, fp);
            Themis_DebugAssert(nRet == 1);
            fseek(fp, SEEK_SET, n);
        }
    }
    void Flush() const { fflush(fp); }
    FILE* GetHandle() { return fp; }
};

/**
 * IFilevector&lt;uint8_t&gt;ŁB
 */
class CMemoryFile : public IFile {
    vector<uint8_t> m_memory;
    size_t m_pos;
    auto_ptr<CStdFile> m_tmpfile;
public:
    CMemoryFile(const CFileName& oFileName, const vector<uint8_t>& r = vector<uint8_t>())
        : IFile(oFileName), m_memory(r), m_pos(0) {}
    ~CMemoryFile() {
        if (m_tmpfile.get() != NULL) {  //e|t@CĂ폜
            string str = m_tmpfile->GetName().Get();
            m_tmpfile.reset();
            remove(Encoding::GetDefault().GetBytes(str).c_str());
        }
    }

    int GetType() const { return 0; }
    void Seek(long offset, int origin) {
        switch (origin) {
        case SEEK_SET: m_pos = offset; break;
        case SEEK_CUR: m_pos += offset; break;
        case SEEK_END: m_pos = m_memory.size() - offset; break;
        default: Themis_DebugTrespass();
        }
    }
    long Tell() const { return static_cast<long>(m_pos); }
    size_t GetSize() const { return m_memory.size(); }
    void Read(void* buffer, size_t size) {
        if (m_memory.size() < m_pos + size) {
            throw IOException(GetName().Get() + L" ̓ǂݍ݂Ɏs܂");
        }
        memcpy(buffer, &m_memory[m_pos], size);
        m_pos += size;
    }
    void Write(const void* buffer, size_t size) {
        Themis_DebugAssert(m_pos <= m_memory.size());
        if (m_memory.size() < m_pos + size) {
            m_memory.resize(m_pos + size);
        }
        memcpy(&m_memory[m_pos], buffer, size);
    }
    void Reserve(size_t size) { m_memory.reserve(size); }
    void Flush() const {}
    FILE* GetHandle() {
        if (m_tmpfile.get() == NULL) {
            string file = string::Format(L"\\~th(%x-%p).tmp", ::GetCurrentProcessId(), this);
            char szDir[_MAX_PATH];
            ::GetTempPath(_MAX_PATH, szDir);
            CFileName fileName(Encoding::GetDefault().GetString(szDir) + file);
            m_tmpfile.reset(new CStdFile(fileName, L"w+b"));
            if (!m_memory.empty()) {
                m_tmpfile->Write(&m_memory[0], m_memory.size());
            }
        }
        return m_tmpfile->GetHandle();
    }

    /// ڎ擾
    vector<uint8_t>& GetMemory() { return m_memory; }
};

/**
 * APIgIFile̎B
 */
class CApiFile : public IFile {
    HANDLE handle;
public:
    enum EMode {
        eModeOpen,
        eModeCreate,
        eModeOpenNoLock, // bNȂŊJBRead()ƂˑRG\AH
    };
    enum EAccess {
        eAccessNormal,
        eAccessSequential,
        eAccessRandom,
    };

    CApiFile(const CFileName& oFileName, EMode eMode, EAccess eAccess = eAccessNormal)
        : IFile(oFileName), handle(INVALID_HANDLE_VALUE) {
        DWORD dwAccess, dwShare, dwDesp, dwAttr;
        switch (eMode) {
        case eModeOpen:
            dwDesp = OPEN_EXISTING;
            dwAccess = GENERIC_READ;
            dwShare = FILE_SHARE_READ;
            break;
        case eModeCreate:
            dwDesp = CREATE_ALWAYS;
            dwAccess = GENERIC_READ | GENERIC_WRITE;
            dwShare = 0;
            break;
        case eModeOpenNoLock:
            dwDesp = OPEN_EXISTING;
            dwAccess = GENERIC_READ;
            dwShare = FILE_SHARE_READ | FILE_SHARE_WRITE;
            if (NativeString::IsUnicode()) {
                dwShare |= FILE_SHARE_DELETE;
            }
            break;
        default:
            Themis_DebugTrespass();
        }
        switch (eAccess) {
        case eAccessNormal: dwAttr = FILE_ATTRIBUTE_NORMAL; break;
        case eAccessSequential: dwAttr = FILE_FLAG_SEQUENTIAL_SCAN; break;
        case eAccessRandom: dwAttr = FILE_FLAG_RANDOM_ACCESS; break;
        default: Themis_DebugTrespass();
        }
     // string fileName = L"\\\\?\\" + oFileName.Get();
        string fileName = oFileName.Get();
        handle = NativeString::IsUnicode() ?
            ::CreateFileW(fileName.ToCharPtr(),                              dwAccess, dwShare, NULL, dwDesp, dwAttr, NULL) :
            ::CreateFileA(Encoding::GetDefault().GetBytes(fileName).c_str(), dwAccess, dwShare, NULL, dwDesp, dwAttr, NULL);
        if (handle == INVALID_HANDLE_VALUE) {
            Themis_DebugApiLog();
            throw IOException(GetName().Get() + L" ̃I[vE쐬Ɏs");
        }
    }
    ~CApiFile() {
        if (handle != INVALID_HANDLE_VALUE) {
            // AzȏSPIςȂƂႤ悤Ȃ̂ő΍(L`)
            PushErrorMode errorMode(SEM_FAILCRITICALERRORS | SEM_NOGPFAULTERRORBOX | SEM_NOOPENFILEERRORBOX);
            BOOL result = ::CloseHandle(handle);
            Themis_DebugAssertApiLog(result != FALSE);
        }
    }

    int GetType() const { return 2; }
    void Seek(long offset, int origin) {
        DWORD dwMethod;
        if (origin == SEEK_SET) {
            dwMethod = FILE_BEGIN;
        } else if (origin == SEEK_CUR) {
            dwMethod = FILE_CURRENT;
        } else {
            Themis_DebugAssert(origin == SEEK_END);
            dwMethod = FILE_END;
        }
        if (::SetFilePointer(handle, offset, NULL, dwMethod) == INVALID_SET_FILE_POINTER) {
            Themis_DebugApiLog();
            throw IOException(GetName().Get() + L" ̃t@C|C^̐ݒɎs");
        }
    }
    long Tell() const {
        return ::SetFilePointer(handle, 0, NULL, FILE_CURRENT);
    }
    size_t GetSize() const {
        DWORD dw = ::GetFileSize(handle, NULL);
        if (dw == static_cast<DWORD>(-1)) {
            throw IOException(GetName().Get() + L" ̃t@CTCY̎擾Ɏs");
        }
        return dw;
    }
    void Read(void* buffer, size_t size) {
        DWORD dwSize = 0;
        if (0 < size && ::ReadFile(handle, buffer, static_cast<DWORD>(size), &dwSize, NULL) == FALSE) {
            Themis_DebugApiLog();
            throw IOException(GetName().Get() + L" ̓ǂݍ݂Ɏs");
        }
    }
    void Write(const void* buffer, size_t size) {
        DWORD dwSize = 0;
        if (0 < size && ::WriteFile(handle, buffer, static_cast<DWORD>(size), &dwSize, NULL) == FALSE) {
            Themis_DebugApiLog();
            throw IOException(GetName().Get() + L" ւ݂̏Ɏs");
        }
    }
    void Reserve(size_t size) {
        if (1024 < size) {
            long n = Tell();
            Seek(static_cast<long>(size), SEEK_SET);    //ӂ̊댯AȂ̂ŒӁc
            ::SetEndOfFile(handle);
            Seek(n, SEEK_SET);
        }
    }
    void Flush() const { ::FlushFileBuffers(handle); }
    FILE* GetHandle() { Themis_DebugWarning(L"CApiFile::GetHandle()Ă΂ꂽ"); return NULL; }
    HANDLE GetApiHandle() { return handle; }
};

}
