#include "../../TimeChecker.h"
namespace Themis {

namespace {
    static bool useMMX = true;
    static bool use3DNow = true;
    static bool useParallel = true;
}
/// MMXĝ
bool CPlane::IsUseMMX() { return useMMX; }
/// MMXĝ
void CPlane::SetUseMMX(bool b) { useMMX = b; }
/// 3DNowĝ
bool CPlane::IsUse3DNow() { return use3DNow; }
/// 3DNowĝ
void CPlane::SetUse3DNow(bool b) { use3DNow = b; }
/// Parallelĝ
bool CPlane::IsUseParallel() { return useParallel; }
/// Parallelĝ
void CPlane::SetUseParallel(bool b) { useParallel = b; }

void CPlane::SetSize(const CSize& sz) {
    if (!IsCreated() || m_pImpl->IsReAllocatable()) {
        if (0 < sz.cx && 0 < sz.cy) {
            m_pImpl->Create(sz, GetBpp());
        }
    } else {
        CSize szOld = GetCapacity();
        if (szOld.cx < sz.cx) {
            m_pImpl->Create(CSize(sz.cx, Max(szOld.cy, sz.cy)), GetBpp());
        } else if (szOld.cy < sz.cy) {
            m_pImpl->Create(CSize(szOld.cx, sz.cy), GetBpp());
        } else {
            /*
            // cɔ{ȏȂ_CGbgĂ݂B
            if (sz.cx * 2 < GetSize().cx &&
                sz.cy * 2 < GetSize().cy ) {
                m_pImpl->Create(GetSize() / 2, GetBpp());
                Themis_DebugLog(string::Format(L"CPlaneߖ񏈗: (%d x %d) => (%d x %d)", GetSize(), GetSize() / 2));
            }
            //*/
        }
    }
    m_oSize = sz;
}

void CPlane::Fill(COLORREF rgb) {
    CRect rcDst(0, 0, GetSize());
    Fill(rgb, rcDst);
}

void CPlane::Fill(COLORREF rgb, CRect rcDst) {
    if (CClipper(GetSize()).Clipping(rcDst)) {
        CPlane::CLock oDstLock(*this);
        uint8_t* pby = oDstLock.GetByte(rcDst.GetPoint());
        uint16_t* pw = oDstLock.GetWord(rcDst.GetPoint());
        uint32_t* pdw = oDstLock.GetDWord(rcDst.GetPoint());
        const int nDstPitch = oDstLock.GetPitch();
        const unsigned r = GetRValue(rgb);
        const unsigned g = GetGValue(rgb);
        const unsigned b = GetBValue(rgb);
        const CSize size = rcDst.GetSize();
        const int nBpp = GetBpp();
        if (nBpp == 8) {
            const uint8_t byPixel = static_cast<uint8_t>((r*77 + g*151 + b*28) >> 8);
            for (int y=0, cy=size.cy; y<cy; y++) {
                for (int x=0, cx=size.cx; x<cx; x++) {
                    pby[x] = byPixel;
                }
                pby += nDstPitch / sizeof(uint8_t);
            }
        } else if (nBpp == 16) {
            const unsigned nr = r >> 3;
            const unsigned ng = g >> 3;
            const unsigned nb = b >> 3;
            const uint16_t wPixel = static_cast<uint16_t>((nr<<10) | (ng<<5) | nb);
            for (int y=0, cy=size.cy; y<cy; y++) {
                for (int x=0, cx=size.cx; x<cx; x++) {
                    pw[x] = wPixel;
                }
                pw += nDstPitch / sizeof(uint16_t);
            }
        } else {
            //24C32
            if (nBpp == 24) {
                for (int y=0, cy=size.cy; y<cy; y++) {
                    for (int x=0, cx=size.cx*3; x<cx; x+=3) {
                        pby[x+0] = static_cast<uint8_t>(b);
                        pby[x+1] = static_cast<uint8_t>(g);
                        pby[x+2] = static_cast<uint8_t>(r);
                    }
                    pby += nDstPitch / sizeof(uint8_t);
                }
            } else {  //32
                const uint32_t dwPixel = (r<<16) | (g<<8) | b;
                for (int y=0, cy=size.cy; y<cy; y++) {
                    for (int x=0, cx=size.cx; x<cx; x++) {
                        pdw[x] = dwPixel;
                    }
                    pdw += nDstPitch / sizeof(uint32_t);
                }
            }
        }
    }
}

void CPlane::Blt(const CPlane& rSrc, const CClipper& rClip) {
    if (this != &rSrc) {
        Blt(rSrc, CPoint(), CRect(rSrc.GetSize()), rClip);
    }
}

void CPlane::Blt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        CPlane::CLock oDstLock(*this);
        CPlane::CLock oSrcLock(rSrc);
        uint8_t* pbyDst = oDstLock.GetByte(ptDst);
        uint8_t* pbySrc = oSrcLock.GetByte(rcSrc.GetPoint());
        uint16_t* pwDst = oDstLock.GetWord(ptDst);
        uint16_t* pwSrc = oSrcLock.GetWord(rcSrc.GetPoint());
        uint32_t* pdwDst = oDstLock.GetDWord(ptDst);
        uint32_t* pdwSrc = oSrcLock.GetDWord(rcSrc.GetPoint());
        const int nDstPitch = oDstLock.GetPitch();
        const int nSrcPitch = oSrcLock.GetPitch();
        const CSize size = rcSrc.GetSize();
        const int nBppDst = GetBpp();
        const int nBppSrc = rSrc.GetBpp();
        if (nBppDst == nBppSrc) {
          #if 1
            if (nBppDst == 32) {
          #else
            if (false) {
          #endif
              /*
                for (int y = 0, cy = size.cy; y < cy; y++) {
                    for (int x = 0, cx = size.cx; x < cx; x++) {
                        pdwDst[i] = pdwSrc[i];
                    }
                    pdwDst += nDstPitch / sizeof(uint32_t);
                    pdwSrc += nSrcPitch / sizeof(uint32_t);
                }
              /*/
                int nSizeX = size.cx;
                int nSizeY = size.cy;
                int nDstPitchRemain = nDstPitch - nSizeX * 4;
                int nSrcPitchRemain = nSrcPitch - nSizeX * 4;
                _asm {
                    cld
                    mov ebx, nSizeX
                    mov eax, nSizeY
                    mov edi, pbyDst
                    mov esi, pbySrc
                    mov ecx, ebx
                ___loop32:
                        rep movsd
                        add edi, nDstPitchRemain
                        add esi, nSrcPitchRemain
                        mov ecx, ebx
                    dec eax
                    jnz ___loop32
                }
              //*/
            } else {
              /*
                int nSize = size.cx * nBppDst / 8;
                for (int y=0, cy=size.cy; y<cy; y++) {
                    memcpy(pbyDst, pbySrc, nSize);
                    pbyDst += nDstPitch / sizeof(uint8_t);
                    pbySrc += nSrcPitch / sizeof(uint8_t);
                }
              /*/
                const int nSizeY = size.cy;
                Themis_DebugAssert(0 < nSizeY);
                const int nByteSize = size.cx * nBppDst / 8;
                const int nDWordSize = nByteSize / 4;         //SoCg]ȃTCY
                const int nByteRemain = nByteSize % 4;        //c
                const int nDstRemainPitch = nDstPitch - nByteSize;
                const int nSrcRemainPitch = nSrcPitch - nByteSize;
                _asm {
                    cld
                    mov edx, nByteRemain
                    mov ebx, nDWordSize
                    mov edi, pbyDst
                    mov esi, pbySrc
                    mov eax, nSizeY
                    mov ecx, ebx    ; ecx = ebx = nDWordSize
                ___loop:
                        rep movsd
                        mov ecx, edx
                        rep movsb
                        add edi, nDstRemainPitch
                        add esi, nSrcRemainPitch
                        mov ecx, ebx    ; ecx = ebx = nDWordSize
                    dec eax
                    jnz ___loop
                }
              //*/
            }
        } else if (nBppDst == 8) {
            //  bright = r*0.3 + g*0.59 + b*.11
            //        == (r*77 + g*151 + b*28) >> 8
            if (nBppSrc == 16) {
                uint16_t awTableB[1<<5];
                uint16_t awTableG[1<<5];
                uint16_t awTableR[1<<5];
                for (int i = 0; i<(1<<5); i++) {
                    const int n = (i<<3) | (i>>2);    //5bit -> 8bit
                    awTableB[i] = static_cast<uint16_t>(n * 28);
                    awTableG[i] = static_cast<uint16_t>(n * 151);
                    awTableR[i] = static_cast<uint16_t>(n * 77);
                }
                for (int y=0, cy=size.cy; y<cy; y++) {
                    for (int x=0, cx=size.cx; x<cx; x++) {
                        pbyDst[x] = static_cast<uint8_t>((
                            awTableB[(pwSrc[x]      ) & 0x1f] +
                            awTableG[(pwSrc[x] >> 5 ) & 0x1f] +
                            awTableR[(pwSrc[x] >> 10) & 0x1f]
                            ) >> 8);
                    }
                    pbyDst += nDstPitch / sizeof(uint8_t);
                    pwSrc += nSrcPitch / sizeof(uint16_t);
                }
            } else if (nBppSrc == 24) {
                uint16_t awTableB[0x100];
                uint16_t awTableG[0x100];
                uint16_t awTableR[0x100];
                for (int i = 0; i < 0x100; i++) {
                    awTableB[i] = static_cast<uint16_t>(i * 28);
                    awTableG[i] = static_cast<uint16_t>(i * 151);
                    awTableR[i] = static_cast<uint16_t>(i * 77);
                }
                for (int y=0, cy=size.cy; y<cy; y++) {
                    for (int x=0, x3=0, cx=size.cx; x<cx; x++, x3+=3) {
                        pbyDst[x] = static_cast<uint8_t>((
                            awTableB[pbySrc[x3+0]] +
                            awTableG[pbySrc[x3+1]] +
                            awTableR[pbySrc[x3+2]] )
                            >> 8);
                    }
                    pbyDst += nDstPitch / sizeof(uint8_t);
                    pbySrc += nSrcPitch / sizeof(uint8_t);
                }
            } else if (nBppSrc == 32) {
                uint16_t awTableB[0x100];
                uint16_t awTableG[0x100];
                uint16_t awTableR[0x100];
                for (int i = 0; i < 0x100; i++) {
                    awTableB[i] = static_cast<uint16_t>(i * 28);
                    awTableG[i] = static_cast<uint16_t>(i * 151);
                    awTableR[i] = static_cast<uint16_t>(i * 77);
                }
                for (int y=0, cy=size.cy; y<cy; y++) {
                    for (int x=0, cx=size.cx; x<cx; x++) {
                        pbyDst[x] = static_cast<uint8_t>(static_cast<unsigned>(
                            awTableB[(pdwSrc[x]      ) & 0xff] +
                            awTableG[(pdwSrc[x] >> 8 ) & 0xff] +
                            awTableR[(pdwSrc[x] >> 16) & 0xff] )
                            >> 8);
                    }
                    pbyDst += nDstPitch / sizeof(uint8_t);
                    pdwSrc += nSrcPitch / sizeof(uint32_t);
                }
            }
        } else if (nBppDst == 16) {
            if (nBppSrc == 8) {
                //            bbbb bbbb
                //  xrrr rrgg gggb bbbb
                for (int y=0, cy=size.cy; y<cy; y++) {
                    for (int x=0, cx=size.cx; x<cx; x++) {
                        pwDst[x] = static_cast<uint16_t>(
                            (static_cast<unsigned>((pbySrc[x]) >> 3)) |              //b
                            (static_cast<unsigned>((pbySrc[x]) << 2) & 0x03e0) |     //g
                            (static_cast<unsigned>((pbySrc[x]) << 7) & 0x7c00) );    //r
                    }
                    pwDst += nDstPitch / sizeof(uint16_t);
                    pbySrc += nSrcPitch / sizeof(uint8_t);
                }
            } else if (nBppSrc == 24) {
                //rrrr rrrr gggg gggg bbbb bbbb
                //          xrrr rrgg gggb bbbb
                for (int y=0, cy=size.cy; y<cy; y++) {
                    for (int x=0, x3=0, cx=size.cx; x<cx; x++, x3+=3) {
                        pwDst[x] = static_cast<uint16_t>(
                            (static_cast<unsigned>(pbySrc[x3+0]) >> 3) |           //b
                            (static_cast<unsigned>(pbySrc[x3+1]) << 2) & 0x03e0 |  //g
                            (static_cast<unsigned>(pbySrc[x3+2]) << 7) & 0x7c00 ); //r
                    }
                    pwDst += nDstPitch / sizeof(uint16_t);
                    pbySrc += nSrcPitch / sizeof(uint8_t);
                }
            } else {    // 32
                //aaaa aaaa rrrr rrrr gggg gggg bbbb bbbb
                //                    xrrr rrgg gggb bbbb
                for (int y=0, cy=size.cy; y<cy; y++) {
                    for (int x=0, cx=size.cx; x<cx; x++) {
                        pwDst[x] = static_cast<uint16_t>(
                            (pdwSrc[x] >> 9) & 0x7c00 | //b
                            (pdwSrc[x] >> 6) & 0x03e0 | //g
                            (pdwSrc[x] >> 3) & 0x001f); //r
                    }
                    pwDst += nDstPitch / sizeof(uint16_t);
                    pdwSrc += nSrcPitch / sizeof(uint32_t);
                }
            }
        } else if (nBppDst == 24) {
            if (nBppSrc == 8) {
                for (int y=0, cy=size.cy; y<cy; y++) {
                    for (int x=0, x3=0, cx=size.cx; x<cx; x++, x3+=3) {
                        pbyDst[x3+0] = pbySrc[x];
                        pbyDst[x3+1] = pbySrc[x];
                        pbyDst[x3+2] = pbySrc[x];
                    }
                    pbyDst += nDstPitch / sizeof(uint8_t);
                    pbySrc += nSrcPitch / sizeof(uint8_t);
                }
            } else if (nBppSrc == 16) {
                //          xrrr rrgg gggb bbbb
                //rrrr rrrr gggg gggg bbbb bbbb
                uint8_t aTable5[0x20]; //5т  8тƂ̕ϊĂԂ
                for (int i = 0; i<0x20; i++) {
                    aTable5[i] = static_cast<uint8_t>((i<<3) | (i>>2));
                }
                for (int y=0, cy=size.cy; y<cy; y++) {
                    for (int x=0, x3=0, cx=size.cx; x<cx; x++, x3+=3) {
                        pbyDst[x3+0] = aTable5[(pwSrc[x] & 0x001f)];         //b
                        pbyDst[x3+1] = aTable5[(pwSrc[x] & 0x03e0) >> 5];    //g
                        pbyDst[x3+2] = aTable5[(pwSrc[x] & 0x7c00) >> 10];   //r
                    }
                    pbyDst += nDstPitch / sizeof(uint8_t);
                    pwSrc += nSrcPitch / sizeof(uint16_t);
                }
            } else {    // 32
                //aaaa aaaa rrrr rrrr gggg gggg bbbb bbbb
                //          rrrr rrrr gggg gggg bbbb bbbb
                for (int y=0, cy=size.cy; y<cy; y++) {
                    for (int x=0, x3=0, cx=size.cx; x<cx; x++, x3+=3) {
                        pbyDst[x3+0] = static_cast<uint8_t>(pdwSrc[x]);         //b
                        pbyDst[x3+1] = static_cast<uint8_t>(pdwSrc[x] >> 8);    //g
                        pbyDst[x3+2] = static_cast<uint8_t>(pdwSrc[x] >> 16);   //r
                    }
                    pbyDst += nDstPitch / sizeof(uint8_t);
                    pdwSrc += nSrcPitch / sizeof(uint32_t);
                }
            }
        } else {  // 32
            if (nBppSrc == 8) {
                for (int y=0, cy=size.cy; y<cy; y++) {
                    for (int x=0, cx=size.cx; x<cx; x++) {
                        pdwDst[x] = (pbySrc[x]) |
                            (static_cast<uint32_t>(pbySrc[x]) << 8) |
                            (static_cast<uint32_t>(pbySrc[x]) << 16);
                    }
                    pdwDst += nDstPitch / sizeof(uint32_t);
                    pbySrc += nSrcPitch / sizeof(uint8_t);
                }
            } else if (nBppSrc == 16) {
                //                    xrrr rrgg gggb bbbb
                //aaaa aaaa rrrr rrrr gggg gggg bbbb bbbb
                unsigned aTable5[0x20];    //5т  8тƂ̕ϊĂԂ
                for (int i = 0; i<0x20; i++) {
                    aTable5[i] = (i<<3) | (i>>2);
                }
                for (int y=0, cy=size.cy; y<cy; y++) {
                    for (int x=0, cx=size.cx; x<cx; x++) {
                        pdwDst[x] = aTable5[(pwSrc[x] & 0x001f)]                //b
                                  | aTable5[(pwSrc[x] & 0x03e0) >> 5] << 8      //g
                                  | aTable5[(pwSrc[x] & 0x7c00) >> 10] << 16;   //r
                    }
                    pdwDst += nDstPitch / sizeof(uint32_t);
                    pwSrc += nSrcPitch / sizeof(uint16_t);
                }
            } else {    // 24
                Detail::Blt24to32(pdwDst, nDstPitch / sizeof(uint32_t), pbySrc, nSrcPitch / sizeof(uint8_t), size.cx, size.cy);
            }
        }
    }
}

void CPlane::BltFromDIBits(const BITMAPINFO& rBmpInfo, const void* pBits, CPoint ptDst, CRect rcSrc, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), CSize(abs(rBmpInfo.bmiHeader.biWidth), abs(rBmpInfo.bmiHeader.biHeight)), ptDst, rcSrc)) {
        if (GetBpp() == 32) {
            BltFromDIBits32(rBmpInfo, pBits, ptDst, rcSrc, rClip);
        } else {
            Themis_DebugLog(L"rbg}bv̕`GDIgp܂ (]BPPF" + string::Format(L"%d", GetBpp()) + L")");
            BltFromDIBitsGDI(rBmpInfo, pBits, ptDst, rcSrc, rClip);
        }
    }
}

void CPlane::BltFromDIBits32(const BITMAPINFO& rBmpInfo, const void* p, CPoint ptDst, CRect rcSrc, const CClipper& rClip) {
    //ǂݍ݂ʓ|łȂ`ۂȂ玩Oœǂݍ
    //(HDCɂfoCXւ̈ˑ邽߂Ȃ񂾂ǂȂ蔲)
    const uint8_t* pBits = static_cast<const uint8_t*>(p);
    if (pBits != NULL && 0 < rBmpInfo.bmiHeader.biWidth && 0 < rBmpInfo.bmiHeader.biHeight && rBmpInfo.bmiHeader.biCompression == BI_RGB) {
        const int sw = rcSrc.GetWidth();
        const int sh = rcSrc.GetHeight();
        if (rBmpInfo.bmiHeader.biBitCount == 24) {
            int nSrcPitch = (sw * 3 + 3) & ~3;
            CPlane::CLock oLock(*this);
            uint32_t* p = oLock.GetDWord(CPoint(ptDst.x, sh - 1 - ptDst.y));
            int nPitch = oLock.GetPitch() / sizeof(*p);
        #if 0
            CTimeCheckerLog oTimeChecker(L"Blt24to32");
            for (int i = 0; i < 1000; i++)
        #endif
            Detail::Blt24to32(p, - nPitch, pBits, nSrcPitch, sw, sh);
        } else if (rBmpInfo.bmiHeader.biBitCount == 8) {
            const RGBQUAD* pPalette = rBmpInfo.bmiColors;
            int nSrcPitch = (sw * 1 + 3) & ~3;
            CPlane::CLock oLock(*this);
            uint32_t* p = oLock.GetDWord(CPoint(ptDst.x, sh - 1 - ptDst.y));
            int nPitch = oLock.GetPitch() / sizeof(*p);
            for (int y = 0, cy = sh; y < cy; y++) {
                for (int x = 0, cx = sw; x < cx; x++) {
                    p[x] = static_cast<uint32_t>(pPalette[pBits[x]].rgbBlue) |
                        (static_cast<uint32_t>(pPalette[pBits[x]].rgbGreen) << 8) |
                        (static_cast<uint32_t>(pPalette[pBits[x]].rgbRed) << 16);
                }
                p -= nPitch;
                pBits += nSrcPitch;
            }
        } else if (rBmpInfo.bmiHeader.biBitCount == 32) {
            int nSrcPitch = sw * 4; // + 3) & ~3
            CPlane::CLock oLock(*this);
            uint32_t* p = oLock.GetDWord(CPoint(ptDst.x, sh - 1 - ptDst.y));
            int nPitch = oLock.GetPitch() / sizeof(*p);
            for (int y = 0, cy = sh; y < cy; y++) {
                memcpy(p, pBits, sw * sizeof(uint32_t));
                p -= nPitch;
                pBits += nSrcPitch;
            }
        } else {
            Themis_DebugLog(string::Format(L"rbg}bv̕`GDIgp܂ (nBpp:%d)", rBmpInfo.bmiHeader.biBitCount));
            BltFromDIBitsGDI(rBmpInfo, pBits, ptDst, rcSrc, rClip);
        }
    } else {
        Themis_DebugLog(string::Format(L"rbg}bv̕`GDIgp܂ (pBits:%p, nWidth:%d, nHeight:%d, nComp:%d)",
            pBits, rBmpInfo.bmiHeader.biWidth, rBmpInfo.bmiHeader.biHeight, rBmpInfo.bmiHeader.biCompression));
        BltFromDIBitsGDI(rBmpInfo, pBits, ptDst, rcSrc, rClip);
    }
}

void CPlane::BltFromDIBitsGDI(const BITMAPINFO& rBmpInfo, const void* pBits, CPoint ptDst, CRect rcSrc, const CClipper& rClip) {
    int w = rcSrc.GetWidth();
    int h = rcSrc.GetHeight();
 // int nRet = ::StretchDIBits(CPlane::CGdiLock(*this), ptDst.x, ptDst.y, w, h,
 //     rcSrc.left, rcSrc.top, w, h, pBits, &rBmpInfo, DIB_RGB_COLORS, SRCCOPY);
    int nRet = ::SetDIBitsToDevice(CPlane::CGdiLock(*this), ptDst.x, ptDst.y, w, h,
        rcSrc.left, rcSrc.top, 0, h, pBits, &rBmpInfo, DIB_RGB_COLORS);
    if (nRet == 0) {
        throw Win32Exception(L"rbg}bv̕`Ɏs܂");
    }
}

}
