#include "asm.h"
#include "Lanczos3.h"
#include "PixelAveration.h"
#include "../GDI/DC.h"
#include "../../Windows/CpuId.h"
#include "../../Thread/Parallel.h"
#include "../../Utility/Algorithm.h"
#include <math.h>       // sin, cos
#include <limits>       // numeric_limits<T>
namespace Themis {

void CPlane32::ReverseFill(CRect rcDst) {
    if (CClipper(GetSize()).Clipping(rcDst)) {
        Themis_PlaneLock4(*this, rcDst.GetPoint(), pDst, nDstPitch);
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        for (int y = 0; y < nDstH; y++) {
            for (int x = 0; x < nDstW; x++) {
                pDst[x] ^= 0xffffff;
            }
            pDst += nDstPitch;
        }
    }
}

void CPlane32::AlphaFill(COLORREF rgb, CRect rcDst, int nAlpha) {
    if (nAlpha <= 0) {
    } else if (0x100 <= nAlpha) {
        Fill(rgb, rcDst);
    } else if (CClipper(GetSize()).Clipping(rcDst)) {
        Themis_PlaneLock4(*this, rcDst.GetPoint(), pDst, nDstPitch);
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        Themis_PlaneGetMMXWidth32(nDstW, nMMXSize, nCppSize);
        uint32_t dwColor = Detail::RGBtoBGR32(rgb);
        if (0 < nMMXSize) {
            Detail::MMXAlphaFill32(pDst, nDstPitch, nMMXSize, nDstH, dwColor, nAlpha);
        }
        //csNZ̏(xĂc)
        Themis_PlaneGetRGB(rgb, nSrcR, nSrcG, nSrcB);
        for (int y = 0; y < nDstH; y++) {
            for (int i = 0; i < nCppSize; i++) {
                Themis_PlaneGetBGR32(pDst[nMMXSize * 2 + i], nDstR, nDstG, nDstB);
                pDst[nMMXSize * 2 + i] = static_cast<uint32_t>(
                    (((nSrcR - nDstR) * nAlpha << 8) + (nDstR << 16)) & 0xff0000 |
                    (((nSrcG - nDstG) * nAlpha     ) + (nDstG << 8 )) & 0x00ff00 |
                    (((nSrcB - nDstB) * nAlpha >> 8) + (nDstB      )) & 0x0000ff
                    );
            }
            pDst += nDstPitch;
        }
    }
}

void CPlane32::SatAddFill(COLORREF rgb, CRect rcDst) {
    if (CClipper(GetSize()).Clipping(rcDst)) {
        Themis_PlaneLock4(*this, rcDst.GetPoint(), pDst, nDstPitch);
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        Themis_PlaneGetMMXWidth32(nDstW, nMMXSize, nCppSize);
        uint32_t dwColor = Detail::RGBtoBGR32(rgb);
        if (0 < nMMXSize) {
            Detail::MMXSatAddFill32(pDst, nDstPitch, nMMXSize, nDstH, dwColor);
        }
        //csNZ̏
        uint32_t dwSrc = dwColor;
        for (int y = 0; y < nDstH; y++) {
            for (int i = 0; i < nCppSize; i++) {
                const uint32_t dwDst = pDst[nMMXSize * 2 + i];
                const uint32_t c = (((dwDst & dwSrc) << 1) + ((dwDst ^ dwSrc) & 0xfefefe)) & 0x1010100;
                const uint32_t d = ((c >> 5) + 0x7f7f7f) ^ 0x7f7f7f;
                pDst[nMMXSize * 2 + i] = (dwDst + dwSrc - d) | d;
            }
            pDst += nDstPitch;
        }
    }
}

void CPlane32::SatSubFill(COLORREF rgb, CRect rcDst) {
    if (CClipper(GetSize()).Clipping(rcDst)) {
        Themis_PlaneLock4(*this, rcDst.GetPoint(), pDst, nDstPitch);
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        Themis_PlaneGetMMXWidth32(nDstW, nMMXSize, nCppSize);
        uint32_t dwColor = Detail::RGBtoBGR32(rgb);
        if (0 < nMMXSize) {
            Detail::MMXSatSubFill32(pDst, nDstPitch, nMMXSize, nDstH, dwColor);
        }
        //csNZ̏
        uint32_t dwSrc = dwColor;
        for (int y = 0; y < nDstH; y++) {
            for (int i = 0; i < nCppSize; i++) {
                const uint32_t dwDst = pDst[nMMXSize * 2 + i];
                const uint32_t c = (((~dwDst & dwSrc) << 1) + ((~dwDst ^ dwSrc) & 0xfefefe)) & 0x1010100;
                const uint32_t d = ((c >> 5) + 0x7f7f7f) ^ 0x7f7f7f;
                pDst[nMMXSize * 2 + i] = (dwDst | d) - (dwSrc | d);
            }
            pDst += nDstPitch;
        }
    }
}

namespace {
    inline void FillMosaic(uint32_t* p, const int nPitch, const int cx, const int cy) {
        //p[^́A捶_ւ̃|C^Av[Ŝ̃sb`ATCY(x, y)
        const uint32_t dw = p[cx / 2 + (cy / 2) * nPitch]; //F
        for (int y = 0; y < cy; y++) {
            for (int x = 0; x < cx; x++) {
                p[x] = dw;
            }
            p += nPitch;
        }
    }
}
void CPlane32::MosaicFill(CRect rcDst, int nLevel) {
    // jF`nLevel ~ nLevelɋ؂āA̒̃sNZ̐FœhԂ
    //       [ׂ̈ɍEEEEEEEEEEE(ɂP`XƂ)ɕĂKv
    if (1 < nLevel) {
        if (CClipper(GetSize()).Clipping(rcDst)) {
            Themis_PlaneLock4(*this, rcDst.GetPoint(), pDst, nDstPitch);
            Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
            CSize szDst = rcDst.GetSize();
            // O
            const CSize szMosaic = szDst / nLevel;          //UCNȋ搔
            const CSize szError = szDst % nLevel;           //덷sNZ
            const CSize szError1 = szError / 2;             //Ə㑤ɔz덷sNZ
            const CSize szError2 = szError - szError1;      //EƉɔz덷sNZ
            if (0 < szError1.cy) {
                // (1)
                if (0 < szError1.cx) {
                    FillMosaic(&pDst[0], nDstPitch, szError1.cx, szError1.cy);
                }
                // (2)
                for (int x = 0, i = 0, ci = szMosaic.cx; i < ci; i++, x += nLevel) {
                    FillMosaic(&pDst[szError1.cx + x], nDstPitch, nLevel, szError1.cy);
                }
                // E(3)
                if (0 < szError2.cx) {
                    FillMosaic(&pDst[szError1.cx + szMosaic.cx * nLevel], nDstPitch, szError2.cx, szError1.cy);
                }
                pDst += szError1.cy * nDstPitch;    //ȍ~㑤͍lȂ
            }
            if (0 < szError1.cx) {
                // (4)
                uint32_t* pDst2 = &pDst[0];
                for (int j = 0, cj = szMosaic.cy; j < cj; j++) {
                    FillMosaic(&pDst2[0], nDstPitch, szError1.cx, nLevel);
                    pDst2 += nLevel * nDstPitch;
                }
                // (7)
                if (0 < szError2.cy) {
                    FillMosaic(&pDst2[0], nDstPitch, szError1.cx, szError2.cy);
                }
                pDst += szError1.cx;    //ȍ~͍lȂ
            }
            for (int j = 0, cj = szMosaic.cy; j < cj; j++) {
                // (5)
                for (int x = 0, i = 0, ci = szMosaic.cx; i < ci; i++, x += nLevel) {
                    FillMosaic(&pDst[x], nDstPitch, nLevel, nLevel);
                }
                // E(6)
                if (0 < szError2.cx) {
                    FillMosaic(&pDst[szMosaic.cx * nLevel], nDstPitch, szError2.cx, nLevel);
                }
                pDst += nDstPitch * nLevel;
            }
            if (0 < szError2.cy) {
                // (8)
                for (int x = 0, i = 0, ci = szMosaic.cx; i < ci; i++, x += nLevel) {
                    FillMosaic(&pDst[x], nDstPitch, nLevel, szError2.cy);
                }
                // E(9)
                if (0 < szError2.cx) {
                    FillMosaic(&pDst[szMosaic.cx * nLevel], nDstPitch, szError2.cx, szError2.cy);
                }
            }
        }
    }
}

//FillMosaic()ƋFႤB
namespace {
    inline void FillMosaicAverage(uint32_t* p, const int nPitch, const int cx, const int cy) {
        const uint32_t dw = Detail::GetAverage(p, nPitch, cx, cy, cx * cy); //F
        for (int y = 0; y < cy; y++) {
            for (int x = 0; x < cx; x++) {
                p[x] = dw;
            }
            p += nPitch;
        }
    }
}
void CPlane32::MosaicAverageFill(CRect rcDst, int nLevel) {
    if (1 < nLevel) {
        if (CClipper(GetSize()).Clipping(rcDst)) {
            Themis_PlaneLock4(*this, rcDst.GetPoint(), pDst, nDstPitch);
            Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
            CSize szDst = rcDst.GetSize();
            // O
            const CSize szMosaic = szDst / nLevel;          //UCNȋ搔
            const CSize szError = szDst % nLevel;           //덷sNZ
            const CSize szError1 = szError / 2;             //Ə㑤ɔz덷sNZ
            const CSize szError2 = szError - szError1;      //EƉɔz덷sNZ
            if (0 < szError1.cy) {
                // (1)
                if (0 < szError1.cx) {
                    FillMosaicAverage(&pDst[0], nDstPitch, szError1.cx, szError1.cy);
                }
                // (2)
                for (int x = 0, i = 0, ci = szMosaic.cx; i < ci; i++, x += nLevel) {
                    FillMosaicAverage(&pDst[szError1.cx + x], nDstPitch, nLevel, szError1.cy);
                }
                // E(3)
                if (0 < szError2.cx) {
                    FillMosaicAverage(&pDst[szError1.cx + szMosaic.cx * nLevel], nDstPitch, szError2.cx, szError1.cy);
                }
                pDst += szError1.cy * nDstPitch;    //ȍ~㑤͍lȂ
            }
            if (0 < szError1.cx) {
                // (4)
                uint32_t* pDst2 = &pDst[0];
                for (int j = 0, cj = szMosaic.cy; j < cj; j++) {
                    FillMosaicAverage(&pDst2[0], nDstPitch, szError1.cx, nLevel);
                    pDst2 += nLevel * nDstPitch;
                }
                // (7)
                if (0 < szError2.cy) {
                    FillMosaicAverage(&pDst2[0], nDstPitch, szError1.cx, szError2.cy);
                }
                pDst += szError1.cx;    //ȍ~͍lȂ
            }
            for (int j = 0, cj = szMosaic.cy; j < cj; j++) {
                // (5)
                for (int x = 0, i = 0, ci = szMosaic.cx; i < ci; i++, x += nLevel) {
                    FillMosaicAverage(&pDst[x], nDstPitch, nLevel, nLevel);
                }
                // E(6)
                if (0 < szError2.cx) {
                    FillMosaicAverage(&pDst[szMosaic.cx * nLevel], nDstPitch, szError2.cx, nLevel);
                }
                pDst += nDstPitch * nLevel;
            }
            if (0 < szError2.cy) {
                // (8)
                for (int x = 0, i = 0, ci = szMosaic.cx; i < ci; i++, x += nLevel) {
                    FillMosaicAverage(&pDst[x], nDstPitch, nLevel, szError2.cy);
                }
                // E(9)
                if (0 < szError2.cx) {
                    FillMosaicAverage(&pDst[szMosaic.cx * nLevel], nDstPitch, szError2.cx, szError2.cy);
                }
            }
        }
    }
}

void CPlane32::LevelRevTableFill(CRect rcDst, uint8_t tbl[0x100]) {
    if (CClipper(GetSize()).Clipping(rcDst)) {
        uint32_t adwTable[0x100] = {0}; //ƂFX̂ňx[JɃRs[
        for (int i = 0; i < 0x100; i++) {
            adwTable[i] = tbl[i];
        }
        Themis_PlaneLock4(*this, rcDst.GetPoint(), pDst, nDstPitch);
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        for (int y = 0; y < nDstH; y++) {
            for (int x = 0; x < nDstW; x++) {
                // RGBꂼPɕϊ
                Themis_PlaneGetBGR32(pDst[x], r, g, b);
                pDst[x] = Detail::BGR32(adwTable[r], adwTable[g], adwTable[b]);
            }
            pDst += nDstPitch;
        }
    }
}

void CPlane32::GrayFill(CRect rcDst) {
    if (CClipper(GetSize()).Clipping(rcDst)) {
        uint32_t adwTable[0x100 * 3];
        for (int i = 0, i3 = 0; i < 0x100; i++, i3 += 3) {
            adwTable[i3] = adwTable[i3 + 1] = adwTable[i3 + 2] = Detail::BGR32(i, i, i);
        }
        Themis_PlaneLock4(*this, rcDst.GetPoint(), pDst, nDstPitch);
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        for (int y = 0; y < nDstH; y++) {
            for (int x = 0; x < nDstW; x++) {
                Themis_PlaneGetBGR32(pDst[x], r, g, b);
                pDst[x] = adwTable[r + g + b];
            }
            pDst += nDstPitch;
        }
    }
}

void CPlane32::GrayFillStrict(CRect rcDst) {
    if (CClipper(GetSize()).Clipping(rcDst)) {
        uint16_t awTableR[0x100];
        uint16_t awTableG[0x100];
        uint16_t awTableB[0x100];
        for (int i = 0; i < 0x100; i++) {
            awTableR[i] = static_cast<uint16_t>(i * 77);
            awTableG[i] = static_cast<uint16_t>(i * 151);
            awTableB[i] = static_cast<uint16_t>(i * 28);
        }
        uint32_t adwTable[0x100];
        for (int i = 0; i < 0x100; i++) {
            adwTable[i] = Detail::BGR32(i, i, i);
        }
        Themis_PlaneLock4(*this, rcDst.GetPoint(), pDst, nDstPitch);
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        for (int y = 0; y < nDstH; y++) {
            for (int x = 0; x < nDstW; x++) {
                Themis_PlaneGetBGR32(pDst[x], r, g, b);
                pDst[x] = adwTable[(awTableR[r] + awTableG[g] + awTableB[b]) >> 8];
            }
            pDst += nDstPitch;
        }
    }
}

void CPlane32::MakeBargraph(CRect rcDst, double adData[], int nDataCount, double dDataMin, double dDataMax, COLORREF rgbBg) {
    const int nGraphWidthRate = 4;
    const int nGraphHeightRate = 4;
    const int nGraphWidth = nDataCount * nGraphWidthRate;
    const int nGraphHeight = rcDst.GetHeight() * nGraphHeightRate;
    const uint32_t dwBgColor = Detail::RGBtoBGR32(rgbBg);
    // e|̂P̍쐬
    CMemoryPlane32 oTmp;
    oTmp.SetSize(CSize(nGraphHeight, nGraphWidth));
    Themis_PlaneLock4(oTmp, CPoint(), pdw, nPitch);
    for (int y = 0; y < nGraphWidth; y++) {
        int nHistogram = static_cast<int>((adData[y / nGraphWidthRate] - dDataMin) * nGraphHeight / dDataMax);
        for (int x = 0, nHistogramEnd = Min(nHistogram, nGraphHeight); x < nHistogramEnd; x++) {
            pdw[x] = 0;
        }
        for (int x2 = nHistogram; x2 < nGraphHeight; x2++) {
            pdw[x2] = dwBgColor;
        }
        pdw += nPitch;
    }
    // e|̂Q̍쐬
    CMemoryPlane32 oTmp2;
    oTmp2.SetSize(CSize(nGraphWidth, nGraphHeight));
    oTmp2.Rot90AntiClockwiseBlt(oTmp, CPoint(), oTmp.GetSize().ToRect());
    // `֏k]
    StretchBltLanczos3(oTmp2, rcDst, oTmp2.GetSize().ToRect());
}

void CPlane32::GetHistogram(int anHistogram[0x100], CRect rcDst) {
    std::fill(anHistogram, anHistogram + 0x100, 0);
    if (CClipper(GetSize()).Clipping(rcDst)) {
        Themis_PlaneLock4(*this, rcDst.GetPoint(), pDst, nDstPitch);
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        for (int y = 0; y < nDstH; y++) {
            for (int x = 0; x < nDstW; x++) {
                Themis_PlaneGetBGR32(pDst[x], r, g, b);
                anHistogram[(r * 77 + g * 151 + b * 28) >> 8]++;
            }
            pDst += nDstPitch;
        }
    }
}

void CPlane32::GetHistogramRGBAverage(int anHistogram[0x100], CRect rcDst) {
    std::fill(anHistogram, anHistogram + 0x100, 0);
    int anTbl[0x100 * 3];
    for (int i = 0, j = 0; i < 0x100; i++, j += 3) {
        anTbl[j] = anTbl[j + 1] = anTbl[j + 2] = i;
    }
    if (CClipper(GetSize()).Clipping(rcDst)) {
        Themis_PlaneLock4(*this, rcDst.GetPoint(), pDst, nDstPitch);
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        for (int y = 0; y < nDstH; y++) {
            for (int x = 0; x < nDstW; x++) {
                Themis_PlaneGetBGR32(pDst[x], r, g, b);
                anHistogram[anTbl[r + g + b]]++;
            }
            pDst += nDstPitch;
        }
    }
}

void CPlane32::GetHistogramRGB(int anHistogramR[0x100], int anHistogramG[0x100], int anHistogramB[0x100], CRect rcDst) {
    std::fill(anHistogramR, anHistogramR + 0x100, 0);
    std::fill(anHistogramG, anHistogramG + 0x100, 0);
    std::fill(anHistogramB, anHistogramB + 0x100, 0);
    if (CClipper(GetSize()).Clipping(rcDst)) {
        Themis_PlaneLock4(*this, rcDst.GetPoint(), pDst, nDstPitch);
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        for (int y = 0; y < nDstH; y++) {
            for (int x = 0; x < nDstW; x++) {
                Themis_PlaneGetBGR32(pDst[x], r, g, b);
                anHistogramR[r]++;
                anHistogramG[g]++;
                anHistogramB[b]++;
            }
            pDst += nDstPitch;
        }
    }
}

int CPlane32::GetColorCount(CRect rcDst) {
    if (CClipper(GetSize()).Clipping(rcDst)) {
        const int nFlagBits = 8 * sizeof(unsigned);
        const int nFlagSize = (0x100 * 0x100 * 0x100) / nFlagBits;
        boost::scoped_array<unsigned> abyFlags(new unsigned[nFlagSize]);
        std::fill(abyFlags.get(), abyFlags.get() + nFlagSize, 0x00000000);
        int nCount = 0;
        Themis_PlaneLock4(*this, rcDst.GetPoint(), pDst, nDstPitch);
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        for (int y = 0; y < nDstH; y++) {
            for (int x = 0; x < nDstW; x++) {
                uint32_t dw = pDst[x] & 0xffffff;
                if ((abyFlags[dw / nFlagBits] & (1 << (dw % nFlagBits))) == 0) {
                    abyFlags[dw / nFlagBits] |= (1 << (dw % nFlagBits));
                    nCount++;
                }
            }
            pDst += nDstPitch;
        }
        return nCount;
    }
    return 0;
}

COLORREF CPlane32::GetAverageColor(CRect rcDst) {
    if (CClipper(GetSize()).Clipping(rcDst)) {
        Themis_PlaneLock4(*this, rcDst.GetPoint(), pDst, nDstPitch);
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        const uint32_t dwAverage = Detail::GetAverage(pDst, nDstPitch, nDstW, nDstH, nDstW * nDstH);
        Themis_PlaneGetBGR32(dwAverage, r, g, b);
        return RGB(r, g, b);
    }
    return RGB(0x00, 0x00, 0x00);
}

void CPlane32::ClearAlpha() {
    Themis_PlaneLock4(*this, CPoint(), pDst, nDstPitch);
    Themis_PlaneGetSize(GetSize(), nDstW, nDstH);
    for (int y = 0; y < nDstH; y++) {
        for (int x = 0; x < nDstW; x++) {
            pDst[x] &= 0x00ffffff;
        }
        pDst += nDstPitch;
    }
}

void CPlane32::ReverseAlpha() {
    Themis_PlaneLock4(*this, CPoint(), pDst, nDstPitch);
    Themis_PlaneGetSize(GetSize(), nDstW, nDstH);
    for (int y = 0; y < nDstH; y++) {
        for (int x = 0; x < nDstW; x++) {
            pDst[x] ^= 0xff000000u;
        }
        pDst += nDstPitch;
    }
}

void CPlane32::BrightToAlpha() {
    unsigned adwTableR[0x100];
    unsigned adwTableG[0x100];
    unsigned adwTableB[0x100];
    for (int i = 0; i < 0x100; i++) {
        adwTableR[i] = i * 77 << 16;
        adwTableG[i] = i * 151 << 16;
        adwTableB[i] = i * 28 << 16;
    }
    Themis_PlaneLock4(*this, CPoint(), pDst, nDstPitch);
    Themis_PlaneGetSize(GetSize(), nDstW, nDstH);
    for (int y = 0; y < nDstH; y++) {
        for (int x = 0; x < nDstW; x++) {
            Themis_PlaneGetBGR32(pDst[x], r, g, b);
            pDst[x] &= 0x00ffffff;
            pDst[x] |= (adwTableR[r] + adwTableG[g] + adwTableB[b]) & 0xff000000u;
        }
        pDst += nDstPitch;
    }
}

namespace {
    inline uint32_t GetDiff32(uint32_t dw1, uint32_t dw2) {
        const uint32_t t = ((dw1 | 0x808080) - dw2) & 0x00ffffff; //1̍ŏʃrbg1ɂĈZ
        return (t ^ (dw1 & 0x808080) ^ 0x808080); //1̍ŏʃrbg0Ȃ甽]
    }
}
void CPlane32::DiffToAlpha(const CPlane& rSrc) {
    Themis_DebugAssert(rSrc.GetBpp() == 32);
    uint32_t adwTable[0x100 * 3];
    for (uint32_t i = 0; i < 0x100; i++) {
        adwTable[i * 3 + 0] = i << 24;
        adwTable[i * 3 + 1] = i << 24;
        adwTable[i * 3 + 2] = i << 24;
    }
    Themis_PlaneLock4(*this, CPoint(), pDst, nDstPitch);
    Themis_PlaneLock4(rSrc, CPoint(), pSrc, nSrcPitch);
    Themis_PlaneGetSize(GetSize().GetIntersect(rSrc.GetSize()), nDstW, nDstH);
    for (int y = 0; y < nDstH; y++) {
        for (int x = 0; x < nDstW; x++) {
            const uint32_t dwDiff = GetDiff32(pDst[x], pSrc[x]);
            Themis_PlaneGetBGR32(dwDiff, r, g, b);
            pDst[x] &= 0x00ffffff;
            pDst[x] |= adwTable[r + g + b];
        }
        pDst += nDstPitch;
        pSrc += nSrcPitch;
    }
}

void CPlane32::DiffToAlphaStrict(const CPlane& rSrc) {
    Themis_DebugAssert(rSrc.GetBpp() == 32);
    uint32_t adwTableR[0x100];
    uint32_t adwTableG[0x100];
    uint32_t adwTableB[0x100];
    for (int i = 0; i < 0x100; i++) {
        adwTableR[i] = i * 77 << 16;
        adwTableG[i] = i * 151 << 16;
        adwTableB[i] = i * 28 << 16;
    }
    Themis_PlaneLock4(*this, CPoint(), pDst, nDstPitch);
    Themis_PlaneLock4(rSrc, CPoint(), pSrc, nSrcPitch);
    Themis_PlaneGetSize(GetSize().GetIntersect(rSrc.GetSize()), nDstW, nDstH);
    for (int y = 0; y < nDstH; y++) {
        for (int x = 0; x < nDstW; x++) {
            const uint32_t dwDiff = GetDiff32(pDst[x], pSrc[x]);
            Themis_PlaneGetBGR32(dwDiff, r, g, b);
            pDst[x] &= 0x00ffffff;
            pDst[x] |= (adwTableR[r] + adwTableG[g] + adwTableB[b]) &  0xff000000u;
        }
        pDst += nDstPitch;
        pSrc += nSrcPitch;
    }
}

bool CPlane32::HasAlphaChannel(CRect rcDst) {
    if (CClipper(GetSize()).Clipping(rcDst)) {
        Themis_PlaneLock4(*this, rcDst.GetPoint(), pDst, nDstPitch);
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        const int nMin = nDstW * nDstH * 5 / 100; // 5%ȏ̃sNZOȊOȂtrueƂ
        int nCount = 0;
        for (int y = 0; y < nDstH; y++) {
            for (int x = 0; x < nDstW; x++) {
                if ((pDst[x] & 0xff000000) != 0) {
                    // [ȂS~Ă\̂BB
                    if (nMin <= ++nCount) {
                        return true;
                    }
                }
            }
            pDst += nDstPitch;
        }
    } else {
        Themis_DebugWarning(L"CPlane32::HasAlphaChannel()rcDst");
    }
    return false;
}

//////////////////////////////////////////////////////////////////////

void CPlane32::TableBlt(const CPlane8& rSrc, CPoint ptDst, CRect rcSrc, COLORREF rgbTable[0x100], const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        uint32_t dwTable[0x100];
        for (int i = 0; i < 0x100; i++) {
            dwTable[i] = Detail::RGBtoBGR32(rgbTable[i]);
        }
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock1(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        for (int y = 0; y < nSrcH; y++) {
            for (int x = 0; x < nSrcW; x++) {
                pDst[x] = dwTable[pSrc[x]];
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
        }
    }
}

void CPlane32::BltTrans(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, COLORREF rgbTrans, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        const uint32_t dwTrans = Detail::RGBtoBGR32(rgbTrans);
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        for (int y = 0; y < nSrcH; y++) {
            for (int x = 0; x < nSrcW; x++) {
                if (dwTrans != pSrc[x]) {
                    pDst[x] = pSrc[x];
                }
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
        }
    }
}

namespace {
    inline uint32_t Befog2(uint32_t dwSrc, uint32_t dwAround1, uint32_t dwAround2) {
        return Detail::GetAverage(dwSrc, Detail::GetAverage(dwAround1, dwAround2));
    }
    inline uint32_t Befog3(uint32_t dwSrc, uint32_t dwAround1, uint32_t dwAround2, uint32_t dwAround3) {
        return Detail::GetAverage(dwSrc, Detail::GetAverage(dwAround1, dwAround2, dwAround3));
    }
    inline uint32_t Befog4(uint32_t dwSrc, uint32_t dwAround1, uint32_t dwAround2, uint32_t dwAround3, uint32_t dwAround4) {
        return Detail::GetAverage(dwSrc, Detail::GetAverage(dwAround1, dwAround2, dwAround3, dwAround4));
    }
}
void CPlane32::BefogBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        if (rcSrc.GetWidth() <= 1 || rcSrc.GetHeight() <= 1) {
            Blt(rSrc, ptDst, rcSrc, rClip);
            // ͖̂ʓ|Ȃ̂ŏ
        } else {
            Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
            Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
            Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
            // 
            pDst[0] = Befog2(pSrc[0], pSrc[0 + 1], pSrc[0 + nSrcPitch]);
            // 
            for (int x = 0 + 1; x < nSrcW - 1; x++) {
                pDst[x] = Befog3(pSrc[x], pSrc[x - 1], pSrc[x + 1], pSrc[x + nSrcPitch]);
            }
            // E
            pDst[nSrcW - 1] = Befog2(pSrc[nSrcW - 1], pSrc[nSrcW - 2], pSrc[nSrcW - 1 + nSrcPitch]);
            pDst += nDstPitch;
            pSrc += nSrcPitch;
            for (int y = 0 + 1; y < nSrcH - 1; y++) {
                // 
                pDst[0] = Befog3(pSrc[0], pSrc[0 - nSrcPitch], pSrc[0 + 1], pSrc[0 + nSrcPitch]);
                // 
                for (int x = 0 + 1; x < nSrcW - 1; x++) {
                    pDst[x] = Befog4(pSrc[x], pSrc[x - nSrcPitch], pSrc[x - 1], pSrc[x + 1], pSrc[x + nSrcPitch]);
                }
                // E
                pDst[nSrcW - 1] = Befog3(pSrc[nSrcW - 1], pSrc[nSrcW - 1 - nSrcPitch], pSrc[nSrcW - 2], pSrc[nSrcW - 1 + nSrcPitch]);
                pDst += nDstPitch;
                pSrc += nSrcPitch;
            }
            // 
            pDst[0] = Befog2(pSrc[0], pSrc[0 - nSrcPitch], pSrc[0 + 1]);
            // 
            for (int x = 0 + 1; x < nSrcW - 1; x++) {
                pDst[x] = Befog3(pSrc[x], pSrc[x - nSrcPitch], pSrc[x - 1], pSrc[x + 1]);
            }
            // E
            pDst[nSrcW - 1] = Befog2(pSrc[nSrcW - 1], pSrc[nSrcW - 1 - nSrcPitch], pSrc[nSrcW - 2]);
        }
    }
}

namespace {
    inline uint32_t Sharp(uint32_t dwSrc, uint32_t dwAround) {
        // x + (x - y) / 2
        Themis_PlaneGetBGR32(dwSrc, sr, sg, sb);
        Themis_PlaneGetBGR32(dwAround, ar, ag, ab);
        const int r = Detail::GetFixByte(sr + int(sr - ar) / 2);
        const int g = Detail::GetFixByte(sg + int(sg - ag) / 2);
        const int b = Detail::GetFixByte(sb + int(sb - ab) / 2);
        return Detail::BGR32(r, g, b);
    }
    inline uint32_t Sharp2(uint32_t dwSrc, uint32_t dwAround1, uint32_t dwAround2) {
        return Sharp(dwSrc, Detail::GetAverage(dwAround1, dwAround2));
    }
    inline uint32_t Sharp3(uint32_t dwSrc, uint32_t dwAround1, uint32_t dwAround2, uint32_t dwAround3) {
        return Sharp(dwSrc, Detail::GetAverage(dwAround1, dwAround2, dwAround3));
    }
    inline uint32_t Sharp4(uint32_t dwSrc, uint32_t dwAround1, uint32_t dwAround2, uint32_t dwAround3, uint32_t dwAround4) {
        return Sharp(dwSrc, Detail::GetAverage(dwAround1, dwAround2, dwAround3, dwAround4));
    }
}
void CPlane32::SharpBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, const CClipper& rClip) {
    // BefogBltRsyBefogSharpB
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        if (rcSrc.GetWidth() <= 1 || rcSrc.GetHeight() <= 1) {
            Blt(rSrc, ptDst, rcSrc, rClip);
            // ͖̂ʓ|Ȃ̂ŏ
        } else {
            Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
            Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
            Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
            // 
            pDst[0] = Sharp2(pSrc[0], pSrc[0 + 1], pSrc[0 + nSrcPitch]);
            // 
            for (int x = 0 + 1; x < nSrcW - 1; x++) {
                pDst[x] = Sharp3(pSrc[x], pSrc[x - 1], pSrc[x + 1], pSrc[x + nSrcPitch]);
            }
            // E
            pDst[nSrcW - 1] = Sharp2(pSrc[nSrcW - 1], pSrc[nSrcW - 2], pSrc[nSrcW - 1 + nSrcPitch]);
            pDst += nDstPitch;
            pSrc += nSrcPitch;
            for (int y = 0 + 1; y < nSrcH - 1; y++) {
                // 
                pDst[0] = Sharp3(pSrc[0], pSrc[0 - nSrcPitch], pSrc[0 + 1], pSrc[0 + nSrcPitch]);
                // 
                for (int x = 0 + 1; x < nSrcW - 1; x++) {
                    pDst[x] = Sharp4(pSrc[x], pSrc[x - nSrcPitch], pSrc[x - 1], pSrc[x + 1], pSrc[x + nSrcPitch]);
                }
                // E
                pDst[nSrcW - 1] = Sharp3(pSrc[nSrcW - 1], pSrc[nSrcW - 1 - nSrcPitch], pSrc[nSrcW - 2], pSrc[nSrcW - 1 + nSrcPitch]);
                pDst += nDstPitch;
                pSrc += nSrcPitch;
            }
            // 
            pDst[0] = Sharp2(pSrc[0], pSrc[0 - nSrcPitch], pSrc[0 + 1]);
            // 
            for (int x = 0 + 1; x < nSrcW - 1; x++) {
                pDst[x] = Sharp3(pSrc[x], pSrc[x - nSrcPitch], pSrc[x - 1], pSrc[x + 1]);
            }
            // E
            pDst[nSrcW - 1] = Sharp2(pSrc[nSrcW - 1], pSrc[nSrcW - 1 - nSrcPitch], pSrc[nSrcW - 2]);
        }
    }
}

namespace {
    inline uint32_t HardSharp(uint32_t dwSrc, uint32_t dwAround) {
        // x + (x - y)
        // = x * 2 - y
        const int r = Detail::GetFixByte(((dwSrc >> 15) & 0x1fe) - ((dwAround >> 16) & 0xff));
        const int g = Detail::GetFixByte(((dwSrc >> 7) & 0x1fe) - ((dwAround >> 8) & 0xff));
        const int b = Detail::GetFixByte(((dwSrc << 1) & 0x1fe) - (dwAround & 0xff));
        return Detail::BGR32(r, g, b);
    }
    inline uint32_t HardSharp2(uint32_t dwSrc, uint32_t dwAround1, uint32_t dwAround2) {
        return HardSharp(dwSrc, Detail::GetAverage(dwAround1, dwAround2));
    }
    inline uint32_t HardSharp3(uint32_t dwSrc, uint32_t dwAround1, uint32_t dwAround2, uint32_t dwAround3) {
        return HardSharp(dwSrc, Detail::GetAverage(dwAround1, dwAround2, dwAround3));
    }
    inline uint32_t HardSharp4(uint32_t dwSrc, uint32_t dwAround1, uint32_t dwAround2, uint32_t dwAround3, uint32_t dwAround4) {
        return HardSharp(dwSrc, Detail::GetAverage(dwAround1, dwAround2, dwAround3, dwAround4));
    }
}
void CPlane32::HardSharpBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, const CClipper& rClip) {
    // SharpBltRsySharpHardSharpB
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        if (rcSrc.GetWidth() <= 1 || rcSrc.GetHeight() <= 1) {
            Blt(rSrc, ptDst, rcSrc, rClip);
            // ͖̂ʓ|Ȃ̂ŏ
        } else {
            Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
            Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
            Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
            // 
            pDst[0] = HardSharp2(pSrc[0], pSrc[0 + 1], pSrc[0 + nSrcPitch]);
            // 
            for (int x = 0 + 1; x < nSrcW - 1; x++) {
                pDst[x] = HardSharp3(pSrc[x], pSrc[x - 1], pSrc[x + 1], pSrc[x + nSrcPitch]);
            }
            // E
            pDst[nSrcW - 1] = HardSharp2(pSrc[nSrcW - 1], pSrc[nSrcW - 2], pSrc[nSrcW - 1 + nSrcPitch]);
            pDst += nDstPitch;
            pSrc += nSrcPitch;
            for (int y = 0 + 1; y < nSrcH - 1; y++) {
                // 
                pDst[0] = HardSharp3(pSrc[0], pSrc[0 - nSrcPitch], pSrc[0 + 1], pSrc[0 + nSrcPitch]);
                // 
                for (int x = 0 + 1; x < nSrcW - 1; x++) {
                    pDst[x] = HardSharp4(pSrc[x], pSrc[x - nSrcPitch], pSrc[x - 1], pSrc[x + 1], pSrc[x + nSrcPitch]);
                }
                // E
                pDst[nSrcW - 1] = HardSharp3(pSrc[nSrcW - 1], pSrc[nSrcW - 1 - nSrcPitch], pSrc[nSrcW - 2], pSrc[nSrcW - 1 + nSrcPitch]);
                pDst += nDstPitch;
                pSrc += nSrcPitch;
            }
            // 
            pDst[0] = HardSharp2(pSrc[0], pSrc[0 - nSrcPitch], pSrc[0 + 1]);
            // 
            for (int x = 0 + 1; x < nSrcW - 1; x++) {
                pDst[x] = HardSharp3(pSrc[x], pSrc[x - nSrcPitch], pSrc[x - 1], pSrc[x + 1]);
            }
            // E
            pDst[nSrcW - 1] = HardSharp2(pSrc[nSrcW - 1], pSrc[nSrcW - 1 - nSrcPitch], pSrc[nSrcW - 2]);
        }
    }
}

namespace {
    inline void GaussBefogBltH(const CPlane& rDst, const CPlane& rSrc, const CPoint& ptDst, const CRect& rcSrc, const int nRange, const unsigned anTable[]) {
        Themis_PlaneLock4(rDst, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        for (int y = 0; y < nSrcH; y++) {
            for (int x = 0; x < nSrcW; x++) {
                unsigned dr = 0, dg = 0, db = 0, dw = 0;
                for (int i = -nRange; i <= nRange; i++) {
                    if (0 <= x + i && x + i < nSrcW) {
                        const uint32_t dwSrc = pSrc[x + i];
                        Themis_PlaneGetBGR32(dwSrc, r, g, b);
                        const unsigned nGauss = anTable[abs(i)];
                        dr += r * nGauss;
                        dg += g * nGauss;
                        db += b * nGauss;
                        dw += nGauss;
                    }
                }
                dr = dr / dw;
                dg = dg / dw;
                db = db / dw;
                pDst[x] = Detail::BGR32(dr, dg, db);
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
        }
    }
    inline void GaussBefogBltV(const CPlane& rDst, const CPlane& rSrc, const CPoint& ptDst, const CRect& rcSrc, const int nRange, const unsigned anTable[]) {
        Themis_PlaneLock4(rDst, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        for (int y = 0; y < nSrcH; y++) {
            for (int x = 0; x < nSrcW; x++) {
                unsigned dr = 0, dg = 0, db = 0, dw = 0;
                for (int i = -nRange; i <= nRange; i++) {
                    if (0 <= y + i && y + i < nSrcH) {
                        const uint32_t dwSrc = pSrc[x + nSrcPitch * i];
                        Themis_PlaneGetBGR32(dwSrc, r, g, b);
                        const unsigned nGauss = anTable[abs(i)];
                        dr += r * nGauss;
                        dg += g * nGauss;
                        db += b * nGauss;
                        dw += nGauss;
                    }
                }
                dr = dr / dw;
                dg = dg / dw;
                db = db / dw;
                pDst[x] = Detail::BGR32(dr, dg, db);
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
        }
    }
}
void CPlane32::GaussBefogBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, double r, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        int nRange = static_cast<int>(r * 3);
        if (nRange <= 0) {
            Themis_DebugLog(L"GaussBefogBltǕʂBltB");
            Blt(rSrc, ptDst, rcSrc);
        } else {
            boost::scoped_array<unsigned> anTable(new unsigned[nRange + 1]);
            for (int i = 0; i <= nRange; i++) {
                double d = i;
                double g = exp(- d * d / (2.0 * r * r));
                anTable[i] = static_cast<unsigned>(g * (1 << 6));
            }
            CMemoryPlane32 oTmp;
            oTmp.SetSize(rcSrc.GetSize());
            GaussBefogBltH(oTmp, rSrc, CPoint(), rcSrc, nRange, anTable.get());
            GaussBefogBltV(*this, oTmp, ptDst, oTmp.GetSize().ToRect(), nRange, anTable.get());
        }
    }
}

void CPlane32::UnsharpMaskBlt(const CPlane& rSrc, const CPlane& rMsk, CPoint ptDst, CRect rcSrc, CPoint ptMsk, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneLock4(rMsk, ptMsk, pMsk, nMskPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        for (int y = 0; y < nSrcH; y++) {
            for (int x = 0; x < nSrcW; x++) {
                pDst[x] = Sharp(pSrc[x], pMsk[x]);
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
            pMsk += nMskPitch;
        }
    }
}

namespace {
    inline void MedianInsert(unsigned* a, uint32_t* b, int n, int i, unsigned x, uint32_t y) {
        for (int j = n; i < j; j--) {
            a[j] = a[j - 1];
            b[j] = b[j - 1];
        }
        a[i] = x;
        b[i] = y;
    }
    inline void MedianInsert(unsigned ab[9], uint32_t ap[9], int n, uint32_t dwSrc) {
        Themis_PlaneGetBGR32(dwSrc, r, g, b);
        const unsigned rgb = r + g + b;
        int i;
        for (i = 0; i < n; i++) { if (rgb < ab[i]) { break; } }
        MedianInsert(ab, ap, n, i, rgb, dwSrc);
    }
}
void CPlane32::MedianFilterBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc) && 3 <= rcSrc.GetWidth() && 3 <= rcSrc.GetHeight()) {
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        for (int y = 0 + 1; y < nSrcH - 1; y++) {
            pDst += nDstPitch;
            pSrc += nSrcPitch;
            for (int x = 0 + 1; x < nSrcW - 1; x++) {
                unsigned ab[9];
                uint32_t ap[9];
                MedianInsert(ab, ap, 0, pSrc[x - 1 - nSrcPitch]);
                MedianInsert(ab, ap, 1, pSrc[x + 0 - nSrcPitch]);
                MedianInsert(ab, ap, 2, pSrc[x + 1 - nSrcPitch]);
                MedianInsert(ab, ap, 3, pSrc[x - 1]);
                MedianInsert(ab, ap, 4, pSrc[x + 0]);
                MedianInsert(ab, ap, 5, pSrc[x + 1]);
                MedianInsert(ab, ap, 6, pSrc[x - 1 + nSrcPitch]);
                MedianInsert(ab, ap, 7, pSrc[x + 0 + nSrcPitch]);
                MedianInsert(ab, ap, 8, pSrc[x + 1 + nSrcPitch]);
             #ifndef NDEBUG
                unsigned atest[9];
                memcpy(atest, ab, sizeof(ab));
                std::sort(atest, atest + 9);
                Themis_DebugAssert(memcmp(atest, ab, sizeof(unsigned) * 9) == 0);
             #endif
                pDst[x] = ap[4];
            }
        }
    }
}

void CPlane32::BefogEdgeBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, double dRate, const CClipper& rClip) {
    Themis_DebugWarning(L"");
}

void CPlane32::BefogNotEdgeBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, double dRate, const CClipper& rClip) {
    Themis_DebugWarning(L"");
}

namespace {
    inline int Edge2(uint32_t dw1, uint32_t dw2, const int (& anTable)[0xff * 2 + 1]) {
        Themis_PlaneGetBGR32(dw1, r1, g1, b1);
        Themis_PlaneGetBGR32(dw2, r2, g2, b2);
        int t = anTable[r1 - r2 + 0x100];
        t += anTable[g1 - g2 + 0x100];
        t += anTable[b1 - b2 + 0x100];
        return t;
    }
}
void CPlane32::EdgeBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, double dRate, const CClipper& rClip) {
    // ʓ|Ȃ̂Œ̂ݏ邱ƂɂB
    // dRate  0.001 ` 1.0 炢œK

    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        if (rcSrc.GetWidth() <= 2 || rcSrc.GetHeight() <= 2) {
            Blt(rSrc, ptDst, rcSrc, rClip);
            // ͖̂ʓ|Ȃ̂ŏ
        } else {
            const int nMul = 0x100;
            int anTable[0xff * 2 + 1];
            for (int i = 0; i < 0x100; i++) {
                const int t = int(i * i * dRate * nMul);
                anTable[0x100 + i] = t;
                anTable[0x100 - i] = t;
            }
            Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
            Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
            Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
            for (int y = 0 + 1; y < nSrcH - 1; y++) {
                for (int x = 0 + 1; x < nSrcW - 1; x++)  {
                    int t = Edge2(pSrc[x], pSrc[x - nSrcPitch], anTable);
                    t += Edge2(pSrc[x], pSrc[x - 1], anTable);
                    t += Edge2(pSrc[x], pSrc[x + 1], anTable);
                    t += Edge2(pSrc[x], pSrc[x + nSrcPitch], anTable);
                    int u = Detail::GetFixByte(t / nMul);
                    pDst[x] = Detail::BGR32(u, u, u);
                }
                pSrc += nSrcPitch;
                pDst += nDstPitch;
            }
        }
    }
}

void CPlane32::ChannelMixerBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, double rr, double rg, double rb, double gr, double gg, double gb, double br, double bg, double bb, const CClipper& rClip) {
    if (!rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) return;
    Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
    Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
    Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
    int nrr = (int)(rr * 0x10000), nrg = (int)(rg * 0x10000), nrb = (int)(rb * 0x10000);
    int ngr = (int)(gr * 0x10000), ngg = (int)(gg * 0x10000), ngb = (int)(gb * 0x10000);
    int nbr = (int)(br * 0x10000), nbg = (int)(bg * 0x10000), nbb = (int)(bb * 0x10000);
    for (int y = 0; y < nSrcH; y++) {
        for (int x = 0; x < nSrcW; x++) {
            Themis_PlaneGetBGRA32(pSrc[x], a, r, g, b);
            pDst[x] = Detail::BGRA32(a,
                Detail::GetFixByte((r * nrr + g * nrg + b * nrb) / 0x10000),
                Detail::GetFixByte((r * ngr + g * ngg + b * ngb) / 0x10000),
                Detail::GetFixByte((r * nbr + g * nbg + b * nbb) / 0x10000));
        }
        pDst += nDstPitch;
        pSrc += nSrcPitch;
    }
}

void CPlane32::ChannelMixerBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, double kr, double kg, double kb, const CClipper& rClip) {
    if (!rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) return;
    Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
    Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
    Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
    int nkr = (int)(kr * 0x10000), nkg = (int)(kg * 0x10000), nkb = (int)(kb * 0x10000);
    for (int y = 0; y < nSrcH; y++) {
        for (int x = 0; x < nSrcW; x++) {
            Themis_PlaneGetBGRA32(pSrc[x], a, r, g, b);
            int k = Detail::GetFixByte((r * nkr + g * nkg + b * nkb) / 0x10000);
            pDst[x] = Detail::BGRA32(a, k, k, k);
        }
        pDst += nDstPitch;
        pSrc += nSrcPitch;
    }
}

void CPlane32::OrBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        for (int y = 0; y < nSrcH; y++) {
            for (int x = 0; x < nSrcW; x++) {
                pDst[x] |= pSrc[x];
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
        }
    }
}

void CPlane32::AndBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        for (int y = 0; y < nSrcH; y++) {
            for (int x = 0; x < nSrcW; x++) {
                pDst[x] &= pSrc[x];
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
        }
    }
}

void CPlane32::XorBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        for (int y = 0; y < nSrcH; y++) {
            for (int x = 0; x < nSrcW; x++) {
                pDst[x] &= pSrc[x];
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
        }
    }
}

void CPlane32::SatAddBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        Themis_PlaneGetMMXWidth32(nSrcW, nMMXSize, nCppSize);
        if (0 < nMMXSize) {
            Detail::MMXSatAddBlt32(pDst, nDstPitch, pSrc, nSrcPitch, nMMXSize, nSrcH);
        }
        //csNZ̏(xĂc)
        for (int y = 0; y < nSrcH; y++) {
            for (int i = 0; i < nCppSize; i++) {
                const uint32_t dwDst = pDst[nMMXSize * 2 + i];
                const uint32_t dwSrc = pSrc[nMMXSize * 2 + i];
                const uint32_t c = (((dwDst & dwSrc) << 1) + ((dwDst ^ dwSrc) & 0xfefefe)) & 0x1010100;
                const uint32_t d = ((c >> 5) + 0x7f7f7f) ^ 0x7f7f7f;
                pDst[nMMXSize * 2 + i] = (dwDst + dwSrc - d) | d;
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
        }
    }
}

void CPlane32::SatSubBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        Themis_PlaneGetMMXWidth32(nSrcW, nMMXSize, nCppSize);
        if (0 < nMMXSize) {
            Detail::MMXSatSubBlt32(pDst, nDstPitch, pSrc, nSrcPitch, nMMXSize, nSrcH);
        }
        //csNZ̏(xĂc)
        for (int y = 0; y < nSrcH; y++) {
            for (int i = 0; i < nCppSize; i++) {
                const uint32_t dwDst = pDst[nMMXSize * 2 + i];
                const uint32_t dwSrc = pSrc[nMMXSize * 2 + i];
                const uint32_t c = (((~dwDst & dwSrc) << 1) + ((~dwDst ^ dwSrc) & 0xfefefe)) & 0x1010100;
                const uint32_t d = ((c >> 5) + 0x7f7f7f) ^ 0x7f7f7f;
                pDst[nMMXSize * 2 + i] = (dwDst | d) - (dwSrc | d);
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
        }
    }
}

void CPlane32::FadeBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, int nFade, const CClipper& rClip) {
    if (nFade <= 0) {
        Blt(rSrc, ptDst, rcSrc, rClip);
    } else if (0x100 <= nFade) {
        Fill(RGB(0x00, 0x00, 0x00), CRect(ptDst, rcSrc.GetSize()));
    } else if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        Themis_PlaneGetMMXWidth32(nSrcW, nMMXSize, nCppSize);
        nFade = 0x100 - nFade;          //l𔽓]Ă
        if (0 < nMMXSize) {
            Detail::MMXFadeBlt32(pDst, nDstPitch, pSrc, nSrcPitch, nMMXSize, nSrcH, nFade);
        }
        //csNZ̏(xĂc)
        for (int y = 0; y < nSrcH; y++) {
            for (int i = 0; i < nCppSize; i++) {
                const uint32_t nSrcRB = pSrc[nMMXSize * 2 + i] & 0xff00ff;
                const uint32_t nSrcG = pSrc[nMMXSize * 2 + i] & 0x00ff00;
                pDst[nMMXSize * 2 + i] = static_cast<uint32_t>((nSrcRB * nFade >> 8) & 0xff00ff | (nSrcG * nFade >> 8) & 0x00ff00);
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
        }
    }
}

void CPlane32::BrightBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, int nBright, const CClipper& rClip) {
    if (nBright <= 0) {
        Blt(rSrc, ptDst, rcSrc, rClip);
    } else if (0x100 <= nBright) {
        Fill(RGB(0xff, 0xff, 0xff), CRect(ptDst, rcSrc.GetSize()));
    } else if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        Themis_PlaneGetMMXWidth32(nSrcW, nMMXSize, nCppSize);
        if (0 < nMMXSize) {
            Detail::MMXBrightBlt32(pDst, nDstPitch, pSrc, nSrcPitch, nMMXSize, nSrcH, nBright);
        }
        //csNZ̏(xĂc)
        for (int y = 0; y < nSrcH; y++) {
            for (int i = 0; i < nCppSize; i++) {
                const int nSrcRB = pSrc[nMMXSize * 2 + i] & 0xff00ff;
                const int nSrcG = pSrc[nMMXSize * 2 + i] & 0x00ff00;
                pDst[nMMXSize * 2 + i] = static_cast<uint32_t>(
                    (((0xff00ff - nSrcRB) * nBright >> 8) + nSrcRB) & 0xff00ff |
                    (((0x00ff00 - nSrcG) * nBright >> 8) + nSrcG) & 0x00ff00);
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
        }
    }
}

void CPlane32::AlphaBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, int nAlpha, const CClipper& rClip) {
    if (nAlpha <= 0) {
    } else if (0x100 <= nAlpha) {
        Blt(rSrc, ptDst, rcSrc, rClip);
    } else if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        Themis_PlaneGetMMXWidth32(nSrcW, nMMXSize, nCppSize);
        if (0 < nMMXSize) {
            Detail::MMXAlphaBlt32(pDst, nDstPitch, pDst, nDstPitch, pSrc, nSrcPitch, nMMXSize, nSrcH, nAlpha);
        }
        //csNZ̏(xĂc)
        for (int y = 0; y < nSrcH; y++) {
            for (int i = 0; i < nCppSize; i++) {
                uint32_t nSrc = pSrc[nMMXSize * 2 + i];
                uint32_t nDst = pDst[nMMXSize * 2 + i];
                uint32_t nSrcRB = nSrc & 0xff00ff;
                uint32_t nDstRB = nDst & 0xff00ff;
                uint32_t nSrcG = nSrc & 0xff00;
                uint32_t nDstG = nDst & 0xff00;
                pDst[nMMXSize * 2 + i] = static_cast<uint32_t>(
                    (((nSrcRB - nDstRB) * nAlpha >> 8) + nDstRB) & 0xff00ff |
                    (((nSrcG  - nDstG)  * nAlpha >> 8) + nDstG)  & 0x00ff00);
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
        }
    }
}

void CPlane32::BlendBlt(const CPlane& rSrc, const CPlane& rSrc2, CPoint ptDst, CRect rcSrc, int nAlpha, const CClipper& rClip) {
    if (nAlpha <= 0) {
        Blt(rSrc2, ptDst, rcSrc, rClip);
    } else if (0x100 <= nAlpha) {
        Blt(rSrc, ptDst, rcSrc, rClip);
    } else if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneLock4(rSrc2, rcSrc.GetPoint(), pSrc2, nSrcPitch2);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        Themis_PlaneGetMMXWidth32(nSrcW, nMMXSize, nCppSize);
        if (0 < nMMXSize) {
            Detail::MMXAlphaBlt32(pDst, nDstPitch, pSrc2, nSrcPitch2, pSrc, nSrcPitch, nMMXSize, nSrcH, nAlpha);
        }
        //csNZ̏(xĂc)
        for (int y = 0; y < nSrcH; y++) {
            for (int x = 0; x < nCppSize; x++) {
                uint32_t nSrc = pSrc[nMMXSize * 2 + x];
                uint32_t nDst = pSrc2[nMMXSize * 2 + x];
                uint32_t nSrcRB = nSrc & 0xff00ff;
                uint32_t nDstRB = nDst & 0xff00ff;
                uint32_t nSrcG = nSrc & 0xff00;
                uint32_t nDstG = nDst & 0xff00;
                pDst[nMMXSize * 2 + x] = static_cast<uint32_t>(
                    (((nSrcRB - nDstRB) * nAlpha >> 8) + nDstRB) & 0xff00ff |
                    (((nSrcG  - nDstG)  * nAlpha >> 8) + nDstG)  & 0x00ff00);
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
            pSrc2 += nSrcPitch2;
        }
    }
}

void CPlane32::MaskBlt(const CPlane& rSrc, const CPlane8& rMask, CPoint ptDst, CRect rcSrc, int nLow, int nHigh, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        bool abTrans[0x100] = {false};
        for (int i = Max<long>(nLow, 0), n = Min<long>(nHigh + 1, 0x100); i < n; i++) {
            abTrans[i] = true;
        }
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneLock1(rMask, rcSrc.GetPoint(), pMsk, nMskPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        for (int y = 0; y < nSrcH; y++) {
            for (int x = 0; x < nSrcW; x++) {
                if (abTrans[pMsk[x]]) {
                    pDst[x] = pSrc[x];
                }
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
            pMsk += nMskPitch;
        }
    }
}

void CPlane32::MaskFadeBlt(const CPlane& rSrc, const CPlane8& rMask, CPoint ptDst, CRect rcSrc, int nBorder, int nVague, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        int anFade[0x100] = {0};
        for (int i = Max<long>(0, nBorder), n = Min<long>(nVague + nBorder, 0x100); i < n; i++) {
            anFade[i] = 0x100 * (i + 1 - nBorder) / (nVague + 1);     //0`0x100܂ŒiKIɕςĂ
        }
        for (int i = Max<long>(0, nBorder + nVague); i < 0x100; i++) {
            anFade[i] = 0x100;                                  //nBorder + nVagueȏ͖Oa
        }
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneLock1(rMask, rcSrc.GetPoint(), pMsk, nMskPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        for (int y = 0; y < nSrcH; y++) {
            for (int x = 0; x < nSrcW; x++) {
                int nFade = anFade[static_cast<unsigned>(pMsk[x])];
                if (nFade == 0x00) {
                    pDst[x] = pSrc[x];  //P]
                } else if (nFade == 0x100) {
                    pDst[x] = 0x000000; //^
                } else {
                    nFade = 0x100 - nFade;
                    uint32_t nSrcRB = pSrc[x] & ~0xff00ff00;
                    uint32_t nSrcG = (pSrc[x] >> 8) & 0xff;
                    pDst[x] = static_cast<uint32_t>((nSrcRB * nFade >> 8) & 0xff00ff | (nSrcG * nFade) & 0xff00);
                }
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
            pMsk += nMskPitch;
        }
    }
}

void CPlane32::MaskBrightBlt(const CPlane& rSrc, const CPlane8& rMask, CPoint ptDst, CRect rcSrc, int nBorder, int nVague, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        int anBright[0x100] = {0};
        for (int i = Max<long>(0, nBorder), n = Min<long>(nVague + nBorder, 0x100); i < n; i++) {
            anBright[i] = 0x100 * (i + 1 - nBorder) / (nVague + 1);     //0`0x100܂ŒiKIɕςĂ
        }
        for (int i = Max<long>(0, nBorder + nVague); i < 0x100; i++) {
            anBright[i] = 0x100;                                  //nBorder + nVagueȏ͖Oa
        }
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneLock1(rMask, rcSrc.GetPoint(), pMsk, nMskPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        for (int y = 0; y < nSrcH; y++) {
            for (int x = 0; x < nSrcW; x++) {
                int nBright = anBright[static_cast<unsigned>(pMsk[x])];
                if (nBright == 0x00) {
                    pDst[x] = pSrc[x];      //P]
                } else if (nBright == 0x100) {
                    pDst[x] = 0xffffffff;   //^
                } else {
                    uint32_t nSrcRB = pSrc[x] & 0xff00ff;
                    uint32_t nSrcG = pSrc[x] & 0xff00;
                    pDst[x] = static_cast<uint32_t>(
                        (((0xff00ff - nSrcRB) * nBright >> 8) + nSrcRB) & 0xff00ff |
                        (((0xff00 - nSrcG) * nBright >> 8) + nSrcG) & 0xff00
                        );
                }
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
            pMsk += nMskPitch;
        }
    }
}

void CPlane32::MaskBlendBlt(const CPlane& rSrc, const CPlane& rSrc2, const CPlane8& rMask, CPoint ptDst, CRect rcSrc, int nBorder, int nVague, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        int anAlpha[0x100] = {0};
        for (int i = Max<long>(0, nBorder), n = Min<long>(nVague + nBorder, 0x100); i < n; i++) {
            anAlpha[i] = 0x100 * (i + 1 - nBorder) / (nVague + 1);        //0`0x100܂ŒiKIɕςĂ
        }
        for (int i = Max<long>(0, nBorder + nVague); i < 0x100; i++) {
            anAlpha[i] = 0x100;                                     //nBorder + nVagueȏ͖Oa
        }
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneLock4(rSrc2, rcSrc.GetPoint(), pSrc2, nSrcPitch2);
        Themis_PlaneLock1(rMask, rcSrc.GetPoint(), pMsk, nMskPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        for (int y = 0; y < nSrcH; y++) {
            for (int x = 0; x < nSrcW; x++) {
                int nAlpha = anAlpha[static_cast<unsigned>(pMsk[x])];
                if (nAlpha == 0x00) {
                    pDst[x] = pSrc[x];      //P]
                } else if (nAlpha == 0x100) {
                    pDst[x] = pSrc2[x];     //P]
                } else {
                    uint32_t nSrc2R = pSrc2[x] & 0xff0000;
                    uint32_t nSrcR = pSrc[x] & 0xff0000;
                    uint32_t nSrc2G = pSrc2[x] & 0xff00;
                    uint32_t nSrcG = pSrc[x] & 0xff00;
                    uint32_t nSrc2B = pSrc2[x] & 0xff;
                    uint32_t nSrcB = pSrc[x] & 0xff;
                    pDst[x] = static_cast<uint32_t>(
                        (((nSrc2R - nSrcR) * nAlpha >> 8) + nSrcR) & 0xff0000 |
                        (((nSrc2G - nSrcG) * nAlpha >> 8) + nSrcG) & 0xff00 |
                        (((nSrc2B - nSrcB) * nAlpha >> 8) + nSrcB) & 0xff
                        );
                }
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
            pSrc2 += nSrcPitch2;
            pMsk += nMskPitch;
        }
    }
}

void CPlane32::BltAlpha(const CPlane32& rSrc, CPoint ptDst, CRect rcSrc, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        for (int y = 0; y < nSrcH; y++) {
            for (int x = 0; x < nSrcW; x++) {
                const uint32_t dwSrc = pSrc[x];
                const uint32_t dwDst = pDst[x];
                const unsigned nAlpha = (dwSrc >> 24); // & 0xff
                if (nAlpha == 0x00) {       //
                } else if (nAlpha == 0xff) {  //s
                    pDst[x] = dwSrc;
                } else {                      //
                    const uint32_t nSrcRB = dwSrc & 0xff00ff;
                    const uint32_t nDstRB = dwDst & 0xff00ff;
                    const uint32_t nSrcG = dwSrc & 0xff00;
                    const uint32_t nDstG = dwDst & 0xff00;
                    const unsigned nRB = (((nSrcRB - nDstRB) * nAlpha >> 8) + nDstRB) & 0xff00ff;
                    const unsigned nG  = (((nSrcG  - nDstG)  * nAlpha >> 8) + nDstG) & 0x00ff00;
                    pDst[x] = nRB | nG;
                }
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
        }
    }
}

namespace {
    /// {]ƃNbsO1/2]ƁB
    bool StretchBltGeneral(CPlane32& rDst, const CPlane& rSrc, CRect& rcDst, CRect& rcSrc, const CClipper& rClip, bool bBiLinear) {
        if (CSize(rcDst.GetSize()) == rcSrc.GetSize()) {
            rDst.Blt(rSrc, rcDst.GetPoint(), rcSrc, rClip);
            return true;
        }
        if (!rClip.Clipping(rDst.GetSize(), rSrc.GetSize(), rcDst, rcSrc)) {
            return true;
        }
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        if (bBiLinear && nDstW * 2 == nSrcW && nDstH * 2 == nSrcH) {
            //c҂1/2{B
            Themis_PlaneLock4(rDst, rcDst.GetPoint(), pDst, nDstPitch);
            Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
            for (int y = 0; y < nDstH; y++) {
                for (int x = 0; x < nDstW; x++) {
                    uint32_t rb = pSrc[x * 2] & 0xff00ff;
                    uint32_t g  = pSrc[x * 2] & 0x00ff00;
                    rb += pSrc[x * 2 + 1] & 0xff00ff;
                    g  += pSrc[x * 2 + 1] & 0x00ff00;
                    rb += pSrc[x * 2 + nSrcPitch] & 0xff00ff;
                    g  += pSrc[x * 2 + nSrcPitch] & 0x00ff00;
                    rb += pSrc[x * 2 + nSrcPitch + 1] & 0xff00ff;
                    g  += pSrc[x * 2 + nSrcPitch + 1] & 0x00ff00;
                    pDst[x] = (rb / 4 & 0xff00ff) | (g / 4 & 0x00ff00);
                }
                pDst += nDstPitch;
                pSrc += nSrcPitch * 2;
            }
            return true;
        } else if (nDstW == nSrcW * 2 && nDstH == nSrcH * 2) {
            //c҂2{B
            Themis_PlaneLock4(rDst, rcDst.GetPoint(), pDst, nDstPitch);
            Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
            for (int y = 0; y < nSrcH; y++) {
                for (int x = 0; x < nSrcW; x++) {
                    pDst[x * 2] = pSrc[x];
                    pDst[x * 2 + 1] = pSrc[x];
                    pDst[x * 2 + nDstPitch] = pSrc[x];
                    pDst[x * 2 + 1 + nDstPitch] = pSrc[x];
                }
                pDst += nDstPitch * 2;
                pSrc += nSrcPitch;
            }
            return true;
        } else if (bBiLinear && nDstW * 4 == nSrcW && nDstH * 4 == nSrcH) {
            //c҂1/4{
            Themis_PlaneLock4(rDst, rcDst.GetPoint(), pDst, nDstPitch);
            Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
            for (int y = 0; y < nDstH; y++) {
                for (int x = 0; x < nDstW; x++) {
                    uint32_t rb = pSrc[x * 4] & 0xff00ff;
                    uint32_t g  = pSrc[x * 4] & 0x00ff00;
                    rb += pSrc[x * 4 + 1] & 0xff00ff;
                    g  += pSrc[x * 4 + 1] & 0x00ff00;
                    rb += pSrc[x * 4 + 2] & 0xff00ff;
                    g  += pSrc[x * 4 + 2] & 0x00ff00;
                    rb += pSrc[x * 4 + 3] & 0xff00ff;
                    g  += pSrc[x * 4 + 3] & 0x00ff00;
                    rb += pSrc[x * 4 + nSrcPitch] & 0xff00ff;
                    g  += pSrc[x * 4 + nSrcPitch] & 0x00ff00;
                    rb += pSrc[x * 4 + nSrcPitch + 1] & 0xff00ff;
                    g  += pSrc[x * 4 + nSrcPitch + 1] & 0x00ff00;
                    rb += pSrc[x * 4 + nSrcPitch + 2] & 0xff00ff;
                    g  += pSrc[x * 4 + nSrcPitch + 2] & 0x00ff00;
                    rb += pSrc[x * 4 + nSrcPitch + 3] & 0xff00ff;
                    g  += pSrc[x * 4 + nSrcPitch + 3] & 0x00ff00;
                    rb += pSrc[x * 4 + nSrcPitch * 2] & 0xff00ff;
                    g  += pSrc[x * 4 + nSrcPitch * 2] & 0x00ff00;
                    rb += pSrc[x * 4 + nSrcPitch * 2 + 1] & 0xff00ff;
                    g  += pSrc[x * 4 + nSrcPitch * 2 + 1] & 0x00ff00;
                    rb += pSrc[x * 4 + nSrcPitch * 2 + 2] & 0xff00ff;
                    g  += pSrc[x * 4 + nSrcPitch * 2 + 2] & 0x00ff00;
                    rb += pSrc[x * 4 + nSrcPitch * 2 + 3] & 0xff00ff;
                    g  += pSrc[x * 4 + nSrcPitch * 2 + 3] & 0x00ff00;
                    rb += pSrc[x * 4 + nSrcPitch * 3] & 0xff00ff;
                    g  += pSrc[x * 4 + nSrcPitch * 3] & 0x00ff00;
                    rb += pSrc[x * 4 + nSrcPitch * 3 + 1] & 0xff00ff;
                    g  += pSrc[x * 4 + nSrcPitch * 3 + 1] & 0x00ff00;
                    rb += pSrc[x * 4 + nSrcPitch * 3 + 2] & 0xff00ff;
                    g  += pSrc[x * 4 + nSrcPitch * 3 + 2] & 0x00ff00;
                    rb += pSrc[x * 4 + nSrcPitch * 3 + 3] & 0xff00ff;
                    g  += pSrc[x * 4 + nSrcPitch * 3 + 3] & 0x00ff00;
                    pDst[x] = (rb / 16 & 0xff00ff) | (g / 16 & 0x00ff00);
                }
                pDst += nDstPitch;
                pSrc += nSrcPitch * 4;
            }
            return true;
        } else if (nDstW == nSrcW * 4 && nDstH == nSrcH * 4) {
            //c҂4{
            Themis_PlaneLock4(rDst, rcDst.GetPoint(), pDst, nDstPitch);
            Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
            for (int y = 0; y < nSrcH; y++) {
                for (int x = 0; x < nSrcW; x++) {
                    pDst[x * 4] = pSrc[x];
                    pDst[x * 4 + 1] = pSrc[x];
                    pDst[x * 4 + 2] = pSrc[x];
                    pDst[x * 4 + 3] = pSrc[x];
                    pDst[x * 4 + nDstPitch] = pSrc[x];
                    pDst[x * 4 + nDstPitch + 1] = pSrc[x];
                    pDst[x * 4 + nDstPitch + 2] = pSrc[x];
                    pDst[x * 4 + nDstPitch + 3] = pSrc[x];
                    pDst[x * 4 + nDstPitch * 2] = pSrc[x];
                    pDst[x * 4 + nDstPitch * 2 + 1] = pSrc[x];
                    pDst[x * 4 + nDstPitch * 2 + 2] = pSrc[x];
                    pDst[x * 4 + nDstPitch * 2 + 3] = pSrc[x];
                    pDst[x * 4 + nDstPitch * 3] = pSrc[x];
                    pDst[x * 4 + nDstPitch * 3 + 1] = pSrc[x];
                    pDst[x * 4 + nDstPitch * 3 + 2] = pSrc[x];
                    pDst[x * 4 + nDstPitch * 3 + 3] = pSrc[x];
                }
                pDst += nDstPitch * 4;
                pSrc += nSrcPitch;
            }
            return true;
        }
        return false;
    }
};

void CPlane32::StretchBltNearest(const CPlane& rSrc, CRect rcDst, CRect rcSrc, const CClipper& rClip) {
    if (!StretchBltGeneral(*this, rSrc, rcDst, rcSrc, rClip, false)) {
        Themis_PlaneLock4(*this, rcDst.GetPoint(), pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        const fint fRateW2 = ToFint(nSrcW) / nDstW;
        const fint fRateH2 = ToFint(nSrcH) / nDstH;
        boost::scoped_array<int> anSrcX(new int[nDstW]);
        for (int x = 0; x < nDstW; x++) {
            //]悘Wɑ΂]̂WvZ
            anSrcX[x] = FintToInt(x * fRateW2 + fRateW2 / 2 + c_nFint_1 / 2);
            if (nSrcW < anSrcX[x] + 1) { // Ô(?)͂ݏo␳
                anSrcX[x] = nSrcW - 1;
            }
        }
        for (int y = 0; y < nDstH; y++) {
            int nSrcY = FintToInt(y * fRateH2 + fRateH2 / 2 + c_nFint_1 / 2);
            if (nSrcH < nSrcY + 1) { // Ô(?)͂ݏo␳
                nSrcY = nSrcH - 1;
            }
            uint32_t* pSrc2 = pSrc + nSrcPitch * nSrcY;
            for (int x = 0; x < nDstW; x++) {
                pDst[x] = pSrc2[anSrcX[x]];
            }
            pDst += nDstPitch;
        }
    }
}

void CPlane32::StretchBlt(const CPlane& rSrc, CRect rcDst, CRect rcSrc, const CClipper& rClip) {
    if (!StretchBltGeneral(*this, rSrc, rcDst, rcSrc, rClip, true)) {
        Themis_PlaneLock4(*this, rcDst.GetPoint(), pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        const fint fRateW2 = ToFint(nSrcW) / nDstW;
        const fint fRateH2 = ToFint(nSrcH) / nDstH;
        boost::scoped_array<int> anSrcX1(new int[nDstW]);
        boost::scoped_array<int> anSrcX2(new int[nDstW]);
        for (int x = 0; x < nDstW; x++) {
            //]悘Wɑ΂]̂W͈̔͂vZ
            fint fX = x * fRateW2;
            anSrcX1[x] = FintCeilToInt(fX);
            anSrcX2[x] = FintToInt(fX + fRateW2);
            Themis_DebugAssert(0 <= anSrcX1[x] && anSrcX1[x] < nSrcW + 1);
            Themis_DebugAssert(0 <= anSrcX2[x] && anSrcX2[x] < nSrcW + 1);
            if (nSrcW <= anSrcX1[x]) {
                anSrcX1[x] = nSrcW - 1;
            }
            if (nSrcW <= anSrcX2[x]) {
                anSrcX2[x] = nSrcW - 1;
            }
        }
        for (int y = 0; y < nDstH; y++) {
            //]悙Wɑ΂]̂W͈̔͂vZ
            fint fY = y * fRateH2;
            int nY1 = FintCeilToInt(fY);
            int nY2 = FintToInt(fY + fRateH2);
            Themis_DebugAssert(0 <= nY1 && nY1 < nSrcH + 1);
            Themis_DebugAssert(0 <= nY2 && nY2 < nSrcH + 1);
            if (nSrcH <= nY1) {
                nY1 = nSrcH - 1;
            }
            if (nSrcH <= nY2) {
                nY2 = nSrcH - 1;
            }
            for (int x = 0; x < nDstW; x++) {
                int nX1 = anSrcX1[x];
                int nX2 = anSrcX2[x];
                // ]iSsNZ̕ϒlj
                uint32_t* ps1 = pSrc + nSrcPitch * nY1;
                uint32_t* ps2 = pSrc + nSrcPitch * nY2;
                pDst[x] = Detail::GetAverage(ps1[nX1], ps1[nX2], ps2[nX1], ps2[nX2]);
            }
            pDst += nDstPitch;
        }
    }
}

// قځ̂Rsy
void CPlane32::StretchBltAlpha(const CPlane& rSrc, CRect rcDst, CRect rcSrc, const CClipper& rClip) {
    if (!StretchBltGeneral(*this, rSrc, rcDst, rcSrc, rClip, true)) {
        Themis_PlaneLock4(*this, rcDst.GetPoint(), pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        const fint fRateW2 = ToFint(nSrcW) / nDstW;
        const fint fRateH2 = ToFint(nSrcH) / nDstH;
        boost::scoped_array<int> anSrcX1(new int[nDstW]);
        boost::scoped_array<int> anSrcX2(new int[nDstW]);
        for (int x = 0; x < nDstW; x++) {
            //]悘Wɑ΂]̂W͈̔͂vZ
            fint fX = x * fRateW2;
            anSrcX1[x] = FintCeilToInt(fX);
            anSrcX2[x] = FintToInt(fX + fRateW2);
            Themis_DebugAssert(0 <= anSrcX1[x] && anSrcX1[x] < nSrcW + 1);
            Themis_DebugAssert(0 <= anSrcX2[x] && anSrcX2[x] < nSrcW + 1);
            if (nSrcW <= anSrcX1[x]) {
                anSrcX1[x] = nSrcW - 1;
            }
            if (nSrcW <= anSrcX2[x]) {
                anSrcX2[x] = nSrcW - 1;
            }
        }
        for (int y = 0; y < nDstH; y++) {
            //]悙Wɑ΂]̂W͈̔͂vZ
            fint fY = y * fRateH2;
            int nY1 = FintCeilToInt(fY);
            int nY2 = FintToInt(fY + fRateH2);
            Themis_DebugAssert(0 <= nY1 && nY1 < nSrcH + 1);
            Themis_DebugAssert(0 <= nY2 && nY2 < nSrcH + 1);
            if (nSrcH <= nY1) {
                nY1 = nSrcH - 1;
            }
            if (nSrcH <= nY2) {
                nY2 = nSrcH - 1;
            }
            for (int x = 0; x < nDstW; x++) {
                int nX1 = anSrcX1[x];
                int nX2 = anSrcX2[x];
                // ܂܂sNZ̐v
                int a = 0, r = 0, g = 0, b = 0;
                uint32_t* ps1 = pSrc + nSrcPitch * nY1;
                uint32_t* ps2 = pSrc + nSrcPitch * nY2;
                b += ((ps1[nX1]) & 0xff);
                g += ((ps1[nX1]>>8) & 0xff);
                r += ((ps1[nX1]>>16) & 0xff);
                a += ((ps1[nX1]>>24) & 0xff);
                b += ((ps1[nX2]) & 0xff);
                g += ((ps1[nX2]>>8) & 0xff);
                r += ((ps1[nX2]>>16) & 0xff);
                a += ((ps1[nX2]>>24) & 0xff);
                b += ((ps2[nX1]) & 0xff);
                g += ((ps2[nX1]>>8) & 0xff);
                r += ((ps2[nX1]>>16) & 0xff);
                a += ((ps2[nX1]>>24) & 0xff);
                b += ((ps2[nX2]) & 0xff);
                g += ((ps2[nX2]>>8) & 0xff);
                r += ((ps2[nX2]>>16) & 0xff);
                a += ((ps2[nX2]>>24) & 0xff);
                // ]sNZ̐vZ
                pDst[x] = b >> 2;
                pDst[x] |= (g << 6) & 0xff00;
                pDst[x] |= (r << 14) & 0xff0000;
                pDst[x] |= (a << 22) & 0xff000000;
            }
            pDst += nDstPitch;
        }
    }
}

namespace {
    static void PixelAverationH(CPlane& rDst, const CPlane& rSrc, const CPoint& ptDst, const CPoint& ptSrc, int nDstW, int nSrcW, int nHeight) {
        Themis_PlaneLock4(rDst, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, ptSrc, pSrc, nSrcPitch);
        Detail::SPixelAverationData oData;
        Detail::SetPixelAverationParam(nDstW, nSrcW, oData);
        if (CPlane::IsUseParallel()) {
            Parallel& pp = Parallel::GetInstance();
            pp.For(std::tr1::bind(&Detail::PixelAverationH, pDst, nDstPitch, pSrc, nSrcPitch,
                nDstW, nSrcW, _1, _2, std::tr1::cref(oData)), 0, nHeight);
        } else {
            Detail::PixelAverationH(pDst, nDstPitch, pSrc, nSrcPitch, nDstW, nSrcW, 0, nHeight, oData);
        }
    }
    static void PixelAverationHAlpha(CPlane& rDst, const CPlane& rSrc, const CPoint& ptDst, const CPoint& ptSrc, int nDstW, int nSrcW, int nHeight) {
        Themis_PlaneLock4(rDst, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, ptSrc, pSrc, nSrcPitch);
        Detail::SPixelAverationData oData;
        Detail::SetPixelAverationParam(nDstW, nSrcW, oData);
        if (CPlane::IsUseParallel()) {
            Parallel& pp = Parallel::GetInstance();
            pp.For(std::tr1::bind(&Detail::PixelAverationHAlpha, pDst, nDstPitch, pSrc, nSrcPitch,
                nDstW, nSrcW, _1, _2, std::tr1::cref(oData)), 0, nHeight);
        } else {
            Detail::PixelAverationHAlpha(pDst, nDstPitch, pSrc, nSrcPitch, nDstW, nSrcW, 0, nHeight, oData);
        }
    }
    static void PixelAverationV(CPlane& rDst, const CPlane& rSrc, const CPoint& ptDst, const CPoint& ptSrc, int nWidth, int nDstH, int nSrcH) {
        Themis_PlaneLock4(rDst, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, ptSrc, pSrc, nSrcPitch);
        Detail::SPixelAverationData oData;
        Detail::SetPixelAverationParam(nDstH, nSrcH, oData);
        if (CPlane::IsUseParallel()) {
            Parallel& pp = Parallel::GetInstance();
            pp.For(std::tr1::bind(&Detail::PixelAverationV, pDst, nDstPitch, pSrc, nSrcPitch,
                nWidth, _1, _2, nSrcH, std::tr1::cref(oData)), 0, nDstH);
        } else {
            Detail::PixelAverationV(pDst, nDstPitch, pSrc, nSrcPitch, nWidth, 0, nDstH, nSrcH, oData);
        }
    }
    static void PixelAverationVAlpha(CPlane& rDst, const CPlane& rSrc, const CPoint& ptDst, const CPoint& ptSrc, int nWidth, int nDstH, int nSrcH) {
        Themis_PlaneLock4(rDst, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, ptSrc, pSrc, nSrcPitch);
        Detail::SPixelAverationData oData;
        Detail::SetPixelAverationParam(nDstH, nSrcH, oData);
        if (CPlane::IsUseParallel()) {
            Parallel& pp = Parallel::GetInstance();
            pp.For(std::tr1::bind(&Detail::PixelAverationVAlpha, pDst, nDstPitch, pSrc, nSrcPitch,
                nWidth, _1, _2, nSrcH, std::tr1::cref(oData)), 0, nDstH);
        } else {
            Detail::PixelAverationVAlpha(pDst, nDstPitch, pSrc, nSrcPitch, nWidth, 0, nDstH, nSrcH, oData);
        }
    }
}
void CPlane32::StretchBltPixelAveration(const CPlane& rSrc, CRect rcDst, CRect rcSrc, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), rcDst, rcSrc)) {
        CMemoryPlane32 oTmp;
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        if (nDstW * nSrcH <= nSrcW * nDstH) { // H  V
            oTmp.SetSize(CSize(nDstW, nSrcH));
            PixelAverationH(oTmp, rSrc, CPoint(), rcSrc.GetPoint(), nDstW, nSrcW, nSrcH);
            PixelAverationV(*this, oTmp, rcDst.GetPoint(), CPoint(), nDstW, nDstH, nSrcH);
        } else { // V  H
            oTmp.SetSize(CSize(nSrcW, nDstH));
            PixelAverationV(oTmp, rSrc, CPoint(), rcSrc.GetPoint(), nSrcW, nDstH, nSrcH);
            PixelAverationH(*this, oTmp, rcDst.GetPoint(), CPoint(), nDstW, nSrcW, nDstH);
        }
    }
}

void CPlane32::StretchBltPixelAverationAlpha(const CPlane& rSrc, CRect rcDst, CRect rcSrc, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), rcDst, rcSrc)) {
        CMemoryPlane32 oTmp;
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        if (nDstW * nSrcH <= nSrcW * nDstH) { // H  V
            oTmp.SetSize(CSize(nDstW, nSrcH));
            PixelAverationHAlpha(oTmp, rSrc, CPoint(), rcSrc.GetPoint(), nDstW, nSrcW, nSrcH);
            PixelAverationVAlpha(*this, oTmp, rcDst.GetPoint(), CPoint(), nDstW, nDstH, nSrcH);
        } else { // V  H
            oTmp.SetSize(CSize(nSrcW, nDstH));
            PixelAverationVAlpha(oTmp, rSrc, CPoint(), rcSrc.GetPoint(), nSrcW, nDstH, nSrcH);
            PixelAverationHAlpha(*this, oTmp, rcDst.GetPoint(), CPoint(), nDstW, nSrcW, nDstH);
        }
    }
}

namespace {
    static void LanczosH(CPlane& rDst, const CPlane& rSrc, const CPoint& ptDst, const CPoint& ptSrc, int nDstW, int nSrcW, int nHeight) {
        Themis_PlaneLock4(rDst, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, ptSrc, pSrc, nSrcPitch);
        if (CPlane::IsUse3DNow() && CCpuId::Is3DNow()) {
            Detail::SLanczosData3DNow oData;
            Detail::SetLanczosParam3DNow(nDstW, nSrcW, oData);
            if (CPlane::IsUseParallel()) {
                Parallel& pp = Parallel::GetInstance();
                pp.For(std::tr1::bind(&Detail::_3DNowLanczosH, pDst, nDstPitch, pSrc, nSrcPitch,
                    nDstW, nSrcW, _1, _2, std::tr1::cref(oData)), 0, nHeight);
            } else {
                Detail::_3DNowLanczosH(pDst, nDstPitch, pSrc, nSrcPitch, nDstW, nSrcW, 0, nHeight, oData);
            }
        } else {
            Detail::SLanczosData oData;
            Detail::SetLanczosParam(nDstW, nSrcW, oData);
            if (CPlane::IsUseParallel()) {
                Parallel& pp = Parallel::GetInstance();
                pp.For(std::tr1::bind(&Detail::LanczosH, pDst, nDstPitch, pSrc, nSrcPitch,
                    nDstW, nSrcW, _1, _2, std::tr1::cref(oData)), 0, nHeight);
            } else {
                Detail::LanczosH(pDst, nDstPitch, pSrc, nSrcPitch, nDstW, nSrcW, 0, nHeight, oData);
            }
        }
    }
    static void LanczosHAlpha(CPlane& rDst, const CPlane& rSrc, const CPoint& ptDst, const CPoint& ptSrc, int nDstW, int nSrcW, int nHeight) {
        Themis_PlaneLock4(rDst, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, ptSrc, pSrc, nSrcPitch);
        if (CPlane::IsUse3DNow() && CCpuId::Is3DNow()) {
            Detail::SLanczosData3DNow oData;
            Detail::SetLanczosParam3DNow(nDstW, nSrcW, oData);
            if (CPlane::IsUseParallel()) {
                Parallel& pp = Parallel::GetInstance();
                pp.For(std::tr1::bind(&Detail::_3DNowLanczosH, pDst, nDstPitch, pSrc, nSrcPitch,
                    nDstW, nSrcW, _1, _2, std::tr1::cref(oData)), 0, nHeight);
            } else {
                Detail::_3DNowLanczosH(pDst, nDstPitch, pSrc, nSrcPitch, nDstW, nSrcW, 0, nHeight, oData);
            }
        } else {
            Detail::SLanczosData oData;
            Detail::SetLanczosParam(nDstW, nSrcW, oData);
            if (CPlane::IsUseParallel()) {
                Parallel& pp = Parallel::GetInstance();
                pp.For(std::tr1::bind(&Detail::LanczosHAlpha, pDst, nDstPitch, pSrc, nSrcPitch,
                    nDstW, nSrcW, _1, _2, std::tr1::cref(oData)), 0, nHeight);
            } else {
                Detail::LanczosHAlpha(pDst, nDstPitch, pSrc, nSrcPitch, nDstW, nSrcW, 0, nHeight, oData);
            }
        }
    }
    static void LanczosV(CPlane& rDst, const CPlane& rSrc, const CPoint& ptDst, const CPoint& ptSrc, int nWidth, int nDstH, int nSrcH) {
        Themis_PlaneLock4(rDst, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, ptSrc, pSrc, nSrcPitch);
        if (CPlane::IsUse3DNow() && CCpuId::Is3DNow()) {
            Detail::SLanczosData3DNow oData;
            Detail::SetLanczosParam3DNow(nDstH, nSrcH, oData);
            if (CPlane::IsUseParallel()) {
                Parallel& pp = Parallel::GetInstance();
                pp.For(std::tr1::bind(&Detail::_3DNowLanczosV, pDst, nDstPitch, pSrc, nSrcPitch,
                    nWidth, _1, _2, nSrcH, std::tr1::cref(oData)), 0, nDstH);
            } else {
                Detail::_3DNowLanczosV(pDst, nDstPitch, pSrc, nSrcPitch, nWidth, 0, nDstH, nSrcH, oData);
            }
        } else {
            Detail::SLanczosData oData;
            Detail::SetLanczosParam(nDstH, nSrcH, oData);
            if (CPlane::IsUseParallel()) {
                Parallel& pp = Parallel::GetInstance();
                pp.For(std::tr1::bind(&Detail::LanczosV, pDst, nDstPitch, pSrc, nSrcPitch,
                    nWidth, _1, _2, nSrcH, std::tr1::cref(oData)), 0, nDstH);
            } else {
                Detail::LanczosV(pDst, nDstPitch, pSrc, nSrcPitch, nWidth, 0, nDstH, nSrcH, oData);
            }
        }
    }
    static void LanczosVAlpha(CPlane& rDst, const CPlane& rSrc, const CPoint& ptDst, const CPoint& ptSrc, int nWidth, int nDstH, int nSrcH) {
        Themis_PlaneLock4(rDst, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, ptSrc, pSrc, nSrcPitch);
        if (CPlane::IsUse3DNow() && CCpuId::Is3DNow()) {
            Detail::SLanczosData3DNow oData;
            Detail::SetLanczosParam3DNow(nDstH, nSrcH, oData);
            if (CPlane::IsUseParallel()) {
                Parallel& pp = Parallel::GetInstance();
                pp.For(std::tr1::bind(&Detail::_3DNowLanczosV, pDst, nDstPitch, pSrc, nSrcPitch,
                    nWidth, _1, _2, nSrcH, std::tr1::cref(oData)), 0, nDstH);
            } else {
                Detail::_3DNowLanczosV(pDst, nDstPitch, pSrc, nSrcPitch, nWidth, 0, nDstH, nSrcH, oData);
            }
        } else {
            Detail::SLanczosData oData;
            Detail::SetLanczosParam(nDstH, nSrcH, oData);
            if (CPlane::IsUseParallel()) {
                Parallel& pp = Parallel::GetInstance();
                pp.For(std::tr1::bind(&Detail::LanczosVAlpha, pDst, nDstPitch, pSrc, nSrcPitch,
                    nWidth, _1, _2, nSrcH, std::tr1::cref(oData)), 0, nDstH);
            } else {
                Detail::LanczosVAlpha(pDst, nDstPitch, pSrc, nSrcPitch, nWidth, 0, nDstH, nSrcH, oData);
            }
        }
    }
}
void CPlane32::StretchBltLanczos3(const CPlane& rSrc, CRect rcDst, CRect rcSrc, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), rcDst, rcSrc)) {
        CMemoryPlane32 oTmp;
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        if (nDstW * nSrcH <= nSrcW * nDstH) { // H  V
            oTmp.SetSize(CSize(nDstW, nSrcH));
            LanczosH(oTmp, rSrc, CPoint(), rcSrc.GetPoint(), nDstW, nSrcW, nSrcH);
            LanczosV(*this, oTmp, rcDst.GetPoint(), CPoint(), nDstW, nDstH, nSrcH);
        } else { // V  H
            oTmp.SetSize(CSize(nSrcW, nDstH));
            LanczosV(oTmp, rSrc, CPoint(), rcSrc.GetPoint(), nSrcW, nDstH, nSrcH);
            LanczosH(*this, oTmp, rcDst.GetPoint(), CPoint(), nDstW, nSrcW, nDstH);
        }
    }
}

void CPlane32::StretchBltLanczos3Alpha(const CPlane& rSrc, CRect rcDst, CRect rcSrc, const CClipper& rClip) {
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), rcDst, rcSrc)) {
        CMemoryPlane32 oTmp;
        Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        if (nDstW * nSrcH <= nSrcW * nDstH) { // H  V
            oTmp.SetSize(CSize(nDstW, nSrcH));
            LanczosHAlpha(oTmp, rSrc, CPoint(), rcSrc.GetPoint(), nDstW, nSrcW, nSrcH);
            LanczosVAlpha(*this, oTmp, rcDst.GetPoint(), CPoint(), nDstW, nDstH, nSrcH);
        } else { // V  H
            oTmp.SetSize(CSize(nSrcW, nDstH));
            LanczosVAlpha(oTmp, rSrc, CPoint(), rcSrc.GetPoint(), nSrcW, nDstH, nSrcH);
            LanczosHAlpha(*this, oTmp, rcDst.GetPoint(), CPoint(), nDstW, nSrcW, nDstH);
        }
    }
}

void CPlane32::ClearTypeBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, const CClipper& rClip) {
    Themis_DebugAssert(rcSrc.GetWidth() % 3 == 0); // R̔{ɂƂׂ
    CRect rcDst(ptDst, rcSrc.GetWidth() / 3, rcSrc.GetHeight());
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), rcDst, rcSrc)) {
        Themis_DebugAssert(rcSrc.GetWidth() % 3 == 0); // NbsOŒ[oH
        if (rSrc.GetBpp() == 8) {
            Themis_PlaneLock4(*this, rcDst.GetPoint(), pDst, nDstPitch);
            Themis_PlaneLock1(rSrc,  rcSrc.GetPoint(), pSrc, nSrcPitch);
            Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
            for (int y = 0; y < nDstH; y++) {
                for (int x = 0, x3 = 0; x < nDstW; x++, x3 += 3) {
                    pDst[x] = Detail::BGR32(pSrc[x3], pSrc[x3 + 1], pSrc[x3 + 2]);
                }
                pDst += nDstPitch;
                pSrc += nSrcPitch;
            }
        } else if (rSrc.GetBpp() == 32) {
            Themis_PlaneLock4(*this, rcDst.GetPoint(), pDst, nDstPitch);
            Themis_PlaneLock4(rSrc,  rcSrc.GetPoint(), pSrc, nSrcPitch);
            Themis_PlaneGetSize(rcDst.GetSize(), nDstW, nDstH);
         // uint16_t tr[0x100], tg[0x100], tb[0x100];
         // CreateBrightnessTable(tr, tg, tb);
#if 1
            Themis_PlaneGetMMXWidth32(nDstW, nMMXSize, nCppSize);
            if (0 < nMMXSize) {
                Detail::MMXClearTypeBlt32(pDst, nDstPitch, pSrc, nSrcPitch, nMMXSize, nDstH);
            }
            for (int y = 0; y < nDstH; y++) {
                for (int x = 0, x3 = 0; x < nCppSize; x++, x3 += 3) {
                    const int t = nMMXSize * 2 + x3;
                    // ̃sNZ͔Ô߃lgB
                    pDst[nMMXSize * 2 + x] = (pSrc[t] & 0x00ff0000) | (pSrc[t + 1] & 0xff00ff00) | (pSrc[t + 2] & 0x000000ff);
                    // 邳lC邪BB
                }
                pDst += nDstPitch;
                pSrc += nSrcPitch;
            }
#else
            for (int y = 0; y < nDstH; y++) {
                for (int x = 0, x3 = 0; x < nDstW; x++, x3 += 3) {
                    // ̃sNZ͔Ô߃lgB
                    pDst[x] = (pSrc[x3] & 0x00ff0000) | (pSrc[x3 + 1] & 0xff00ff00) | (pSrc[x3 + 2] & 0x000000ff);
                }
                pDst += nDstPitch;
                pSrc += nSrcPitch;
            }
#endif
        } else {
            CMemoryPlane8 oTmp;
            oTmp.SetSize(rcSrc.GetSize());
            oTmp.Blt(rSrc, CPoint(), rcSrc);
            ClearTypeBlt(oTmp, ptDst, oTmp.GetSize().ToRect(), rClip);
        }
    }
}

void CPlane32::MirrorHBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, const CClipper& rClip) {
    Themis_DebugAssert(this != &rSrc);
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        for (int y = 0; y < nSrcH; y++) {
            for (int x = 0; x < nSrcW; x++) {
                pDst[nSrcW - x - 1] = pSrc[x];
            }
            pDst += nDstPitch;
            pSrc += nSrcPitch;
        }
    }
}

void CPlane32::MirrorVBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, const CClipper& rClip) {
    Themis_DebugAssert(this != &rSrc);
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        boost::scoped_array<int> anDstPitch(new int[nSrcH]);
        for (int y = 0; y < nSrcH; y++) {
            anDstPitch[nSrcH - y - 1] = nDstPitch * y;
        }
        for (int y = 0; y < nSrcH; y++) {
            for (int x = 0; x < nSrcW; x++) {
                pDst[x + anDstPitch[y]] = pSrc[x];
            }
            pSrc += nSrcPitch;
        }
    }
}

void CPlane32::Rot180Blt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, const CClipper& rClip) {
    Themis_DebugAssert(this != &rSrc);
    if (rClip.Clipping(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        boost::scoped_array<int> anDstPitch(new int[nSrcH]);
        for (int i = 0, n = nSrcH; i < n; i++) {
            anDstPitch[n - i - 1] = nDstPitch * i;
        }
        for (int y = 0; y < nSrcH; y++) {
            for (int x = 0; x < nSrcW; x++) {
                pDst[(nSrcW - x - 1) + anDstPitch[y]] = pSrc[x];
            }
            pSrc += nSrcPitch;
        }
    }
}

void CPlane32::Rot90ClockwiseBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, const CClipper& rClip) {
    Themis_DebugAssert(this != &rSrc);
    if (rClip.ClippingRot90(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        boost::scoped_array<int> anDstPitch(new int[nSrcW]);
        for (int x = 0; x < nSrcW; x++) {
            anDstPitch[x] = nDstPitch * x;
        }
        for (int y = 0; y < nSrcH; y++) {
            for (int x = 0; x < nSrcW; x++) {
                pDst[(nSrcH - y - 1) + anDstPitch[x]] = pSrc[x];
            }
            pSrc += nSrcPitch;
        }
    }
}

void CPlane32::Rot90AntiClockwiseBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, const CClipper& rClip) {
    Themis_DebugAssert(this != &rSrc);
    if (rClip.ClippingRot90(GetSize(), rSrc.GetSize(), ptDst, rcSrc)) {
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        boost::scoped_array<int> anDstPitch(new int[nSrcW]);
        for (int x = 0; x < nSrcW; x++) {
            anDstPitch[nSrcW - x - 1] = nDstPitch * x;
        }
        for (int y = 0; y < nSrcH; y++) {
            for (int x = 0; x < nSrcW; x++) {
                pDst[y + anDstPitch[x]] = pSrc[x];
            }
            pSrc += nSrcPitch;
        }
    }
}

CSize CPlaneEx::GetRotSize(const CSize& szSrc, double dAngle) {
    // px̕␳ƐE]
    while (360.0 <= dAngle) { dAngle -= 360.0; }
    while (dAngle < 0) { dAngle += 360.0; }
    const double pi = 3.1415926535897932384626433832795;
    fint fiSin = ToFint(sin(2 * pi * dAngle / 360.0));
    fint fiCos = ToFint(cos(2 * pi * dAngle / 360.0));
    // e_̉]W
    CPoint ptRot1 = CPoint(FintToInt(+szSrc.cx * fiCos), FintToInt(szSrc.cx * fiSin));  //E
    CPoint ptRot2 = CPoint(FintToInt(-szSrc.cy * fiSin), FintToInt(szSrc.cy * fiCos));  //
    CPoint ptRot3 = CPoint(FintToInt(szSrc.cx * fiCos - szSrc.cy * fiSin),
                           FintToInt(szSrc.cx * fiSin + szSrc.cy * fiCos));             //E
    // e_̉]W̍őlƍŏl
    CPoint ptMin(Min<long>(Min<long>(Min<long>(ptRot1.x, ptRot2.x), ptRot3.x), 0), Min<long>(Min<long>(Min<long>(ptRot1.y, ptRot2.y), ptRot3.y), 0));
    CPoint ptMax(Max<long>(Max<long>(Max<long>(ptRot1.x, ptRot2.x), ptRot3.x), 0), Max<long>(Max<long>(Max<long>(ptRot1.y, ptRot2.y), ptRot3.y), 0));
    // ]TCYvZ
    return CSize(ptMax - ptMin);
}

void CPlane32::RotBlt(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, double dAngle, COLORREF rgbFill, const CClipper& rClip) {
    // dx = sx*cos - sy*sin
    // dy = sx*sin + sy*cos
    // sx = dy*sin - dx*cos
    // sy = dy*cos + dx*sin
    // px̕␳ƐE]
    while (360.0 <= dAngle) { dAngle -= 360.0; }
    while (dAngle < 0) { dAngle += 360.0; }
    // ̊֐ւ̈Ϗ\`FbN
    if (fabs(dAngle - 0.0) < 0.0001) {
        Blt(rSrc, ptDst, rcSrc, rClip);
    } else if (fabs(dAngle - 90.0) < 0.0001) {
        Rot90ClockwiseBlt(rSrc, ptDst, rcSrc, rClip);
    } else if (fabs(dAngle - 180.0) < 0.0001) {
        Rot180Blt(rSrc, ptDst, rcSrc, rClip);
    } else if (fabs(dAngle - 270.0) < 0.0001) {
        Rot90AntiClockwiseBlt(rSrc, ptDst, rcSrc, rClip);
    } else {
        const double pi = 3.1415926535897932384626433832795;
        const fint fiSin = ToFint(sin(2 * pi * dAngle / 360.0));
        const fint fiCos = ToFint(cos(2 * pi * dAngle / 360.0));
        // e_̉]W
        const CSize szSrc = rcSrc.GetSize();
        const CPoint ptRot1 = CPoint(FintToInt(+szSrc.cx * fiCos), FintToInt(szSrc.cx * fiSin));  //E
        const CPoint ptRot2 = CPoint(FintToInt(-szSrc.cy * fiSin), FintToInt(szSrc.cy * fiCos));  //
        const CPoint ptRot3 = CPoint(FintToInt(szSrc.cx * fiCos - szSrc.cy * fiSin),
                                     FintToInt(szSrc.cx * fiSin + szSrc.cy * fiCos));             //E
        // e_̉]W̍őlƍŏl
        const CPoint ptMin(Min<long>(Min<long>(Min<long>(ptRot1.x, ptRot2.x), ptRot3.x), 0), Min<long>(Min<long>(Min<long>(ptRot1.y, ptRot2.y), ptRot3.y), 0));
        const CPoint ptMax(Max<long>(Max<long>(Max<long>(ptRot1.x, ptRot2.x), ptRot3.x), 0), Max<long>(Max<long>(Max<long>(ptRot1.y, ptRot2.y), ptRot3.y), 0));
        // ]TCYvZ
        const CSize szDst(ptMax - ptMin);
        // hׂF̕ϊ
        const uint32_t dwFill = Detail::RGBtoBGR32(rgbFill);
        // ]̓]ɑΉWxNgZo
        fint fSrcX = (ptMin.y * fiSin + ptMin.x * fiCos), fSrcY = (ptMin.y * fiCos - ptMin.x * fiSin);
        const fint fSrcXWX = fiCos, fSrcXWY = (0 - fiSin);
        const fint fSrcYHX = fiSin, fSrcYHY = fiCos;
        // ]
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(CSize(ptMax - ptMin), nDstW, nDstH);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        for (int y = 0; y < nDstH; y++) {
            fint fSrcX2 = fSrcX, fSrcY2 = fSrcY;
            for (int x = 0; x < nDstW; x++) {
                const int sx = FintToInt(fSrcX2);
                const int sy = FintToInt(fSrcY2);
                const bool bx1 = 0 <= sx   && sx   < nSrcW;
                const bool bx2 = 0 <= sx+1 && sx+1 < nSrcW;
                const bool by1 = 0 <= sy   && sy   < nSrcH;
                const bool by2 = 0 <= sy+1 && sy+1 < nSrcH;
                const int nPitchY = sy * nSrcPitch;
                const uint32_t s11 = (bx1 && by1)? pSrc[sx   + nPitchY]: dwFill;
                const uint32_t s21 = (bx2 && by1)? pSrc[sx+1 + nPitchY]: dwFill;
                const uint32_t s12 = (bx1 && by2)? pSrc[sx   + nPitchY + nSrcPitch]: dwFill;
                const uint32_t s22 = (bx2 && by2)? pSrc[sx+1 + nPitchY + nSrcPitch]: dwFill;
                pDst[x] = Detail::GetMidPoint(s11, s21, s12, s22, FintDecimal(fSrcX2), FintDecimal(fSrcY2));
                fSrcX2 += fSrcXWX;
                fSrcY2 += fSrcXWY;
            }
            pDst += nDstPitch;
            fSrcX += fSrcYHX;
            fSrcY += fSrcYHY;
        }
    }
}

void CPlane32::RotBltDraft(const CPlane& rSrc, CPoint ptDst, CRect rcSrc, double dAngle, COLORREF rgbFill, const CClipper& rClip) {
    // px̕␳ƐE]
    while (360.0 <= dAngle) { dAngle -= 360.0; }
    while (dAngle < 0) { dAngle += 360.0; }
    // ̊֐ւ̈Ϗ\`FbN
    if (fabs(dAngle - 0.0) < 0.0001) {
        Blt(rSrc, ptDst, rcSrc, rClip);
    } else if (fabs(dAngle - 90.0) < 0.0001) {
        Rot90ClockwiseBlt(rSrc, ptDst, rcSrc, rClip);
    } else if (fabs(dAngle - 180.0) < 0.0001) {
        Rot180Blt(rSrc, ptDst, rcSrc, rClip);
    } else if (fabs(dAngle - 270.0) < 0.0001) {
        Rot90AntiClockwiseBlt(rSrc, ptDst, rcSrc, rClip);
    } else {
        const double pi = 3.1415926535897932384626433832795;
        const fint fiSin = ToFint(sin(2 * pi * dAngle / 360.0));
        const fint fiCos = ToFint(cos(2 * pi * dAngle / 360.0));
        // e_̉]W
        const CSize szSrc = rcSrc.GetSize();
        const CPoint ptRot1 = CPoint(FintToInt(+szSrc.cx * fiCos), FintToInt(szSrc.cx * fiSin));  //E
        const CPoint ptRot2 = CPoint(FintToInt(-szSrc.cy * fiSin), FintToInt(szSrc.cy * fiCos));  //
        const CPoint ptRot3 = CPoint(FintToInt(szSrc.cx * fiCos - szSrc.cy * fiSin),
                                     FintToInt(szSrc.cx * fiSin + szSrc.cy * fiCos));             //E
        // e_̉]W̍őlƍŏl
        const CPoint ptMin(Min<long>(Min<long>(Min<long>(ptRot1.x, ptRot2.x), ptRot3.x), 0), Min<long>(Min<long>(Min<long>(ptRot1.y, ptRot2.y), ptRot3.y), 0));
        const CPoint ptMax(Max<long>(Max<long>(Max<long>(ptRot1.x, ptRot2.x), ptRot3.x), 0), Max<long>(Max<long>(Max<long>(ptRot1.y, ptRot2.y), ptRot3.y), 0));
        // ]TCYvZ
        const CSize szDst(ptMax - ptMin);

        // hׂF̕ϊ
        const uint32_t dwFill = Detail::RGBtoBGR32(rgbFill);
        // ]̓]ɑΉWxNgZo
        fint fSrcX = (ptMin.y * fiSin + ptMin.x * fiCos), fSrcY = (ptMin.y * fiCos - ptMin.x * fiSin);
        const fint fSrcXWX = fiCos, fSrcXWY = (0 - fiSin);
        const fint fSrcYHX = fiSin, fSrcYHY = fiCos;
        // ]
        Themis_PlaneLock4(*this, ptDst, pDst, nDstPitch);
        Themis_PlaneLock4(rSrc, rcSrc.GetPoint(), pSrc, nSrcPitch);
        Themis_PlaneGetSize(CSize(ptMax - ptMin), nDstW, nDstH);
        Themis_PlaneGetSize(rcSrc.GetSize(), nSrcW, nSrcH);
        for (int y = 0; y < nDstH; y++) {
            fint fSrcX2 = fSrcX, fSrcY2 = fSrcY;
            for (int x = 0; x < nDstW; x++) {
                const int sx = FintToInt(fSrcX2 + c_nFint_1 / 2);
                const int sy = FintToInt(fSrcY2 + c_nFint_1 / 2);
                pDst[x] = (0 <= sx && sx < nSrcW && 0 <= sy && sy < nSrcH)?
                    pSrc[sx + sy * nSrcPitch]: dwFill;
                fSrcX2 += fSrcXWX;
                fSrcY2 += fSrcXWY;
            }
            pDst += nDstPitch;
            fSrcX += fSrcYHX;
            fSrcY += fSrcYHY;
        }
    }
}

}
