#include "stdafx.h"
#include "../File.h"
#include "../Graphics.h"
#include "Argb.h"
namespace Themis {

bool CArgbLoader::Load(const std::tr1::shared_ptr<CFileData>& fileData, bool bRenderOnlyAtOnce) {
    const vector<uint8_t>& fileMemory = fileData->GetMemory();
    memcpy(&m_oHeader, &fileMemory[0], sizeof(m_oHeader));
    if (m_oHeader.dwIdentifier != 0x41524742) {
        return false;
    }

    int nBpp;
    if      (m_oHeader.dwPixelFormat == 0x08080808) nBpp = 32;
    else if (m_oHeader.dwPixelFormat == 0x10101010) nBpp = 64;
    else return false; //T|[gȃtH[}bg

    size_t nFileSize = fileMemory.size();
    size_t nPixel = m_oHeader.dwWidth * m_oHeader.dwHeight * nBpp/32;
    nPixel = Max<size_t>(nPixel, (nFileSize - sizeof(m_oHeader)) / 32);
    if (nPixel <= 0) {
        return false;
    }
    m_adwPixels.reset(new uint32_t[nPixel]);
    size_t nFileReadSize = Min<size_t>(nPixel * sizeof(uint32_t), nFileSize - sizeof(m_oHeader));
    // K؂t@Cȏꍇ nFileReadSize < nPixel ɂȂ肤
    memcpy(m_adwPixels.get(), &fileMemory[0] + sizeof(m_oHeader), nFileReadSize);
    return true;
}

CSize CArgbLoader::GetSize() const {
    return CSize(m_oHeader.dwWidth, m_oHeader.dwHeight);
}

void CArgbLoader::Render(CPlane& dst, const CPoint& pt, bool bRenderOnlyAtOnce) {
    try {
        if (dst.GetBpp() != 32) {
            CMemoryPlane oTmpPlane(32);
            CSize img = GetSize();
            CSize pln = dst.GetSize() - pt;
            oTmpPlane.SetSize(CSize(Min(img.cx, pln.cx), Min(img.cy, pln.cy)));  //ɍ킹
            Render(oTmpPlane, CPoint(), bRenderOnlyAtOnce);
            dst.Blt(oTmpPlane, pt, CRect(oTmpPlane.GetSize()));
        } else {
            Themis_PlaneLock4(dst, pt, p, nPitch);
            uint32_t* pdwPixels = m_adwPixels.get();
            if (m_oHeader.dwPixelFormat == 0x08080808) {    //32bpp
                for (int y=0, cy=m_oHeader.dwHeight; y<cy; y++) {
                    memcpy(p, pdwPixels, m_oHeader.dwWidth * 4);
                    p += nPitch;
                    pdwPixels += m_oHeader.dwWidth;
                }
            } else {  //64bpp
                for (int y=0, cy=m_oHeader.dwHeight; y<cy; y++) {
                    for (int x=0, cx=m_oHeader.dwWidth; x<cx; x++) {
                        p[x] = ((pdwPixels[x*2+0] >> 8) & 0xff) | ((pdwPixels[x*2+0] >> 24) & 0xff00) |
                               ((pdwPixels[x*2+1] << 8) & 0xff0000) | ((pdwPixels[x*2+1]) & 0xff000000);
                    }
                    p += nPitch;
                    pdwPixels += m_oHeader.dwWidth * 2;
                }
            }
        }
    } catch (std::exception&) {
        throw;
    }
}

void CArgbSaver::Save(const CFileName& oFileName, const CPlane& src, const CRect& rc) {
    try {
        Themis_DebugAssert(src.GetBpp() == 32);
        //32BPPȊOłArgbȈӖ̂Łc

        Detail::SArgbHeader oHeader;
        oHeader.dwIdentifier = 0x41524742;
        oHeader.dwPixelFormat = 0x08080808; //32BPPŒ
        oHeader.dwWidth = rc.GetWidth();
        oHeader.dwHeight = rc.GetHeight();
        CFile(oFileName).CreateDir();
        CApiFile oFile(oFileName, CApiFile::eModeCreate, CApiFile::eAccessSequential);
        oFile.Reserve(sizeof(Detail::SArgbHeader) + rc.GetSize().GetSpace() * 4);
        oFile.Write(&oHeader, sizeof(oHeader));
        Themis_PlaneLock4(src, rc.GetPoint(), p, nPitch);
        for (int y = 0, cy = rc.GetHeight(), cx = rc.GetWidth(); y < cy; y++) {
            oFile.Write(p, cx * sizeof(uint32_t));
            p += nPitch;
        }
        Themis_DebugAssert(oFile.Tell() == sizeof(Detail::SArgbHeader) + rc.GetSize().GetSpace() * 4);
    } catch (std::exception&) {
        throw ImageSaveException(oFileName);
    }
}

}
