#include "stdafx.h"
#include "../File.h"
#include "../Graphics.h"
#include "Bmp.h"
namespace Themis {

namespace {
    size_t GetMemorySize(const BITMAPINFOHEADER& rbih) {
        size_t nPaletteSize = 256 * sizeof(RGBQUAD);
        size_t nBitsSize = ((rbih.biWidth + 3) & ~3) * rbih.biHeight * 32/8;
        return sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + nPaletteSize + nBitsSize;
    }
}
bool CBmpLoader::Load(const std::tr1::shared_ptr<CFileData>& fileData, bool bRenderOnlyAtOnce) {
    const vector<uint8_t>& fileMemory = fileData->GetMemory();
    if (fileMemory.size() <= sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER)) {
        return false; //炩ɃTCYȂ
    }
    // t@Cwb_
    const BITMAPFILEHEADER* bfh = reinterpret_cast<const BITMAPFILEHEADER*>(&fileMemory[0]);
    if (bfh->bfType != 0x4d42) {    //L"BM"
        return false;
    }
    // CtHwb_
    m_bi = reinterpret_cast<const BITMAPINFO*>(&fileMemory[sizeof(BITMAPFILEHEADER)]);
    size_t nSize = Max<size_t>(GetMemorySize(m_bi->bmiHeader), fileMemory.size());
    if (nSize <= 0) {
        return false;
    }
    // rbgf[^
    size_t nOffBits = bfh->bfOffBits;
    if (nOffBits == 0) {
        //bfOffBits0ɂȂĂbmpt@CȂ葽c
        //0ȊOȂAƂ肠MłlƂƂɂĂ
        size_t nPalEntrys = (m_bi->bmiHeader.biBitCount < 8) ? (1 << m_bi->bmiHeader.biBitCount) : 0;
        nOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + nPalEntrys * sizeof(RGBQUAD);
    }
    int nPitch = (m_bi->bmiHeader.biWidth * m_bi->bmiHeader.biBitCount / 8 + 3) & ~3;
    int nHeight = abs(m_bi->bmiHeader.biHeight);
    size_t nBitSize = nPitch * nHeight;
    if (fileMemory.size() < nOffBits + nBitSize) { // K؂t@C΍
        m_abyBits.reset(new uint8_t[nBitSize]);
        if (nOffBits < fileMemory.size()) {
            size_t nCopySize = fileMemory.size() - nOffBits;
            memcpy(m_abyBits.get(), &fileMemory[nOffBits], nCopySize);
            // Ƃ肠[߂ƂBB
            memset(m_abyBits.get() + nCopySize, 0x00, nBitSize - nCopySize);
        }
        m_pBits = m_abyBits.get();
    } else { // ʏ펞
        m_pBits = &fileMemory[nOffBits];
    }
    return true;
}

CSize CBmpLoader::GetSize() const {
    return CSize(m_bi->bmiHeader.biWidth, abs(m_bi->bmiHeader.biHeight));
}

void CBmpLoader::Render(CPlane& dst, const CPoint& pt, bool bRenderOnlyAtOnce) {
    try {
        dst.BltFromDIBits(*m_bi, m_pBits, pt, CRect(CSize::GetLarge()));
    } catch (std::exception&) {
        throw;
    }
}

bool CBmpLoader::HasAlpha() const {
    // BPP == 32Ȃ烿`ltƂĂ܂BƂC͂邪BB
    return m_bi->bmiHeader.biBitCount == 32;
}

void CBmpSaver::Save(const CFileName& oFileName, const CPlane& src, const CRect& rc) {
    try {
        if (src.GetBpp() == 8) {
            int nDstPitch = (rc.GetWidth() + 3) & ~3;
            BITMAPFILEHEADER bfh = {0};
            bfh.bfType = 0x4d42; //L'BM'
            bfh.bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * 0x100;
            bfh.bfSize = bfh.bfOffBits + nDstPitch * rc.GetHeight();
            CFile(oFileName).CreateDir();
            CApiFile oFile(oFileName, CApiFile::eModeCreate, CApiFile::eAccessSequential);
            oFile.Reserve(bfh.bfSize);
            oFile.Write(&bfh, sizeof(bfh));

            BITMAPINFOHEADER bi = {0};
            bi.biSize = sizeof(BITMAPINFOHEADER);
            bi.biWidth = rc.GetWidth();
            bi.biHeight = rc.GetHeight();
            bi.biPlanes = 1;
            bi.biBitCount = 8;
            bi.biCompression = BI_RGB;
            bi.biXPelsPerMeter = 3780;
            bi.biYPelsPerMeter = 3780;
            bi.biClrUsed = 0x100;
            oFile.Write(&bi, sizeof(bi));

            RGBQUAD rq[0x100] = {0};
            for (int i = 0; i < 0x100; i++) {
                rq[i].rgbRed = rq[i].rgbGreen = rq[i].rgbBlue = static_cast<uint8_t>(i);
            }
            oFile.Write(&rq, sizeof(rq));

            Themis_PlaneLock1(src, CPoint(rc.left, rc.bottom - 1), p, nPitch);
            const uint8_t abyZero[4] = {0};
            const int nZero = nDstPitch - rc.GetWidth();
            if (0 < nZero) {
                for (int y=0, cy=rc.GetHeight(); y<cy; y++) {
                    oFile.Write(p, rc.GetWidth());
                    oFile.Write(abyZero, nZero);
                    p -= nPitch;
                }
            } else {
                for (int y=0, cy=rc.GetHeight(); y<cy; y++) {
                    oFile.Write(p, rc.GetWidth());
                    p -= nPitch;
                }
            }
            Themis_DebugAssert(oFile.Tell() == bfh.bfSize);
        } else if (src.GetBpp() == 24) {
            int nDstPitch = (rc.GetWidth() * 3 + 3) & ~3;
            BITMAPFILEHEADER bfh = {0};
            bfh.bfType = 0x4d42; //L'BM'
            bfh.bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);
            bfh.bfSize = bfh.bfOffBits + nDstPitch * rc.GetHeight();
            CFile(oFileName).CreateDir();
            CApiFile oFile(oFileName, CApiFile::eModeCreate, CApiFile::eAccessSequential);
            oFile.Reserve(bfh.bfSize);
            oFile.Write(&bfh, sizeof(bfh));

            BITMAPINFOHEADER bi = {0};
            bi.biSize = sizeof(BITMAPINFOHEADER);
            bi.biWidth = rc.GetWidth();
            bi.biHeight = rc.GetHeight();
            bi.biPlanes = 1;
            bi.biBitCount = 24;
            bi.biCompression = BI_RGB;
            bi.biXPelsPerMeter = 3780;
            bi.biYPelsPerMeter = 3780;
            oFile.Write(&bi, sizeof(bi));

            Themis_PlaneLock1(src, CPoint(rc.left, rc.bottom - 1), p, nPitch);
            const uint8_t abyZero[4] = {0};
            const int nZero = nDstPitch - rc.GetWidth() * 3;
            if (0 < nZero) {
                for (int y=0, cy=rc.GetHeight(); y<cy; y++) {
                    oFile.Write(p, rc.GetWidth() * 3);
                    oFile.Write(abyZero, nZero);
                    p -= nPitch;
                }
            } else {
                for (int y=0, cy=rc.GetHeight(); y<cy; y++) {
                    oFile.Write(p, rc.GetWidth() * 3);
                    p -= nPitch;
                }
            }
            Themis_DebugAssert(oFile.Tell() == bfh.bfSize);
        } else {
            CMemoryPlane oTmpPlane(24);
            oTmpPlane.SetSize(rc.GetSize());
            oTmpPlane.Blt(src, CPoint(), rc);
            Save(oFileName, oTmpPlane, rc.GetSize().ToRect());
        }
    } catch (std::exception&) {
        throw;
    }
}

}
