/** @file
 * WE`̈ʒuE傫
 *
 * @author aki.
 */
#pragma once
namespace Themis {

/**
 * W\NX
 */
class CPoint : public POINT,
    private boost::totally_ordered<CPoint>,
    private boost::integer_arithmetic2<CPoint, long> {
public:
    inline CPoint() { Set(); }
    inline CPoint(long px, long py) { Set(px, py); }
    inline CPoint(const POINT& p) { Set(p); }
    inline CPoint(const POINTS& p) { Set(p); }
    inline CPoint(const SIZE& p) { Set(p); }

    inline void Set() {
        this->x = 0;
        this->y = 0;
    }
    inline void Set(long px, long py) {
        this->x = px;
        this->y = py;
    }
    inline void Set(const POINT& p) {
        this->x = p.x;
        this->y = p.y;
    }
    inline void Set(const POINTS& p) {
        this->x = p.x;
        this->y = p.y;
    }
    inline void Set(const SIZE& p) {
        this->x = p.cx;
        this->y = p.cy;
    }

    inline bool operator==(const CPoint& r) const {
        return this->y == r.y && this->x == r.x;
    }
    inline bool operator<(const CPoint& r) const {
        unsigned ey = static_cast<unsigned>(r.y == this->y);    // == ̎P
        unsigned ly = static_cast<unsigned>(this->y - r.y);     // <  ̎P
        unsigned lx = static_cast<unsigned>(this->x - r.x);     // <  ̎P
        ey <<= sizeof(unsigned)*8 - 1;
        ly &= 1 << (sizeof(unsigned)*8 - 1);
        return (ly | (ey & lx)) != 0;
    }

    inline CPoint& operator+=(const CPoint& pt) {
        this->x += pt.x;
        this->y += pt.y;
        return *this;
    }
    inline CPoint& operator-=(const CPoint& pt) {
        this->x -= pt.x;
        this->y -= pt.y;
        return *this;
    }
    inline CPoint& operator+=(const SIZE& si) { return (*this += CPoint(si)); }
    inline CPoint& operator-=(const SIZE& si) { return (*this -= CPoint(si)); }

    inline CPoint& operator+=(long n) {
        this->x += n;
        this->y += n;
        return *this;
    }
    inline CPoint& operator-=(long n) {
        this->x -= n;
        this->y -= n;
        return *this;
    }
    inline CPoint& operator*=(long n) {
        this->x *= n;
        this->y *= n;
        return *this;
    }
    inline CPoint& operator/=(long n) {
        this->x /= n;
        this->y /= n;
        return *this;
    }
    inline CPoint& operator%=(long n) {
        this->x %= n;
        this->y %= n;
        return *this;
    }

    inline const CPoint& operator+() const { return *this; }
    inline const CPoint& operator-() const { SIZE size = {this->x, this->y}; return (CPoint() -= size); }

    static inline CPoint GetLarge() { return CPoint(LONG_MAX / 2, LONG_MAX / 2); }
    static inline CPoint GetSmall() { return CPoint(LONG_MIN / 2, LONG_MIN / 2); }
};

class CRect;

/**
 * `̑傫\NX
 */
class CSize : public SIZE,
    private boost::totally_ordered<CSize>,
    private boost::integer_arithmetic2<CSize, long> {
public:
    inline CSize() { Set(); }
    inline CSize(long px, long py) { Set(px, py); }
    inline CSize(const SIZE& p) { Set(p); }
    inline CSize(const POINT& p) { Set(p); }

    inline void Set() {
        this->cx = 0;
        this->cy = 0;
    }
    inline void Set(long pcx, long pcy) {
        this->cx = pcx;
        this->cy = pcy;
    }
    inline void Set(const SIZE& p) {
        this->cx = p.cx;
        this->cy = p.cy;
    }
    inline void Set(const POINT& p) {
        this->cx = p.x;
        this->cy = p.y;
    }

    /// Lȋ`Ȃ̂
    inline bool IsValid() const { return (0 < this->cx && 0 < this->cy); }
    /// ʐςԂ
    inline long GetSpace() const { return (this->cx * this->cy); }
    /// 90x](Aւ)̂Ԃ
    inline CSize GetRot90() const { return CSize(this->cy, this->cx); }

    /// CRectւ̕ϊ
    inline CRect ToRect() const;

    inline bool operator==(const CSize& r) const {
        return this->cy == r.cy && this->cx == r.cx;
    }
    inline bool operator<(const CSize& r) const {
        unsigned ey = static_cast<unsigned>(r.cy == this->cy);    // == ̎P
        unsigned ly = static_cast<unsigned>(this->cy - r.cy);     // <  ̎P
        unsigned lx = static_cast<unsigned>(this->cx - r.cx);     // <  ̎P
        ey <<= sizeof(unsigned)*8 - 1;
        ly &= 1 << (sizeof(unsigned)*8 - 1);
        return (ly | (ey & lx)) != 0;
    }

    inline CSize& operator+=(const CSize& sz) {
        this->cx += sz.cx;
        this->cy += sz.cy;
        return *this;
    }
    inline CSize& operator-=(const CSize& sz) {
        this->cx -= sz.cx;
        this->cy -= sz.cy;
        return *this;
    }
    inline CSize& operator+=(const POINT& pt) { return (*this += CSize(pt)); }
    inline CSize& operator-=(const POINT& pt) { return (*this -= CSize(pt)); }

    inline CSize& operator+=(long n) {
        this->cx += n;
        this->cy += n;
        return *this;
    }
    inline CSize& operator-=(long n) {
        this->cx -= n;
        this->cy -= n;
        return *this;
    }
    inline CSize& operator*=(long n) {
        this->cx *= n;
        this->cy *= n;
        return *this;
    }
    inline CSize& operator/=(long n) {
        this->cx /= n;
        this->cy /= n;
        return *this;
    }
    inline CSize& operator%=(long n) {
        this->cx %= n;
        this->cy %= n;
        return *this;
    }

    inline const CSize& operator+() const { return *this; }
    inline const CSize& operator-() const { return (CSize(0, 0) -= *this); }

    CSize GetUnion(const CSize& sz) const;
    CSize GetIntersect(const CSize& sz) const;
    inline void SetUnion(const CSize& sz) { *this = GetUnion(sz); }
    inline void SetIntersect(const CSize& sz) { *this = GetIntersect(sz); }

    /// sz̋`(0, 0, *this)̒ɒuƂ̍W擾
    inline CPoint GetCenterOf(const CSize& sz) const {
        return CPoint((this->cx - sz.cx) / 2, (this->cy - sz.cy) / 2);
    }

    static inline CSize GetLarge() { return CSize(LONG_MAX / 2, LONG_MAX / 2); }
    static inline CSize GetSmall() { return CSize(LONG_MIN / 2, LONG_MIN / 2); }
};

inline const CPoint operator+(const CPoint& l, const CPoint& r)  { return (CPoint(l) += r); }
inline const CPoint operator-(const CPoint& l, const CPoint& r)  { return (CPoint(l) -= r); }
inline const CPoint operator+(const CPoint& l, const CSize& r)  { return (CPoint(l) += r); }
inline const CPoint operator-(const CPoint& l, const CSize& r)  { return (CPoint(l) -= r); }
inline const CSize operator+(const CSize& l, const CPoint& r)  { return (CSize(l) += r); }
inline const CSize operator-(const CSize& l, const CPoint& r)  { return (CSize(l) -= r); }
inline const CSize operator+(const CSize& l, const CSize& r)  { return (CSize(l) += r); }
inline const CSize operator-(const CSize& l, const CSize& r)  { return (CSize(l) -= r); }

/**
 * `\NXB
 *
 * RXgN^Ȃǂň̂x, y, w, hłāA
 * x1, y1, x2, y2ł͂Ȃ̂ŒӂׂB
 *
 * ftHgRXgN^ł͖ȋ`ݒ肷B
 */
class CRect : public RECT,
    private boost::equality_comparable<CRect>,
    private boost::additive<CRect, CPoint>,
    private boost::additive<CRect, CSize> {
public:
    inline CRect() { Set(); }
    inline CRect(const RECT& rc) { Set(rc); }
    inline CRect(const CPoint& pt, const CSize& sz) { Set(pt, sz); }
    inline CRect(long x, long y, const CSize& sz) { Set(x, y, sz); }
    inline CRect(const CPoint& pt, long cx, long cy) { Set(pt, cx, cy); }
    inline CRect(long x, long y, long cx, long cy) { Set(x, y, cx, cy); }
    /// 0, 0, sz̈ӖƂBȃjAXȂ̂ŁAÖق̕ϊ͋֎~B
    inline explicit CRect(const CSize& sz) { Set(sz); }

    inline void Set() { Set(0, 0, 0, 0); }
    inline void Set(const CPoint& pt, const CSize& sz) { Set(pt.x, pt.y, sz.cx, sz.cy); }
    inline void Set(long x, long y, const CSize& sz) { Set(x, y, sz.cx, sz.cy); }
    inline void Set(const CPoint& pt, long cx, long cy) { Set(pt.x, pt.y, cx, cy); }
    inline void Set(const CSize& sz) { Set(0, 0, sz.cx, sz.cy); }
    inline void Set(long x, long y, long cx, long cy) {
        this->left   = x;
        this->top    = y;
        this->right  = x + cx;
        this->bottom = y + cy;
    }
    inline void Set(const RECT& rc) {
        this->left   = rc.left;
        this->top    = rc.top;
        this->right  = rc.right;
        this->bottom = rc.bottom;
    }

    inline long GetX() const { return this->left; }
    inline long GetY() const { return this->top; }
    inline long GetWidth() const { return (this->right - this->left); }
    inline long GetHeight() const { return (this->bottom - this->top); }
    inline long GetSpace() const { return (GetWidth() * GetHeight()); }

    inline CPoint GetPoint() const { return CPoint(this->left, this->top); }
    inline CPoint GetPoint2() const { return CPoint(this->right, this->bottom); }
    inline CSize GetSize() const { return CSize(GetWidth(), GetHeight()); }

    inline void SetWidth(long n) { this->right = this->left + n; }
    inline void SetHeight(long n) { this->bottom = this->top + n; }
    inline void SetX(long n) { this->right = n + GetWidth(); this->left = n; }
    inline void SetY(long n) { this->bottom = n + GetHeight(); this->top = n; }
    inline void SetPoint(const CPoint& n) { SetX(n.x); SetY(n.y); }
    inline void SetSize(const CSize& n) { SetWidth(n.cx); SetHeight(n.cy); }

    inline bool operator==(const CRect& other) const {
        return this->left == other.left &&
            this->top == other.top &&
            this->right == other.right &&
            this->bottom == other.bottom;
    }

    /// x, yŎW`̒Ȃtrue(ӂ܂)
    inline bool IsIn(const CPoint& pt) const {
        return 
            this->left <= pt.x &&   //pt.xleftȏォ
            pt.x < this->right &&   //pt.xright
            this->top  <= pt.y &&   //pt.ytopȏォ
            pt.y < this->bottom;    //pt.ybottom
    }
    /// rc `̒ɊSɊ܂܂Ȃtrue
    inline bool IsIn(const CRect& rc) const {
        return
            this->left <= rc.left &&    //rc.leftleftȏォ
            rc.right <= this->right &&  //rc.rightrightȉ
            this->top <= rc.top &&      //rc.toptopȏォ
            rc.bottom <= this->bottom;  //rc.bottombottomȉ
    }

    inline CRect& operator=(const CPoint& pt) { SetX(pt.x); SetY(pt.y); return *this; }
    inline CRect& operator=(const CSize& sz) { SetWidth(sz.cx); SetHeight(sz.cy); return *this; }
    inline CRect& operator+=(const CPoint& pt) { SetX(GetX() + pt.x); SetY(GetY() + pt.y); return *this; }
    inline CRect& operator+=(const CSize& sz) { SetWidth(GetWidth() + sz.cx); SetHeight(GetHeight() + sz.cy); return *this; }
    inline CRect& operator-=(const CPoint& pt) { SetX(GetX() - pt.x); SetY(GetY() - pt.y); return *this; }
    inline CRect& operator-=(const CSize& sz) { SetWidth(GetWidth() - sz.cx); SetHeight(GetHeight() - sz.cy); return *this; }

    //@{ ̋`ݍލŏ̋`
    void SetUnion(const RECT& s);
    inline CRect GetUnion(const RECT& s) const { CRect rc=*this; rc.SetUnion(s); return rc; }
    //@}

    //@{ Q̋`̌Ă`
    void SetIntersect(const RECT& s);
    inline CRect GetIntersect(const RECT& s) const { CRect rc=*this; rc.SetIntersect(s); return rc; }
    //@}

    /// Lȋ`Ȃ̂ǂ(傫OƂ)
    inline bool IsValid() const {
        return ((this->left < this->right) && (this->top < this->bottom));
    }

    /// sz̋`Rect̒ɒuƂ̍W擾
    inline CPoint GetCenterOf(const CSize& sz) const {
        return CPoint((this->GetWidth() - sz.cx) / 2 + this->left,
                      (this->GetHeight() - sz.cy) / 2 + this->top);
    }

    /// this̒rcu̗]Zo
    vector<CRect> GetBlanks(const CRect& rcRect) const;

    /// CRectŕ\oő̋`Ԃ
    static inline CRect GetMax() {
        CRect rc;
        rc.left = rc.top = LONG_MIN / 2;
        rc.right = rc.bottom = LONG_MAX / 2;
        return rc;
    }
};

/** CRectւ̕ϊB

    CRect̃RXgN^ƗɏĂ܂ǂ̂ŁB
    ƂexplicitÔςȃoO肻cB
*/
inline CRect CSize::ToRect() const {
    return CRect(*this);
}

}
