#pragma once
#include "DialogResult.h"
#include "../String/Encoding.h"
namespace Themis {

namespace MessageBoxButtons {
    enum T {
        OK                  = MB_OK,
        OKCancel            = MB_OKCANCEL,
        AbortRetryIgnore    = MB_ABORTRETRYIGNORE,
        YesNoCancel         = MB_YESNOCANCEL,
        YesNo               = MB_YESNO,
        RetryCancel         = MB_RETRYCANCEL,
    };
}

namespace MessageBoxIcon {
    enum T {
        None            = 0,
        Error           = MB_ICONERROR,
        Hand            = MB_ICONHAND,
        Stop            = MB_ICONSTOP,
        Question        = MB_ICONQUESTION,
        Exclamation     = MB_ICONEXCLAMATION,
        Warning         = MB_ICONWARNING,
        Information     = MB_ICONINFORMATION,
        Asterisk        = MB_ICONASTERISK,
    };
}

namespace MessageBoxDefaultButton {
    enum T {
        Button1 = MB_DEFBUTTON1,
        Button2 = MB_DEFBUTTON2,
        Button3 = MB_DEFBUTTON3,
    };
}

namespace MessageBoxOptions {
    //[Flags]
    typedef UINT T;
    static const UINT DefaultDesktopOnly    = MB_DEFAULT_DESKTOP_ONLY;
    static const UINT RightAlign            = MB_RIGHT;
    static const UINT RtlReading            = MB_RTLREADING;
    static const UINT ServiceNotification   = MB_SERVICE_NOTIFICATION;
}

//MB_APPLMODAL
//MB_SYSTEMMODAL
//MB_TASKMODAL
//MB_SETFOREGROUND
//MB_TOPMOST


/**
 * .NETSystem.Windows.Forms.MessageBoxڐAĂ݂B
 */
class MessageBox {
    typedef const string& string;
public:
    static DialogResult::T Show(string text) {
        return Show(NULL, text);
    }
    static DialogResult::T Show(string text, string caption) {
        return Show(NULL, text, caption);
    }
    static DialogResult::T Show(string text, string caption, MessageBoxButtons::T buttons) {
        return Show(NULL, text, caption, buttons);
    }
    static DialogResult::T Show(string text, string caption, MessageBoxButtons::T buttons, MessageBoxIcon::T icon) {
        return Show(NULL, text, caption, buttons, icon);
    }
    static DialogResult::T Show(string text, string caption, MessageBoxButtons::T buttons, MessageBoxIcon::T icon, MessageBoxDefaultButton::T defaultButton) {
        return Show(NULL, text, caption, buttons, icon, defaultButton);
    }
    static DialogResult::T Show(string text, string caption, MessageBoxButtons::T buttons, MessageBoxIcon::T icon, MessageBoxDefaultButton::T defaultButton, MessageBoxOptions::T options) {
        return Show(NULL, text, caption, buttons, icon, defaultButton, options);
    }
    static DialogResult::T Show(HWND owner, string text) {
        return Show(owner, text, string::Empty);
    }
    static DialogResult::T Show(HWND owner, string text, string caption) {
        return Show(owner, text, caption, MessageBoxButtons::OK);
    }
    static DialogResult::T Show(HWND owner, string text, string caption, MessageBoxButtons::T buttons) {
        return Show(owner, text, caption, buttons, MessageBoxIcon::None);
    }
    static DialogResult::T Show(HWND owner, string text, string caption, MessageBoxButtons::T buttons, MessageBoxIcon::T icon) {
        return Show(owner, text, caption, buttons, icon, MessageBoxDefaultButton::Button1);
    }
    static DialogResult::T Show(HWND owner, string text, string caption, MessageBoxButtons::T buttons, MessageBoxIcon::T icon, MessageBoxDefaultButton::T defaultButton) {
        return Show(owner, text, caption, buttons, icon, defaultButton, 0);
    }
    static DialogResult::T Show(HWND owner, string text, string caption, MessageBoxButtons::T buttons, MessageBoxIcon::T icon, MessageBoxDefaultButton::T defaultButton, MessageBoxOptions::T options) {
        UINT flags = 0;
        flags |= buttons;
        flags |= icon;
        flags |= defaultButton;
        flags |= options;
        std::string captionStr = Encoding::GetDefault().GetBytes(caption);
        return (DialogResult::T)::MessageBox(owner, Encoding::GetDefault().GetBytes(text).c_str(),
            captionStr.empty() ? NULL : captionStr.c_str(), flags);
    }
 // static DialogResult::T Show(HWND owner, string text, string caption, MessageBoxButtons::T buttons, MessageBoxIcon icon, MessageBoxDefaultButton defaultButton, MessageBoxOptions options, string helpFilePath);
 // static DialogResult::T Show(HWND owner, string text, string caption, MessageBoxButtons::T buttons, MessageBoxIcon icon, MessageBoxDefaultButton defaultButton, MessageBoxOptions options, string helpFilePath, HelpNavigator navigator);
 // static DialogResult::T Show(HWND owner, string text, string caption, MessageBoxButtons::T buttons, MessageBoxIcon icon, MessageBoxDefaultButton defaultButton, MessageBoxOptions options, string helpFilePath, string keyword);
 // static DialogResult::T Show(HWND owner, string text, string caption, MessageBoxButtons::T buttons, MessageBoxIcon icon, MessageBoxDefaultButton defaultButton, MessageBoxOptions options, string helpFilePath, HelpNavigator navigator, object param);
 // static DialogResult::T Show(string text, string caption, MessageBoxButtons::T buttons, MessageBoxIcon icon, MessageBoxDefaultButton defaultButton, MessageBoxOptions options, bool displayHelpButton);
 // static DialogResult::T Show(string text, string caption, MessageBoxButtons::T buttons, MessageBoxIcon icon, MessageBoxDefaultButton defaultButton, MessageBoxOptions options, string helpFilePath);
 // static DialogResult::T Show(string text, string caption, MessageBoxButtons::T buttons, MessageBoxIcon icon, MessageBoxDefaultButton defaultButton, MessageBoxOptions options, string helpFilePath, HelpNavigator navigator);
 // static DialogResult::T Show(string text, string caption, MessageBoxButtons::T buttons, MessageBoxIcon icon, MessageBoxDefaultButton defaultButton, MessageBoxOptions options, string helpFilePath, string keyword);
 // static DialogResult::T Show(string text, string caption, MessageBoxButtons::T buttons, MessageBoxIcon icon, MessageBoxDefaultButton defaultButton, MessageBoxOptions options, string helpFilePath, HelpNavigator navigator, object param);
};

}
