#include "stdafx.h"
#include "Process.h"
#include "NativeString.h"
#include "Environment.h"
#include "../String/StringBuilder.h"
#include <shellapi.h>
#pragma comment(lib, "shell32.lib")
namespace Themis {

void Process::Close() {
    if (processHandle != NULL && processHandle != INVALID_HANDLE_VALUE) {
        BOOL result = ::CloseHandle(processHandle);
        Themis_DebugAssertApi(result != FALSE);
        processHandle = INVALID_HANDLE_VALUE;
    }
}

/*
bool Process::CloseMainWindow() {
    IntPtr mainWindowHandle = MainWindowHandle;
    if (mainWindowHandle == NULL) {
        return false;
    }
    if ((GetWindowLong(new HandleRef(this, mainWindowHandle), -16) & 0x8000000) != 0) {
        return false;
    }
    PostMessage(new HandleRef(this, mainWindowHandle), 0x10, 0, 0);
    return true;
}
*/

namespace {
    template<class charT> bool StartWithCreateProcessT(const ProcessStartInfo& startInfo,
        const NativeString& cmdLine, HANDLE& processHandle, charT dummy) {
        typedef typename NativeStruct<charT, STARTUPINFOA, STARTUPINFOW>::T STARTUPINFO;
        NativeString dir = startInfo.WorkingDirectory;
        if (dir.GetLength() <= 0) {
            dir = Environment::GetCurrentDirectory();
        }

        STARTUPINFO si = {sizeof(si)};
        PROCESS_INFORMATION pi = {0};
        int creationFlags = 0;
        if (startInfo.CreateNoWindow) {
            creationFlags |= CREATE_NO_WINDOW;
        }
        BOOL result = NativeString::IsUnicode() ?
            ::CreateProcessW(NULL, (LPWSTR)cmdLine.Get(dummy), NULL, NULL, TRUE,
                creationFlags, NULL, (LPCWSTR)dir.Get(dummy), (LPSTARTUPINFOW)&si, &pi) :
            ::CreateProcessA(NULL, (LPSTR)cmdLine.Get(dummy), NULL, NULL, TRUE,
                creationFlags, NULL, (LPCSTR)dir.Get(dummy), (LPSTARTUPINFOA)&si, &pi);
        if (pi.hThread != NULL && pi.hThread != INVALID_HANDLE_VALUE) {
            ::CloseHandle(pi.hThread);
            pi.hThread = INVALID_HANDLE_VALUE;
        }
        if (pi.hProcess != NULL && pi.hProcess != INVALID_HANDLE_VALUE) {
            processHandle = pi.hProcess;
        }
        if (result == FALSE) {
            throw Win32Exception(L"vZX̍쐬Ɏs");
        }
        return true;
    }

    template<class charT> bool StartWithShellExecuteExT(const ProcessStartInfo& startInfo, HANDLE& processHandle, charT dummy) {
        typedef typename NativeStruct<charT, SHELLEXECUTEINFOA, SHELLEXECUTEINFOW>::T SHELLEXECUTEINFO;
        NativeString file = startInfo.FileName;
        NativeString verb = startInfo.Verb;
        NativeString arg = startInfo.Arguments;
        NativeString dir = startInfo.WorkingDirectory;

        SHELLEXECUTEINFO si = {sizeof(si)};
        si.fMask = SEE_MASK_NOCLOSEPROCESS;
        if (startInfo.ErrorDialog) {
            si.hwnd = startInfo.ErrorDialogParentHandle;
        } else {
            si.fMask |= SEE_MASK_FLAG_NO_UI;
        }
        si.nShow = (int)startInfo.WindowStyle;
        if (0 < file.GetLength()) si.lpFile = file.Get(dummy);
        if (0 < verb.GetLength()) si.lpVerb = verb.Get(dummy);
        if (0 < arg.GetLength()) si.lpParameters = arg.Get(dummy);
        if (0 < dir.GetLength()) si.lpDirectory = dir.Get(dummy);
        si.fMask |= SEE_MASK_FLAG_DDEWAIT;
        BOOL r = NativeString::IsUnicode() ?
            ::ShellExecuteExW((LPSHELLEXECUTEINFOW)&si) :
            ::ShellExecuteExA((LPSHELLEXECUTEINFOA)&si);
        if (r == FALSE) {
            throw Win32Exception(L"vZX̍쐬Ɏs");
        }
        if (si.hProcess != NULL) {
            processHandle = si.hProcess;
            return true;
        }
        return false;
    }
}

bool Process::StartWithCreateProcess(const ProcessStartInfo& startInfo) {
    string cmdLine = BuildCommandLine(startInfo.FileName, startInfo.Arguments);
    return NativeString::IsUnicode() ?
        StartWithCreateProcessT(startInfo, cmdLine, processHandle, wchar_t(0)) :
        StartWithCreateProcessT(startInfo, cmdLine, processHandle, char(0));
}

bool Process::StartWithShellExecuteEx(const ProcessStartInfo& startInfo) {
    return NativeString::IsUnicode() ?
        StartWithShellExecuteExT(startInfo, processHandle, wchar_t(0)) :
        StartWithShellExecuteExT(startInfo, processHandle, char(0));
}

string Process::BuildCommandLine(const string& executableFileName, const string& arguments) {
    StringBuilder builder;
    string str = executableFileName.Trim();
    bool quarted = str.StartsWith(L"\"") && str.EndsWith(L"\"");
    if (!quarted) {
        builder.Append(L"\"");
    }
    builder.Append(str);
    if (!quarted) {
        builder.Append(L"\"");
    }
    if (!string::IsNullOrEmpty(arguments)) {
        builder.Append(L" ");
        builder.Append(arguments);
    }
    return builder.ToString();
}

}
