#include "stdafx.h"
#include <Themis/Config2.h>
#include <Themis/Windows/System.h>
#include <Themis/Windows/MessageBox.h>

void SetRelation(const string& strSuffix, bool bRegister, bool& bFileType) {
    try {
        if (bRegister) {
            CSystem::RegisterRelation(strSuffix, L"gv2-img-files");
            bFileType = true;
        } else {
            CSystem::DeleteRelation(strSuffix, L"gv2-img-files");
        }
    } catch (...) {
        MessageBox::Show(L"." + strSuffix +
            (bRegister ? L" ̊֘AtɎs܂B" : L" ̊֘AÂ̍폜Ɏs܂B"),
            L"G[", MessageBoxButtons::OK, MessageBoxIcon::Information, MessageBoxDefaultButton::Button1,
            MB_SYSTEMMODAL | MB_TOPMOST | MB_SETFOREGROUND);
    }
}

void SetRelationCommand(const string& strSuffix, bool bRegister, const string& strMenuName, CConfig2& config) {
    try {
        if (bRegister) {
            string strFileType = CSystem::GetRelationFileType(strSuffix);
            if (string::IsNullOrEmpty(strFileType)) {
                MessageBox::Show(L"." + strSuffix +
                    L" ֘AtĂ܂BENbNj[̓o^sꍇA֘AtsĂĂB",
                    L"G[", MessageBoxButtons::OK, MessageBoxIcon::Information, MessageBoxDefaultButton::Button1,
                    MB_SYSTEMMODAL | MB_TOPMOST | MB_SETFOREGROUND);
                return;
            }
            CSystem::SRelationFileTypeInfo oInfo;
            oInfo.strCommand = L"open-by-gv2";
            oInfo.strMenuName = strMenuName;
            oInfo.strCommandLine = L"\"" + config.Line(L"ModuleFileName").Get() + L"\" \"%1\"";
            CSystem::RegisterRelationFileType(strFileType, oInfo);
        } else {
            CSystem::DeleteRelationCommand(CSystem::GetRelationFileType(strSuffix), L"open-by-gv2");
        }
    } catch (...) {
        MessageBox::Show(L"." + strSuffix +
            (bRegister ? L" ̉ENbNj[o^Ɏs܂B" : L" ̉ENbNj[폜Ɏs܂B"),
            L"G[", MessageBoxButtons::OK, MessageBoxIcon::Information, MessageBoxDefaultButton::Button1,
            MB_SYSTEMMODAL | MB_TOPMOST | MB_SETFOREGROUND);
    }
}

void SetRelationFolder(bool bRegister, const string& strMenuName, CConfig2& config) {
    try {
        if (bRegister) {
            CSystem::SRelationFileTypeInfo oInfo;
            oInfo.strCommand = L"open-by-gv2";
            oInfo.strMenuName = strMenuName;
            oInfo.strCommandLine = L"\"" + config.Line(L"ModuleFileName").Get() + L"\" \"%1\"";
            CSystem::RegisterRelationFileType(L"Folder", oInfo);
        } else {
            CSystem::DeleteRelationCommand(L"Folder", L"open-by-gv2");
        }
    } catch (...) {
        MessageBox::Show(L"tH_̉ENbNj[" + string::Empty +
            (bRegister ? L" ̓o^Ɏs܂B" : L" ̍폜Ɏs܂B"),
            L"G[", MessageBoxButtons::OK, MessageBoxIcon::Information, MessageBoxDefaultButton::Button1,
            MB_SYSTEMMODAL | MB_TOPMOST | MB_SETFOREGROUND);
    }
}

void DoRegisterRelations(CConfig2& config) {
    // ֘AÂx폜
    {
        vector<string> vct = config.Str(L"EnvSetOthersOld").Get().Split(Array(L';'), StringSplitOptions::RemoveEmptyEntries);
        for (size_t i = 0, n = vct.size(); i < n; i++) {
            if (string::IsNullOrEmpty(vct[i])) continue;
            try {
                CSystem::DeleteRelation(vct[i], L"gv2-img-files");
            } catch (std::exception&) {
            }
        }
    }
    {
        vector<string> vct = config.Str(L"EnvRClickOthersOld").Get().Split(Array(L';'), StringSplitOptions::RemoveEmptyEntries);
        for (size_t i = 0, n = vct.size(); i < n; i++) {
            if (string::IsNullOrEmpty(vct[i])) continue;
            try {
                CSystem::DeleteRelationCommand(CSystem::GetRelationFileType(vct[i]), L"open-by-gv2");
            } catch (std::exception&) {
            }
        }
    }
    // ȑÕo[WŊԈႦēo^Ă\ŁBB
    CSystem::DeleteRelationCommand(L"gv2-img-files", L"open-by-gv2");

    // ֘At
    bool bFileType = false;
    SetRelation(L"jpg", config.Bool(L"RelationJPG").Get(), bFileType);
    SetRelation(L"png", config.Bool(L"RelationPNG").Get(), bFileType);
    SetRelation(L"bmp", config.Bool(L"RelationBMP").Get(), bFileType);
    SetRelation(L"gif", config.Bool(L"RelationGIF").Get(), bFileType);
    SetRelation(L"argb", config.Bool(L"RelationARGB").Get(), bFileType);
    SetRelation(L"cbz", config.Bool(L"RelationCBZ").Get(), bFileType);
    SetRelation(L"cbr", config.Bool(L"RelationCBR").Get(), bFileType);
    {
        vector<string> vct = config.Str(L"EnvSetOthers").Get().Split(Array(L';'), StringSplitOptions::RemoveEmptyEntries);
        for (size_t i = 0, n = vct.size(); i < n; i++) {
            if (string::IsNullOrEmpty(vct[i])) continue;
            SetRelation(vct[i], true, bFileType);
        }
    }
    try {
        if (bFileType) {
            CSystem::SRelationFileTypeInfo oInfo;
            oInfo.strCommandLine = L"\"" + config.Line(L"ModuleFileName").Get() + L"\" \"%1\"";
            oInfo.strDescription = config.Line(L"EnvDesc").Get();
            oInfo.strIconFile = config.Line(L"ModuleFileName").Get();
            oInfo.nIcon = 1;
            CSystem::RegisterRelationFileType(L"gv2-img-files", oInfo);
        } else {
            CSystem::DeleteRelationFileType(L"gv2-img-files");
        }
    } catch (...) {
        MessageBox::Show(L"֘AtɎs܂B",
            L"G[", MessageBoxButtons::OK, MessageBoxIcon::Information, MessageBoxDefaultButton::Button1,
            MB_SYSTEMMODAL | MB_TOPMOST | MB_SETFOREGROUND);
    }
    // Ԃ܂
    try {
        if (config.Bool(L"RelationGVB").Get()) {
            CSystem::SRelationFileTypeInfo oInfo;
            oInfo.strCommandLine = L"\"" + config.Line(L"ModuleFileName").Get() + L"\" \"%1\"";
            oInfo.strDescription = config.Line(L"EnvDescGvb").Get();
            oInfo.strIconFile = config.Line(L"ModuleFileName").Get();
            oInfo.nIcon = 2;
            CSystem::RegisterRelationFileType(L"gv2-bkmk-files", oInfo);
            CSystem::RegisterRelation(L"gvb", L"gv2-bkmk-files");
        } else {
            CSystem::DeleteRelationFileType(L"gv2-bkmk-files");
        }
    } catch (...) {
        MessageBox::Show(L"ubN}[N̊֘AtɎs܂B",
            L"G[", MessageBoxButtons::OK, MessageBoxIcon::Information, MessageBoxDefaultButton::Button1,
            MB_SYSTEMMODAL | MB_TOPMOST | MB_SETFOREGROUND);
    }

    // 
    string strMenuName = config.Line(L"EnvMenuName").Get();
    SetRelationCommand(L"zip", config.Bool(L"MenuZIP").Get(), strMenuName, config);
    SetRelationCommand(L"rar", config.Bool(L"MenuRAR").Get(), strMenuName, config);
    SetRelationCommand(L"lzh", config.Bool(L"MenuLZH").Get(), strMenuName, config);
    SetRelationCommand(L"gca", config.Bool(L"MenuGCA").Get(), strMenuName, config);
    SetRelationCommand(L"cbz", config.Bool(L"MenuCBZ").Get(), strMenuName, config);
    SetRelationCommand(L"cbr", config.Bool(L"MenuCBR").Get(), strMenuName, config);
    SetRelationFolder(config.Bool(L"MenuFolder").Get(), strMenuName, config);
    {
        vector<string> vct = config.Str(L"EnvRClickOthers").Get().Split(Array(L';'), StringSplitOptions::RemoveEmptyEntries);
        for (size_t i = 0, n = vct.size(); i < n; i++) {
            if (string::IsNullOrEmpty(vct[i])) continue;
            SetRelationCommand(vct[i], true, strMenuName, config);
        }
    }

    // XVVXeɒʒm
    CSystem::NotifyRelation();
}

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow) {
    // ݒt@Cǂݍ
    CConfig2 config;
    if (!config.GetStorage().Load(CFileName(Encoding::GetDefault().GetString(lpCmdLine)))) {
        Themis_DebugWarning(L"t@Cǂݍ݂ɎsB");
        return 1;
    }

    // ֘At
    try {
        DoRegisterRelations(config);
    } catch (std::exception& e) {
        MessageBox::Show(Encoding::GetDefault().GetString(e.what()), L"G[",
            MessageBoxButtons::OK, MessageBoxIcon::Information, MessageBoxDefaultButton::Button1,
            MB_SYSTEMMODAL | MB_TOPMOST | MB_SETFOREGROUND);
        return 2;
    }

    // I
    return 0;
}
