#include "stdafx.h"
#include "AppWindow.h"
#include "Dialogs.h"
#include "ConfigDialog.h"
#include "EnvConfigDialog.h"
#include "NetworkUpdater.h"
#include "Document.h"
#include "KeyConfig.h"
#include "ConfigEx.h"
#include "FileListWindow.h"
#include "AppWindowMenu.h"
#include "MenuID.h"
#include "resource.h"
#include <Themis/Windows/System.h>
#include <Themis/Windows/Process.h>
#include <Themis/Windows/Environment.h>
#include <Themis/Windows/MessageBox.h>
#include <Themis/Windows/ScrollBar.h>
#include <Themis/ScreenChanger.h>
#include <Themis/CursorHider.h>
#include <Themis/Graphics.h>
#include <Themis/Image.h>
#include <Themis/File.h>
#include <Themis/Layout.h>
#include <Themis/Display.h>
#include <Themis/Archive.h>
#include <Themis/Rand.h>
#include <Themis/WindowSnapper.h>
#include <Themis/RectSelector.h>
#include <Themis/Utility/StringUtility.h>
#include <time.h>


const int c_nVersion = 219;

#define CHECK_INITIALIZE_TIME 0



CAppWindow::CAppWindow(const string& strConfigPrefix): m_strConfigPrefix(strConfigPrefix),
m_bSlideShow(false), m_bPosRestoreDisable(false) {
    m_pSpi.reset(new CSpi);
    m_pJpgLoaderFactory.reset(new CJpgLoaderFactory);
    m_pPngLoaderFactory.reset(new CPngLoaderFactory);
    m_pBmpLoaderFactory.reset(new CBmpLoaderFactory);
    m_pSpiLoaderFactory.reset(new CSpiLoaderFactory(m_pSpi));
    m_pArgbLoaderFactory.reset(new CArgbLoaderFactory);
    m_pTgaLoaderFactory.reset(new CTgaLoaderFactory);
    m_pOleLoaderFactory.reset(new COleLoaderFactory);
    m_pZipArchiverFactory.reset(new CUnZipArchiverFactory);
    m_p7ZipArchiverFactory.reset(new C7ZipArchiverFactory);
    m_pRarArchiverFactory.reset(new CUnRarArchiverFactory);
    m_pLhaArchiverFactory.reset(new CUnLhaArchiverFactory);
    m_pSpiArchiverFactory.reset(new CSpiArchiverFactory(m_pSpi));
    m_pZip2ArchiverFactory.reset(new CZipArchiverFactory);
    m_pGcaArchiverFactory.reset(new CUnGcaArchiverFactory);
    rectSelector.reset(new CRectSelector());

    ConfigInitialize(m_oConfig, CFileNameModuleDir(m_strConfigPrefix + L".cfg"));

    appWindowMenu.reset(new AppWindowMenu());
    {
        CWindowParameter windowParameter;
        windowParameter.SetMenu(appWindowMenu->GetHandle());
        windowParameter.GetStyle() |= WS_THICKFRAME | WS_MAXIMIZEBOX; //TCYύX
        m_pWindow.reset(new CWindow);
        m_pWindow->GetHook().Add(this);
        m_pWindow->CreateWithThread(windowParameter);
    }

    m_pDocument.reset(new CDocument(*this, *this, m_pWindow->GetHandle(), m_oConfig));

    m_oChooseFile.SetOwner(m_pWindow->GetHandle());
    m_oChooseDir.SetOwner(m_pWindow->GetHandle());
    m_oChooseColor.SetOwner(m_pWindow->GetHandle());
    rectSelector->SetWindow(m_pWindow->GetHandle());
    m_pScreenChanger.reset(new CScreenChanger(*m_pWindow));
    m_pMagnifierWindow.reset(new CMagnifierWindow(*m_pWindow, *this));
    m_pFileListWindow.reset(new CFileListWindow(*m_pDocument));
    m_pFileListWindow->SetInitRect(m_oConfig.Rect(L"FileListWindowRect"));
    m_pDocument->GetFileList().SetWindow(m_pFileListWindow.get());
    m_pCursorHider.reset(new CCursorHider(*m_pWindow));

    ApplyConfigData();
}

CAppWindow::~CAppWindow() {
    m_oConfig.Rect(L"FileListWindowRect", m_pFileListWindow->GetInitRect());
    if (!m_bPosRestoreDisable) {
        m_oConfig.Rect(L"WindowPosRestoreRect", m_rcWindow);
    }
    m_oConfig.Str(L"ChooseColor", m_oChooseColor.SaveToString());
    m_oConfig.Save(CFileNameModuleDir(m_strConfigPrefix + L".cfg"));

    // ɍ폜ƂȂƃoO\cB
    m_pDocument.reset();
}

#if CHECK_INITIALIZE_TIME
#include <Themis/TimeChecker.h>
auto_ptr<CTimeChecker> g_pChecker(new CTimeChecker(L""));
#endif

int CAppWindow::Run(const vector<CFileName>& vctFiles, int nCmdShow) {
    boost::optional<CSize> szClient;
    try {
        if (!vctFiles.empty()) {
            OpenFiles(vctFiles, false);
        }
        if (m_pDocument->IsReady()) { //ǂݍ܂ꂽ
            CSize szMax = CSystem::GetWorkArea().GetSize() * 2 / 3;
            //摜ƓcɁB
            if (m_pDocument->GetImageSize().cx < m_pDocument->GetImageSize().cy) {
                szClient = CSize(szMax.cy * m_pDocument->GetImageSize().cx / m_pDocument->GetImageSize().cy, szMax.cy);
            } else {
                szClient = CSize(szMax.cx, szMax.cx * m_pDocument->GetImageSize().cy / m_pDocument->GetImageSize().cx);
            }
        } else { //ǂݍ܂ĂȂ
            throw std::exception();
        }
    } catch (std::exception&) {
        m_pDocument->Clear();
     // throw;
    }

    boost::optional<CPoint> ptWindow;
    if (m_oConfig.Bool(L"WindowPosRestore")) {
        if (szClient && m_oConfig.Bool(L"WindowPosRestoreDisableOnLoad")) {
            m_bPosRestoreDisable = true;
            m_pWindow->SetClientSize(*szClient);
        } else {
            CRect rc = m_oConfig.Rect(L"WindowPosRestoreRect");
            ptWindow = rc.GetPoint();
            szClient = rc.GetSize();
            m_pWindow->SetClientSize(*szClient);
            m_pWindow->SetPosClip(*ptWindow);
        }
    } else {
        if (szClient) {
            m_pWindow->SetClientSize(*szClient);
        } else {
            szClient = CSystem::GetWorkArea().GetSize() / 2;
            m_pWindow->SetClientSize(*szClient);
        }
    }

#if CHECK_INITIALIZE_TIME
g_pChecker.reset();
#endif

    // \
    m_pWindow->Show(nCmdShow);
    m_pWindow->PostMsg(WM_APP + 0, 0, 0); // SetForeground

    // ʒuE傫̍Đݒ
    if (szClient) { m_pWindow->SetClientSize(*szClient); }
    if (ptWindow) { m_pWindow->SetPosClip(*ptWindow); }

    // L[RtBO
    m_pKeyConfig.reset(new CKeyConfigEx(*m_pWindow, m_strConfigPrefix));

    // j[XV
    m_pKeyConfig->UpdateMenubar(m_pWindow->GetMenu());
    UpdateMenu();

    // R_CAOƂKɏ
    m_oChooseColor.LoadFromString(m_oConfig.Str(L"ChooseColor"));

    // RtBÔ܂dvȂB
    ConfigDelayInitialize(m_oConfig, CFileNameModuleDir(m_strConfigPrefix + L".cfg"));

    // JgfBNg͖QȂƂɁB
    Environment::SetCurrentDirectory(CFileNameModuleDir().RemoveDirSeparator());

    // ܂łЂ҂[
    m_pWindow->WaitForQuit();
    return 0;
}

// bZ[W̏
bool CAppWindow::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_COMMAND) {
        bool bRet = OnCommand(rPrms);
        if (m_bSlideShow && bRet && rPrms.GetWPrmLoWord() != GV_MENUID(56) &&
            m_oConfig.Bool(L"SlideShowStopWithCommand")) {
            m_pWindow->UnsetTimer(100);
            m_bSlideShow = false;
            UpdateMenu();
        }
        return bRet;
    } else if (rPrms.GetMsg() == WM_TIMER) {
        OnTimer();
        return true;
    } else if (rPrms.GetMsg() == WM_LBUTTONDOWN) {
        rectSelector->OnButtonDown(m_pDocument->IsReadyImage());
        SetCaption();
        return true;
    } else if (rPrms.GetMsg() == WM_LBUTTONDBLCLK) {
    //*
        if (rectSelector->IsSelected()) {
            rectSelector->OnButtonDown(m_pDocument->IsReadyImage());
            SetCaption();
            return true;
        }
        return false;
    /*/
        _uNbNł̍ő剻ƏՓ˂ĂȂ肤ۂ
        rectSelector->OnButtonDown(m_pDocument->IsReadyImage());
        return true;
    //*/
    } else if (rPrms.GetMsg() == WM_LBUTTONUP) {
        rectSelector->OnButtonUp();
        SetCaption();
        return true;
    } else if (rPrms.GetMsg() == WM_MOUSEMOVE) {
        if (m_pCursorHider.get() != NULL) {
            m_pCursorHider->Reset();
        }
        rectSelector->OnCursorMove();
        if (rectSelector->IsSelected()) { // IĂȂԂȂ瓮ƂŕςȂ͂
            SetCaption();
        }
        return true;
    } else if (rPrms.GetMsg() == WM_SETCURSOR) {
        return rectSelector->OnSetCursor();
    } else if (rPrms.GetMsg() == WM_SIZE) {
        m_pDocument->SetScreen(m_pWindow->GetClientSize());
     // LayoutChanged();
        SetCaption();
        // EBhEʒu̕ۑ
        if (m_pWindow->IsShowNormal() && m_pScreenChanger.get() != NULL && !m_pScreenChanger->IsFullScreen()) {
            m_rcWindow = CRect(m_pWindow->GetPos(), m_pWindow->GetClientSize()); // ȂGetClientRect/SetClientRectClipƃoÔŁBB
        }
        return false;
    } else if (rPrms.GetMsg() == WM_ENTERSIZEMOVE) {
        if (snapper.get() != NULL) {
            return snapper->OnEnterSizeMove(rPrms);
        }
        return false;
    } else if (rPrms.GetMsg() == WM_EXITSIZEMOVE) {
        if (snapper.get() != NULL) {
            return snapper->OnExitSizeMove(rPrms);
        }
        return false;
    } else if (rPrms.GetMsg() == WM_MOVING) {
        if (snapper.get() != NULL) {
            return snapper->OnMoving(rPrms);
        }
        return false;
    } else if (rPrms.GetMsg() == WM_MOVE) {
        // EBhEʒu̕ۑ
        if (m_pWindow->IsShowNormal() && m_pScreenChanger.get() != NULL && !m_pScreenChanger->IsFullScreen()) {
            m_rcWindow = CRect(m_pWindow->GetPos(), m_pWindow->GetClientSize()); // ȂGetClientRect/SetClientRectClipƃoÔŁBB
        }
        return false;
    } else if (rPrms.GetMsg() == WM_HSCROLL) {
        if (hScrollBar.get() != NULL) {
            return hScrollBar->OnScroll(rPrms);
        }
        return false;
    } else if (rPrms.GetMsg() == WM_VSCROLL) {
        if (vScrollBar.get() != NULL) {
            return vScrollBar->OnScroll(rPrms);
        }
        return false;
    } else if (rPrms.GetMsg() == WM_PAINT) {
        std::tr1::shared_ptr<PAINTSTRUCT> ps = m_pWindow->BeginPaint();
        m_pDocument->Draw(*ps);
        if (m_pDocument->IsReady()) {
            m_pDocument->DrawMark(*ps);
            m_pDocument->DrawFilterMark(*ps);
        }
        if (m_pDocument->IsReadyImage()) {
            rectSelector->OnPaint(*ps);
        }
        return false;
    } else if (rPrms.GetMsg() == WM_DROPFILES) {
        OpenFiles(m_pWindow->GetDropedFiles(rPrms.GetWPrm()), true);
        return true;
    } else if (rPrms.GetMsg() == WM_CREATE) {
        // XN[o[ȃIuWFNgB
        hScrollBar.reset(new HScrollBar(m_pWindow->GetHandle()));
        vScrollBar.reset(new VScrollBar(m_pWindow->GetHandle()));
        hScrollBar->SetScrollEvent(std::tr1::bind(&CAppWindow::OnHScroll, this, _1, _2, _3));
        vScrollBar->SetScrollEvent(std::tr1::bind(&CAppWindow::OnVScroll, this, _1, _2, _3));
        // X^C̐ݒȂ
        m_pWindow->SetIcon(IDI_ICON1);
        m_pWindow->SetAcceptFiles(true);
        return true;
    } else if (rPrms.GetMsg() == WM_CLOSE) {
        if (DeleteAllMarkedOrClear()) {
            m_pDocument->Clear(); // I񂪁AEBhEăvZXc̓}V(L`)
            m_pFileListWindow->Hide(false);
            m_pScreenChanger.reset();
            m_pWindow->Delete();
        } else {
            // (Ȃ)
        }
        return true;
    } else if (rPrms.GetMsg() == WM_APP + 0) {
        m_pWindow->SetForeground();
        OnUpdateDocument(CRect()); // ÔߍXV
        return true;
    } else {
        return false;
    }
}

// WM_COMMAND̏(̂œƗƂ)
bool CAppWindow::OnCommand(CMsgHookPrms& rPrms) {
    switch (LOWORD(rPrms.GetWPrm())) {
    case GV_MENUID(0): // VK쐬FEBhE
        Process::Start(CSystem::GetModuleFileName());
        break;
    case GV_MENUID(1): // t@CJ
        m_oChooseFile.SetTitle(L"t@CJ");
        m_oChooseFile.ClearNameFilter();
        m_oChooseFile.AddNameFilter(L"All files", L"*.*");
        m_oChooseFile.SetDefaultExt(string());
        m_oChooseFile.SetCurrentDir(CFileNameModuleDir(m_oConfig.File(L"CurrentDir")));
        if (m_oChooseFile.ShowOpenMulti()) {
            OpenFiles(m_oChooseFile.GetResults(), false);
        }
        break;
    case GV_MENUID(2): // tH_J
        m_oChooseDir.SetDefault(CFileName(m_oConfig.File(L"CurrentDir")));
        m_oChooseDir.SetTitle(L"tH_J");
        if (m_oChooseDir.Show()) {
            vector<CFileName> vct;
            vct.push_back(m_oChooseDir.GetResult());
            OpenFiles(vct, false);
            m_oConfig.File(L"CurrentDir", m_oChooseDir.GetResult().Get());
        }
        break;
    case GV_MENUID(139): // J
        {
            CHistoryDialog dialog(*this);
            dialog.Open(m_pWindow->GetHandle());
            if (dialog.IsOk()) {
                m_pDocument->LoadHistory(dialog.GetNo());
            }
        }
        break;
    case GV_MENUID(3): // t@CSĕ
        if (DeleteAllMarkedOrClear()) {
            m_pDocument->Clear();
        }
        break;
    case GV_MENUID(4): // }[Nݒ
        m_pDocument->SetMark(true);
        break;
    case GV_MENUID(5): // }[N
        m_pDocument->SetMark(false);
        break;
    case GV_MENUID(6): // SẴ}[N
        if (0 < m_pDocument->GetMarkCount()) {
            if (MessageBox::Show(m_pWindow->GetHandle(), 
                L"" + ItoA(m_pDocument->GetMarkCount()) +  L" ̃}[NSĉ܂?",
                L"SẴ}[N̊mF", MessageBoxButtons::YesNo, MessageBoxIcon::Question) == DialogResult::Yes) {
                m_pDocument->ClearMarks();
            }
        }
        break;
    case GV_MENUID(7): // SẴ}[N𔽓]
        m_pDocument->ReverseAllMarks();
        break;
    case GV_MENUID(97): // ŏ猻݂܂ł̃}[N𔽓]
        m_pDocument->ReverseFirstToCurrentMarks();
        break;

    case GV_MENUID(98): // }[Nt@CXg폜
        if (0 < m_pDocument->GetMarkCount()) {
            if (MessageBox::Show(m_pWindow->GetHandle(), 
                L"}[Nt@C " + ItoA(m_pDocument->GetMarkCount()) +  L" Xg폜܂?",
                L"Xg폜̊mF", MessageBoxButtons::YesNo, MessageBoxIcon::Question) == DialogResult::Yes) {
                m_pDocument->GetFileList().RemoveAllMarks(false, false, NULL);
                m_pDocument->Reload();
            }
        }
        break;
    case GV_MENUID(8): // }[Nt@CSɍ폜
        if (0 < m_pDocument->GetMarkCount()) {
            if (MessageBox::Show(m_pWindow->GetHandle(), 
                L"}[Nt@C " + ItoA(m_pDocument->GetMarkCount()) +  L" Sɍ폜܂?",
                L"t@C폜̊mF", MessageBoxButtons::YesNo, MessageBoxIcon::Question) == DialogResult::Yes) {
                m_pDocument->GetFileList().RemoveAllMarks(true, m_oConfig.Bool(L"DeleteByShell"), m_pWindow->GetHandle());
                m_pDocument->Reload();
            }
        }
        break;
    case GV_MENUID(104): // }[Nt@Cړ
        if (0 < m_pDocument->GetMarkCount()) {
            m_oChooseDir.SetDefault(CFileName(m_oConfig.File(L"CurrentDir")));
            m_oChooseDir.SetTitle(L"}[Nt@C " + ItoA(m_pDocument->GetMarkCount()) +  L" ړ");
            if (m_oChooseDir.Show()) {
                m_oConfig.File(L"CurrentDir", m_oChooseDir.GetResult().Get());
                m_pDocument->GetFileList().RenameAllMarks(m_oChooseDir.GetResult(), m_oConfig.Bool(L"RemoveOnMove"));
                m_pDocument->Reload();
            }
        }
        break;
    case GV_MENUID(105): // }[Nt@C𕡐
        if (0 < m_pDocument->GetMarkCount()) {
            m_oChooseDir.SetDefault(CFileName(m_oConfig.File(L"CurrentDir")));
            m_oChooseDir.SetTitle(L"}[Nt@C " + ItoA(m_pDocument->GetMarkCount()) +  L" 𕡐");
            if (m_oChooseDir.Show()) {
                m_oConfig.File(L"CurrentDir", m_oChooseDir.GetResult().Get());
                m_pDocument->GetFileList().CopyAllMarks(m_oChooseDir.GetResult());
            }
        }
        break;
    case GV_MENUID(120): // }[Nt@CRs[
        if (0 < m_pDocument->GetMarkCount()) {
            CSystem::SetClipboard(string::Join(L"\r\n", m_pDocument->GetMarkedFiles()));
        }
        break;
    case GV_MENUID(9): // ݂̉摜̂tH_J
        if (m_pDocument->IsReady()) {
            Process::Start(L"Explorer.exe", L"/select," + m_pDocument->GetCurrentExistFileName().Get());
        }
        break;

    case GV_MENUID(11): // t@CNbv{[hփRs[
        if (m_pDocument->IsReady()) {
            CSystem::SetClipboard(m_pDocument->GetCurrentFileName().Get());
            //t@CɂcH
        }
        break;

    case GV_MENUID(61): // t@CXg폜
        if (m_pDocument->IsReady()) {
            CFileName oFileName = m_pDocument->GetCurrentFileName();
            if (!m_oConfig.Bool(L"QuestionRemoveFile") ||
                MessageBox::Show(m_pWindow->GetHandle(), 
                L"'" + oFileName.Get() + L"' Xg폜܂?",
                L"Xg폜̊mF", MessageBoxButtons::YesNo, MessageBoxIcon::Question) == DialogResult::Yes) {
                m_pDocument->Remove();
            }
        }
        break;
    case GV_MENUID(12): // t@C폜
        if (m_pDocument->IsReady()) {
            CFileName oFileName = m_pDocument->GetCurrentFileName();
            if (MessageBox::Show(m_pWindow->GetHandle(), 
                L"'" + oFileName.Get() + L"' 폜܂?",
                L"t@C폜̊mF", MessageBoxButtons::YesNo, MessageBoxIcon::Question) == DialogResult::Yes) {
                m_pDocument->Delete(m_pWindow->GetHandle());
            }
        }
        break;
    case GV_MENUID(13): // t@Cړ
        if (m_pDocument->IsReady()) {
            CFileName oOldName = m_pDocument->GetCurrentFileName();
            string strExt = oOldName.GetSuffix();
            m_oChooseFile.SetTitle(L"t@Cړ");
            m_oChooseFile.ClearNameFilter();
            m_oChooseFile.AddNameFilter(strExt.ToUpper() + L" Files", L"*." + strExt);
            m_oChooseFile.AddNameFilter(L"All Files", L"*.*");
            m_oChooseFile.SetDefaultExt(strExt);
            m_oChooseFile.SetDefault(GetSaveDefaultName());
            m_oChooseFile.SetCurrentDir(CFileNameModuleDir(m_oConfig.File(L"CurrentDir")));
            if (m_oChooseFile.ShowSave()) {
                m_pDocument->Rename(m_oChooseFile.GetResult());
             // m_pDocument->Remove();
                m_oConfig.File(L"CurrentDir", m_oChooseFile.GetCurrentDir().Get());
            }
        }
        break;
    case GV_MENUID(14): // t@C𕡐
        if (m_pDocument->IsReady()) {
            CFileName oOldName(m_pDocument->GetCurrentFileName().GetPureName());
            string strExt = oOldName.GetSuffix();
            m_oChooseFile.SetTitle(L"t@C𕡐");
            m_oChooseFile.ClearNameFilter();
            m_oChooseFile.AddNameFilter(strExt.ToUpper() + L" Files", L"*." + strExt);
            m_oChooseFile.AddNameFilter(L"All Files", L"*.*");
            m_oChooseFile.SetDefaultExt(strExt);
            m_oChooseFile.SetDefault(GetSaveDefaultName());
            m_oChooseFile.SetCurrentDir(CFileNameModuleDir(m_oConfig.File(L"CurrentDir")));
            if (m_oChooseFile.ShowSave()) {
                m_pDocument->Copy(m_oChooseFile.GetResult());
                m_oConfig.File(L"CurrentDir", m_oChooseFile.GetCurrentDir().Get());
            }
        }
        break;
    case GV_MENUID(15): // EBhEŏ
        m_pWindow->Minimize();
        break;
    case GV_MENUID(10): // EBhEő剻
        if (m_pWindow->IsMaximized()) {
            m_pWindow->ShowRestore();
        } else {
            m_pWindow->Maximize();
        }
        break;
    case GV_MENUID(16): // I
        m_pWindow->Close();
        break;


    case GV_MENUID(17): // 摜̍ēǂݍ
        m_pDocument->Reload(true);
        break;
    case GV_MENUID(18): // Nbv{[h\t
        m_pDocument->Paste();
        break;

    case GV_MENUID(19): // E]
        m_pDocument->MirrorH();
        break;
    case GV_MENUID(20): // ㉺]
        m_pDocument->MirrorV();
        break;
    case GV_MENUID(21): // v90x
        m_pDocument->Rot90();
        break;
    case GV_MENUID(22): // v90x
        m_pDocument->Rot90A();
        break;
    case GV_MENUID(23): // 180x
        m_pDocument->Rot180();
        break;
    case GV_MENUID(24): // pxw]
        if (m_pDocument->IsReadyImage()) {
            CRotDialog(*this).Open(m_pWindow->GetHandle());
        }
        break;
    case GV_MENUID(25): // 𑜓x̕ύX
        if (m_pDocument->IsReadyImage()) {
            CStretchDialog oDialog(m_pDocument->GetImageSize(), m_oConfig.Bool(L"StretchKeepRate"));
            oDialog.Open(m_pWindow->GetHandle());
            boost::optional<CSize> oSize = oDialog.GetResult();
            if (oSize) {
                m_oConfig.Bool(L"StretchKeepRate", oDialog.IsKeepRate());    //ݒۑ
                m_pDocument->Stretch(*oSize);
            }
        }
        break;
    case GV_MENUID(124): // I͈͂
        rectSelector->ClearSelect();
        OnUpdateDocument(CRect());
        break;
    case GV_MENUID(26): // 摜̐؂蔲
        if (!rectSelector->IsSelected()) {
            MessageBox::Show(m_pWindow->GetHandle(), L"؂蔲͈̔͂w肳Ă܂B\r\nhbOŔ͈͂w肵Ă", L"G[");
        } else {
            CRect rc = GetSelectedRect();
            rectSelector->ClearSelect();
            m_pDocument->Cut(rc);
        }
        break;
    case GV_MENUID(27): // hׂ
        if (m_pDocument->IsReadyImage() && m_oChooseColor.Show()) {
            m_pDocument->Fill(m_oChooseColor.GetResult(), GetSelectedRect());
        }
        break;
    case GV_MENUID(28): // x␳
        if (m_pDocument->IsReadyImage()) {
            CLevelRevDialog(*this, GetSelectedRect(), false).Open(m_pWindow->GetHandle());
        }
        break;
    case GV_MENUID(106): // K}␳
        if (m_pDocument->IsReadyImage()) {
            CGammaRevDialog(*this, GetSelectedRect(), false).Open(m_pWindow->GetHandle());
        }
        break;
    case GV_MENUID(116): // 邳ƃRgXg
        if (m_pDocument->IsReadyImage()) {
            CBrightContrastDialog(*this, GetSelectedRect(), false).Open(m_pWindow->GetHandle());
        }
        break;
    case GV_MENUID(29): // UCN
        if (m_pDocument->IsReadyImage()) {
            CMosaicDialog(*this, GetSelectedRect()).Open(m_pWindow->GetHandle());
        }
        break;
    case GV_MENUID(32): // F𔽓]
        if (m_pDocument->IsReadyImage()) {
            m_pDocument->Reverse(GetSelectedRect());
        }
        break;
    case GV_MENUID(33): // ȈՃO[XP[
        if (m_pDocument->IsReadyImage()) {
            m_pDocument->Gray(GetSelectedRect(), false);
        }
        break;
    case GV_MENUID(34): // O[XP[
        if (m_pDocument->IsReadyImage()) {
            m_pDocument->Gray(GetSelectedRect(), true);
        }
        break;
    case GV_MENUID(30): // ڂ
        if (m_pDocument->IsReadyImage()) {
            m_pDocument->Befog(GetSelectedRect());
        }
        break;
    case GV_MENUID(99): // ڂ()
        if (m_pDocument->IsReadyImage()) {
            m_pDocument->HardBefog(GetSelectedRect());
        }
        break;
    case GV_MENUID(31): // V[v
        if (m_pDocument->IsReadyImage()) {
            m_pDocument->Sharp(GetSelectedRect());
        }
        break;
    case GV_MENUID(100): // V[v()
        if (m_pDocument->IsReadyImage()) {
            m_pDocument->HardSharp(GetSelectedRect());
        }
        break;
    case GV_MENUID(150): // KEXڂ
        if (m_pDocument->IsReadyImage()) {
            CGaussDialog(*this, GetSelectedRect(), false, true).Open(m_pWindow->GetHandle());
        }
        break;
    case GV_MENUID(151): // AV[v}XN
        if (m_pDocument->IsReadyImage()) {
            CGaussDialog(*this, GetSelectedRect(), false, false).Open(m_pWindow->GetHandle());
        }
        break;
    case GV_MENUID(114): // fBAtB^
        if (m_pDocument->IsReadyImage()) {
            m_pDocument->MedianFilter(GetSelectedRect());
        }
        break;
    case GV_MENUID(136): // `l̔f
        if (m_pDocument->IsReadyImage()) {
            CAlphaDialog(*this, GetSelectedRect(), false).Open(m_pWindow->GetHandle());
        }
        break;
    case GV_MENUID(35): // Nbv{[hփRs[
        if (m_pDocument->IsReadyImage()) {
            m_pDocument->CopyToClipboard(GetSelectedRect());
        }
        break;
    case GV_MENUID(36): // JPGo
        if (m_pDocument->IsReadyImage()) {
            m_oChooseFile.SetTitle(rectSelector->IsSelected()? L"I͈͂t@Cɏo_CAO": L"t@Cɏo_CAO");
            m_oChooseFile.ClearNameFilter();
            m_oChooseFile.AddNameFilter(L"JPEG Files", L"*.jpg");
            m_oChooseFile.AddNameFilter(L"All Files", L"*.*");
            m_oChooseFile.SetDefaultExt(L"jpg");
            m_oChooseFile.SetDefault(GetSaveDefaultName());
            m_oChooseFile.SetCurrentDir(CFileNameModuleDir(m_oConfig.File(L"CurrentDir")));
            if (m_oChooseFile.ShowSave()) {
                m_pDocument->SaveJpg(m_oChooseFile.GetResult(), GetSelectedRect());
                m_oConfig.File(L"CurrentDir", m_oChooseFile.GetCurrentDir().Get());
            }
        }
        break;
    case GV_MENUID(37): // BMPo
        if (m_pDocument->IsReadyImage()) {
            m_oChooseFile.SetTitle(rectSelector->IsSelected()? L"I͈͂t@Cɏo_CAO": L"t@Cɏo_CAO");
            m_oChooseFile.ClearNameFilter();
            m_oChooseFile.AddNameFilter(L"Bitmap Files", L"*.bmp");
            m_oChooseFile.AddNameFilter(L"All Files", L"*.*");
            m_oChooseFile.SetDefaultExt(L"bmp");
            m_oChooseFile.SetDefault(GetSaveDefaultName());
            m_oChooseFile.SetCurrentDir(CFileNameModuleDir(m_oConfig.File(L"CurrentDir")));
            if (m_oChooseFile.ShowSave()) {
                m_pDocument->SaveBmp(m_oChooseFile.GetResult(), GetSelectedRect());
                m_oConfig.File(L"CurrentDir", m_oChooseFile.GetCurrentDir().Get());
            }
        }
        break;
    case GV_MENUID(38): // PNGo
        if (m_pDocument->IsReadyImage()) {
            m_oChooseFile.SetTitle(rectSelector->IsSelected()? L"I͈͂t@Cɏo_CAO": L"t@Cɏo_CAO");
            m_oChooseFile.ClearNameFilter();
            m_oChooseFile.AddNameFilter(L"PNG Files", L"*.png");
            m_oChooseFile.AddNameFilter(L"All Files", L"*.*");
            m_oChooseFile.SetDefaultExt(L"png");
            m_oChooseFile.SetDefault(GetSaveDefaultName());
            m_oChooseFile.SetCurrentDir(CFileNameModuleDir(m_oConfig.File(L"CurrentDir")));
            if (m_oChooseFile.ShowSave()) {
                m_pDocument->SavePng(m_oChooseFile.GetResult(), GetSelectedRect());
                m_oConfig.File(L"CurrentDir", m_oChooseFile.GetCurrentDir().Get());
            }
        }
        break;
    case GV_MENUID(39): // ARGBo
        if (m_pDocument->IsReadyImage()) {
            m_oChooseFile.SetTitle(rectSelector->IsSelected()? L"I͈͂t@Cɏo_CAO": L"t@Cɏo_CAO");
            m_oChooseFile.ClearNameFilter();
            m_oChooseFile.AddNameFilter(L"ARGB Files", L"*.argb");
            m_oChooseFile.AddNameFilter(L"All Files", L"*.*");
            m_oChooseFile.SetDefaultExt(L"argb");
            m_oChooseFile.SetDefault(GetSaveDefaultName());
            m_oChooseFile.SetCurrentDir(CFileNameModuleDir(m_oConfig.File(L"CurrentDir")));
            if (m_oChooseFile.ShowSave()) {
                m_pDocument->SaveArgb(m_oChooseFile.GetResult(), GetSelectedRect());
                m_oConfig.File(L"CurrentDir", m_oChooseFile.GetCurrentDir().Get());
            }
        }
        break;
    case GV_MENUID(40): // ǎݒ
        if (m_pDocument->IsReady()) {
            string strDir = CFileNameModuleDir(m_oConfig.File(L"WallpaperDir")).Get();
            bool bUnifyFile = m_oConfig.Bool(L"WallpaperUnifyFile");
            string strUnifyFileName = m_oConfig.File(L"WallpaperUnifyFileName");
            bool bResize = m_oConfig.Bool(L"WallpaperResize");
            m_pDocument->SetToWallpaper(strDir, bUnifyFile, strUnifyFileName, bResize);
        }
        break;
    case GV_MENUID(125): OpenExViewer(L"ExViewer0"); break; // Or[ÂPŊJ
    case GV_MENUID(126): OpenExViewer(L"ExViewer1"); break; // Or[ÂQŊJ
    case GV_MENUID(127): OpenExViewer(L"ExViewer2"); break; // Or[ÂRŊJ

    case GV_MENUID(95): // tB^Lɂ
        m_oConfig.Bool(L"Filter", !m_oConfig.Bool(L"Filter"));
        m_pDocument->Reload();
        UpdateMenu();
        break;
    case GV_MENUID(107): // tB^FE]
        if (m_oConfig.Num(L"FilterRot1") == 1) {
            m_oConfig.Num(L"FilterRot1", 0);
        } else {
            m_oConfig.Num(L"FilterRot1", 1);
        }
        m_pDocument->Reload();
        UpdateMenu();
        break;
    case GV_MENUID(108): // tB^F㉺]
        if (m_oConfig.Num(L"FilterRot1") == 2) {
            m_oConfig.Num(L"FilterRot1", 0);
        } else {
            m_oConfig.Num(L"FilterRot1", 2);
        }
        m_pDocument->Reload();
        UpdateMenu();
        break;
    case GV_MENUID(109): // tB^F180x]
        if (m_oConfig.Num(L"FilterRot1") == 3) {
            m_oConfig.Num(L"FilterRot1", 0);
        } else {
            m_oConfig.Num(L"FilterRot1", 3);
        }
        m_pDocument->Reload();
        UpdateMenu();
        break;
    case GV_MENUID(110): // tB^Fv90x
        if (m_oConfig.Num(L"FilterRot2") == 1) {
            m_oConfig.Num(L"FilterRot2", 0);
        } else {
            m_oConfig.Num(L"FilterRot2", 1);
        }
        m_pDocument->Reload();
        UpdateMenu();
        break;
    case GV_MENUID(111): // tB^Fv90x
        if (m_oConfig.Num(L"FilterRot2") == 2) {
            m_oConfig.Num(L"FilterRot2", 0);
        } else {
            m_oConfig.Num(L"FilterRot2", 2);
        }
        m_pDocument->Reload();
        UpdateMenu();
        break;
    case GV_MENUID(41): // tB^Fx␳
        if (!m_oConfig.Bool(L"FilterLevelRev")) {
            CLevelRevDialog(*this, CRect(), true).Open(m_pWindow->GetHandle());
        } else {
            m_oConfig.Bool(L"FilterLevelRev", false);
        }
        m_pDocument->Reload();
        UpdateMenu();
        break;
    case GV_MENUID(115): // tB^FK}␳
        if (!m_oConfig.Bool(L"FilterGammaRev")) {
            CGammaRevDialog(*this, CRect(), true).Open(m_pWindow->GetHandle());
        } else {
            m_oConfig.Bool(L"FilterGammaRev", false);
        }
        m_pDocument->Reload();
        UpdateMenu();
        break;
    case GV_MENUID(117): // tB^F邳ƃRgXg
        if (!m_oConfig.Bool(L"FilterBC")) {
            CBrightContrastDialog(*this, CRect(), true).Open(m_pWindow->GetHandle());
        } else {
            m_oConfig.Bool(L"FilterBC", false);
        }
        m_pDocument->Reload();
        UpdateMenu();
        break;
    case GV_MENUID(43): // tB^FȈՃO[XP[
        if (m_oConfig.Num(L"FilterGray") == 1) {
            m_oConfig.Num(L"FilterGray", 0);
        } else {
            m_oConfig.Num(L"FilterGray", 1);
        }
        m_pDocument->Reload();
        UpdateMenu();
        break;
    case GV_MENUID(44): // tB^FO[XP[
        if (m_oConfig.Num(L"FilterGray") == 2) {
            m_oConfig.Num(L"FilterGray", 0);
        } else {
            m_oConfig.Num(L"FilterGray", 2);
        }
        m_pDocument->Reload();
        UpdateMenu();
        break;
    case GV_MENUID(45): // tB^Fڂ
        if (m_oConfig.Num(L"FilterSharp") == 1) {
            m_oConfig.Num(L"FilterSharp", 0);
        } else {
            m_oConfig.Num(L"FilterSharp", 1);
        }
        m_pDocument->Reload();
        UpdateMenu();
        break;
    case GV_MENUID(91): // tB^Fڂ()
        if (m_oConfig.Num(L"FilterSharp") == 3) {
            m_oConfig.Num(L"FilterSharp", 0);
        } else {
            m_oConfig.Num(L"FilterSharp", 3);
        }
        m_pDocument->Reload();
        UpdateMenu();
        break;
    case GV_MENUID(152): // tB^FKEXڂ
        if (!m_oConfig.Bool(L"FilterGauss")) {
            CGaussDialog(*this, CRect(), true, true).Open(m_pWindow->GetHandle());
        } else {
            m_oConfig.Bool(L"FilterGauss", false);
        }
        m_pDocument->Reload();
        UpdateMenu();
        break;
    case GV_MENUID(153): // tB^FAV[v}XN
        if (!m_oConfig.Bool(L"FilterUnsharp")) {
            CGaussDialog(*this, CRect(), true, false).Open(m_pWindow->GetHandle());
        } else {
            m_oConfig.Bool(L"FilterUnsharp", false);
        }
        m_pDocument->Reload();
        UpdateMenu();
        break;
    case GV_MENUID(46): // tB^FV[v
        if (m_oConfig.Num(L"FilterSharp") == 2) {
            m_oConfig.Num(L"FilterSharp", 0);
        } else {
            m_oConfig.Num(L"FilterSharp", 2);
        }
        m_pDocument->Reload();
        UpdateMenu();
        break;
    case GV_MENUID(92): // tB^FV[v()
        if (m_oConfig.Num(L"FilterSharp") == 4) {
            m_oConfig.Num(L"FilterSharp", 0);
        } else {
            m_oConfig.Num(L"FilterSharp", 4);
        }
        m_pDocument->Reload();
        UpdateMenu();
        break;
    case GV_MENUID(113): // tB^FfBAtB^
        m_oConfig.Bool(L"FilterMedian", !m_oConfig.Bool(L"FilterMedian"));
        m_pDocument->Reload();
        UpdateMenu();
        break;
    case GV_MENUID(112): // tB^FF̔]
        m_oConfig.Bool(L"FilterNeg", !m_oConfig.Bool(L"FilterNeg"));
        m_pDocument->Reload();
        UpdateMenu();
        break;
    case GV_MENUID(137): // tB^F`l̔f
        if (!m_oConfig.Bool(L"FilterAlpha")) {
            CAlphaDialog(*this, CRect(), true).Open(m_pWindow->GetHandle());
        } else {
            m_oConfig.Bool(L"FilterAlpha", false);
        }
        m_pDocument->Reload();
        UpdateMenu();
        break;

    case GV_MENUID(103): // t@CXg̕\
        if (m_pFileListWindow->IsVisible()) {
            m_pFileListWindow->Hide();
        } else {
            m_pFileListWindow->Show(m_pWindow->GetHandle());
        }
        m_pWindow->SetForeground();
        m_pWindow->PostMsg(WM_APP + 0, 0, 0);
        break;
    case GV_MENUID(47): // Ỏ摜
        m_pDocument->ToPrior();
        break;
    case GV_MENUID(48): // ̉摜
        m_pDocument->ToNext();
        break;
    case GV_MENUID(49): // 1y[WỎ摜
        m_pDocument->ToPrior(1);
        break;
    case GV_MENUID(50): // 1y[W̉摜
        m_pDocument->ToNext(1);
        break;
    case GV_MENUID(51): // 5y[WỎ摜
        m_pDocument->ToPrior(5);
        break;
    case GV_MENUID(52): // 5y[W̉摜
        m_pDocument->ToNext(5);
        break;
    case GV_MENUID(53): // 50y[WỎ摜
        m_pDocument->ToPrior(50);
        break;
    case GV_MENUID(54): // 50y[W̉摜
        m_pDocument->ToNext(50);
        break;
    case GV_MENUID(93): // ÕtH_
        m_pDocument->ToPriorDir();
        break;
    case GV_MENUID(94): // ̃tH_
        m_pDocument->ToNextDir();
        break;
    case GV_MENUID(118): // Õ}[N摜
        m_pDocument->ToPriorMarkFile();
        break;
    case GV_MENUID(119): // ̃}[N摜
        m_pDocument->ToNextMarkFile();
        break;
    case GV_MENUID(55): // ŏ
        m_pDocument->ToFirst();
        break;
    case GV_MENUID(140): // Ō
        m_pDocument->ToLast();
        break;
    case GV_MENUID(56): // y[W
        if (m_pDocument->IsReady()) {
            if (m_bSlideShow) {
                m_pWindow->UnsetTimer(100);
                m_bSlideShow = false;
            } else {
                CSlideShowDialog oDialog(*this);
                oDialog.Open(m_pWindow->GetHandle());
                if (oDialog.IsOk()) {
                    m_pWindow->SetTimer(100, m_oConfig.Num(L"SlideShowFreq"));
                    m_bSlideShow = true;
                }
            }
            UpdateMenu();
        }
        break;
    case GV_MENUID(57): m_pDocument->Sort(0); break; // Xg\[g
    case GV_MENUID(129): m_pDocument->Sort(1); break; // 啶ʖŃ\[g
    case GV_MENUID(130): m_pDocument->Sort(2); break; // tH_Ń\[g
    case GV_MENUID(131): m_pDocument->Sort(3); break; // tH_/啶ʖŃ\[g
    case GV_MENUID(132): m_pDocument->Sort(4); break; // t@CTCYŃ\[g
    case GV_MENUID(133): m_pDocument->Sort(5); break; // ŏIXVŃ\[g
    case GV_MENUID(138): m_pDocument->Sort(6); break; // lŃ\[g
    case GV_MENUID(58): m_pDocument->Shuffle(); break; // XgVbt
    case GV_MENUID(134): m_pDocument->ShuffleDir(); break; // tH_̏Vbt
    case GV_MENUID(59): // ꎞIɃ\[gđỎ摜
        m_pDocument->ToSortedPrior();
        break;
    case GV_MENUID(60): // ꎞIɃ\[gđỎ摜
        m_pDocument->ToSortedNext();
        break;
    case GV_MENUID(62): // ubN}[N̕ۑ
        if (m_pDocument->IsReady()) {
            m_oChooseFile.SetTitle(L"ubN}[No_CAO");
            m_oChooseFile.ClearNameFilter();
            m_oChooseFile.AddNameFilter(L"GVB File", L"*.gvb");
            m_oChooseFile.SetDefaultExt(L"gvb");
            if (m_pDocument->GetBookmarkFileName().IsEmpty()) {
                CFileNameModuleDir oBookmarkDir(m_oConfig.File(L"BookmarkDir"));
                CFile(oBookmarkDir).CreateDir();
                m_oChooseFile.SetCurrentDir(oBookmarkDir);
                m_oChooseFile.SetDefault(m_pDocument->GetFileList().GetBookmarkDefaultName());
            } else {
                m_oChooseFile.SetCurrentDir(CFileName(m_pDocument->GetBookmarkFileName().GetDir()));
                m_oChooseFile.SetDefault(m_pDocument->GetBookmarkFileName().GetPrefix());
            }
            if (m_oChooseFile.ShowSave()) {
                m_pDocument->SaveBookmark(m_oChooseFile.GetResult());
            }
        }
        break;
    case GV_MENUID(63): // ubN}[N̓ǂݍ
        {
            m_oChooseFile.SetTitle(L"ubN}[Nǂݍނ_CAO");
            m_oChooseFile.ClearNameFilter();
            m_oChooseFile.AddNameFilter(L"GVB File", L"*.gvb");
            m_oChooseFile.SetDefaultExt(L"gvb");
            if (m_pDocument->GetBookmarkFileName().IsEmpty()) {
                CFileNameModuleDir oBookmarkDir(m_oConfig.File(L"BookmarkDir"));
                CFile(oBookmarkDir).CreateDir();
                m_oChooseFile.SetCurrentDir(oBookmarkDir);
                m_oChooseFile.SetDefault(string());
            } else {
                m_oChooseFile.SetCurrentDir(CFileName(m_pDocument->GetBookmarkFileName().GetDir()));
                m_oChooseFile.SetDefault(m_pDocument->GetBookmarkFileName().GetPureName());
            }
            if (m_oChooseFile.ShowOpen()) {
                m_pDocument->LoadBookmark(m_oChooseFile.GetResult());
            }
        }
        break;
    case GV_MENUID(121): // ubN}[NGfB^ŊJ
        {
            m_oChooseFile.SetTitle(L"ubN}[NGfB^ŊJ_CAO");
            m_oChooseFile.ClearNameFilter();
            m_oChooseFile.AddNameFilter(L"GVB File", L"*.gvb");
            m_oChooseFile.SetDefaultExt(L"gvb");
            if (m_pDocument->GetBookmarkFileName().IsEmpty()) {
                CFileNameModuleDir oBookmarkDir(m_oConfig.File(L"BookmarkDir"));
                CFile(oBookmarkDir).CreateDir();
                m_oChooseFile.SetCurrentDir(oBookmarkDir);
                m_oChooseFile.SetDefault(string());
            } else {
                m_oChooseFile.SetCurrentDir(CFileName(m_pDocument->GetBookmarkFileName().GetDir()));
                m_oChooseFile.SetDefault(m_pDocument->GetBookmarkFileName().GetPureName());
            }
            if (m_oChooseFile.ShowOpen()) {
                // m_pDocument->SetBookmarkFileName(); ƂăubN}[Nt@CoH
                // .txt̊֘AtŊJ
                Process::Start(CSystem::FindExec(L"txt"), m_oChooseFile.GetResult().Get());
            }
        }
        break;
    case GV_MENUID(88): // g勾\
        if (m_pDocument->IsReadyImage()) {
            m_pMagnifierWindow->Show();
        }
        break;
    case GV_MENUID(89): // 摜\
        if (m_pDocument->IsReady()) {
            CFileName o = m_pDocument->GetCurrentFileName();
            CRect rc = m_pDocument->ClientToImage(GetSelectedRect());
            COLORREF rgb = m_pDocument->GetAverageColor(GetSelectedRect());
            vector<string> vct;
            vct.push_back(o.GetDir());
            vct.push_back(o.GetPureName());
            vct.push_back(string::Format(L"%d / %d (%d)",
                m_pDocument->GetFileIndex() + 1, m_pDocument->GetFileCount(), m_pDocument->GetMarkCount()));
            vct.push_back(string::Format(L"%d (KB)", static_cast<int>(m_pDocument->GetCurrentFileSize() / 1024)));
            vct.push_back(m_pDocument->GetCurrentLastUpdate());
            vct.push_back(m_pDocument->GetImageTypeName());
            vct.push_back(string::Format(L"%d x %d", m_pDocument->GetImageSize()));
            vct.push_back(FtoA(int(m_pDocument->GetLayout()->GetRateV() * 1000) / 10.f) + L"%");
            vct.push_back(string::Format(L"(%d, %d) - [%d x %d]", rc.left, rc.top, rc.GetWidth(), rc.GetHeight()));
            vct.push_back(string::Format(L"%d", m_pDocument->GetColorCount(GetSelectedRect())));
            vct.push_back(string::Format(L"R, G, B = %d, %d, %d (#rrggbb = #%02x%02x%02x)",
                GetRValue(rgb), GetGValue(rgb), GetBValue(rgb),
                GetRValue(rgb), GetGValue(rgb), GetBValue(rgb)));
            vct.push_back(m_pDocument->GetFileMD5());
            /* ǉ\
                t@CSHA1H
                pXʂꍇ̃pX[h
            */
            CImageInfoDialog oDialog;
            oDialog.SetText(vct);
            oDialog.Open(m_pWindow->GetHandle());
        }
        break;

    case GV_MENUID(64): ConfigDialog(0); break; // \ݒ
    case GV_MENUID(122): ConfigDialog(1); break; // 摜ݒ
    case GV_MENUID(142): ConfigDialog(2); break; // `ݒ
    case GV_MENUID(143): ConfigDialog(3); break; // Xgݒ
    case GV_MENUID(123): ConfigDialog(4); break; // Sʐݒ
    case GV_MENUID(66): ConfigDialog(5); break; // vOCݒ
    case GV_MENUID(128): ConfigDialog(6); break; // Or[Aݒ
    case GV_MENUID(65): // ݒ
        CEnvConfigDialog(m_oConfig).Open(m_pWindow->GetHandle());
        m_oConfig.Save(CFileNameModuleDir(m_strConfigPrefix + L".cfg"));
        ApplyConfigData();
        m_pWindow->SetForeground();
        break;
    case GV_MENUID(67): // L[ݒ
        m_pKeyConfig->ShowDialog();
        // j[o[XV
        m_pKeyConfig->UpdateMenubar(m_pScreenChanger->GetMenu());
        break;
    case GV_MENUID(68): // J\
        if (m_oConfig.Num(L"ImageSpreadView_") != 1) {
            m_oConfig.Num(L"ImageSpreadView_", 1);
        } else {
            m_oConfig.Num(L"ImageSpreadView_", 0);
            if (m_pDocument->IsViewSpreaded()) {
                if (m_oConfig.Bool(L"NextOnCancelSpread")) {
                    m_pDocument->ToNext(1, false);
                }
            }
        }
        UpdateMenu();
        m_pDocument->OnChangeConfig(false);
        m_pDocument->Reload();
        break;
    case GV_MENUID(102): // J\(Ԃ)
        if (m_oConfig.Num(L"ImageSpreadView_") != 2) {
            m_oConfig.Num(L"ImageSpreadView_", 2);
        } else {
            m_oConfig.Num(L"ImageSpreadView_", 0);
            if (m_pDocument->IsViewSpreaded()) {
                if (m_oConfig.Bool(L"NextOnCancelSpread")) {
                    m_pDocument->ToNext(1, false);
                }
            }
        }
        UpdateMenu();
        m_pDocument->OnChangeConfig(false);
        m_pDocument->Reload();
        break;
    case GV_MENUID(70): // k\
        m_oConfig.Num(L"DrawStretchType", 0);
        UpdateMenu();
        m_pDocument->OnChangeConfig();
        break;
    case GV_MENUID(69): // kEg\
        m_oConfig.Num(L"DrawStretchType", 1);
        UpdateMenu();
        m_pDocument->OnChangeConfig();
        break;
    case GV_MENUID(144): // g\
        m_oConfig.Num(L"DrawStretchType", 2);
        UpdateMenu();
        m_pDocument->OnChangeConfig();
        break;
    case GV_MENUID(149): // \
        m_oConfig.Num(L"DrawStretchType", 3);
        UpdateMenu();
        m_pDocument->OnChangeConfig();
        break;
    case GV_MENUID(146): // Œ{\
        m_oConfig.Num(L"DrawStretchType", 4);
        UpdateMenu();
        m_pDocument->OnChangeConfig();
        break;
    case GV_MENUID(71): // ͂ݏő
        m_oConfig.Bool(L"DrawStickOut", !m_oConfig.Bool(L"DrawStickOut"));
        UpdateMenu();
        m_pDocument->OnChangeConfig();
        break;
    case GV_MENUID(147): // {0.7{
        m_oConfig.Num(L"DrawStretchType", 4); // Œ{\ɕύX
        m_oConfig.Float(L"DrawStretchRate", Max(m_oConfig.Float(L"DrawStretchRate") / sqrt(2.0), 1.0 / 16));
        UpdateMenu();
        m_pDocument->OnChangeConfig();
        break;
    case GV_MENUID(148): // {1.4{
        m_oConfig.Num(L"DrawStretchType", 4); // Œ{\ɕύX
        m_oConfig.Float(L"DrawStretchRate", Min(m_oConfig.Float(L"DrawStretchRate") * sqrt(2.0), 8.0));
        UpdateMenu();
        m_pDocument->OnChangeConfig();
        break;
    case GV_MENUID(145): // {1.0ɂ
        m_oConfig.Num(L"DrawStretchType", 4); // Œ{\ɕύX
        m_oConfig.Float(L"DrawStretchRate", 1.0);
        UpdateMenu();
        m_pDocument->OnChangeConfig();
        break;
    case GV_MENUID(101): // ]t
        m_oConfig.Bool(L"Padding", !m_oConfig.Bool(L"Padding"));
        UpdateMenu();
        m_pDocument->OnChangeConfig();
        break;
    case GV_MENUID(72): // J[\B
        m_oConfig.Bool(L"FullScreenCursorHide", !m_oConfig.Bool(L"FullScreenCursorHide"));
        UpdateMenu();
        m_pCursorHider->SetForceHide(m_pScreenChanger->IsFullScreen() &&
            m_oConfig.Bool(L"FullScreenCursorHide"));
        break;
    case GV_MENUID(73): // j[o[Ot
        if (m_pScreenChanger->GetMenu() == NULL) {
            m_pScreenChanger->SetMenu(appWindowMenu->GetHandle());
            m_pKeyConfig->UpdateMenubar(m_pScreenChanger->GetMenu());
            UpdateMenu();
        } else {
            m_pScreenChanger->UnsetMenu();
        }
        break;
    case GV_MENUID(141): // ɎOɕ\
        m_oConfig.Bool(L"WindowTopMost", !m_oConfig.Bool(L"WindowTopMost"));
        UpdateMenu();
        m_pScreenChanger->SetTopMost(m_oConfig.Bool(L"WindowTopMost"));
        break;
    case GV_MENUID(74): // Sʕ\؂ւ
        if (m_pScreenChanger->IsFullScreen()) {
            m_pScreenChanger->SetSize(m_rcWindow.GetSize());
            m_pScreenChanger->ToWindowMode();
        } else {
            m_pScreenChanger->SetSize(CDisplay::GetCurrentMode().oSize);
            m_pScreenChanger->SetDefaultBpp();
            m_pScreenChanger->ToFullScreen();
        }
        m_pCursorHider->SetForceHide(m_pScreenChanger->IsFullScreen() &&
            m_oConfig.Bool(L"FullScreenCursorHide"));
        break;
    case GV_MENUID(75): // tXN[؂ւ
        if (m_pScreenChanger->IsFullScreen()) {
            m_pScreenChanger->SetSize(m_rcWindow.GetSize());
            m_pScreenChanger->ToWindowMode();
        } else {
            m_pScreenChanger->SetSize(m_oConfig.Size(L"FullScreen"));
            m_pScreenChanger->SetBpp(m_oConfig.Num(L"FullScreenBpp"));
            m_pScreenChanger->ToFullScreen();
        }
        m_pCursorHider->SetForceHide(m_pScreenChanger->IsFullScreen() &&
            m_oConfig.Bool(L"FullScreenCursorHide"));
        break;
    case GV_MENUID(79): // st@ĈtH_J
        Process::Start(CFileNameModuleDir().Get());
        break;
    case GV_MENUID(80): // ubN}[ÑtH_J
        Process::Start(CFileNameModuleDir(m_oConfig.File(L"BookmarkDir")).Get());
        break;
    case GV_MENUID(81): // SPĨtH_J
        Process::Start(CFileNameModuleDir(m_oConfig.File(L"SpiPluginsDir")).Get());
        break;
    case GV_MENUID(82): // ꎞtH_J
        Process::Start(CFileName(L"$т$", CFileNameModuleDir(m_oConfig.File(L"TmpDir"))).Get());
        break;
    case GV_MENUID(83): // ǎۑtH_J
        Process::Start(CFileNameModuleDir(m_oConfig.File(L"WallpaperDir")).Get());
        break;


    case GV_MENUID(84): // т.txtJ
        Process::Start(CFileNameModuleDir(L"т.txt").Get());
        break;
    case GV_MENUID(85): // o[W
        {
            string env;
            env += string::Format(L"󂫃:\t%dMB\r\n", m_pDocument->GetNowFreeMemory() / 1024 / 1024);
            env += string::Format(L"LbV:\t%d\r\n", m_pDocument->GetNowCacheCount());
            env += string::Format(L"kLbV:\t%s\r\n", m_pDocument->IsNowStretchCaching() ? L"ON" : L"OFF");
            env += string::Format(L"̧كLbV:\t%s\r\n", m_pDocument->IsNowLoaderCaching() ? L"ON" : L"OFF");
            CVersionDialog(env).Open(rPrms.GetWnd());
        }
        break;
    case GV_MENUID(135): // ҂փ[𑗂
        Process::Start(L"http://tqzh.tk:24497/site/mail/");
        break;
    case GV_MENUID(86): // zzTCgJ
        Process::Start(L"http://hp.vector.co.jp/authors/VA039571/gv2/");
        break;
    case GV_MENUID(90): // wv
        Process::Start(L"http://hp.vector.co.jp/authors/VA039571/gv2/help/");
        break;
    case GV_MENUID(87): // ˂Ƃ킭XV
        {
            CNetworkUpdater oUpdater(*m_pWindow, m_oConfig);
            oUpdater.Open(rPrms.GetWnd());
            if (oUpdater.IsUpdate()) {
                m_pWindow->Close();
            }
        }
        break;

    default:
        return false;
    }
    return true;
}

// t@CJB
void CAppWindow::OpenFiles(const vector<CFileName>& vct, bool dragdrop) {
    /*
    std::tr1::shared_ptr<MsgDialog> dialog(new MsgDialog());
    dialog->OpenModelessWithThread(m_pWindow->GetHandle());
    dialog->SetCaption(L"т : t@CXg̍\z...");
    /*/
    std::tr1::shared_ptr<MsgDialog> dialog;
    //*/
    // JB
    if (m_oConfig.Bool(L"FileAddMode")) {
        m_pDocument->Open(vct, dialog.get());
    } else {
        if (DeleteAllMarkedOrClear()) {
            m_pDocument->Clear();
            m_pDocument->Open(vct, dialog.get());
        }
    }
    // JgfBNg̐ݒBƂ肠OԖڂ̂B
    if (dragdrop && !m_oConfig.Bool(L"CDOnDrop")) {
        // D&DCDOnDrop == falseȂ珈ȂB
    } else {
        if (!vct.empty()) {
            m_oConfig.File(L"CurrentDir", vct[0].GetDir());
        }
    }
}

// ۑ̃_CAOɃZbgftHg̃t@C擾
string CAppWindow::GetSaveDefaultName() {
    string str;
    std::tr1::shared_ptr<CFileInfo> fi;
    if (m_pDocument->IsReady()) {
        fi = m_pDocument->GetFileInfoClone(m_pDocument->GetFileIndex());
        if (fi->IsArchiveFile()) {
            if (m_oConfig.Bool(L"SaveNameUseArchive")) {
                CFileName name = fi->GetExistFileName();
                str += name.GetPrefix();
                str += L"\\";
            }
            if (m_oConfig.Bool(L"SaveNameUseArchiveDir")) {
                str += fi->GetViewName().GetDir().Substring(fi->GetExistFileName().Get().GetLength() + 1);
                //str += L"\\";
            }
        }
    }
    str = str
        .Replace(L'/', L'\\')
        .Replace(L"\\\\", L"\\")
        .Replace(L"\\", m_oConfig.Str(L"SaveNameSeparator"));
    return str + m_pDocument->GetCurrentFileName().GetPrefix();
}

// ^C}[ɂĒIɃR[obN֐
void CAppWindow::OnTimer() {
    bool bStop = false;
    int nOldIndex = m_pDocument->GetFileIndex();
    if (m_oConfig.Num(L"SlideShowDist") == 0) {
        // ̃t@C
        m_pDocument->ToNext();
        if (m_oConfig.Bool(L"SlideShowStopWithLast")) {
            if (m_pDocument->IsLastFile()) {
                // Ō܂ōsI
                bStop = true;
            } else if (m_pDocument->GetFileIndex() < nOldIndex) {
                // Ȃ̂ɌĂ(s߂ۂ)ꍇ
                bStop = true;
            }
        }
    } else {
        // Õt@C
        m_pDocument->ToPrior();
        if (m_oConfig.Bool(L"SlideShowStopWithLast")) {
            if (m_pDocument->IsFirstFile()) {
                // ŏ܂ōsI
                bStop = true;
            } else if (nOldIndex < m_pDocument->GetFileIndex()) {
                // tȂ̂ɑĂ(s߂ۂ)ꍇ
                bStop = true;
            }
        }
    }
    if (bStop) {
        m_pWindow->UnsetTimer(100);
        m_bSlideShow = false;
        UpdateMenu();
    }
}

// RtBO_CAO̕\
void CAppWindow::ConfigDialog(int nPage) {
    CConfigDialog(m_oConfig, *m_pSpi, nPage).Open(m_pWindow->GetHandle());
    ApplyConfigData();
    m_pWindow->SetForeground();
}

void CAppWindow::ApplyConfigData() {
    m_oConfig.Save(CFileNameModuleDir(m_strConfigPrefix + L".cfg"));

    //LoaderFactory
    m_pSpi->SetDir(CFileNameModuleDir(m_oConfig.File(L"SpiPluginsDir")));
    typedef PriorityObject<IFactory<IImageLoader> > ILO;
    std::multiset<ILO> oILFL;
    oILFL.insert(ILO(m_pJpgLoaderFactory, m_oConfig.Num(L"LoaderPriorityJ")));
    oILFL.insert(ILO(m_pPngLoaderFactory, m_oConfig.Num(L"LoaderPriorityP")));
    oILFL.insert(ILO(m_pBmpLoaderFactory, m_oConfig.Num(L"LoaderPriorityB")));
    oILFL.insert(ILO(m_pSpiLoaderFactory, m_oConfig.Num(L"LoaderPriorityS")));
    oILFL.insert(ILO(m_pArgbLoaderFactory, m_oConfig.Num(L"LoaderPriorityA")));
    oILFL.insert(ILO(m_pTgaLoaderFactory, m_oConfig.Num(L"LoaderPriorityT")));
    oILFL.insert(ILO(m_pOleLoaderFactory, m_oConfig.Num(L"LoaderPriorityO")));
    oILFL.insert(ILO(std::tr1::shared_ptr<IFactory<IImageLoader> >(), m_oConfig.Num(L"LoaderPriorityX")));
    CImageLoader::GetFactoryList().Clear();
    for (std::multiset<ILO>::iterator it = oILFL.begin(); it != oILFL.end(); ++it) {
        if ((*it).Get() == NULL) {
            break;
        } else {
            CImageLoader::GetFactoryList().Add(*it);
        }
    }

    //ArchiverFactory
    typedef PriorityObject<IFactory<IArchiver> > ALO;
    std::multiset<ALO> oALFL;
    oALFL.insert(ALO(m_pSpiArchiverFactory, m_oConfig.Num(L"ArchiverPriorityS")));
    oALFL.insert(ALO(m_pZipArchiverFactory, m_oConfig.Num(L"ArchiverPriorityZ")));
    oALFL.insert(ALO(m_pRarArchiverFactory, m_oConfig.Num(L"ArchiverPriorityR")));
    oALFL.insert(ALO(m_pLhaArchiverFactory, m_oConfig.Num(L"ArchiverPriorityL")));
    oALFL.insert(ALO(m_pZip2ArchiverFactory, m_oConfig.Num(L"ArchiverPriorityZL")));
    oALFL.insert(ALO(m_p7ZipArchiverFactory, m_oConfig.Num(L"ArchiverPriority7Z")));
    oALFL.insert(ALO(m_pGcaArchiverFactory, m_oConfig.Num(L"ArchiverPriorityG")));
    oALFL.insert(ALO(std::tr1::shared_ptr<IFactory<IArchiver> >(), m_oConfig.Num(L"ArchiverPriorityX")));
    CArchiver::GetFactoryList().Clear();
    for (std::multiset<ALO>::iterator it = oALFL.begin(); it != oALFL.end(); ++it) {
        if ((*it).Get() == NULL) {
            break;
        } else {
            CArchiver::GetFactoryList().Add(*it);
        }
    }

    //Document
    m_pDocument->OnChangeConfig();
    m_pDocument->SetScreen(m_pWindow->GetClientSize());

    //̃EBhEƂ
    Themis_DebugAssert(m_pScreenChanger.get() != NULL);
    Themis_DebugAssert(m_pCursorHider.get() != NULL);
    Themis_DebugAssert(m_pMagnifierWindow.get() != NULL);
    m_pCursorHider->SetEnable(m_oConfig.Bool(L"AutoCursorHide"));
    m_pCursorHider->SetTimeout(m_oConfig.Num(L"AutoCursorHideTime"));
    if (m_pScreenChanger->IsFullScreen()) {
        if (m_oConfig.Bool(L"FullScreenCursorHide")) {
            m_pCursorHider->ForceHide();
        } else {
            m_pCursorHider->UnforceHide();
        }
    } else {
        m_pCursorHider->UnforceHide();
    }
    m_pScreenChanger->SetMinimizeOnDeactivate(m_oConfig.Bool(L"FullScreenMinOnDeactive"));
    m_pScreenChanger->SetTopMost(m_oConfig.Bool(L"WindowTopMost"));
    m_pMagnifierWindow->SetSize(m_oConfig.Size(L"Magnifier"));
    m_pMagnifierWindow->SetShape(m_oConfig.Num(L"MagnifierShape"));
    if (m_oConfig.Bool(L"Snap")) {
        snapper.reset(new CWindowSnapper());
        snapper->SetSnapWorkArea(true);
        snapper->SetSnapDist(32);
    } else {
        snapper.reset();
    }
}

// j[o[̕\XV
void CAppWindow::UpdateMenu() {
    CMenu oMenu(m_pScreenChanger->GetMenu());

    oMenu.SetCheck(GV_MENUID(107), m_oConfig.Num(L"FilterRot1") == 1);
    oMenu.SetCheck(GV_MENUID(108), m_oConfig.Num(L"FilterRot1") == 2);
    oMenu.SetCheck(GV_MENUID(109), m_oConfig.Num(L"FilterRot1") == 3);
    oMenu.SetCheck(GV_MENUID(110), m_oConfig.Num(L"FilterRot2") == 1);
    oMenu.SetCheck(GV_MENUID(111), m_oConfig.Num(L"FilterRot2") == 2);
    oMenu.SetCheck(GV_MENUID(41), m_oConfig.Bool(L"FilterLevelRev"));
    oMenu.SetCheck(GV_MENUID(115), m_oConfig.Bool(L"FilterGammaRev"));
    oMenu.SetCheck(GV_MENUID(117), m_oConfig.Bool(L"FilterBC"));
    oMenu.SetCheck(GV_MENUID(43), m_oConfig.Num(L"FilterGray") == 1);
    oMenu.SetCheck(GV_MENUID(44), m_oConfig.Num(L"FilterGray") == 2);
    oMenu.SetCheck(GV_MENUID(45), m_oConfig.Num(L"FilterSharp") == 1);
    oMenu.SetCheck(GV_MENUID(46), m_oConfig.Num(L"FilterSharp") == 2);
    oMenu.SetCheck(GV_MENUID(91), m_oConfig.Num(L"FilterSharp") == 3);
    oMenu.SetCheck(GV_MENUID(92), m_oConfig.Num(L"FilterSharp") == 4);
    oMenu.SetCheck(GV_MENUID(152), m_oConfig.Bool(L"FilterGauss"));
    oMenu.SetCheck(GV_MENUID(153), m_oConfig.Bool(L"FilterUnsharp"));
    oMenu.SetCheck(GV_MENUID(113), m_oConfig.Bool(L"FilterMedian"));
    oMenu.SetCheck(GV_MENUID(112), m_oConfig.Bool(L"FilterNeg"));
    oMenu.SetCheck(GV_MENUID(137), m_oConfig.Bool(L"FilterAlpha"));
    bool bFilter = m_oConfig.Bool(L"Filter");
    oMenu.SetCheck(GV_MENUID(95), bFilter);
    oMenu.SetEnable(GV_MENUID(96), bFilter); // TCY
    oMenu.SetEnable(GV_MENUID(107), bFilter); // ]
    oMenu.SetEnable(GV_MENUID(108), bFilter); // ]
    oMenu.SetEnable(GV_MENUID(109), bFilter); // ]
    oMenu.SetEnable(GV_MENUID(110), bFilter); // ]
    oMenu.SetEnable(GV_MENUID(111), bFilter); // ]
    oMenu.SetEnable(GV_MENUID(41), bFilter); // x␳
    oMenu.SetEnable(GV_MENUID(115), bFilter); // K}␳
    oMenu.SetEnable(GV_MENUID(117), bFilter); // 邳ƃRgXg
    oMenu.SetEnable(GV_MENUID(43), bFilter); // O[XP[
    oMenu.SetEnable(GV_MENUID(44), bFilter); // O[XP[
    oMenu.SetEnable(GV_MENUID(45), bFilter); // ڂ
    oMenu.SetEnable(GV_MENUID(91), bFilter); // ڂ
    oMenu.SetEnable(GV_MENUID(46), bFilter); // V[v
    oMenu.SetEnable(GV_MENUID(92), bFilter); // V[v
    oMenu.SetEnable(GV_MENUID(152), bFilter); // KEX
    oMenu.SetEnable(GV_MENUID(153), bFilter); // AV[v
    oMenu.SetEnable(GV_MENUID(113), bFilter); // fBAtB^
    oMenu.SetEnable(GV_MENUID(112), bFilter); // ]
    oMenu.SetEnable(GV_MENUID(137), bFilter); // `l̔f

    oMenu.SetCheck(GV_MENUID(56), m_bSlideShow);

    oMenu.SetCheck(GV_MENUID(68), m_oConfig.Num(L"ImageSpreadView_") == 1);
    oMenu.SetCheck(GV_MENUID(102), m_oConfig.Num(L"ImageSpreadView_") == 2);
    oMenu.SetCheck(GV_MENUID(70), m_oConfig.Num(L"DrawStretchType") == 0);
    oMenu.SetCheck(GV_MENUID(69), m_oConfig.Num(L"DrawStretchType") == 1);
    oMenu.SetCheck(GV_MENUID(144), m_oConfig.Num(L"DrawStretchType") == 2);
    oMenu.SetCheck(GV_MENUID(149), m_oConfig.Num(L"DrawStretchType") == 3);
    oMenu.SetCheck(GV_MENUID(146), m_oConfig.Num(L"DrawStretchType") == 4);
    oMenu.SetCheck(GV_MENUID(71), m_oConfig.Bool(L"DrawStickOut"));
    oMenu.SetCheck(GV_MENUID(101), m_oConfig.Bool(L"Padding"));
    oMenu.SetCheck(GV_MENUID(72), m_oConfig.Bool(L"FullScreenCursorHide"));
    oMenu.SetCheck(GV_MENUID(141), m_oConfig.Bool(L"WindowTopMost"));

    m_pScreenChanger->DrawMenubar();
}

// LZȂ߂lfalse
bool CAppWindow::DeleteAllMarkedOrClear() {
    if (0 < m_pDocument->GetMarkCount()) {
        if (MessageBox::Show(m_pWindow->GetHandle(), 
            ItoA(m_pDocument->GetMarkCount()) +  L" ̃}[Nt@C܂B\r\nsꍇASẴ}[NNA܂",
            L"mF", MessageBoxButtons::OKCancel, MessageBoxIcon::Information) == DialogResult::OK) {
            m_pDocument->ClearMarks();
        } else {
            return false;
        }
    }
    return true;
}

// CAEgύXꂽCxg
void CAppWindow::LayoutChanged() {
//*
    if (hScrollBar.get() != NULL && vScrollBar.get() != NULL) {
        CSize clientSize = m_pWindow->GetClientSize();
        CSize ranges = m_pDocument->GetLayout()->GetScrollRanges();
        CPoint pos = m_pDocument->GetLayout()->GetScrollPos();
        bool disableNoScrooll = 
            m_oConfig.Num(L"DrawStretchType") <= 2 && m_oConfig.Bool(L"DrawStickOut");
        // gEkł͂ݏoAȂXN[o[͋Iɏɕ\B
        try {
            hScrollBar->SuspendLayout();
            hScrollBar->SetMinimum(0);
            hScrollBar->SetMaximum(ranges.cx);
            hScrollBar->SetLargeChange(clientSize.cx);
            hScrollBar->SetSmallChange(hScrollBar->GetLargeChange() / 8);
            hScrollBar->SetValue(pos.x);
            hScrollBar->SetDisableNoScroll(disableNoScrooll);
            hScrollBar->SetVisible(disableNoScrooll ? true : 0 < ranges.cx);
            hScrollBar->SetEnabled(disableNoScrooll ? 0 < ranges.cx : true);
            hScrollBar->ResumeLayout();
        } catch (ScrollBarAlreadyResumedLayoutException&) {
            // Ȃ񂩍ċAĂ镗(L`)
        }
        try {
            vScrollBar->SuspendLayout();
            vScrollBar->SetMinimum(0);
            vScrollBar->SetMaximum(ranges.cy);
            vScrollBar->SetLargeChange(clientSize.cy);
            vScrollBar->SetSmallChange(vScrollBar->GetLargeChange() / 8);
            vScrollBar->SetValue(pos.y);
            vScrollBar->SetDisableNoScroll(disableNoScrooll);
            vScrollBar->SetVisible(disableNoScrooll ? true : 0 < ranges.cy);
            vScrollBar->SetEnabled(disableNoScrooll ? 0 < ranges.cy : true);
            vScrollBar->ResumeLayout();
        } catch (ScrollBarAlreadyResumedLayoutException&) {
            // Ȃ񂩍ċAĂ镗(L`)
        }
    }
//*/
}

void CAppWindow::OnUpdateDocument(const CRect& rc) {
    if (m_pWindow.get() != NULL && m_pWindow->Exists()) {
        // ̃^Cg̐ݒ
        SetCaption();
        // EBhEĕ`
        if (rc.IsValid()) {
            m_pWindow->Update(rc);
        } else {
            m_pWindow->Update();
        }
    }
}

/// XN[̏
void CAppWindow::OnHScroll(ScrollBar& sender, int oldValue, int newValue) {
    Themis_DebugAssert(hScrollBar->GetValue() == newValue);
    int x = Min(Max(hScrollBar->GetValue(), hScrollBar->GetMinimum()), hScrollBar->GetMaximum());
    int y = Min(Max(vScrollBar->GetValue(), vScrollBar->GetMinimum()), vScrollBar->GetMaximum());
    m_pDocument->SetScrollPos(CPoint(x, y));
}

/// XN[̏
void CAppWindow::OnVScroll(ScrollBar& sender, int oldValue, int newValue) {
    Themis_DebugAssert(vScrollBar->GetValue() == newValue);
    int x = Min(Max(hScrollBar->GetValue(), hScrollBar->GetMinimum()), hScrollBar->GetMaximum());
    int y = Min(Max(vScrollBar->GetValue(), vScrollBar->GetMinimum()), vScrollBar->GetMaximum());
    m_pDocument->SetScrollPos(CPoint(x, y));
}

// ^Cg̐ݒ
void CAppWindow::SetCaption() {
    if (m_pWindow.get() != NULL && m_pWindow->Exists()) {
        vector<pair<string, string> > vct;
        CFileName oFile(m_pDocument->IsReady() ? m_pDocument->GetCurrentFileName().Get() : string());
        CSize szImage = m_pDocument->IsReady() ? m_pDocument->GetImageSize() : CSize(0, 0);
        int nFileSize = m_pDocument->IsReady() ? static_cast<int>(m_pDocument->GetCurrentFileSize() / 1024) : 0;
        CRect rc = m_pDocument->IsReady() ? m_pDocument->ClientToImage(GetSelectedRect()) : CRect();
        string strImgType = m_pDocument->IsReady() ? m_pDocument->GetImageTypeName() : string();
        vct.push_back(make_pair(L"$(dir)", CFileName::RemoveDirSeparatorForce(oFile.GetDir())));
        vct.push_back(make_pair(L"$(file)", oFile.GetPrefix()));
        vct.push_back(make_pair(L"$(ext)", oFile.GetSuffix()));
        vct.push_back(make_pair(L"$(name)", oFile.GetPureName()));
        vct.push_back(make_pair(L"$(full)", oFile.Get()));
        vct.push_back(make_pair(L"$(index)", ItoA(m_pDocument->GetFileIndex() + 1)));
        vct.push_back(make_pair(L"$(count)", ItoA(m_pDocument->GetFileCount())));
        vct.push_back(make_pair(L"$(marks)", ItoA(m_pDocument->GetMarkCount())));
        vct.push_back(make_pair(L"$(imgw)", ItoA(szImage.cx)));
        vct.push_back(make_pair(L"$(imgh)", ItoA(szImage.cy)));
        vct.push_back(make_pair(L"$(fsize)", ItoA(nFileSize)));
        vct.push_back(make_pair(L"$(imgtype)", strImgType));
        vct.push_back(make_pair(L"$(viewrate)", FtoA(int(m_pDocument->GetLayout()->GetRateV() * 1000) / 10.f) + L"%"));
        vct.push_back(make_pair(L"$(selx)", ItoA(rc.left)));
        vct.push_back(make_pair(L"$(sely)", ItoA(rc.top)));
        vct.push_back(make_pair(L"$(selw)", ItoA(rc.GetWidth())));
        vct.push_back(make_pair(L"$(selh)", ItoA(rc.GetHeight())));
        vct.push_back(make_pair(L"$(build)", GetVersionText()));
        string str = Replace(m_oConfig.Indirect(L"Titlebar"), vct);
#ifndef NDEBUG
        str = L"{DEBUG-BUILD} " + str;
#endif
        if (str != m_pWindow->GetText()) { // h~
            m_pWindow->SetText(str);
        }
    }
}

void CAppWindow::OpenExViewer(const string& strCfgPrefix) {
    if (m_pDocument->IsReady()) {
        CFileName oFileName = m_pDocument->GetCurrentExistFileName();
        vector<pair<string, string> > vct1, vct2;
        vct1.push_back(make_pair(L"$(dir)", CFileName::RemoveDirSeparatorForce(oFileName.GetDir())));
        vct1.push_back(make_pair(L"$(file)", oFileName.GetPrefix()));
        vct1.push_back(make_pair(L"$(ext)", oFileName.GetSuffix()));
        vct1.push_back(make_pair(L"$(name)", oFileName.GetPureName()));
        vct1.push_back(make_pair(L"$(full)", oFileName.Get()));
        vct2.push_back(vct1[0]);
        string strFile = CFileNameNoDir(m_oConfig.File(strCfgPrefix + L"File")).Get();
        string strArg = Replace(m_oConfig.Indirect(strCfgPrefix + L"Arg"), vct1);
        string strDir = Replace(m_oConfig.Indirect(strCfgPrefix + L"Dir"), vct2);
        ProcessStartInfo info(strFile, strArg);
        info.WorkingDirectory = CFileNameNoDir(strDir).Get();
        Process::Start(info);
    }
}

// Ï̎擾(NCAgW)
CRect CAppWindow::GetSelectedRect() {
    if (!rectSelector->IsSelected()) {
        return CRect();
    }
    return rectSelector->GetSelect();
}

void CAppWindow::OnPaint(PAINTSTRUCT& ps, const CSize& szDst, const CPoint& ptSrcCenter) {
    if (GetDocument().IsReadyImage()) {
        int nMagnification = m_oConfig.Num(L"Magnification");
        double rateH = 100.0f / nMagnification / m_pDocument->GetLayout()->GetRateH();
        double rateV = 100.0f / nMagnification / m_pDocument->GetLayout()->GetRateV();
        CSize szSrc = CSize(long(szDst.cx * rateH + 0.5), long(szDst.cy * rateV + 0.5)); //g勾ɕ\摜̑傫
        CPoint ptSrcCenter2 = GetDocument().GetLayout()->ScreenToDocument(ptSrcCenter); //g勾̒
        SimpleLayout layout;
        layout.SetScreen(szDst.ToRect()); // g勾S
        layout.SetDocument(m_pDocument->GetLayout()->GetDocument()); // 摜S
        layout.SetDocumentViewArea(CRect(ptSrcCenter2 - szSrc / 2, szSrc)); // 摜̕\̈
        GetDocument().DrawStretch(ps, layout.GetDst(), layout.GetSrc());
        CDC dc(ps.hdc);
        vector<CRect> vct = szDst.ToRect().GetBlanks(layout.GetDst());
        for (size_t i = 0, n = vct.size(); i < n; i++) {
            dc.FillBlack(vct[i]);
        }
        //ꉞŜĕ`
        m_pWindow->PostUpdate();
    }
}

int CAppWindow::GetVersion() {
    return c_nVersion;
}

string CAppWindow::GetVersionText() {
    return L"build-" + ItoA(GetVersion());
}

/// ɂ̃pX[h₢킹B
bool CAppWindow::QueryPassword(const CFileName& archiveName, string& pass, int tryCount) {
    CPassDialog oDialog(*this, archiveName);
    //oDialog.Open(m_pWindow->GetHandle());
    oDialog.Open(NULL);
    if (oDialog.IsOk()) {
        pass = oDialog.GetPass();
        return true;
    }
    return false;
}
