#pragma once
#include "MagnifierWindow.h"
#include "DocView.h"
#include <Themis/Factory.h>
#include <Themis/AppFrameBase.h>
#include <Themis/Window.h>
#include <Themis/CommDlg.h>
#include <Themis/Config.h>
#include <Themis/Archive.h>
namespace Themis {
class CScreenChanger;
class CCursorHider;
class IImageLoader;
class IArchiver;
class CSpi;
class CWindowSnapper;
class ScrollBar;
class HScrollBar;
class VScrollBar;
class CRectSelector;
}

class CKeyConfigEx;
class CDocument;
class CFileListWindow;
class AppWindowMenu;

class CAppWindow : private IMsgHooker, private CMagnifierWindow::IOwner, private IDocView, private IQueryArchivePassword {
public:
    CAppWindow(const string& strConfigPrefix);
    ~CAppWindow();

    int Run(const vector<CFileName>& vctFiles, int nCmdShow);

    const auto_ptr<CWindow>& GetWindow() const { return m_pWindow; }
    CDocument& GetDocument() { return *m_pDocument; }
    CConfig& GetConfig() { return m_oConfig; }
    CChooseFile& GetChooseFile() { return m_oChooseFile; }

    static int GetVersion();
    static string GetVersionText();

private:
    bool HookMsg(CMsgHookPrms& rPrms);
    bool OnCommand(CMsgHookPrms& rPrms);

    void OpenFiles(const vector<CFileName>& vct, bool dragdrop);
    string GetSaveDefaultName();

    void OnTimer();
    void ConfigDialog(int nPage);
    void ApplyConfigData();
    void UpdateMenu();
    bool DeleteAllMarkedOrClear();
    void OnOpenFile();

    void LayoutChanged();
    void OnUpdateDocument(const CRect& rc);

    void OnHScroll(ScrollBar& sender, int oldValue, int newValue);
    void OnVScroll(ScrollBar& sender, int oldValue, int newValue);

    void OnChangeImage();
    void SetCaption();
    void OpenExViewer(const string& strCfgPrefix);
    CRect GetSelectedRect();
    void OnPaint(PAINTSTRUCT& ps, const CSize& szDst, const CPoint& ptSrcCenter);
    /// ɂ̃pX[h₢킹B
    bool QueryPassword(const CFileName& archiveName, string& pass, int tryCount);

private:
    const string m_strConfigPrefix;

    CConfig m_oConfig;

    auto_ptr<CWindow> m_pWindow;
    auto_ptr<CScreenChanger> m_pScreenChanger;
    auto_ptr<CMagnifierWindow> m_pMagnifierWindow;
    auto_ptr<CCursorHider> m_pCursorHider;
    auto_ptr<CWindowSnapper> snapper;
    auto_ptr<AppWindowMenu> appWindowMenu;
    auto_ptr<HScrollBar> hScrollBar;
    auto_ptr<VScrollBar> vScrollBar;

    std::tr1::shared_ptr<CSpi> m_pSpi;
    std::tr1::shared_ptr<IFactory<IImageLoader> > m_pJpgLoaderFactory;
    std::tr1::shared_ptr<IFactory<IImageLoader> > m_pPngLoaderFactory;
    std::tr1::shared_ptr<IFactory<IImageLoader> > m_pBmpLoaderFactory;
    std::tr1::shared_ptr<IFactory<IImageLoader> > m_pSpiLoaderFactory;
    std::tr1::shared_ptr<IFactory<IImageLoader> > m_pArgbLoaderFactory;
    std::tr1::shared_ptr<IFactory<IImageLoader> > m_pTgaLoaderFactory;
    std::tr1::shared_ptr<IFactory<IImageLoader> > m_pOleLoaderFactory;
    std::tr1::shared_ptr<IFactory<IArchiver> > m_pZipArchiverFactory;
    std::tr1::shared_ptr<IFactory<IArchiver> > m_p7ZipArchiverFactory;
    std::tr1::shared_ptr<IFactory<IArchiver> > m_pRarArchiverFactory;
    std::tr1::shared_ptr<IFactory<IArchiver> > m_pLhaArchiverFactory;
    std::tr1::shared_ptr<IFactory<IArchiver> > m_pGcaArchiverFactory;
    std::tr1::shared_ptr<IFactory<IArchiver> > m_pSpiArchiverFactory;
    std::tr1::shared_ptr<IFactory<IArchiver> > m_pZip2ArchiverFactory;

    auto_ptr<CDocument> m_pDocument;
    auto_ptr<CKeyConfigEx> m_pKeyConfig;
    auto_ptr<CFileListWindow> m_pFileListWindow;

    auto_ptr<CRectSelector> rectSelector;

    CChooseFile m_oChooseFile;
    CChooseDir m_oChooseDir;
    CChooseColor m_oChooseColor;

    CFileName m_oBookmarkName;

    CRect m_rcWindow;

    bool m_bSlideShow;

    bool m_bPosRestoreDisable;
};




