#include "stdafx.h"
#include "ConfigDialog.h"
#include "AppWindow.h"
#include "resource.h"
#include <Themis/File.h>
#include <Themis/Tooltip.h>
#include <Themis/CommDlg.h>
#include <Themis/KeyTable.h>
#include <Themis/WndCtrlData.h>
#include <Themis/CtrlEnable.h>

class CConfigDialogTabBase : public CTabPage {
public:
    CConfigDialogTabBase(int nResourceId) : CTabPage(nResourceId) {}
    virtual void LoadFromConfig(CConfig& m_oConfig) = 0;
    virtual void SaveToConfig(CConfig& m_oConfig) = 0;
};

#include "ConfigDialogGUIPage.h"
#include "ConfigDialogImagePage.h"
#include "ConfigDialogDrawPage.h"
#include "ConfigDialogListPage.h"
#include "ConfigDialogWholePage.h"
#include "ConfigDialogPluginPage.h"
#include "ConfigDialogExternPage.h"

CConfigDialog::CConfigDialog(CConfig& m_oConfig, CSpi& r, int nDefaultPage)
: CResourceDialogBase(IDD_CFG), m_oConfig(m_oConfig), m_rSpi(r), m_nDefaultPage(nDefaultPage) {
}

CConfigDialog::~CConfigDialog() {
}

bool CConfigDialog::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_COMMAND) {
        switch (rPrms.GetWPrmLoWord()) {
        case IDOK:
            for (int i = 0, n = m_pTabCtrl->GetPageCount(); i < n; i++) {
                static_cast<CConfigDialogTabBase&>(m_pTabCtrl->GetPage(i)).SaveToConfig(m_oConfig);
            }
            m_pTabCtrl.reset();
            GetDialog().Close();
            break;
        case IDCANCEL:
            m_pTabCtrl.reset();
            GetDialog().Close();
            break;
        default:
            return false;
        }
        return true;
/*
    } else if (rPrms.GetMsg() == WM_KEYDOWN) {
        if (rPrms.GetWPrm() == VK_PRIOR) {
            if (CKeyTable::GetModifiers() == CKeyTable::eModCtrl) {
                int nPage = m_pTabCtrl->GetPage();
                m_pTabCtrl->SetPage(Max(nPage - 1, 0));
                return true;
            }
        } else if (rPrms.GetWPrm() == VK_NEXT) {
            if (CKeyTable::GetModifiers() == CKeyTable::eModCtrl) {
                int nPage = m_pTabCtrl->GetPage();
                m_pTabCtrl->SetPage(Min(nPage + 1, m_pTabCtrl->GetPageCount() - 1));
                return true;
            }
        }
        return false;
*/
    } else if (rPrms.GetMsg() == WM_INITDIALOG) {
        CWnd oParentWindow(GetDialog().GetParent());
        GetDialog().SetPosClip(oParentWindow.ToScreen(oParentWindow.GetClientSize().GetCenterOf(GetDialog().GetSize())));  //
        // ^u쐬
        m_pTabCtrl.reset(new CTabCtrl(*this, GetDialog(), IDC_TAB1));
        m_pTabCtrl->AddPage(L"\", new CConfigDialogGUIPage);
        m_pTabCtrl->AddPage(L"摜", new CConfigDialogImagePage);
        m_pTabCtrl->AddPage(L"`", new CConfigDialogDrawPage);
        m_pTabCtrl->AddPage(L"Xg", new CConfigDialogListPage);
        m_pTabCtrl->AddPage(L"S", new CConfigDialogWholePage);
        m_pTabCtrl->AddPage(L"vOC", new CConfigDialogPluginPage(m_rSpi));
        m_pTabCtrl->AddPage(L"Or[A", new CConfigDialogExternPage);
        // RtBOǂݍ
        for (int i = 0, n = m_pTabCtrl->GetPageCount(); i < n; i++) {
            static_cast<CConfigDialogTabBase&>(m_pTabCtrl->GetPage(i)).LoadFromConfig(m_oConfig);
        }
        // y[Wݒ
        m_pTabCtrl->SetPage(m_nDefaultPage);
        return true;
    } else if (rPrms.GetMsg() == WM_CLOSE) {
        GetDialog().Delete();
        return true;
    } else {
        return false;
    }
}

void CConfigDialog::OnShowTabPage(CTabCtrl& rTabCtrl, int nPage) {
}

bool CConfigDialog::HookMsgTabPages(CTabCtrl& rTabCtrl, int nPage, CMsgHookPrms& rPrms) {
    return false;
}
