#include "stdafx.h"
#include <Themis/Graphics.h>
#include <Themis/Image.h>
#include <Themis/Tooltip.h>
#include <Themis/WndCtrlData.h>
#include <math.h>
#include "resource.h"
#include "AppWindow.h"
#include "Document.h"
#include "Dialogs.h"

CImageInfoDialog::CImageInfoDialog() : CResourceDialogBase(IDD_IMGINFO) {
}

CImageInfoDialog::~CImageInfoDialog() {
}

void CImageInfoDialog::SetText(const vector<string>& vctText) {
    m_vctText = vctText;
}

bool CImageInfoDialog::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_CLOSE) {
        GetDialog().Delete();
        return true;
    } else if (rPrms.GetMsg() == WM_INITDIALOG) {
        CWnd oParentWindow(GetDialog().GetParent());
        GetDialog().SetPosClip(oParentWindow.ToScreen(oParentWindow.GetClientSize().GetCenterOf(GetDialog().GetSize())));  //
     // GetDialog().SetPosClip(oParentWindow.ToScreen(CPoint(oParentWindow.GetClientSize().cx - GetDialog().GetSize().cx, 0)));  //E
        Themis_DebugAssert(m_vctText.size() == 12);
        GetDialog().Editbox(IDC_EDIT1).SetText(m_vctText[0]);
        GetDialog().Editbox(IDC_EDIT2).SetText(m_vctText[1]);
        GetDialog().Editbox(IDC_EDIT3).SetText(m_vctText[2]);
        GetDialog().Editbox(IDC_EDIT4).SetText(m_vctText[3]);
        GetDialog().Editbox(IDC_EDIT5).SetText(m_vctText[4]);
        GetDialog().Editbox(IDC_EDIT6).SetText(m_vctText[5]);
        GetDialog().Editbox(IDC_EDIT7).SetText(m_vctText[6]);
        GetDialog().Editbox(IDC_EDIT8).SetText(m_vctText[7]);
        GetDialog().Editbox(IDC_EDIT9).SetText(m_vctText[8]);
        GetDialog().Editbox(IDC_EDIT19).SetText(m_vctText[9]);
        GetDialog().Editbox(IDC_EDIT29).SetText(m_vctText[10]);
        GetDialog().Editbox(IDC_EDIT30).SetText(m_vctText[11]);
        return true;
    } else if (rPrms.GetMsg() == WM_LBUTTONUP || rPrms.GetMsg() == WM_MBUTTONUP || rPrms.GetMsg() == WM_RBUTTONUP) {
        GetDialog().Close();
        return true;
    } else if (rPrms.GetMsg() == WM_COMMAND) {
        switch (rPrms.GetWPrmLoWord()) {
        case IDOK:
        case IDCANCEL:
            GetDialog().Close();
            break;
        }
        return true;
    } else {
        return false;
    }
}

//////////////////////////////////////////////////////////////////////

CVersionDialog::CVersionDialog(const string& e) : CResourceDialogBase(IDD_VERSION), env(e) {
}

CVersionDialog::~CVersionDialog() {
}

bool CVersionDialog::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_CLOSE) {
        GetDialog().Delete();
        return true;
    } else if (rPrms.GetMsg() == WM_INITDIALOG) {
        CWnd oParentWindow(GetDialog().GetParent());
        GetDialog().SetPosClip(oParentWindow.ToScreen(oParentWindow.GetClientSize().GetCenterOf(GetDialog().GetSize())));  //
        CStatic(GetDialog().GetCtrl(IDC_VERSIONTEXT)).SetText(L"т  " + CAppWindow::GetVersionText());
        CStatic(GetDialog().GetCtrl(IDC_PNGVERTEXT)).SetText(
            string::Join(L"\r\n", CPngLoader::GetCopyright().Split(Array<string>(L"\n"))));
        CEditbox(GetDialog().GetCtrl(IDC_EDIT1)).SetText(env);
        return false;
    } else if (rPrms.GetMsg() == WM_LBUTTONUP) {
        GetDialog().Close();
        return false;
    } else if (rPrms.GetMsg() == WM_MBUTTONUP) {
        GetDialog().Close();
        return false;
    } else if (rPrms.GetMsg() == WM_RBUTTONUP) {
        GetDialog().Close();
        return false;
    } else if (rPrms.GetMsg() == WM_COMMAND) {
        switch (rPrms.GetWPrmLoWord()) {
        case IDOK:
        case IDCANCEL:
            GetDialog().Close();
            break;
        }
        return false;
    } else {
        return false;
    }
}

//////////////////////////////////////////////////////////////////////

CRotDialog::CRotDialog(CAppWindow& r) : CResourceDialogBase(IDD_ROTATE), m_rAppWindow(r), m_dAngle(0.0) {
}

CRotDialog::~CRotDialog() {
}

bool CRotDialog::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_COMMAND) {
        switch (rPrms.GetWPrmLoWord()) {
        case IDOK:
            m_bEnable = false;
            m_rAppWindow.GetDocument().BackupEndWithRestore();
            m_rAppWindow.GetDocument().Rot(AtoF(GetDialog().Editbox(IDC_EDIT1).GetText()));
            m_rAppWindow.GetWindow()->Update();
            GetDialog().Delete();
            break;

        case IDCANCEL:
            GetDialog().Close();
            break;

        case IDC_EDIT1:
            if (rPrms.GetWPrmHiWord() == EN_CHANGE) {
                if (m_bEnable) {
                    double d = AtoF(GetDialog().Editbox(IDC_EDIT1).GetText());
                    if (d != m_dAngle) {
                        m_dAngle = d;
                        m_rAppWindow.GetDocument().BackupRestore();
                        m_rAppWindow.GetDocument().RotDraft(m_dAngle);
                        m_rAppWindow.GetWindow()->Update();
                        while (d < -180) { d += 360; }
                        while (180 < d) { d -= 360; }
                        GetDialog().Slider(IDC_SLIDER1).Set(int(d * 10));
                    }
                }
            }
            break;
        }
        return false;
    } else if (rPrms.GetMsg() == WM_HSCROLL) {
        // \̍XV
        double dAngle = (double)GetDialog().Slider(IDC_SLIDER1).Get() / 10.0;
        if (AtoF(GetDialog().Editbox(IDC_EDIT1).GetText()) != dAngle) {
            GetDialog().Editbox(IDC_EDIT1).SetText(string::Format(L"%.1f", dAngle));
        }
        return true;
    } else if (rPrms.GetMsg() == WM_INITDIALOG) {
        CWnd oParentWindow(GetDialog().GetParent());
        CPoint ptCenter = oParentWindow.GetClientSize().GetCenterOf(GetDialog().GetSize());
        GetDialog().SetPosClip(oParentWindow.ToScreen(CPoint(ptCenter.x, ptCenter.y / 2)));  //
        m_rAppWindow.GetDocument().BackupBegin();
        CSlider oSlider1 = GetDialog().Slider(IDC_SLIDER1);
        oSlider1.SetRange(-1800, 1800);
        oSlider1.SetLineSize(10);
        oSlider1.SetPageSize(50);
        oSlider1.Set(1800);     // Ȃ񂩂悭񂯂ǂȂƏ肭ʒuȂc
        oSlider1.Set(-1800);    // ˑ̂ŔÔ߂B
        oSlider1.Set(0);
        GetDialog().Editbox(IDC_EDIT1).SetText(L"0.0");
        m_bEnable = true;
        return true;
    } else if (rPrms.GetMsg() == WM_CLOSE) {
        m_bEnable = false;
        m_rAppWindow.GetDocument().BackupEndWithRestore();
        m_rAppWindow.GetWindow()->Update();
        GetDialog().Delete();
        return true;
    } else {
        return false;
    }
}

//////////////////////////////////////////////////////////////////////

CStretchDialog::CStretchDialog(const CSize& szSrc, bool bKeepRate)
: CResourceDialogBase(IDD_STRETCH), m_oSrcSize(szSrc), m_bKeepRate(bKeepRate) {
}

CStretchDialog::~CStretchDialog() {
}

bool CStretchDialog::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_CLOSE) {
        GetDialog().Delete();
        return true;
    } else if (rPrms.GetMsg() == WM_INITDIALOG) {
        CWnd oParentWindow(GetDialog().GetParent());
        GetDialog().SetPosClip(oParentWindow.ToScreen(oParentWindow.GetClientSize().GetCenterOf(GetDialog().GetSize())));  //
        m_oSize = boost::optional<CSize>();
        GetDialog().Editbox(IDC_EDIT1).SetInt(m_oSrcSize.cx);
        GetDialog().Editbox(IDC_EDIT14).SetInt(m_oSrcSize.cy);
        GetDialog().Checkbox(IDC_CHECK4).SetCheck(m_bKeepRate);
        return true;
    } else if (rPrms.GetMsg() == WM_COMMAND) {
        switch (rPrms.GetWPrmLoWord()) {
        case IDOK:
            m_oSize = CSize(GetDialog().Editbox(IDC_EDIT15).GetInt(), GetDialog().Editbox(IDC_EDIT16).GetInt());
            GetDialog().Close();
            break;

        case IDCANCEL:
            GetDialog().Close();
            break;

        case IDC_EDIT15:
            if (rPrms.GetWPrmHiWord() == EN_CHANGE) {
                if (m_bKeepRate) {
                    int n1 = GetDialog().Editbox(IDC_EDIT16).GetInt();
                    int n2 = GetDialog().Editbox(IDC_EDIT15).GetInt() * m_oSrcSize.cy / m_oSrcSize.cx;
                    if (n1 != n2) {
                        m_bKeepRate = false;    //}uBB
                        GetDialog().Editbox(IDC_EDIT16).SetInt(n2);
                        m_bKeepRate = true;
                    }
                }
            }
            break;

        case IDC_EDIT16:
            if (rPrms.GetWPrmHiWord() == EN_CHANGE) {
                if (m_bKeepRate) {
                    int n1 = GetDialog().Editbox(IDC_EDIT15).GetInt();
                    int n2 = GetDialog().Editbox(IDC_EDIT16).GetInt() * m_oSrcSize.cx / m_oSrcSize.cy;
                    if (n1 != n2) {
                        m_bKeepRate = false;    //}uBB
                        GetDialog().Editbox(IDC_EDIT15).SetInt(n2);
                        m_bKeepRate = true;
                    }
                }
            }
            break;

        case IDC_CHECK4:
            m_bKeepRate = GetDialog().Checkbox(IDC_CHECK4).GetCheck();
            break;
        }
        return false;
    } else {
        return false;
    }
}

//////////////////////////////////////////////////////////////////////

CLevelRevDialog::CLevelRevDialog(CAppWindow& r, const CRect& rc, bool bFilter)
: CResourceDialogBase(IDD_LEVELREV), m_rAppWindow(r), m_rcArea(rc), m_bFilter(bFilter) {
}

CLevelRevDialog::~CLevelRevDialog() {
}

bool CLevelRevDialog::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_COMMAND) {
        switch (rPrms.GetWPrmLoWord()) {
        case IDOK:
            // ݒۑ
            CCheckboxData(GetDialog(), IDC_CHECK1) >> m_rAppWindow.GetConfig().BoolData(L"LevelRevReflect");
            CCheckboxData(GetDialog(), IDC_CHECK2) >> m_rAppWindow.GetConfig().BoolData(L"LevelRevOpaque");
            if (m_bFilter) {
                m_rAppWindow.GetConfig().Bool(L"FilterLevelRev", true);
                CSliderData(GetDialog(), IDC_SLIDER1) >> m_rAppWindow.GetConfig().NumData(L"FilterLevelRevLow");
                CSliderData(GetDialog(), IDC_SLIDER2) >> m_rAppWindow.GetConfig().NumData(L"FilterLevelRevHigh");
            }
            // ύX𔽉fďI
            if (!GetDialog().Checkbox(IDC_CHECK1).GetCheck()) {
                UpdateImage(); // fȂ悤ɂȂĂ甽fĂI
            }
            m_rAppWindow.GetDocument().BackupEnd();
            GetDialog().Delete();
            break;

        case IDCANCEL:
            GetDialog().Close();
            break;

        case IDC_BUTTON1:
            {
                CChooseFile& oChooseFile = m_rAppWindow.GetChooseFile();
                oChooseFile.SetTitle(L"t@CJ");
                oChooseFile.ClearNameFilter();
                oChooseFile.AddNameFilter(L"CSV file", L"*.csv");
                oChooseFile.SetDefaultExt(L"csv");
                oChooseFile.SetDefault(CFileName(m_rAppWindow.GetDocument().GetCurrentFileName().GetPureName()).SetSuffix(L"csv").Get());
                if (oChooseFile.ShowSave()) {
                    m_rAppWindow.GetDocument().SaveHistogramGraph(m_rcArea, oChooseFile.GetResult());
                }
            }
            break;

        case IDC_CHECK1:
            if (GetDialog().Checkbox(IDC_CHECK1).GetCheck()) {
                UpdateImage();
            } else {
                m_rAppWindow.GetDocument().BackupRestore();
                m_rAppWindow.GetWindow()->Update();
            }
            break;

        case IDC_CHECK2:
            if (GetDialog().Checkbox(IDC_CHECK2).GetCheck()) {
                GetDialog().SetAlpha(64);  // ɂ
            } else {
                GetDialog().UnsetAlpha();
            }
            break;
        }
        return false;
    } else if (rPrms.GetMsg() == WM_HSCROLL) {
        // \̍XV
        CSlider oSlider1 = GetDialog().Slider(IDC_SLIDER1);
        int nLow = oSlider1.Get();
        if (GetDialog().Editbox(IDC_EDIT1).GetInt() != nLow) {
            GetDialog().Editbox(IDC_EDIT1).SetInt(nLow);
        }
        CSlider oSlider2 = GetDialog().Slider(IDC_SLIDER2);
        int nHigh = oSlider2.Get();
        if (GetDialog().Editbox(IDC_EDIT2).GetInt() != nHigh) {
            GetDialog().Editbox(IDC_EDIT2).SetInt(nHigh);
        }
        // 摜̍XV
        if (GetDialog().Checkbox(IDC_CHECK1).GetCheck()) {
            UpdateImage();
        }
        return true;
    } else if (rPrms.GetMsg() == WM_DRAWITEM) {
        DRAWITEMSTRUCT* di = reinterpret_cast<DRAWITEMSTRUCT*>(rPrms.GetLPrm());
        if (di->CtlID == IDC_PICTURE) {
            CRect rcItem(di->rcItem);
            CDC(di->hDC).Blt(CPlane::CGdiLock(*m_pGraphPlane), rcItem.GetPoint(), rcItem.GetSize().ToRect());
            Themis_DebugAssert(rcItem.GetSize() == m_pGraphPlane->GetSize());
        }
        return false;
    } else if (rPrms.GetMsg() == WM_INITDIALOG) {
        CWnd oParentWindow(GetDialog().GetParent());
        GetDialog().SetPosClip(oParentWindow.ToScreen(oParentWindow.GetClientSize().GetCenterOf(GetDialog().GetSize())));  //

        m_rAppWindow.GetDocument().BackupBegin();
        double dAverage, dStdEvp;
        m_pGraphPlane = m_rAppWindow.GetDocument().GetHistogramGraph(m_rcArea, GetDialog().Static(IDC_PICTURE).GetClientSize(), dAverage, dStdEvp);
        GetDialog().SetText(string::Format(L"x␳@[ρF%.2f@W΍F%.2f]", dAverage, dStdEvp));

        CSlider oSlider1 = GetDialog().Slider(IDC_SLIDER1);
        CSlider oSlider2 = GetDialog().Slider(IDC_SLIDER2);
        oSlider1.SetRange(0, 255);
        oSlider1.SetTicFreq(5);
        oSlider1.SetLineSize(1);
        oSlider1.SetPageSize(16);
        oSlider1.Set(0);
        oSlider2.SetRange(0, 255);
        oSlider2.SetTicFreq(5);
        oSlider2.SetLineSize(1);
        oSlider2.SetPageSize(16);
        oSlider2.Set(255);
        if (m_bFilter) {
            CSliderData(GetDialog(), IDC_SLIDER1) << m_rAppWindow.GetConfig().NumData(L"FilterLevelRevLow");
            CSliderData(GetDialog(), IDC_SLIDER2) << m_rAppWindow.GetConfig().NumData(L"FilterLevelRevHigh");
        }
        GetDialog().Editbox(IDC_EDIT1).SetInt(oSlider1.Get());
        GetDialog().Editbox(IDC_EDIT2).SetInt(oSlider2.Get());
        CCheckboxData(GetDialog(), IDC_CHECK1) << m_rAppWindow.GetConfig().BoolData(L"LevelRevReflect");
        CCheckboxData(GetDialog(), IDC_CHECK2) << m_rAppWindow.GetConfig().BoolData(L"LevelRevOpaque");
        if (m_rAppWindow.GetConfig().Bool(L"LevelRevOpaque")) {
            GetDialog().SetAlpha(64);  // ɂ
        }
        if (GetDialog().Checkbox(IDC_CHECK1).GetCheck()) {
            UpdateImage();
        }
        GetDialog().Update();
        return true;
    } else if (rPrms.GetMsg() == WM_CLOSE) {
        // ݒۑ
        CCheckboxData(GetDialog(), IDC_CHECK1) >> m_rAppWindow.GetConfig().BoolData(L"LevelRevReflect");
        CCheckboxData(GetDialog(), IDC_CHECK2) >> m_rAppWindow.GetConfig().BoolData(L"LevelRevOpaque");
        // obNAbv𕜌ďI
        m_rAppWindow.GetDocument().BackupEndWithRestore();
        m_rAppWindow.GetWindow()->Update();
        GetDialog().Delete();
        return true;
    } else {
        return false;
    }
}

void CLevelRevDialog::UpdateImage() {
    m_rAppWindow.GetDocument().BackupRestore();
    m_rAppWindow.GetDocument().LevelRev(m_rcArea, GetDialog().Slider(IDC_SLIDER1).Get(), GetDialog().Slider(IDC_SLIDER2).Get());
    m_rAppWindow.GetWindow()->Update();
}

//////////////////////////////////////////////////////////////////////

CGaussDialog::CGaussDialog(CAppWindow& r, const CRect& rc, bool bFilter, bool gaussOrUnsharp)
: CResourceDialogBase(IDD_GAUSS), m_rAppWindow(r), m_rcArea(rc), m_bFilter(bFilter), gaussOrUnsharp(gaussOrUnsharp) {
    configName = gaussOrUnsharp ? L"Gauss" : L"Unsharp";
}

CGaussDialog::~CGaussDialog() {
}

namespace {
    int RToSlider(double d) {
        return int(floor(d * 100.0 + 0.5));
    }
    double SliderToR(int n) {
        return double(n) / 100.0;
    }
}

bool CGaussDialog::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_COMMAND) {
        switch (rPrms.GetWPrmLoWord()) {
        case IDOK:
            // ݒۑ
            CCheckboxData(GetDialog(), IDC_CHECK1) >> m_rAppWindow.GetConfig().BoolData(configName + L"Reflect");
            if (m_bFilter) {
                m_rAppWindow.GetConfig().Bool(L"Filter" + configName, true);
                CSliderData(GetDialog(), IDC_SLIDER1) >> m_rAppWindow.GetConfig().NumData(L"Filter" + configName + L"Value");
            }
            // ύX𔽉fďI
            if (!GetDialog().Checkbox(IDC_CHECK1).GetCheck()) {
                UpdateImage(); // fȂ悤ɂȂĂ甽fĂI
            }
            m_rAppWindow.GetDocument().BackupEnd();
            GetDialog().Delete();
            break;

        case IDCANCEL:
            GetDialog().Close();
            break;

        case IDC_EDIT1:
            if (rPrms.GetWPrmHiWord() == EN_CHANGE) {
                double r = AtoF(GetDialog().Editbox(IDC_EDIT1).GetText()); //0.1 - 1 - 10
                GetDialog().Slider(IDC_SLIDER1).Set(RToSlider(r));
                // 摜̍XV
                if (GetDialog().Checkbox(IDC_CHECK1).GetCheck()) {
                    UpdateImage();
                }
                // Ot̍XV
                GetDialog().Update();
            }
            break;

        case IDC_CHECK1: // f
            if (GetDialog().Checkbox(IDC_CHECK1).GetCheck()) {
                UpdateImage();
            } else {
                m_rAppWindow.GetDocument().BackupRestore();
                m_rAppWindow.GetWindow()->Update();
            }
            break;
        }
        return false;
    } else if (rPrms.GetMsg() == WM_HSCROLL) {
        // \̍XV
        CSlider oSlider1 = GetDialog().Slider(IDC_SLIDER1); //-100 - 0 - 100
        string str = FtoA(SliderToR(oSlider1.Get()));
        if (GetDialog().Editbox(IDC_EDIT1).GetText() != str) {
            GetDialog().Editbox(IDC_EDIT1).SetText(str);
        }
        // 摜̍XV
        if (GetDialog().Checkbox(IDC_CHECK1).GetCheck()) {
            UpdateImage();
        }
        // Ot̍XV
        GetDialog().Update();
        return true;
    } else if (rPrms.GetMsg() == WM_INITDIALOG) {
        CWnd oParentWindow(GetDialog().GetParent());
        GetDialog().SetPosClip(oParentWindow.ToScreen(oParentWindow.GetClientSize().GetCenterOf(GetDialog().GetSize())));  //
        m_rAppWindow.GetDocument().BackupBegin();
        // Ƃ̏
        CSlider oSlider1 = GetDialog().Slider(IDC_SLIDER1);
        oSlider1.SetRange(0, 1000);
        oSlider1.SetTicFreq(10);
        oSlider1.SetLineSize(1);
        oSlider1.SetPageSize(10);
        oSlider1.Set(0);
        oSlider1.Set(1000); // oO΍
        oSlider1.Set(150);
        if (m_bFilter) {
            CSliderData(GetDialog(), IDC_SLIDER1) << m_rAppWindow.GetConfig().Num(L"Filter" + configName + L"Value");
        }
        GetDialog().Editbox(IDC_EDIT1).SetText(FtoA(SliderToR(oSlider1.Get())));
        CCheckboxData(GetDialog(), IDC_CHECK1) << m_rAppWindow.GetConfig().BoolData(configName + L"Reflect");
        GetDialog().SetText(gaussOrUnsharp ? L"KEXڂ" : L"AV[v}XN");
        // XV
        if (GetDialog().Checkbox(IDC_CHECK1).GetCheck()) {
            UpdateImage();
        }
        GetDialog().Update();
        return true;
    } else if (rPrms.GetMsg() == WM_CLOSE) {
        // ݒۑ
        CCheckboxData(GetDialog(), IDC_CHECK1) >> m_rAppWindow.GetConfig().BoolData(configName + L"Reflect");
        // obNAbv𕜌ďI
        m_rAppWindow.GetDocument().BackupEndWithRestore();
        m_rAppWindow.GetWindow()->Update();
        GetDialog().Delete();
        return true;
    } else {
        return false;
    }
}

void CGaussDialog::UpdateImage() {
    double r = AtoF(GetDialog().Editbox(IDC_EDIT1).GetText());
    m_rAppWindow.GetDocument().BackupRestore();
    if (gaussOrUnsharp) {
        m_rAppWindow.GetDocument().GaussBefog(m_rcArea, r);
    } else {
        m_rAppWindow.GetDocument().UnsharpMask(m_rcArea, r);
    }
    m_rAppWindow.GetWindow()->Update();
}

//////////////////////////////////////////////////////////////////////

CGammaRevDialog::CGammaRevDialog(CAppWindow& r, const CRect& rc, bool bFilter)
: CResourceDialogBase(IDD_GAMMA), m_rAppWindow(r), m_rcArea(rc), m_bFilter(bFilter) {
    m_pGraphPlane.reset(new CDib32);
}

CGammaRevDialog::~CGammaRevDialog() {
}

namespace {
    int GammaToSlider(double d) {
        return int(floor(log(d) * 100.0 + 0.5));
    }
    double SliderToGamma(int n) {
        return exp((double(n) / 100.0));
    }
}

bool CGammaRevDialog::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_COMMAND) {
        switch (rPrms.GetWPrmLoWord()) {
        case IDOK:
            // ݒۑ
            CCheckboxData(GetDialog(), IDC_CHECK1) >> m_rAppWindow.GetConfig().BoolData(L"GammaRevReflect");
            CCheckboxData(GetDialog(), IDC_CHECK2) >> m_rAppWindow.GetConfig().BoolData(L"GammaRevOpaque");
            if (m_bFilter) {
                m_rAppWindow.GetConfig().Bool(L"FilterGammaRev", true);
                CSliderData(GetDialog(), IDC_SLIDER1) >> m_rAppWindow.GetConfig().NumData(L"FilterGammaRevValue");
            }
            // ύX𔽉fďI
            if (!GetDialog().Checkbox(IDC_CHECK1).GetCheck()) {
                UpdateImage(); // fȂ悤ɂȂĂ甽fĂI
            }
            m_rAppWindow.GetDocument().BackupEnd();
            GetDialog().Delete();
            break;

        case IDCANCEL:
            GetDialog().Close();
            break;

        case IDC_EDIT1:
            if (rPrms.GetWPrmHiWord() == EN_CHANGE) {
                double dGamma = AtoF(GetDialog().Editbox(IDC_EDIT1).GetText()); //0.1 - 1 - 10
                if (dGamma <= 0.00000001) {
                    dGamma = 1.0;
                }
                GetDialog().Slider(IDC_SLIDER1).Set(GammaToSlider(dGamma));
                // 摜̍XV
                if (GetDialog().Checkbox(IDC_CHECK1).GetCheck()) {
                    UpdateImage();
                }
                // Ot̍XV
                UpdateGraph();
                GetDialog().Update();
            }
            break;

        case IDC_CHECK1: // f
            if (GetDialog().Checkbox(IDC_CHECK1).GetCheck()) {
                UpdateImage();
            } else {
                m_rAppWindow.GetDocument().BackupRestore();
                m_rAppWindow.GetWindow()->Update();
            }
            break;

        case IDC_CHECK2: // 
            if (GetDialog().Checkbox(IDC_CHECK2).GetCheck()) {
                GetDialog().SetAlpha(64);  // ɂ
            } else {
                GetDialog().UnsetAlpha();
            }
            break;
        }
        return false;
    } else if (rPrms.GetMsg() == WM_HSCROLL) {
        // \̍XV
        CSlider oSlider1 = GetDialog().Slider(IDC_SLIDER1); //-100 - 0 - 100
        string str = FtoA(SliderToGamma(oSlider1.Get()));
        if (GetDialog().Editbox(IDC_EDIT1).GetText() != str) {
            GetDialog().Editbox(IDC_EDIT1).SetText(str);
        }
        // 摜̍XV
        if (GetDialog().Checkbox(IDC_CHECK1).GetCheck()) {
            UpdateImage();
        }
        // Ot̍XV
        UpdateGraph();
        GetDialog().Update();
        return true;
    } else if (rPrms.GetMsg() == WM_DRAWITEM) {
        DRAWITEMSTRUCT* di = reinterpret_cast<DRAWITEMSTRUCT*>(rPrms.GetLPrm());
        if (di->CtlID == IDC_PICTURE) {
            if (m_pGraphPlane != NULL) {
                CRect rcItem(di->rcItem);
                CDC(di->hDC).Blt(CPlane::CGdiLock(*m_pGraphPlane), rcItem.GetPoint(), rcItem.GetSize().ToRect());
                Themis_DebugAssert(rcItem.GetSize() == m_pGraphPlane->GetSize());
            }
        }
        return false;
    } else if (rPrms.GetMsg() == WM_INITDIALOG) {
        m_pGraphPlane->SetSize(GetDialog().Static(IDC_PICTURE).GetClientSize());
        CWnd oParentWindow(GetDialog().GetParent());
        GetDialog().SetPosClip(oParentWindow.ToScreen(oParentWindow.GetClientSize().GetCenterOf(GetDialog().GetSize())));  //
        m_rAppWindow.GetDocument().BackupBegin();
        // Ƃ̏
        CSlider oSlider1 = GetDialog().Slider(IDC_SLIDER1);
        oSlider1.SetRange(-100, 100);
        oSlider1.SetTicFreq(10);
        oSlider1.SetLineSize(1);
        oSlider1.SetPageSize(10);
        oSlider1.Set(-100);
        oSlider1.Set(100); // oO΍
        oSlider1.Set(0);
        if (m_bFilter) {
            CSliderData(GetDialog(), IDC_SLIDER1) << m_rAppWindow.GetConfig().Num(L"FilterGammaRevValue");
        }
        GetDialog().Editbox(IDC_EDIT1).SetText(FtoA(SliderToGamma(oSlider1.Get())));
        CCheckboxData(GetDialog(), IDC_CHECK1) << m_rAppWindow.GetConfig().BoolData(L"GammaRevReflect");
        CCheckboxData(GetDialog(), IDC_CHECK2) << m_rAppWindow.GetConfig().BoolData(L"GammaRevOpaque");
        if (m_rAppWindow.GetConfig().Bool(L"GammaRevOpaque")) {
            GetDialog().SetAlpha(64);  // ɂ
        }
        // OtȂǂȂǂ̍XV
        UpdateGraph();
        if (GetDialog().Checkbox(IDC_CHECK1).GetCheck()) {
            UpdateImage();
        }
        GetDialog().Update();
        return true;
    } else if (rPrms.GetMsg() == WM_CLOSE) {
        // ݒۑ
        CCheckboxData(GetDialog(), IDC_CHECK1) >> m_rAppWindow.GetConfig().BoolData(L"GammaRevReflect");
        CCheckboxData(GetDialog(), IDC_CHECK2) >> m_rAppWindow.GetConfig().BoolData(L"GammaRevOpaque");
        // obNAbv𕜌ďI
        m_rAppWindow.GetDocument().BackupEndWithRestore();
        m_rAppWindow.GetWindow()->Update();
        GetDialog().Delete();
        return true;
    } else {
        return false;
    }
}

void CGammaRevDialog::UpdateImage() {
    double dGamma = AtoF(GetDialog().Editbox(IDC_EDIT1).GetText());
    m_rAppWindow.GetDocument().BackupRestore();
    m_rAppWindow.GetDocument().GammaRev(m_rcArea, dGamma);
    m_rAppWindow.GetWindow()->Update();
}

void CGammaRevDialog::UpdateGraph() {
    double dGamma = AtoF(GetDialog().Editbox(IDC_EDIT1).GetText());
    uint8_t abyTable[0x100];
    CPlaneEx::MakeGammaRevTable(abyTable, dGamma);
    // Ot쐬
    double adGraph[0x100];
    std_copy(abyTable, abyTable + 0x100, adGraph);
    m_pGraphPlane->MakeBargraph(m_pGraphPlane->GetSize().ToRect(), adGraph, 0x100, 0.0, 0xff, ::GetSysColor(COLOR_3DFACE));
}

//////////////////////////////////////////////////////////////////////

CBrightContrastDialog::CBrightContrastDialog(CAppWindow& r, const CRect& rc, bool bFilter)
: CResourceDialogBase(IDD_BRICON), m_rAppWindow(r), m_rcArea(rc), m_bFilter(bFilter) {
}

CBrightContrastDialog::~CBrightContrastDialog() {
}

bool CBrightContrastDialog::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_COMMAND) {
        switch (rPrms.GetWPrmLoWord()) {
        case IDOK:
            m_rAppWindow.GetDocument().BackupEnd();
            if (m_bFilter) {
                m_rAppWindow.GetConfig().Bool(L"FilterBC", true);
                CSliderData(GetDialog(), IDC_SLIDER1) >> m_rAppWindow.GetConfig().NumData(L"FilterBCBright");
                CSliderData(GetDialog(), IDC_SLIDER2) >> m_rAppWindow.GetConfig().NumData(L"FilterBCContrast");
            }
            GetDialog().Delete();
            break;
        case IDCANCEL:
            GetDialog().Close();
            break;
        }
        return false;
    } else if (rPrms.GetMsg() == WM_HSCROLL) {
        UpdateImage();
        return true;
    } else if (rPrms.GetMsg() == WM_INITDIALOG) {
        CWnd oParentWindow(GetDialog().GetParent());
        CPoint ptCenter = oParentWindow.GetClientSize().GetCenterOf(GetDialog().GetSize());
        GetDialog().SetPosClip(oParentWindow.ToScreen(CPoint(ptCenter.x, ptCenter.y / 2)));  //
        m_rAppWindow.GetDocument().BackupBegin();
        CSlider oSlider1 = GetDialog().Slider(IDC_SLIDER1);
        oSlider1.SetRange(-128, 128);
        oSlider1.SetLineSize(1);
        oSlider1.SetPageSize(8);
        oSlider1.Set(128);      // Ȃ񂩂悭񂯂ǂȂƏ肭ʒuȂc
        oSlider1.Set(-128);     // ˑ̂ŔÔ߂B
        oSlider1.Set(0);
        CSlider oSlider2 = GetDialog().Slider(IDC_SLIDER2);
        oSlider2.SetRange(-128, 128);
        oSlider2.SetLineSize(1);
        oSlider2.SetPageSize(8);
        oSlider2.Set(128);      // Ȃ񂩂悭񂯂ǂȂƏ肭ʒuȂc
        oSlider2.Set(-128);     // ˑ̂ŔÔ߂B
        oSlider2.Set(0);
        if (m_bFilter) {
            CSliderData(GetDialog(), IDC_SLIDER1) << m_rAppWindow.GetConfig().NumData(L"FilterBCBright");
            CSliderData(GetDialog(), IDC_SLIDER2) << m_rAppWindow.GetConfig().NumData(L"FilterBCContrast");
        }
        UpdateImage();
        GetDialog().Update();
        return true;
    } else if (rPrms.GetMsg() == WM_CLOSE) {
        m_rAppWindow.GetDocument().BackupEndWithRestore();
        m_rAppWindow.GetWindow()->Update();
        GetDialog().Delete();
        return true;
    } else {
        return false;
    }
}

void CBrightContrastDialog::UpdateImage() {
    m_rAppWindow.GetDocument().BackupRestore();
    m_rAppWindow.GetDocument().BrightContrast(m_rcArea, GetDialog().Slider(IDC_SLIDER1).Get(), GetDialog().Slider(IDC_SLIDER2).Get());
    m_rAppWindow.GetWindow()->Update();
}

//////////////////////////////////////////////////////////////////////

CMosaicDialog::CMosaicDialog(CAppWindow& r, const CRect& rc)
: CResourceDialogBase(IDD_MOSAIC), m_rAppWindow(r), m_rcArea(rc) {
}

CMosaicDialog::~CMosaicDialog() {
}

bool CMosaicDialog::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_COMMAND) {
        switch (rPrms.GetWPrmLoWord()) {
        case IDOK:
            m_rAppWindow.GetDocument().BackupEnd();
            m_rAppWindow.GetWindow()->Update();
            CEditboxData(GetDialog(), IDC_EDIT1) >> m_rAppWindow.GetConfig().NumData(L"MosaicLevel");
            CRadiosData(GetDialog(), IDC_RADIO1, IDC_RADIO2) >> m_rAppWindow.GetConfig().NumData(L"MosaicType");
            GetDialog().Delete();
            break;

        case IDCANCEL:
            GetDialog().Close();
            break;

        case IDC_EDIT1:
            if (rPrms.GetWPrmHiWord() == EN_CHANGE) {
                UpdateImage();
            }
            break;
        case IDC_RADIO1:
        case IDC_RADIO2:
            UpdateImage();
            break;
        }
        return false;
    } else if (rPrms.GetMsg() == WM_INITDIALOG) {
        CWnd oParentWindow(GetDialog().GetParent());
        CPoint ptCenter = oParentWindow.GetClientSize().GetCenterOf(GetDialog().GetSize());
        GetDialog().SetPosClip(oParentWindow.ToScreen(CPoint(ptCenter.x, ptCenter.y / 2)));  //
        m_rAppWindow.GetDocument().BackupBegin();
        CSlider oSlider1 = GetDialog().Slider(IDC_SLIDER1);
        oSlider1.SetRange(-1800, 1800);
        oSlider1.SetLineSize(10);
        oSlider1.SetPageSize(50);
        oSlider1.Set(1800);     // Ȃ񂩂悭񂯂ǂȂƏ肭ʒuȂc
        oSlider1.Set(-1800);    // ˑ̂ŔÔ߂B
        oSlider1.Set(0);
        GetDialog().Updown(IDC_SPIN1).SetRange(30, 0);
        CEditboxData(GetDialog(), IDC_EDIT1) << m_rAppWindow.GetConfig().NumData(L"MosaicLevel");
        CRadiosData(GetDialog(), IDC_RADIO1, IDC_RADIO2) << m_rAppWindow.GetConfig().NumData(L"MosaicType");
        return true;
    } else if (rPrms.GetMsg() == WM_CLOSE) {
        m_rAppWindow.GetDocument().BackupEndWithRestore();
        m_rAppWindow.GetWindow()->Update();
        CEditboxData(GetDialog(), IDC_EDIT1) >> m_rAppWindow.GetConfig().NumData(L"MosaicLevel");
        CRadiosData(GetDialog(), IDC_RADIO1, IDC_RADIO2) >> m_rAppWindow.GetConfig().NumData(L"MosaicType");
        GetDialog().Delete();
        return true;
    } else {
        return false;
    }
}

void CMosaicDialog::UpdateImage() {
    m_rAppWindow.GetDocument().BackupRestore();
    m_rAppWindow.GetDocument().Mosaic(m_rcArea, GetDialog().Editbox(IDC_EDIT1).GetInt(),
        GetDialog().Radios(IDC_RADIO1, IDC_RADIO2).Get());
    m_rAppWindow.GetWindow()->Update();
}

//////////////////////////////////////////////////////////////////////

CSlideShowDialog::CSlideShowDialog(CAppWindow& r) : CResourceDialogBase(IDD_SSHOW), m_rAppWindow(r) {
}

CSlideShowDialog::~CSlideShowDialog() {
}

bool CSlideShowDialog::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_COMMAND) {
        switch (rPrms.GetWPrmLoWord()) {
        case IDOK:
            m_bOk = true;
            SaveToConfig();
            GetDialog().Close();
            break;
        case IDCANCEL:
            GetDialog().Close();
            break;
        }
        return false;
    } else if (rPrms.GetMsg() == WM_INITDIALOG) {
        m_bOk = false;
        CWnd oParentWindow(GetDialog().GetParent());
        GetDialog().SetPosClip(oParentWindow.ToScreen(oParentWindow.GetClientSize().GetCenterOf(GetDialog().GetSize())));  //
        GetDialog().Updown(IDC_SPIN1).SetRange(5000, 500).SetStep(100);
        LoadFromConfig();
        return true;
    } else if (rPrms.GetMsg() == WM_CLOSE) {
        GetDialog().Delete();
        return true;
    } else {
        return false;
    }
}

void CSlideShowDialog::LoadFromConfig() {
    CEditboxData(GetDialog(), IDC_EDIT1) << m_rAppWindow.GetConfig().NumData(L"SlideShowFreq");
    CRadiosData(GetDialog(), IDC_RADIO1, IDC_RADIO2) << m_rAppWindow.GetConfig().NumData(L"SlideShowDist");
    CCheckboxData(GetDialog(), IDC_CHECK2) << m_rAppWindow.GetConfig().BoolData(L"SlideShowStopWithLast");
    CCheckboxData(GetDialog(), IDC_CHECK3) << m_rAppWindow.GetConfig().BoolData(L"SlideShowStopWithCommand");
}

void CSlideShowDialog::SaveToConfig() {
    CEditboxData(GetDialog(), IDC_EDIT1) >> m_rAppWindow.GetConfig().NumData(L"SlideShowFreq");
    CRadiosData(GetDialog(), IDC_RADIO1, IDC_RADIO2) >> m_rAppWindow.GetConfig().NumData(L"SlideShowDist");
    CCheckboxData(GetDialog(), IDC_CHECK2) >> m_rAppWindow.GetConfig().BoolData(L"SlideShowStopWithLast");
    CCheckboxData(GetDialog(), IDC_CHECK3) >> m_rAppWindow.GetConfig().BoolData(L"SlideShowStopWithCommand");
}

//////////////////////////////////////////////////////////////////////

CAlphaDialog::CAlphaDialog(CAppWindow& r, const CRect& rc, bool bFilter)
: CResourceDialogBase(IDD_ALPHA), m_rAppWindow(r), m_rcArea(rc), m_bFilter(bFilter) {
}

CAlphaDialog::~CAlphaDialog() {
}

bool CAlphaDialog::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_COMMAND) {
        switch (rPrms.GetWPrmLoWord()) {
        case IDOK:
            m_rAppWindow.GetDocument().BackupEnd();
            m_rAppWindow.GetWindow()->Update();
            if (m_bFilter) {
                m_rAppWindow.GetConfig().Bool(L"FilterAlpha", true);
            }
            m_rgbBgColor >> m_rAppWindow.GetConfig().ColorData(L"FilterAlphaBg");
            CCheckboxData(GetDialog(), IDC_CHECK1) >> m_rAppWindow.GetConfig().BoolData(L"FilterAlphaCheck");
            GetDialog().Delete();
            break;

        case IDCANCEL:
            GetDialog().Close();
            break;

        case IDC_STATIC1: // wiF̕ύX
            {
                CChooseColor oChooseColor;
                oChooseColor.SetOwner(GetDialog().GetHandle());
                oChooseColor.LoadFromString(m_rAppWindow.GetConfig().Str(L"ChooseColor"));
                oChooseColor.SetDefault(m_rgbBgColor);
                if (oChooseColor.Show()) {
                    m_rgbBgColor = oChooseColor.GetResult();
                    m_rAppWindow.GetConfig().Str(L"ChooseColor", oChooseColor.SaveToString());
                    CWnd((HWND)rPrms.GetLPrm()).Update();
                    UpdateImage();
                }
            }
            break;
        case IDC_CHECK1:
            UpdateImage();
            break;
        }
        return false;
    } else if (rPrms.GetMsg() == WM_DRAWITEM) {
        DRAWITEMSTRUCT* di = (DRAWITEMSTRUCT*)rPrms.GetLPrm();
        if (di->CtlID == IDC_STATIC1) { // wiF
            CDC(di->hDC).Fill(m_rgbBgColor, di->rcItem);
            return true;
        }
        return false;
    } else if (rPrms.GetMsg() == WM_INITDIALOG) {
        CWnd oParentWindow(GetDialog().GetParent());
        GetDialog().SetPosClip(oParentWindow.ToScreen(oParentWindow.GetClientSize().GetCenterOf(GetDialog().GetSize())));  //
        m_rAppWindow.GetDocument().BackupBegin();
        m_rgbBgColor << m_rAppWindow.GetConfig().ColorData(L"FilterAlphaBg");
        CCheckboxData(GetDialog(), IDC_CHECK1) << m_rAppWindow.GetConfig().BoolData(L"FilterAlphaCheck");
        UpdateImage();
        // 邿
        CDialogTooltip* pTip = new CDialogTooltip(GetDialog());
        pTip->Add(IDC_STATIC1, L"wiFw肵܂B");
        pTip->Add(IDC_CHECK1, L"摜`lĂƎvꍇɂ̂ݏs܂B");
        return true;
    } else if (rPrms.GetMsg() == WM_CLOSE) {
        m_rAppWindow.GetDocument().BackupEndWithRestore();
        m_rAppWindow.GetWindow()->Update();
        GetDialog().Delete();
        return true;
    } else {
        return false;
    }
}

void CAlphaDialog::UpdateImage() {
    m_rAppWindow.GetDocument().BackupRestore();
    m_rAppWindow.GetDocument().Alpha(m_rcArea, m_rgbBgColor, GetDialog().Checkbox(IDC_CHECK1).GetCheck());
    m_rAppWindow.GetWindow()->Update();
}


//////////////////////////////////////////////////////////////////////

CPassDialog::CPassDialog(CAppWindow& r, const CFileName& archiveName)
: CResourceDialogBase(IDD_PASS), m_rAppWindow(r), archiveName(archiveName) {
}

CPassDialog::~CPassDialog() {
}

bool CPassDialog::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_COMMAND) {
        switch (rPrms.GetWPrmLoWord()) {
        case IDOK:
            m_bOk = true;
            m_pass = GetDialog().Editbox(IDC_EDIT1).GetText();
            SaveToConfig();
            GetDialog().Close();
            break;
        case IDCANCEL:
            GetDialog().Close();
            break;
        case IDC_CHECK1:
            UpdateCtrl();
            break;
        }
        return false;
    } else if (rPrms.GetMsg() == WM_INITDIALOG) {
        m_bOk = false;
        //CWnd oParentWindow(GetDialog().GetParent());
        //GetDialog().SetPosClip(oParentWindow.ToScreen(oParentWindow.GetClientSize().GetCenterOf(GetDialog().GetSize())));  //
        GetDialog().Static(IDC_STATIC1).SetText(archiveName.Get());
        LoadFromConfig();
        return true;
    } else if (rPrms.GetMsg() == WM_CLOSE) {
        GetDialog().Delete();
        return true;
    } else {
        return false;
    }
}

void CPassDialog::LoadFromConfig() {
    m_rAppWindow.GetConfig().BoolData(L"ArcPassMask") >> CCheckboxData(GetDialog(), IDC_CHECK1);
    UpdateCtrl();
}

void CPassDialog::SaveToConfig() {
    m_rAppWindow.GetConfig().BoolData(L"ArcPassMask") << CCheckboxData(GetDialog(), IDC_CHECK1);
}

void CPassDialog::UpdateCtrl() {
    GetDialog().Editbox(IDC_EDIT1).SetPasswordMode(GetDialog().Checkbox(IDC_CHECK1).GetCheck());
}

//////////////////////////////////////////////////////////////////////

CHistoryDialog::CHistoryDialog(CAppWindow& r) : CResourceDialogBase(IDD_HISTORY), m_rAppWindow(r) {
}

CHistoryDialog::~CHistoryDialog() {
}

bool CHistoryDialog::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_COMMAND) {
        switch (rPrms.GetWPrmLoWord()) {
        case IDOK:
            m_bOk = true;
            {
                int n = GetDialog().Listview(IDC_LIST1).GetSelected();
                if (n < 0) {
                    // LZɂĂ܂B
                    // OK{^𖳌ɏoΈԂȂ̂BB
                    m_bOk = false;
                } else {
                    m_no = AtoI(GetDialog().Listview(IDC_LIST1).GetSubItemText(n, 0));
                }
            }
            GetDialog().Close();
            break;
        case IDCANCEL:
            GetDialog().Close();
            break;
        }
        return false;
    } else if (rPrms.GetMsg() == WM_NOTIFY) {
        NMHDR* hdr = (NMHDR*)rPrms.GetLPrm();
        if (hdr->idFrom == IDC_LIST1) {
            //if (hdr->code == LVN_ITEMACTIVATE) {
            //}
        }
        return false;
    } else if (rPrms.GetMsg() == WM_INITDIALOG) {
        m_bOk = false;
        CWnd oParentWindow(GetDialog().GetParent());
        GetDialog().SetPosClip(oParentWindow.ToScreen(oParentWindow.GetClientSize().GetCenterOf(GetDialog().GetSize())));  //
        // Xgr[փf[^Zbg
        CListview oList1(GetDialog().GetCtrl(IDC_LIST1));
        oList1.SetExListStyle(0x00000020); // LVS_EX_FULLROWSELECT
        oList1.AddColumn(L"", 24);
        oList1.AddColumn(L"t@C", oList1.GetValidClientSize().cx - 24);
        CFileNameModuleDir historyDir(m_rAppWindow.GetConfig().File(L"HistoryDir"));
        for (int i = 0, n = m_rAppWindow.GetConfig().Num(L"HistoryMax"); i < n; i++) {
            try {
                CFileName oFileName(string::Format(L"%02d.gvb", i), historyDir);
                vector<string> vct = CFile(oFileName).LoadStr().Split(Array<string>(L"\r\n", L"\n", L"\r"));
                int t = 0;
                if (vct[t] != L"[gvbookmark/ver:1]") {
                    continue;
                }
                t++;
                if (vct[t] != L"current:") {
                    continue;
                }
                t++;
                string strCurrent = vct[t];
                int index = oList1.AddItem(string::Format(L"%02d", i));
                oList1.SetSubItem(index, 1, strCurrent);
            } catch (...) {
                // ĎցB
            }
        }
        if (0 < oList1.GetCount()) {
            oList1.SetSelected(0);
        }
        return true;
    } else if (rPrms.GetMsg() == WM_CLOSE) {
        GetDialog().Delete();
        return true;
    } else {
        return false;
    }
}

//////////////////////////////////////////////////////////////////////

MsgDialog::MsgDialog() : CResourceDialogBase(IDD_MSG) {
}

MsgDialog::~MsgDialog() {
}

void MsgDialog::SetCaption(const string& caption) {
    GetDialog().SetText(caption);
}

void MsgDialog::SetText(const string& msg) {
    GetDialog().Static(IDC_STATIC1).SetText(msg);
}

bool MsgDialog::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_COMMAND) {
        switch (rPrms.GetWPrmLoWord()) {
        case IDCANCEL:
            GetDialog().SetResult(DialogResult::Cancel);
            GetDialog().Close();
            break;
        }
        return false;
    } else if (rPrms.GetMsg() == WM_INITDIALOG) {
        CWnd oParentWindow(GetDialog().GetParent());
        GetDialog().SetPosClip(oParentWindow.ToScreen(oParentWindow.GetClientSize().GetCenterOf(GetDialog().GetSize())));  //
        return true;
    } else if (rPrms.GetMsg() == WM_CLOSE) {
        GetDialog().Delete();
        return true;
    } else {
        return false;
    }
}

//////////////////////////////////////////////////////////////////////


