#pragma once
#include <Themis/Windows/ResourceDialogBase.h>
#include <Themis/Config.h>
#include <Themis/MsgHook.h>
#include <Themis/Spi.h>
#include <boost/optional.hpp>
namespace Themis {
class CPlaneEx;
}
class CAppWindow;

//////////////////////////////////////////////////////////////////////

class CImageInfoDialog: public CResourceDialogBase {
public:
    CImageInfoDialog();
    ~CImageInfoDialog();

    void SetText(const vector<string>& vctText);

private:
    bool HookMsg(CMsgHookPrms& rPrms);

    vector<string> m_vctText;
};

//////////////////////////////////////////////////////////////////////

class CVersionDialog: public CResourceDialogBase {
public:
    CVersionDialog(const string& env);
    ~CVersionDialog();

private:
    bool HookMsg(CMsgHookPrms& rPrms);
    string env;
};

//////////////////////////////////////////////////////////////////////

class CRotDialog: public CResourceDialogBase {
public:
    CRotDialog(CAppWindow& r);
    ~CRotDialog();

private:
    bool HookMsg(CMsgHookPrms& rPrms);

private:
    bool            m_bEnable;
    double          m_dAngle;
    CAppWindow&     m_rAppWindow;
};


//////////////////////////////////////////////////////////////////////


class CStretchDialog: public CResourceDialogBase {
public:
    CStretchDialog(const CSize& szSrc, bool bKeepRate);
    ~CStretchDialog();

    const boost::optional<CSize>& GetResult() const { return m_oSize; }
    bool IsKeepRate() const { return m_bKeepRate; }

private:
    bool HookMsg(CMsgHookPrms& rPrms);

private:
    CSize                   m_oSrcSize;
    bool                    m_bKeepRate;
    boost::optional<CSize>  m_oSize;
};


//////////////////////////////////////////////////////////////////////

class CLevelRevDialog: public CResourceDialogBase {
public:
    CLevelRevDialog(CAppWindow& r, const CRect& rc, bool bFilter);
    ~CLevelRevDialog();

private:
    bool HookMsg(CMsgHookPrms& rPrms);
    void UpdateImage();

private:
    CAppWindow&         m_rAppWindow;
    CRect               m_rcArea;
    bool                m_bFilter;
    std::tr1::shared_ptr<CPlaneEx> m_pGraphPlane;
};


//////////////////////////////////////////////////////////////////////

class CGaussDialog: public CResourceDialogBase {
public:
    CGaussDialog(CAppWindow& r, const CRect& rc, bool bFilter, bool gaussOrUnsharp);
    ~CGaussDialog();

private:
    bool HookMsg(CMsgHookPrms& rPrms);
    void UpdateImage();

private:
    CAppWindow&         m_rAppWindow;
    CRect               m_rcArea;
    bool                m_bFilter;
    bool                gaussOrUnsharp;
    string              configName;
};

//////////////////////////////////////////////////////////////////////

class CGammaRevDialog: public CResourceDialogBase {
public:
    CGammaRevDialog(CAppWindow& r, const CRect& rc, bool bFilter);
    ~CGammaRevDialog();

private:
    bool HookMsg(CMsgHookPrms& rPrms);
    void UpdateImage();
    void UpdateGraph();

private:
    CAppWindow&         m_rAppWindow;
    CRect               m_rcArea;
    bool                m_bFilter;
    std::tr1::shared_ptr<CPlaneEx> m_pGraphPlane;
};


//////////////////////////////////////////////////////////////////////

class CBrightContrastDialog: public CResourceDialogBase {
public:
    CBrightContrastDialog(CAppWindow& r, const CRect& rc, bool bFilter);
    ~CBrightContrastDialog();

private:
    bool HookMsg(CMsgHookPrms& rPrms);
    void UpdateImage();

private:
    CAppWindow&     m_rAppWindow;
    CRect           m_rcArea;
    bool            m_bFilter;
};

//////////////////////////////////////////////////////////////////////

class CMosaicDialog: public CResourceDialogBase {
public:
    CMosaicDialog(CAppWindow& r, const CRect& rc);
    ~CMosaicDialog();

private:
    bool HookMsg(CMsgHookPrms& rPrms);
    void UpdateImage();

private:
    CAppWindow&     m_rAppWindow;
    CRect           m_rcArea;
};

//////////////////////////////////////////////////////////////////////

class CSlideShowDialog: public CResourceDialogBase {
public:
    CSlideShowDialog(CAppWindow& r);
    ~CSlideShowDialog();

    bool IsOk() const { return m_bOk; }

private:
    bool HookMsg(CMsgHookPrms& rPrms);
    void LoadFromConfig();
    void SaveToConfig();

private:
    CAppWindow&     m_rAppWindow;
    bool            m_bOk;
};

//////////////////////////////////////////////////////////////////////

class CAlphaDialog: public CResourceDialogBase {
public:
    CAlphaDialog(CAppWindow& r, const CRect& rc, bool bFilter);
    ~CAlphaDialog();

private:
    bool HookMsg(CMsgHookPrms& rPrms);
    void UpdateImage();

private:
    CAppWindow&     m_rAppWindow;
    CRect           m_rcArea;
    COLORREF        m_rgbBgColor;
    bool            m_bFilter;
};

//////////////////////////////////////////////////////////////////////

class CPassDialog: public CResourceDialogBase {
public:
    CPassDialog(CAppWindow& r, const CFileName& archiveName);
    ~CPassDialog();

    bool IsOk() const { return m_bOk; }
    string GetPass() const { return m_pass; }

private:
    bool HookMsg(CMsgHookPrms& rPrms);
    void LoadFromConfig();
    void SaveToConfig();
    void UpdateCtrl();

private:
    CAppWindow&     m_rAppWindow;
    CFileName       archiveName;
    bool            m_bOk;
    string          m_pass;
};


//////////////////////////////////////////////////////////////////////

class CHistoryDialog: public CResourceDialogBase {
public:
    CHistoryDialog(CAppWindow& r);
    ~CHistoryDialog();

    bool IsOk() const { return m_bOk; }
    int GetNo() const { return m_no; }

private:
    bool HookMsg(CMsgHookPrms& rPrms);

private:
    CAppWindow&     m_rAppWindow;
    bool            m_bOk;
    int             m_no;
};

//////////////////////////////////////////////////////////////////////

/// bZ[W_CAOBLZ{^GetResult() == DialogResult::Cancel
class MsgDialog : public CResourceDialogBase {
public:
    MsgDialog();
    ~MsgDialog();

    void SetCaption(const string& caption);
    void SetText(const string& msg);

private:
    bool HookMsg(CMsgHookPrms& rPrms);
};

//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////

