#pragma once
#include <Themis/Image/ThreadedImageLoader.h>
#include <Themis/PageLoaderBase.h>
#include <Themis/Archive.h>
#include <Themis/Graphics.h>
#include "FileList.h"
namespace Themis {
class ILayout;
class CConfig;
class CImageLoader;
class CImageFilter;
class CLocalExclusion;
}
class IDocView;

/**
    t@CXg̊ǗƂ摜Ƃ`ƂsB

    J\@\Aǁ[ƋɂBB
    \̓I}PIɎ̃y[W\Ă܂݂ȊH
    ȎƁALbVȂꍇɂ₱ɂȂ邩BB
*/
class CDocument : private CPageLoaderBase, private IQueryArchivePassword, private ThreadedImageLoader::IOwner {
public:
    CDocument(IDocView& v, IQueryArchivePassword& q, HWND hwndOwner, CConfig& r);
    ~CDocument();

    /// RtBOlύXʒm
    void OnChangeConfig(bool update = true);

    /// EBhETCY̐ݒiς邽тɌĂԂׂj
    void SetScreen(const CSize& szScreen);
    /// XN[ʒũZbg
    void SetScrollPos(const CPoint& pt);

    //@{ t@Cj[n
    void Clear();
    void Open(const CFileName& oFileName, MsgDialog* msgDialog);
    void Open(const vector<CFileName>& vct, MsgDialog* msgDialog);
    void Delete(HWND hwndParent);
    void DeleteByList();
    void Rename(const CFileName& oFileName);
    void Copy(const CFileName& oFileName);
    //@}

    //@{ 摜j[n(ݕ\Ă摜ɑ΂摜Ƃ)
    void Reload(bool clearFailed = false);
    void MirrorH();
    void MirrorV();
    void Rot90();
    void Rot90A();
    void Rot180();
    void Rot(double dAngle);
    void RotDraft(double dAngle);
    void Stretch(const CSize& sz);
    void Cut(const CRect& rc);
    void CopyToClipboard(const CRect& rc);
    void Paste();
    void Fill(COLORREF rgbColor, const CRect& rc);
    void Reverse(const CRect& rc);
    void Mosaic(const CRect& rc, int nLevel, int nType);
    void LevelRev(const CRect& rc, int nLow, int nHigh);
    void GammaRev(const CRect& rc, double dGamma);
    void BrightContrast(const CRect& rc, int nLow, int nHigh);
    void Gray(const CRect& rc, bool bStrict);
    void Befog(const CRect& rc);
    void HardBefog(const CRect& rc);
    void Sharp(const CRect& rc);
    void HardSharp(const CRect& rc);
    void GaussBefog(const CRect& rc, double r);
    void UnsharpMask(const CRect& rc, double r);
    void MedianFilter(const CRect& rc);
    void Alpha(const CRect& rc, COLORREF rgbBgColor, bool bCheck);
    void SaveJpg(const CFileName& oFileName, const CRect& rc);
    void SavePng(const CFileName& oFileName, const CRect& rc);
    void SaveBmp(const CFileName& oFileName, const CRect& rc);
    void SaveArgb(const CFileName& oFileName, const CRect& rc);
    void SetToWallpaper(const string& strDir, bool bUnifyFile, const string& strUnifyFileName, bool bResize);
    //@}

    //@{ t@CXgn
    void ToFirst() { ToIndirect(0); }
    void ToIndirect(int i);
    void ToLast();
    void ToPrior(int nCount = -1, bool update = true);
    void ToNext(int nCount = -1, bool update = true);
    void ToPriorDir();
    void ToNextDir();
    void ToPriorMarkFile();
    void ToNextMarkFile();
    void ToSortedNext();
    void ToSortedPrior();
    void Remove();
    void Sort(int nSortMode);
    void Shuffle();
    void ShuffleDir();
    void LoadHistory(int n);
    void SaveBookmark(const CFileName& oFileName, bool rememberFileName = true);
    void LoadBookmark(const CFileName& oFileName, bool rememberFileName = true);
    const CFileName& GetBookmarkFileName() const { return bookmarkFileName; }
    void SetMark(bool mark);
    void ReverseAllMarks();
    void ReverseFirstToCurrentMarks();
    void ClearMarks();
    int GetMarkCount() const { return m_pFileList->GetMarkCount(); }
    vector<string> GetMarkedFiles() const { return m_pFileList->GetMarkedFiles(); }
    //@}

    //@{ WM_PAINT̎̕`揈
    void Draw(PAINTSTRUCT& ps);
    void DrawMark(PAINTSTRUCT& ps);
    void DrawFilterMark(PAINTSTRUCT& ps);
    void DrawSelectFrame(PAINTSTRUCT& ps, const CRect& rc);
    void DrawStretch(PAINTSTRUCT& ps, const CRect& rcDst, const CRect& rcSrc);
    //@}


    //@{ 摜ǂݍ܂Ă̂ǂ
    bool IsReady() const { return m_pFileList->IsValidIndex() && IsReadyImage(); }
    bool IsReadyImage() const { return GetCurrentImage()->IsCreated(); }
    //@}

    // t@CXg֌W
    int GetFileIndex() const { return m_pFileList->GetCurrentIndex(); }
    int GetFileCount() const { return m_pFileList->GetCount(); }

    //@{ ݂̃t@C̏擾
    CFileName GetCurrentFileName() const {
        return IsReady() ? m_pFileList->GetViewName() : CFileNameSystemDir(L"t@C.bmp");
    }
    CFileName GetCurrentExistFileName() const { return m_pFileList->GetExistFileName(); }
    size_t GetCurrentFileSize() const { return m_pFileList->GetFileSize(); }
    string GetCurrentLastUpdate() const { return m_pFileList->GetLastUpdate(); }
    COLORREF GetAverageColor(const CRect& rc) const;
    int GetColorCount(const CRect& rc) const;
    string GetFileMD5() const;
    bool IsMarked() const { return m_pFileList->IsMarked(); }
    std::tr1::shared_ptr<CFileInfo> GetFileInfoClone(int i) { return m_pFileList->GetFileInfoClone(i); }
    //@}
    /// J\ɂȂĂ̂
    bool IsViewSpreaded() const { return 0 <= GetCurrentPage().m_nIndex2; }
    /// ŏ̃t@CȂ̂ǂ
    bool IsFirstFile() const;
    /// Ō̃t@CȂ̂ǂ
    bool IsLastFile() const;

    //@{ IuWFNg̎擾Ƃ
    CSize GetImageSize() const { return GetCurrentImage()->GetSize(); }
    string GetImageTypeName() const { return GetCurrentPage().m_strTypeName; }
    bool HasImageAlpha() const { return GetCurrentPage().m_bAlpha; }
    const std::tr1::shared_ptr<ILayout>& GetLayout() const { return m_pLayout; }
    CFileList& GetFileList() { return *m_pFileList; }
    //@}

    //@{ obNAbv쐬蕜A
    void BackupBegin();
    void BackupRestore();
    void BackupEnd();
    void BackupEndWithRestore();
    //@}
    /// x␳_CAOpqXgO}̍쐬
    std::tr1::shared_ptr<CPlaneEx> GetHistogramGraph(const CRect& rc, const CSize& szDst, double& dAverage, double& dStdEvp);
    /// qXgO}̃f[^o
    void SaveHistogramGraph(const CRect& rc, const CFileName& oFileName);

    /// NCAgW摜̍W֕ϊ
    CRect ClientToImage(const CRect& rc) const;

    //@{ o[Wp̏擾FX
    uint64_t GetNowFreeMemory();
    int GetNowCacheCount() const { return GetCacheCount(); }
    bool IsNowStretchCaching() const { return useStretchCache; }
    bool IsNowLoaderCaching() const { return !threadedImageLoader->IsCacheless(); }
    //@}

private:
    bool InnerSetScreen(const CSize& szScreen);

    /// ɂ̃pX[h₢킹B
    bool QueryPassword(const CFileName& archiveName, string& pass, int tryCount);

    // ThreadedImageLoaderȏ
    std::tr1::shared_ptr<CImageLoader> CreateImageLoader();
    std::tr1::shared_ptr<CFileData> LoadFile(ThreadedImageLoader::IFileID& fileName);
    void RegisterPreLoad(int n);

    struct SFileID : public ThreadedImageLoader::IFileID {
        std::tr1::shared_ptr<CFileInfo> info;
        SFileID(const std::tr1::shared_ptr<CFileInfo>& p) : info(p) {}
        bool Equals(const IFileID& r) const { return *info == *((SFileID&)r).info; }
    };

    struct SPageData : public CPageLoaderBase::IPageData {
        int                 m_nIndex1;      // Ẽy[W̃t@CCfbNX
        int                 m_nIndex2;      // ̃y[W̃t@CCfbNXB-1ȂPy[W
        std::tr1::shared_ptr<CPlaneEx> m_pPlane;
        std::tr1::shared_ptr<CPlaneEx> stretchCache;   // kLbV
        string              m_strTypeName;
        bool                m_bAlpha;
        bool                filtered;
        SPageData() {}
        /// m_nIndex2 == -1 Ȃ m_nIndex1 AłȂ m_nIndex2 Ԃ
        int GetIndex2Safe() const { return m_nIndex2 < 0 ? m_nIndex1 : m_nIndex2; }
        /// ̃LbṼCfbNX̎擾
        int GetNextIndex(CFileList& r) const;
        /// ÕLbṼCfbNX̎擾
        int GetPriorIndex(CFileList& r) const;
        /// LIndexlĂ̂ǂ
        bool IsReadyIndex() const { return 0 <= m_nIndex1 || 0 <= m_nIndex2; }
        /// indexl̐ݒ
        void SetIndex(int n1, int n2) { m_nIndex1 = n1; m_nIndex2 = n2; }
    };
    std::tr1::shared_ptr<CPageLoaderBase::IPageData> CreatePage();
    void ClearPage(CPageLoaderBase::IPageData& rPage);

    std::tr1::shared_ptr<CPageLoaderBase::ILoadData> PrepareLoadPage(CPageLoaderBase::IPageData& rPage);
    bool LoadPage(CPageLoaderBase::ILoadData& rLoad);
    void DisposeLoadPage(CPageLoaderBase::IPageData& rPage, CPageLoaderBase::ILoadData& rLoad);

    void PrepareCurrentPage(CPageLoaderBase::IPageData& rPage);
    void PrepareForwardPage(CPageLoaderBase::IPageData& rPage, CPageLoaderBase::IPageData& rNext);
    void PrepareBackwardPage(CPageLoaderBase::IPageData& rPage, CPageLoaderBase::IPageData& rPrior);
    bool CreateLoader(std::tr1::shared_ptr<CImageLoader>& pImageLoader, int& nIndex, bool bForward);
    std::tr1::shared_ptr<CPlaneEx> CreatePagePlane();
    std::tr1::shared_ptr<CPlaneEx> CreatePagePlane(const CSize& sz) {
        std::tr1::shared_ptr<CPlaneEx> p = CreatePagePlane(); p->SetSize(sz); return p;
    }

    /// ݂̃y[WύXꂽ
    void CurrentPageChanged();

    bool IsLoadableFileSize(size_t nFileSize);
    bool IsLoadableImage(const CImageLoader& rLoader);
    void CreateStretchCache(std::tr1::shared_ptr<CPlaneEx>& stretchCache, std::tr1::shared_ptr<CPlaneEx>& pPlane);

    void Stretch(std::tr1::shared_ptr<CPlaneEx>& pDst, std::tr1::shared_ptr<CPlaneEx>& pSrc, const CSize& szDst);
    void Stretch(CPlaneEx& rDst, CPlaneEx& rSrc, const CSize& szDst);

    /// ҏWȂǂŉ摜f[^̓eLbVƕςmȂȂRĂŔjĂ
    void ClearCurrentStretchCache() { GetCurrentPage().stretchCache.reset(); }

    /// 摜̍XV
    void UpdateImage() { UpdateImage(CRect()); }
    void UpdateImage(const CRect& rc);
    void InnerUpdateImage(const CRect& rc);

    void InnerDraw(CDC& dc, const CRect& rcPaint);

    inline int GetDrawMode();
    bool IsDrawModeGdi(int n) { return n <= 1; }

    inline const SPageData& GetCurrentPage() const { return static_cast<SPageData&>(CPageLoaderBase::GetCurrentPage()); }
    inline const SPageData& GetPriorPage() const { return static_cast<SPageData&>(CPageLoaderBase::GetPriorPage()); }
    inline const SPageData& GetNextPage() const { return static_cast<SPageData&>(CPageLoaderBase::GetNextPage()); }
    inline const std::tr1::shared_ptr<CPlaneEx>& GetCurrentImage() const { return GetCurrentPage().m_pPlane; }
    inline SPageData& GetCurrentPage() { return static_cast<SPageData&>(CPageLoaderBase::GetCurrentPage()); }
    inline std::tr1::shared_ptr<CPlaneEx>& GetCurrentImage() { return GetCurrentPage().m_pPlane; }

private:
    IDocView&                   m_rView;
    IQueryArchivePassword&      m_rQueryPass;
    CConfig&                    m_oConfig;

    boost::recursive_mutex      mutex; // t@CXg̔rp
    auto_ptr<CFileList>         m_pFileList;

    auto_ptr<ThreadedImageLoader> threadedImageLoader;

    std::tr1::shared_ptr<CPlaneEx> m_pBackupImage;     // obNAbvpv[
    bool                        backupImageHasAlpha;

    CSize                       m_szScreen;
    std::tr1::shared_ptr<ILayout>  m_pLayout;          // CAEg

    auto_ptr<CFont>             m_pFont;            // }[NƂ̕`ptHg

    auto_ptr<CImageFilter>      m_pImageFilter;     // tB^

    auto_ptr<CLocalExclusion> localExclusion;

    COLORREF                    m_rgbBgColor;

    size_t                      m_nStartMemorySize; // ÑB

    int                         m_historyMaxOld;    // ̌ÂlB

    vector<string>              m_vctArcPass;       // IɎspX[h̃Xg

    CFileName bookmarkFileName; // ubN}[Ñt@C

    bool m_bBackup;          // obNAbvȂtrue

    bool useStretchCache; // kLbVpȂtrue

    bool pageChanged;
};

