#include "stdafx.h"
#include "FileInfo.h"
#include <Themis/FileData.h>
#include <Themis/MD5.h>
#include <Themis/StringScanner.h>
#include <Themis/Utility/StringUtility.h>
#ifndef NDEBUG
# include <math.h>
#endif

CFileInfo::CFileInfo() : loadState(0), marked(false), archiveIndex(-1) {
}

CFileInfo::CFileInfo(const CFileName& oViewName, int nSortMode)
: m_oViewName(oViewName), loadState(0), marked(false), archiveIndex(-1) {
    SetSortMode(nSortMode);
}

CFileInfo::CFileInfo(const string& strViewName, const std::tr1::weak_ptr<CArchive>& a, int ai, int nSortMode)
: m_oViewName(strViewName), loadState(0), marked(false), archive(a), archiveIndex(ai) {
    SetSortMode(nSortMode);
}

CFileInfo::~CFileInfo() {
}

/// CX^X𐶐ĕԂB
std::tr1::shared_ptr<CFileInfo> CFileInfo::Clone() const {
    std::tr1::shared_ptr<CFileInfo> p(new CFileInfo());
    p->m_oViewName = m_oViewName;
    p->m_strSortName = m_strSortName;
    p->archive = archive;
    p->archiveIndex = archiveIndex;
    p->loadState = loadState;
    p->marked = marked;
    return p;
}

//------------------------------------------------------------------//

void CFileInfo::SetSortMode(int nMode) {
    switch (nMode) {
    case 0: // ʏ
        m_strSortName = m_oViewName.Get();
        break;
    case 1: // 啶ʖ
        m_strSortName = m_oViewName.Get().ToLower();
        break;
    case 2: // tH_
        m_strSortName = m_oViewName.GetPureName();
        break;
    case 3: // tH_/啶ʖ
        m_strSortName = m_oViewName.GetPureName().ToLower();
        break;
    case 4: // t@CTCY
        Themis_DebugAssert(int(ceil(log10(double(UINT_MAX)))) == 10);
        m_strSortName = string::Format(L"%010d", GetFileSize());
        break;
    case 5: // ŏIXV
        m_strSortName = GetLastUpdate();
        break;
    case 6: // lBWindowsXPH
        {
            static vector<pair<wchar_t, wchar_t> > vct;
            static bool vectorInitialized = false;
            if (!vectorInitialized) {
                vct.push_back(make_pair(L'O', L'0'));
                vct.push_back(make_pair(L'P', L'1'));
                vct.push_back(make_pair(L'Q', L'2'));
                vct.push_back(make_pair(L'R', L'3'));
                vct.push_back(make_pair(L'S', L'4'));
                vct.push_back(make_pair(L'T', L'5'));
                vct.push_back(make_pair(L'U', L'6'));
                vct.push_back(make_pair(L'V', L'7'));
                vct.push_back(make_pair(L'W', L'8'));
                vct.push_back(make_pair(L'X', L'9'));
                vct.push_back(make_pair(L'D', L'.')); // ܂
                vectorInitialized = true;
            }
            string tmp = Replace(m_oViewName.Get(), vct);
            CStringScanner oScanner(tmp);

            StringBuilder result;
            while (oScanner) {
                if (oScanner.IsDigit()) {
                    string tmpDigits;
                    do { oScanner.Copy1Byte(tmpDigits); } while (oScanner.IsDigit());
                    // 8~nɑ
                    int n = static_cast<int>(8 - (tmpDigits.GetLength() % 8)) % 8;
                    for (int i = 0; i < n; i++) {
                        result.Append(L'0');
                    }
                    result.Append(tmpDigits);
                    // _ȉ̏
                    if (oScanner.IsMatch(L'.')) {
                        string tmpDigits;
                        do { oScanner.Copy1Byte(tmpDigits); } while (oScanner.IsDigit());
                        if (3 < tmpDigits.GetLength()) {
                            tmpDigits = tmpDigits.Substring(0, 3); // 4ȍ~͐؂̂(蔲)
                        }
                        result.Append(L'.');
                        result.Append(tmpDigits);
                    } else {
                        result.Append(L".000");
                    }
                } else {
                    oScanner.Copy1(result);
                }
            }
            m_strSortName = result.ToString();
        }
        break;
    }
}

//------------------------------------------------------------------//

void CFileInfo::Copy(const CFileName& oDstName) {
    std::tr1::shared_ptr<CArchive> arcPtr = archive.lock();
    if (arcPtr == NULL) {
        CFile(m_oViewName).Copy(oDstName);
    } else {
        if (oDstName.IsDirPath()) { // t@CtH_
            Copy(CFileName(m_oViewName.GetPureName() , oDstName));
        } else { // t@Ct@C
            if (CFile(oDstName).Exists()) {
                throw IOException(L"t@C̃Rs[Ɏs܂ (" + m_oViewName.Get() + L"  " + oDstName.Get() + L")");
            }
            CFile(oDstName).Save(arcPtr->LoadFile(archiveIndex)->GetMemory());
        }
    }
}

void CFileInfo::Move(const CFileName& oDstName) {
    std::tr1::shared_ptr<CArchive> arcPtr = archive.lock();
    if (arcPtr == NULL) {
        CFile(m_oViewName).Move(oDstName);
    } else {
        if (oDstName.IsDirPath()) { // t@CtH_
            Move(CFileName(m_oViewName.GetPureName() , oDstName));
        } else { // t@Ct@C
            if (CFile(oDstName).Exists()) {
                throw IOException(L"t@C̈ړɎs܂ (" + m_oViewName.Get() + L"  " + oDstName.Get() + L")");
            }
            CFile(oDstName).Save(arcPtr->LoadFile(archiveIndex)->GetMemory());
        }
    }
}

//------------------------------------------------------------------//

size_t CFileInfo::GetFileSize() const {
    std::tr1::shared_ptr<CArchive> arcPtr = archive.lock();
    return arcPtr != NULL ? arcPtr->GetFileSize(archiveIndex) : CFile(m_oViewName).GetSize();
}

string CFileInfo::GetLastUpdate() const {
    std::tr1::shared_ptr<CArchive> arcPtr = archive.lock();
    FILETIME ft = arcPtr != NULL ? arcPtr->GetLastUpdate(archiveIndex) : CFile(m_oViewName).GetLastUpdate();
    SYSTEMTIME st = {0};
    ::FileTimeToSystemTime(&ft, &st);
    Themis_DebugAssert(st.wMilliseconds < 1000);
    return string::Format(L"%04d/%02d/%02d %02d:%02d:%02d:%03d",
        st.wYear, st.wMonth, st.wDay,
        st.wHour, st.wMinute, st.wSecond, st.wMilliseconds);
}

string CFileInfo::GetFileMD5() const {
    std::tr1::shared_ptr<CFileData> data = LoadFile();
    return CMD5::Make(data->GetMemory());
}

CFileName CFileInfo::GetExistFileName() const {
    std::tr1::shared_ptr<CArchive> arcPtr = archive.lock();
    if (arcPtr == NULL) {
        return m_oViewName;
    } else {
        while (true) { // qɂȏꍇeɂ
            std::tr1::shared_ptr<CArchive> p = arcPtr->GetParent().lock();
            if (p == NULL) break;
            arcPtr = p;
        }
        return arcPtr->GetFileName();
    }
}

string CFileInfo::GetBookmarkDefaultName() const {
    std::tr1::shared_ptr<CArchive> arcPtr = archive.lock();
    return arcPtr != NULL ? arcPtr->GetFileName().GetPrefix() : m_oViewName.GetDirName();
}

std::tr1::shared_ptr<CFileData> CFileInfo::LoadFile() const {
    std::tr1::shared_ptr<CArchive> arcPtr = archive.lock();
    return arcPtr != NULL ? arcPtr->LoadFile(archiveIndex) :
        std::tr1::shared_ptr<CFileData>(new CExistFileData(m_oViewName));
}

