#include "stdafx.h"
#include <Themis/Windows/System.h>
#include <Themis/Thread/ThreadPool.h>
#include <Themis/Thread/Parallel.h>
#include <Themis/tpl/map_vector.h>
#include <Themis/Wnd.h>
#include <Themis/Graphics.h>
#include <Themis/File.h>
#include <Themis/Rand.h>
#include <Themis/Serialize.h>
#include <Themis/Image.h> // CImageLoader::GetSupportedSuffixesLc()
#include "FileListWindow.h"
#include "FileList.h"

namespace {
    template<class T> void Reserve(boost::ptr_vector<T>& container, size_t n) {
        container.reserve(n);
    }
    template<class T> void Reserve(boost::ptr_deque<T>& container, size_t n) {
    }
}

CFileList::CFileList(const ArchiveOwner& owner)
: m_pListWindow(NULL), m_nSortMode(-1), m_nCurrentIndex(-1) {
    m_pArchiveLoader.reset(new CArchiveLoader(owner));
}

CFileList::~CFileList() {
    Clear();
}

void CFileList::SetWindow(CFileListWindow* pWindow) {
    m_pListWindow = pWindow;
}

void CFileList::SetTempDir(const CFileName& o) {
    m_pArchiveLoader->SetTempDir(o);
}

void CFileList::SetSortMode(int nMode) {
    if (m_nSortMode != nMode) {
        m_nSortMode = nMode;
        InnerSetSortMode(nMode);
        // \[gƂBcvCȂłȂBB
        if (0 <= m_nCurrentIndex && m_nCurrentIndex < int(m_vctFiles.size())) {
            string strTemp = m_vctFiles[m_nCurrentIndex].GetViewName().Get();
            InnerSort();
            m_nCurrentIndex =  Find(strTemp);
        }
    }
}

void CFileList::InnerSetSortMode(int mode) {
    Parallel& pp = Parallel::GetInstance();
    pp.For(std::tr1::bind(&CFileList::InnerSetSortModePP, this, mode, _1, _2),
        0, static_cast<int>(m_vctFiles.size()));
}

//------------------------------------------------------------------//

void CFileList::Clear() {
    m_vctFiles.clear();
    m_pArchiveLoader->Clear();
    if (m_pListWindow != NULL) {
        m_pListWindow->UpdateAll();
    }
}

namespace {
    template<class C> void std_sort_and_unique(C& c) {
        std::sort(c.begin(), c.end()); 
        c.erase(std::unique(c.begin(), c.end()), c.end());
    }
}
void CFileList::AddInitialize(const AddConfig& config) {
    if (config.followLnk) {
        m_pFollowLnkList.reset(new std::set<string>);
    }
    if (config.ignoreBySuffix) {
        // 
        for (size_t i = 0, n = config.includeSuffixes.size(); i < n; i++) {
            config.includeSuffixes[i] = config.includeSuffixes[i].ToLower();
        }
        // 摜̂ǉ
        vector<string> vct = CImageLoader::GetSupportedSuffixesLc();
        config.includeSuffixes.insert(config.includeSuffixes.end(), vct.begin(), vct.end());
        // ɂ̂ǉ
        vct = CArchiver::GetSupportedSuffixesLc();
        config.includeSuffixes.insert(config.includeSuffixes.end(), vct.begin(), vct.end());
        
    }
    // 
    for (size_t i = 0, n = config.excludeSuffixes.size(); i < n; i++) {
        config.excludeSuffixes[i] = config.excludeSuffixes[i].ToLower();
    }
    std_sort_and_unique(config.includeSuffixes);
    std_sort_and_unique(config.excludeSuffixes);
}

void CFileList::Add(const CFileName& oFileName, const AddConfig& config, MsgDialog* msgDialog) {
    AddInitialize(config);

    size_t sizeBefore = m_vctFiles.size();
    if (oFileName.IsDirPath()) {
        // tH_
        InnerAdd(oFileName, config, false);
    } else {
        vector<CArchive::SFileInfo> vctFiles;
        if (m_pArchiveLoader->Add(oFileName, vctFiles)) {
            // 
            InnerAddArchive(config, vctFiles);
        } else {
            // P̃t@CF̃t@ĈtH_̃t@CSĒǉ
            vector<CFileName> vct = CFile::FindFile(CFileName(oFileName.GetDir()), 0);
            InnerAddPP(vct, config, true);
        }
    }
    UniqueFiles();
    if (sizeBefore < m_vctFiles.size()) { // TCYĂ炻\[g
        InnerSort(m_vctFiles.begin() + sizeBefore, m_vctFiles.end());
    }
    m_nCurrentIndex = m_vctFiles.empty() ? -1 : Find(oFileName.Get());
    if (m_pListWindow != NULL) {
        m_pListWindow->UpdateAll();
    }
    m_pFollowLnkList.reset();
}

void CFileList::Add(const vector<CFileName>& vct, const AddConfig& config, MsgDialog* msgDialog) {
    AddInitialize(config);

    Themis_DebugAssert(!vct.empty());
    if (vct.size() == 1) {
        Add(vct[0], config, msgDialog);
    } else {
        size_t sizeBefore = m_vctFiles.size();
        InnerAddPP(vct, config, false);
        UniqueFiles();
        if (sizeBefore < m_vctFiles.size()) { // TCYĂ炻\[g
            InnerSort(m_vctFiles.begin() + sizeBefore, m_vctFiles.end());
        }
        m_nCurrentIndex = m_vctFiles.empty() ? -1 : Find(vct[0].Get());
        if (m_pListWindow != NULL) {
            m_pListWindow->UpdateAll();
        }
    }
    m_pFollowLnkList.reset();
}

void CFileList::InnerAddPP(const vector<CFileName>& vct, const AddConfig& config, bool bOnlySingleFile) {
    /*
    Parallel& pp = Parallel::GetInstance();
    pp.For(std::tr1::bind(&CFileList::InnerAddPP, this, vct, _1, _2, config, bOnlySingleFile),
        0, static_cast<int>(vct.size()));
    */
    InnerAddPP(vct, 0, static_cast<int>(vct.size()), config, bOnlySingleFile);
}

void CFileList::InnerAdd(const CFileName& oFileName, const AddConfig& config, bool bOnlySingleFile) {
    if (oFileName.IsDirPath()) {
        // tH_(̃tH_ȉSẴt@Cǉ)
        vector<CFileName> vctFiles = CFile::FindFile(CFileName(oFileName.Get()), config.openTopDirOnly ? 0 : -1);
        Reserve(m_vctFiles, m_vctFiles.size() + vctFiles.size());
        for (size_t i = 0, n = vctFiles.size(); i < n; i++) {
            InnerAdd(vctFiles[i], config, false);
        }
    } else {
        // gqɂ閳B
        if (ToBeIgnore(config, oFileName)) {
            return; // I
        }

        vector<CArchive::SFileInfo> vctFiles;
        if ((bOnlySingleFile && m_pArchiveLoader->IsReadable(oFileName)) ||
            (!bOnlySingleFile && m_pArchiveLoader->Add(oFileName, vctFiles))) {
            // 
            if (!bOnlySingleFile) {
                InnerAddArchive(config, vctFiles);
            }
        } else if (config.followLnk && oFileName.GetSuffixLc() == L"lnk") {
            try {
                CSystem::SShortcutInfo oInfo = CSystem::GetShortcutInfo(oFileName);
                if (m_pFollowLnkList->insert(oInfo.strTarget).second) {
                    // `FbNƂΖċA͋NȂ͂BB
                    CFileName target = CFile::SetDirSeparator(CFileName(oInfo.strTarget));
                    if (!bOnlySingleFile || !CFile(target).ExistsDir()) {
                        // bOnlySingleFileŃN悪fBNgȏꍇ͖B
                        InnerAdd(target, config, bOnlySingleFile);
                    }
                }
            } catch (std::exception&) {
                // Ôߒʏ̃t@CƂĒǉĂ
                m_vctFiles.push_back(new CFileInfo(oFileName, m_nSortMode));
            }
        } else {
            // ʏ̃t@C
            m_vctFiles.push_back(new CFileInfo(oFileName, m_nSortMode));
        }
    }
}

void CFileList::InnerAddArchive(const AddConfig& config, const vector<CArchive::SFileInfo>& vct) {
    // ɁF̃t@CSĒǉ
    /*
    // Ȃ񂩂ɎԂĈŕ񉻂Ă݂
    Parallel& pp = Parallel::GetInstance();
    pp.For(std::tr1::bind(&CFileList::InnerAddArchivePP, this, std::tr1::cref(config), std::tr1::cref(vct), _1, _2),
        0, static_cast<int>(vct.size()));
    /*/
    Reserve(m_vctFiles, m_vctFiles.size() + vct.size());
    InnerAddArchivePP(config, vct, 0, static_cast<int>(vct.size()));
    //*/
}

void CFileList::InnerAddArchivePP(const AddConfig& config, const vector<CArchive::SFileInfo>& vct, int begin, int end) {
    for (int i = begin; i < end; i++) {
        if (ToBeIgnore(config, CFileName(vct[i].strViewName))) {
            continue;
        }
        m_vctFiles.push_back(new CFileInfo(
            vct[i].strViewName, vct[i].pArchive, vct[i].nArchiveIndex, m_nSortMode));
    }
}

bool CFileList::ToBeIgnore(const AddConfig& config, const CFileName& fileName) {
    if (config.ignoreBySuffix) {
        string lc = fileName.GetSuffixLc();
        if (std::find(config.includeSuffixes.begin(), config.includeSuffixes.end(), lc) == config.includeSuffixes.end() ||
            std::find(config.excludeSuffixes.begin(), config.excludeSuffixes.end(), lc) != config.excludeSuffixes.end()) {
            Themis_DebugLog(L"gqɂ閳(1): " + fileName.Get());
            return true;
        }
    } else if (!config.excludeSuffixes.empty()) {
        string lc = fileName.GetSuffixLc();
        if (std::find(config.excludeSuffixes.begin(), config.excludeSuffixes.end(), lc) != config.excludeSuffixes.end()) {
            Themis_DebugLog(L"gqɂ閳(2): " + fileName.Get());
            return true;
        }
    }
    return false;
}

void CFileList::InsertDummyFile() {
    m_vctFiles.insert(m_vctFiles.begin() + m_nCurrentIndex,
        new CFileInfo(CFileName(L"<dummy-file>"), m_nSortMode));
}

void CFileList::InsertExistFile(const CFileName& oFileName, bool bNext) {
    Themis_DebugAssert(CFile(oFileName).Exists());
    if (GetViewName() != oFileName) {
        if (bNext) { // ݂̂̎֒ǉ
            m_vctFiles.insert(m_vctFiles.begin() + m_nCurrentIndex + 1,
                new CFileInfo(oFileName, m_nSortMode));
        } else { // Ō֒ǉ
            m_vctFiles.push_back(new CFileInfo(oFileName, m_nSortMode));
        }
        if (m_pListWindow != NULL) {
            m_pListWindow->UpdateAll();
        }
    }
}

//------------------------------------------------------------------//

void CFileList::Remove() {
    m_vctFiles.erase(m_vctFiles.begin() + m_nCurrentIndex);
    m_nCurrentIndex = m_vctFiles.empty() ? -1 : GetRotatedIndexOnInc(m_nCurrentIndex, GetCount());
    if (m_pListWindow != NULL) {
        m_pListWindow->UpdateAll();
    }
}

void CFileList::Remove(const CFileName& oViewName) {
    string strTemp = m_vctFiles[m_nCurrentIndex].GetViewName().Get();
    m_vctFiles.erase(std::find(m_vctFiles.begin(), m_vctFiles.end(), oViewName.Get()));
    m_nCurrentIndex = m_vctFiles.empty() ? -1 : Find(strTemp);
    if (m_pListWindow != NULL) {
        m_pListWindow->UpdateAll();
    }
}

void CFileList::Delete(bool deleteByShell, HWND hwndParent) {
    CFile oFile(m_vctFiles[m_nCurrentIndex].GetViewName());
    if (oFile.Exists()) {
        if (deleteByShell) {
            CSystem::DeleteFile(oFile.GetName(), false, hwndParent);
        } else {
            oFile.Delete();
        }
    }
    Remove();
}

namespace {
    /// Rs[ȃXbhB
    static void CopyThread(std::tr1::shared_ptr<CFileInfo> info, CFileName dstName) {
        info->Copy(dstName);
    }
}

void CFileList::Copy(const CFileName& oNewName) {
    ThreadPool::Start(std::tr1::bind(&CopyThread, GetFileInfoClone(m_nCurrentIndex), oNewName));
}

void CFileList::Rename(const CFileName& oNewName, bool bRemoveOnMove) {
 // m_vctFiles[m_nCurrentIndex].SetMark(false); // }[N
    m_vctFiles[m_nCurrentIndex].Move(oNewName);
    if (bRemoveOnMove) {
        Remove();
    } else {
        m_vctFiles[m_nCurrentIndex].SetViewName(oNewName);
        if (m_pListWindow != NULL) {
            m_pListWindow->UpdateAll();
        }
    }
}

void CFileList::CopyAllMarks(const CFileName& oDir) {
    string strError;
    for (size_t i = 0, n = m_vctFiles.size(); i < n; i++) {
        try {
            if (m_vctFiles[i].IsMarked()) {
                m_vctFiles[i].Copy(oDir);
             // ThreadPool::Start(std::tr1::bind(&CopyThread, GetFileInfo(int(i)), oDir));
            }
        } catch (std::exception& e) {
            strError += Encoding::GetDefault().GetString(e.what());
            strError += L"\r\n";
        }
    }
    if (!string::IsNullOrEmpty(strError)) {
        throw RuntimeException(strError);
    }
}

void CFileList::RenameAllMarks(const CFileName& oDir, bool bRemoveOnMove) {
    int nIndex2 = m_nCurrentIndex;
    string strError;
    for (size_t i = 0; i < m_vctFiles.size(); i++) {
        try {
            if (m_vctFiles[i].IsMarked()) {
                m_vctFiles[i].Move(oDir);
             // m_vctFiles[i].SetMark(false); // }[N
                if (bRemoveOnMove) {
                    m_vctFiles.erase(m_vctFiles.begin() + i);
                    if (int(i) < m_nCurrentIndex) { // ݈ʒuOō폜ꂽ炻̕߂
                        nIndex2--;
                    }
                    i--; //  Ȃ񂩃_TC邯ǎ蔲dlƂƂŁBorz
                } else {
                    m_vctFiles[i].SetViewName(CFileName(oDir.Get() + m_vctFiles[i].GetViewName().GetPureName()));
                }
            }
        } catch (std::exception& e) {
            strError += Encoding::GetDefault().GetString(e.what());
            strError += L"\r\n";
        }
    }
    m_nCurrentIndex = m_vctFiles.empty() ? -1 : GetRotatedIndex(nIndex2, GetCount());
    if (!string::IsNullOrEmpty(strError)) {
        throw RuntimeException(strError);
    }
    if (m_pListWindow != NULL) {
        m_pListWindow->UpdateAll();
    }
}

void CFileList::RemoveAllMarks(bool bDelete, bool deleteByShell, HWND hwndParent) {
    int nIndex2 = m_nCurrentIndex;
    // t@CXgč\z
    vector<CFileName> vctDelete;
    FileListType vctFiles2;
    for (size_t i = 0, n = m_vctFiles.size(); i < n; i++) {
        if (m_vctFiles[i].IsMarked()) {
            if (!m_vctFiles[i].IsArchiveFile()) {
                vctDelete.push_back(m_vctFiles[i].GetViewName());
            } else {
                Themis_DebugLog(L"ɓt@C̍폜͖[Ȃ");
            }
            if (int(i) < m_nCurrentIndex) { // ݈ʒuOō폜ꂽ炻̕߂
                nIndex2--;
            }
        } else {
            vctFiles2.push_back(new CFileInfo(m_vctFiles[i]));
        }
    }
    // t@C̍폜
    if (bDelete) {
        vector<CFileName> files;
        for (size_t i = 0, n = vctDelete.size(); i < n; i++) {
            CFile oFile(vctDelete[i]);
            if (oFile.Exists()) {
                if (deleteByShell) {
                    files.push_back(oFile.GetName());
                } else {
                    oFile.Delete(true);
                }
            }
        }
        if (deleteByShell) {
            CSystem::DeleteFile(files, false, hwndParent);
        }
    }
    // o̐ݒ
    m_vctFiles.swap(vctFiles2);
    m_nCurrentIndex = m_vctFiles.empty() ? -1 : GetRotatedIndex(nIndex2, GetCount());
    if (m_pListWindow != NULL) {
        m_pListWindow->UpdateAll();
    }
}

//------------------------------------------------------------------//

void CFileList::Sort(int nSortMode) {
    if (!m_vctFiles.empty()) {
        string strTemp = m_vctFiles[m_nCurrentIndex].GetViewName().Get();
        if (nSortMode == m_nSortMode) {
            InnerSort();
            m_vctFiles.erase(std::unique(m_vctFiles.begin(), m_vctFiles.end()), m_vctFiles.end()); // łɏd폜
        } else {
            // \[g[h̕ύX
            InnerSetSortMode(nSortMode);
            // \[g
            InnerSort();
            m_vctFiles.erase(std::unique(m_vctFiles.begin(), m_vctFiles.end()), m_vctFiles.end()); // łɏd폜
            // \[g[h̕A
            InnerSetSortMode(m_nSortMode);
        }
        m_nCurrentIndex =  Find(strTemp);
        if (m_pListWindow != NULL) {
            m_pListWindow->UpdateAll();
        }
    }
}

void CFileList::Shuffle() {
    if (!m_vctFiles.empty()) {
        UniqueFiles(); // łɏd폜ƂĂ݂B
        std::swap(m_vctFiles[0], m_vctFiles[m_nCurrentIndex]);
        std::random_shuffle(m_vctFiles.begin() + 1, m_vctFiles.end(), CRand::GetInstance());
        m_nCurrentIndex = 0;
        if (m_pListWindow != NULL) {
            m_pListWindow->UpdateAll();
        }
    }
}

#include <Themis/tpl/smart_obj.h>

void CFileList::ShuffleDir() {
    if (!m_vctFiles.empty()) {
        UniqueFiles(); // łɏd폜ƂĂ݂B
        CFileName oOld = m_vctFiles[m_nCurrentIndex].GetViewName();
        map_vector<string, smart_obj<FileListType> > oList;
        for (size_t i = 0, n = m_vctFiles.size(); i < n; i++) {
            oList[m_vctFiles[i].GetViewName().GetDir()]->push_back(new CFileInfo(m_vctFiles[i]));
        }
        std::random_shuffle(oList.begin(), oList.end(), CRand::GetInstance()); // xBB
        m_vctFiles.clear();
        for (size_t i = 0, n = oList.size(); i < n; i++) {
            m_vctFiles.insert(m_vctFiles.begin(), oList[i].second->begin(), oList[i].second->end());
        }
        m_nCurrentIndex = Find(oOld.Get());
        if (m_pListWindow != NULL) {
            m_pListWindow->UpdateAll();
        }
    }
}

//------------------------------------------------------------------//

void CFileList::SetCurrentIndex(int n) {
    m_nCurrentIndex = m_vctFiles.empty() ? -1 : GetRotatedIndex(n, GetCount());
    if (m_pListWindow != NULL) {
        m_pListWindow->UpdateAll();
    }
}

namespace {
    template<class T> static auto_ptr<T> Clone(const T& files) {
        auto_ptr<T> copy(new T());
        for (size_t i = 0, n = files.size(); i < n; i++) {
            copy->push_back(new CFileInfo(files[i]));
        }
        return copy->release();
    }
}

void CFileList::ToSortedPrior() {
    if (!m_vctFiles.empty()) {
        auto_ptr<FileListType> vctTemp = Clone(m_vctFiles);
        vctTemp->sort(vctTemp->begin(), vctTemp->end());
        FileListType::iterator it = std::find(vctTemp->begin(), vctTemp->end(), m_vctFiles[m_nCurrentIndex]);
        if (it == vctTemp->end()) {
            Themis_DebugWarning(L"CFileList::ToSortedNext()ŃoOH");
            m_nCurrentIndex = 0;
        } else {
            if (it == vctTemp->begin()) {
                it = vctTemp->end();
            }
            --it;
            m_nCurrentIndex = Find((*it).GetViewName().Get());
        }
        if (m_pListWindow != NULL) {
            m_pListWindow->UpdateAll();
        }
    }
}

void CFileList::ToSortedNext() {
    if (!m_vctFiles.empty()) {
        auto_ptr<FileListType> vctTemp = Clone(m_vctFiles);
        vctTemp->sort(vctTemp->begin(), vctTemp->end());
        FileListType::iterator it = std::find(vctTemp->begin(), vctTemp->end(), m_vctFiles[m_nCurrentIndex]);
        if (it == vctTemp->end()) {
            Themis_DebugWarning(L"CFileList::ToSortedNext()ŃoOH");
            m_nCurrentIndex = 0;
        } else {
            if (++it == vctTemp->end()) {
                it = vctTemp->begin();
            }
            m_nCurrentIndex = Find((*it).GetViewName().Get());
        }
        if (m_pListWindow != NULL) {
            m_pListWindow->UpdateAll();
        }
    }
}

namespace {
    inline bool IsEqualDir(const string& strDir, const CFileName& oFile) {
        return strDir == oFile.GetDir();
    }
}
void CFileList::ToPriorDir() {
    if (!m_vctFiles.empty()) {
        int t = m_nCurrentIndex;
        string strDir = m_vctFiles[t].GetViewName().GetDir();
        bool bChanged = false;
        for (int i = 0 + 1, n = GetCount(); i < n; i++) {
            t = GetRotatedIndexOnDec(t - 1, n);
            if (!bChanged) {
                if (strDir != m_vctFiles[t].GetViewName().GetDir()) {
                    // Pڂ̃tH_̕ς
                    bChanged = true;
                    strDir = m_vctFiles[t].GetViewName().GetDir();
                }
            } else {
                if (strDir != m_vctFiles[t].GetViewName().GetDir()) {
                    // Qڂ̃tH_̕ς
                    if (m_pListWindow != NULL) {
                        m_pListWindow->UpdateAll();
                    }
                    break;
                } else {
                    m_nCurrentIndex = t;
                }
            }
        }
    }
}
void CFileList::ToNextDir() {
    if (!m_vctFiles.empty()) {
        int t = m_nCurrentIndex;
        string strDir = m_vctFiles[t].GetViewName().GetDir();
        for (int i = 0 + 1, n = GetCount(); i < n; i++) {
            t = GetRotatedIndexOnInc(t + 1, n);
            if (strDir != m_vctFiles[t].GetViewName().GetDir()) {
                m_nCurrentIndex = t; // ႤtH_ɂȂ炻ɂďI
                break;
            }
        }
    }
}

void CFileList::ToPriorMarkFile() {
    if (!m_vctFiles.empty()) {
        int t = m_nCurrentIndex;
        for (int i = 0 + 1, n = GetCount(); i < n; i++) {
            t = GetRotatedIndexOnDec(t - 1, n);
            if (m_vctFiles[t].IsMarked()) {
                m_nCurrentIndex = t; // }[NĂ炻ɂďI
                break;
            }
        }
    }
}
void CFileList::ToNextMarkFile() {
    if (!m_vctFiles.empty()) {
        int t = m_nCurrentIndex;
        for (int i = 0 + 1, n = GetCount(); i < n; i++) {
            t = GetRotatedIndexOnInc(t + 1, n);
            if (m_vctFiles[t].IsMarked()) {
                m_nCurrentIndex = t; // }[NĂ炻ɂďI
                break;
            }
        }
    }
}

//------------------------------------------------------------------//

void CFileList::SaveBookmark(const CFileName& oFileName) const {
    Themis_DebugAssert(0 <= m_nCurrentIndex);
    Themis_DebugAssert(!m_vctFiles.empty());
    StringBuilder str;
    str.Append(L"[gvbookmark/ver:1]\r\n");
    str.Append(L"current:");
    str.Append(L'\r');
    str.Append(L'\n');
    str.Append(m_vctFiles[m_nCurrentIndex].GetViewName().Get());
    str.Append(L'\r');
    str.Append(L'\n');
    str.Append(L"files:");
    str.Append(L'\r');
    str.Append(L'\n');
    for (size_t i = 0, n = m_vctFiles.size(); i < n; i++) {
        str.Append(m_vctFiles[i].GetViewName().Get());
        str.Append(L'\r');
        str.Append(L'\n');
    }
    if (0 < m_pArchiveLoader->GetCount()) {
        str.Append(L"archives:");
        str.Append(L'\r');
        str.Append(L'\n');
        for (int i = 0, n = m_pArchiveLoader->GetCount(); i < n; i++) {
            str.Append(m_pArchiveLoader->GetFileName(i));
            str.Append(L'\r');
            str.Append(L'\n');
        }
    }
    CFile(oFileName).Save(str.ToString());
}

void CFileList::LoadBookmark(const CFileName& oFileName) {
    Clear();
    try {
        string strCurrent;
        if (!LoadBookmark1(oFileName, strCurrent)) {
            if (!LoadBookmark2(oFileName, strCurrent)) {
                throw RuntimeException(L"ubN}[Nt@C '" + oFileName.Get() + L"' ̓ǂݍ݂Ɏs܂");
            } else {
                m_nCurrentIndex = Find(strCurrent);
                // `Ȃ̂ŁAV`ŏ㏑ۑ(ꉞmF)
                if (CWnd().MsgBoxYesNo(L"mF", L"ǂݍ܂ꂽubN}[Nt@C '" + oFileName.Get() + L"' ͌Ât@C`łB\r\nV`ɕϊď㏑ۑ܂H") == IDYES) {
                    SaveBookmark(oFileName);
                }
            }
        } else {
            m_nCurrentIndex = Find(strCurrent);
        }
        if (m_pListWindow != NULL) {
            m_pListWindow->UpdateAll();
        }
    } catch (std::exception& e) {
        Themis_DebugLog(e.what()); e;
        Clear();
    }
}

bool CFileList::LoadBookmark1(const CFileName& oFileName, string& strCurrent) {
    vector<string> vct = CFile(oFileName).LoadStr().Split(Array<string>(L"\r\n", L"\n", L"\r"));
    size_t i = 0;
    if (vct[i] != L"[gvbookmark/ver:1]") {
        return false;
    }
    i++;
    // current
    if (vct[i] != L"current:") {
        return false;
    }
    i++;
    strCurrent = vct[i];
    i++;
    // files
    if (vct[i] != L"files:") {
        return false;
    }
    i++;
    m_vctFiles.clear();
    for (; i < vct.size(); i++) {
        if (vct[i] == L"archives:") {
            i++;
            break;
        }
        if (string::IsNullOrEmpty(vct[i])) continue; // s͔΂
        m_vctFiles.push_back(new CFileInfo(CFileName(vct[i]), m_nSortMode));
    }
    // archives
    for (; i < vct.size(); i++) {
        if (string::IsNullOrEmpty(vct[i])) continue; // s͔΂
        m_pArchiveLoader->AddFileName(CFileName(vct[i]));
    }
    LoadBookmarkSetArchive();
    return true;
}

bool CFileList::LoadBookmark2(const CFileName& oFileName, string& strCurrent) {
    vector<string> vctTemp;
    auto_ptr<IRestorer> r;//(new CRestorer(oFileName))
    r.reset(new CCompressRestorer(oFileName));
    *r << vctTemp;
    *r << strCurrent;
    *r << *m_pArchiveLoader;
    // f[^o֐ݒ
    m_vctFiles.clear();
    Reserve(m_vctFiles, vctTemp.size());
    for (size_t i = 0, n = vctTemp.size(); i < n; i++) {
        m_vctFiles.push_back(new CFileInfo(CFileName(vctTemp[i]), m_nSortMode));
    }
    LoadBookmarkSetArchive();
    return true;
}

void CFileList::LoadBookmarkSetArchive() {
 // Parallel& pp = Parallel::GetInstance();
 // pp.For(std::tr1::bind(&CFileList::LoadBookmarkSetArchivePP, this, _1, _2), 0, static_cast<int>(m_vctFiles.size()));
    LoadBookmarkSetArchivePP(0, static_cast<int>(m_vctFiles.size()));
}

void CFileList::SetLoadState(int i, int nState) {
    if (0 <= i && i < int(m_vctFiles.size())) {
        m_vctFiles[i].SetLoadState(nState);
        if (m_pListWindow != NULL) {
            m_pListWindow->Update(i);
        }
    }
}

/// ǂݍݎsԂNA
void CFileList::ClearFailed() {
    for (size_t i = 0, n = m_vctFiles.size(); i < n; i++) {
        if (m_vctFiles[i].GetLoadState() == -1) {
            m_vctFiles[i].SetLoadState(0);
        }
    }
    // łɂ܂B
    m_pArchiveLoader->ResetExtractFlags();
}

//------------------------------------------------------------------//

void CFileList::SetMark(bool mark) {
    int i = m_nCurrentIndex;
    if (0 <= i && i < int(m_vctFiles.size())) {
        m_vctFiles[i].SetMark(mark);
        if (m_pListWindow != NULL) {
            m_pListWindow->Update(i);
        }
    }
}

void CFileList::ReverseAllMarks() {
    for (size_t i = 0, n = m_vctFiles.size(); i < n; i++) {
        m_vctFiles[i].ReverseMark();
    }
}

void CFileList::ReverseFirstToCurrentMarks() {
    for (int i = 0, n = m_nCurrentIndex + 1; i < n; i++) {
        m_vctFiles[i].ReverseMark();
    }
}

void CFileList::ClearMarks() {
    for (size_t i = 0, n = m_vctFiles.size(); i < n; i++) {
        m_vctFiles[i].SetMark(false);
    }
}

int CFileList::GetMarkCount() const {
    int c = 0;
    for (size_t i = 0, n = m_vctFiles.size(); i < n; i++) {
        if (m_vctFiles[i].IsMarked()) {
            c++;
        }
    }
    return c;
}

vector<string> CFileList::GetMarkedFiles() const {
    vector<string> files;
    for (size_t i = 0, n = m_vctFiles.size(); i < n; i++) {
        if (m_vctFiles[i].IsMarked()) {
            files.push_back(m_vctFiles[i].GetViewName().Get());
        }
    }
    return files;
}

//------------------------------------------------------------------//

// std::unique() ̓\[gĂȂƎgȂ̂ŁBB
namespace { template<class IT1, class IT2> IT2 unique_copy_no_sort(IT1 itBegin, IT1 itEnd, IT2 itOut) {
    IT2 itOutEnd = itOut;
    for (IT1 it = itBegin; it != itEnd; ++it) {
        if (std::find(itOut, itOutEnd, *it) == itOutEnd) {
            *itOutEnd = *it;
            ++itOutEnd;
        }
    }
    return itOutEnd;
} }
/// dt@C̍폜
void CFileList::UniqueFiles() {
    if (!IsEmpty()) {
        auto_ptr<FileListType> vctTemp = Clone(m_vctFiles);
        m_vctFiles.erase(unique_copy_no_sort(vctTemp->begin(), vctTemp->end(), m_vctFiles.begin()), m_vctFiles.end());
    }
}

int CFileList::Find(const string& strFileName) {
    int n = static_cast<int>(std::find(m_vctFiles.begin(), m_vctFiles.end(), strFileName) - m_vctFiles.begin());
    if (n == GetCount()) {
        Themis_DebugLog(L"t@Ct@CXg猩Ȃ: " + strFileName);
        return 0;
    }
    return n;
}

