#include "stdafx.h"
#include "FileListWindow.h"
#include "Document.h"
#include "resource.h"
#include <Themis/Window.h>
#include <Themis/Windows/ListViewUtility.h>

CFileListWindow::CFileListWindow(CDocument& document) : document(document), m_hParent(NULL), setParentActive(false) {
}

CFileListWindow::~CFileListWindow() {
    Hide();
}

bool CFileListWindow::IsVisible() {
    return m_pWindow.get() != NULL && m_pWindow->Exists();
}

void CFileListWindow::Show(HWND hParent) {
    if (m_pWindow.get() == NULL || !m_pWindow->Exists()) {
        m_hParent = hParent;
        m_pWindow.reset(new CWindow);
        m_pWindow->GetHook().Add(this);
        CDialogParameter oPrm(IDD_FILELIST);
        oPrm.SetActiveOnCreate(false);
        m_pWindow->CreateWithThread(oPrm);
    }
}

void CFileListWindow::Hide(bool parentActive) {
    if (m_pWindow.get() != NULL) {
        setParentActive = parentActive;
        m_pWindow->Close();
        m_pWindow.reset();
    }
}

void CFileListWindow::Update(int i) {
    if (IsVisible()) {
        CListview oListview = m_pWindow->GetCtrl(IDC_LIST1);
        oListview.UpdateItem(i);
    }
}

void CFileListWindow::UpdateAll() {
    if (IsVisible()) {
        CListview oListview = m_pWindow->GetCtrl(IDC_LIST1);
        oListview.SetItemCount(document.GetFileList().GetCount());
        int nIndex = document.GetFileList().GetCurrentIndex();
        if (0 <= nIndex && nIndex < document.GetFileList().GetCount()) {
            oListview.SetSelected(nIndex, true);
            oListview.ScrollToSelectCenter();
         // oListview.ScrollToSelect();
        }
        SetColumns(oListview);
    }
}

bool CFileListWindow::HookMsg(CMsgHookPrms& rPrms) {
    /*
    if (rPrms.GetMsg() == WM_COMMAND) {
        switch (rPrms.GetWPrmLoWord()) {
        default:
            return false;
        }
        return true;

        return false;
    } else */ if (rPrms.GetMsg() == WM_NOTIFY) {
        NMHDR* hdr = (NMHDR*)rPrms.GetLPrm();
        if (hdr->idFrom == IDC_LIST1) {
            //zXgr[ȏBI[i[f[^Ƀ`FbNׂB
            if (hdr->code == LVN_GETDISPINFOA ||
                hdr->code == LVN_GETDISPINFOW) { // \̂̔zuɂĂ͊낤̂c
                LV_DISPINFO* di = (LV_DISPINFO*)hdr;
                if (di->item.mask & LVIF_TEXT) {
                    if (di->item.iItem < document.GetFileList().GetCount()) {
                        if (di->item.iSubItem == 0) {
                            ListViewUtility::SetText(*di, CFileName(document.GetFileList().GetViewName(di->item.iItem)).GetPureName());
                            return true;
                        } else if (di->item.iSubItem == 1) {
                            switch (document.GetFileList().GetLoadState(di->item.iItem)) {
                            case -1: ListViewUtility::SetText(*di, L"~"); break;
                            case 0:  ListViewUtility::SetText(*di, L"");   break;
                            case 1:  ListViewUtility::SetText(*di, L"\"); break;
                            case 2:  ListViewUtility::SetText(*di, L""); break;
                            }
                            return true;
                        } else if (di->item.iSubItem == 2) {
                            if (document.GetFileList().IsMarked(di->item.iItem)) {
                                ListViewUtility::SetText(*di, L"");
                            } else {
                                ListViewUtility::SetText(*di, L"");
                            }
                            return true;
                        }
                    }
                }
            } else if (hdr->code == LVN_GETINFOTIPA ||
                       hdr->code == LVN_GETINFOTIPW) { // \̂̔zuɂĂ͊낤̂c
                NMLVGETINFOTIP* it = (NMLVGETINFOTIP*)hdr;
                if (it->iItem < document.GetFileList().GetCount()) {
                    ListViewUtility::SetText(*it, document.GetFileList().GetViewName(it->iItem).Get());
                    return true;
                }
            } else if (hdr->code == NM_DBLCLK) { // _uدB
                CListview listview = m_pWindow->GetCtrl(IDC_LIST1);
                int n  = listview.GetSelected();
                if (0 <= n && n < document.GetFileCount()) {
                    document.ToIndirect(n);
                    CWnd(m_hParent).SetForeground();
                    return true;
                }
            }
        }
        return false;
    } else if (rPrms.GetMsg() == WM_INITDIALOG) {
        m_pWindow->SetRect(GetInitRect());
        m_pWindow->Show(SW_SHOWNOACTIVATE);
        CListview oListview = m_pWindow->GetCtrl(IDC_LIST1);
        oListview.SetExListStyle(0x00000001 | 0x00000020 | 0x00000400);
        //LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT | LVS_EX_INFOTIP
        //  | LVS_EX_ONECLICKACTIVATE(0x00000040) H
        oListview.AddColumn(0, L"t@C", 1);
        oListview.AddColumn(1, L"", 1); // 
        oListview.AddColumn(2, L"", 1); // }[N
        UpdateAll();
        m_pWindow->Update();
        // xƏ肭悤Ȃ̂łx
        m_pWindow->PostMsg(WM_APP + 0, 0, 0);
        return true;
    } else if (rPrms.GetMsg() == WM_APP + 0) {
        UpdateAll();
        CWnd(m_hParent).SetForeground();
        return true;
    } else if (rPrms.GetMsg() == WM_MOVE) {
        m_ptWindow = m_pWindow->GetPos();
        return true;
    } else if (rPrms.GetMsg() == WM_SIZE) {
        CListview oListview = m_pWindow->GetCtrl(IDC_LIST1);
        oListview.SetRect(m_pWindow->GetClientSize().ToRect());
        SetColumns(oListview);
        m_szWindow = m_pWindow->GetSize();
        return true;
    } else if (rPrms.GetMsg() == WM_CLOSE) {
        m_pWindow->Delete();
        if (setParentActive) {
            CWnd(m_hParent).SetForeground();
        }
        return true;
    } else if (rPrms.GetMsg() == WM_DESTROY) {
        return false;
    } else {
        return false;
    }
}

void CFileListWindow::SetColumns(CListview& oListview) {
    int cx = oListview.GetClientSize().cx;
    oListview.SetColumn(0, L"t@C", cx - 24 * 2);
    oListview.SetColumn(1, L"", 24); // 
    oListview.SetColumn(2, L"", 24); // }[N
}
