#include "stdafx.h"
#include "KeyConfig.h"
#include "MenuID.h"
#include "resource.h"
#include <Themis/KeyConfig.h>
#include <Themis/Tooltip.h>
#include <Themis/Utility/BoostUtility.h>
#include <commctrl.h>

CKeyConfigEx::CKeyConfigEx(CWindow& r, const string& s)
: CResourceDialogBase(IDD_KEYCONFIG), m_rWindow(r), m_strConfigPrefix(s) {
    m_pHotKey.reset(new CHotKey(shared_ptr_fake<CHotKey::IOwner>(this)));
    m_pKeyConfig.reset(new CKeyConfig);

    m_pHotKey->AddToHook(m_rWindow.GetHook());

    // KeyConfig̃ZbgAbv
    #define CATEGORY(_category, _accKey)
    #define ITEM(_id, _category, _text, _accKey, _key)    m_pKeyConfig->Add(_id, _category L": " _text, GV_MENUID(_id))
    #define SEPARATOR()
    #include "MenuData.h"
    #undef SEPARATOR
    #undef ITEM
    #undef CATEGORY

    // HotKeyݒ̓ǂݍ
    LoadFromConfig();
}

CKeyConfigEx::~CKeyConfigEx() {
}

namespace {
    void SetDefaultHotKey(HotKeySetting& hotKey) {
        hotKey.Clear();
        #define CATEGORY(_category, _accKey)
        #define ITEM(_id, _category, _text, _accKey, _key)    hotKey.Add(_id, _key)
        #define SEPARATOR()
        #include "MenuData.h"
        #undef SEPARATOR
        #undef ITEM
        #undef CATEGORY
    }
}
void CKeyConfigEx::LoadFromConfig() {
    HotKeySetting defaultSetting;
    SetDefaultHotKey(defaultSetting);
    m_pHotKey->LoadFromConfig(CFileNameModuleDir(m_strConfigPrefix + L".key.cfg"), defaultSetting);
}

void CKeyConfigEx::LoadDefault() {
    SetDefaultHotKey(*m_pHotKey);
}

void CKeyConfigEx::SaveToConfig() {
    HotKeySetting defaultSetting;
    SetDefaultHotKey(defaultSetting);
    m_pHotKey->SaveToConfig(CFileNameModuleDir(m_strConfigPrefix + L".key.cfg"), defaultSetting);
}

void CKeyConfigEx::ShowDialog() {
    Open(m_rWindow.GetHandle());
}

void CKeyConfigEx::UpdateMenubar(HMENU hMenu) {
    m_pKeyConfig->UpdateMenubar(hMenu, *m_pHotKey);
}

bool CKeyConfigEx::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_COMMAND) {
        switch (rPrms.GetWPrmLoWord()) {
        case IDOK:
            SaveToConfig();
            // ǂRƕςȊBB
            GetDialog().Listview(IDC_LIST3).Clear();
            GetDialog().Delete();
            break;

        case IDCANCEL:
            GetDialog().Close();
            break;

        case IDC_BUTTON1: {
                int n = GetDialog().MsgBoxOkCancel(L"mF", L"݂̐ݒjAݒɖ߂܂");
                if (n == IDOK) {
                    LoadDefault();
                    UpdateListview();
                }
            }
            break;

        case IDC_CHECK1:
        case IDC_CHECK2:
        case IDC_CHECK3:
            UpdateListview();
            break;

        case IDC_LIST1:
            if (rPrms.GetWPrmHiWord() == LBN_DBLCLK) { // _uNbNꂽݒ菈
                CListview oListview = GetDialog().Listview(IDC_LIST3);
                CListbox oListbox = GetDialog().Listbox(IDC_LIST1);
                int n = oListview.GetSelected();
                if (0 <= n) {
                    int nMod = 0;
                    if (GetDialog().Checkbox(IDC_CHECK1).GetCheck()) {
                        nMod = CKeyTable::eModCtrl;
                    }
                    if (GetDialog().Checkbox(IDC_CHECK2).GetCheck()) {
                        nMod |= CKeyTable::eModAlt;
                    }
                    if (GetDialog().Checkbox(IDC_CHECK3).GetCheck()) {
                        nMod |= CKeyTable::eModShift;
                    }

                    m_pHotKey->Remove(CKeyTable::EKey(n), nMod);
                    int nId = m_pKeyConfig->GetIdFromViewName(oListbox.Get());
                    if (0 <= nId) {
                        m_pHotKey->Add(nId, CKeyTable::EKey(n), nMod);
                        oListview.SetSubItem(n, 1, oListbox.Get());
                    } else {
                        oListview.SetSubItem(n, 1, string());
                    }
                }
            }
            break;
        }
        return false;
    } else if (rPrms.GetMsg() == WM_NOTIFY) {
        NMHDR* p = (NMHDR*)rPrms.GetLPrm();
        if (p->idFrom == IDC_LIST3) {
            //LVN_ITEMACTIVATEF(LVN_FIRST-14)
            if (p->code == (LVN_FIRST-14) || p->code == LVN_ITEMCHANGED) {
         // if (p->code == NM_CLICK) {
                UpdateListbox();
            }
            return true;
        }
        return false;
    } else if (rPrms.GetMsg() == WM_INITDIALOG) {
        CWnd oParentWindow(GetDialog().GetParent());
        GetDialog().SetPosClip(oParentWindow.ToScreen(oParentWindow.GetClientSize().GetCenterOf(GetDialog().GetSize())));  //
        CListview oListview = GetDialog().Listview(IDC_LIST3);
        oListview.SetExListStyle(0x00000001 | 0x00000020 | 0x00000040); //LVS_EX_GRIDLINES | LVS_EX_FULLROWSELECT
        //LVS_EX_ONECLICKACTIVATE:0x00000040͂ȂH
        oListview.AddColumn(0, L"L[", 70);
        oListview.AddColumn(1, L"݂̊蓖", 165);
        for (int i = 0; i < CKeyTable::c_nKeyCount; i++) {
            oListview.AddItem(i, CKeyTable::GetKeyName(CKeyTable::EKey(i)));
        }
        CListbox oListbox = GetDialog().Listbox(IDC_LIST1);
        oListbox.Add(L"(蓖)");
        for (int i = 0; i < m_pKeyConfig->GetCount(); i++) {
            oListbox.Add(m_pKeyConfig->GetViewName(i));
        }
        UpdateListview();
        // [邿ՁB
        CDialogTooltip* pTip = new CDialogTooltip(GetDialog());;
        pTip->Add(IDC_LIST3, L"蓖ĂL[I܂B");
     // pTip->Add(IDC_CHECK1, L"");
     // pTip->Add(IDC_CHECK2, L"");
     // pTip->Add(IDC_CHECK3, L"");
        pTip->Add(IDC_LIST1, L"蓖Ă鑀IсA_uNbNŁAIL[Ɋ蓖Ă܂B");
        pTip->Add(IDC_BUTTON1, L"JX^}CYݒjAftHg̃L[ݒɖ߂܂B");
        return true;
    } else if (rPrms.GetMsg() == WM_CLOSE) {
        LoadFromConfig();
        // ǂRƕςȊBB
        GetDialog().Listview(IDC_LIST3).Clear();
        GetDialog().Delete();
        return true;
    } else {
        return false;
    }
}

void CKeyConfigEx::UpdateListview() {
    int nMod = 0;
    if (GetDialog().Checkbox(IDC_CHECK1).GetCheck()) {
        nMod = CKeyTable::eModCtrl;
    }
    if (GetDialog().Checkbox(IDC_CHECK2).GetCheck()) {
        nMod |= CKeyTable::eModAlt;
    }
    if (GetDialog().Checkbox(IDC_CHECK3).GetCheck()) {
        nMod |= CKeyTable::eModShift;
    }

    CListview oListview = GetDialog().Listview(IDC_LIST3);
    for (int i = 0; i < CKeyTable::c_nKeyCount; i++) {
        int nId = m_pHotKey->GetId(CKeyTable::EKey(i), nMod);
        if (0 <= nId) {
            oListview.SetSubItem(i, 1, m_pKeyConfig->GetViewNameFromId(nId));
        } else {
            oListview.SetSubItem(i, 1, string());
        }
    }

    // łɁB
    UpdateListbox();
}

void CKeyConfigEx::UpdateListbox() {
    CListview oListview = GetDialog().Listview(IDC_LIST3);
    GetDialog().Listbox(IDC_LIST1).SetEnable(0 <= oListview.GetSelected());
}

void CKeyConfigEx::OnHotKey(CHotKey* pHotKey, int nId) {
    m_pKeyConfig->SendCommand(m_rWindow.GetHandle(), nId);
}

