#include "stdafx.h"
#include "MagnifierWindow.h"
#include <Themis/Windows/System.h>
#include <Themis/Region.h>
#include <Themis/LocalExclusion.h>

CMagnifierWindow::CMagnifierWindow(CWindow& rWindow, IOwner& pOwner):
m_rWindow(rWindow), m_pOwner(pOwner) {
    m_pExclusion.reset(new CLocalExclusion);
    m_oMagnifierWindow.GetHook().Add(this);
    m_rWindow.GetHook().Add(this);
}

CMagnifierWindow::~CMagnifierWindow() {
    m_oMagnifierWindow.PostClose(); // Ô
}

void CMagnifierWindow::SetSize(const CSize& sz) {
    m_szMagnifier = sz;
    if (m_oMagnifierWindow.Exists()) {
        m_oMagnifierWindow.SetSize(sz);
    }
}

CSize CMagnifierWindow::GetSize() const {
    return m_szMagnifier;
}

void CMagnifierWindow::SetShape(int nShape) {
    if (m_oMagnifierWindow.Exists()) {
        if (nShape == 0) { // [WO
            CRegion().SetToWindow(m_oMagnifierWindow.GetHandle());
        } else { //  if (nShape == 1) [Wt
            CRegion oRegion;
            oRegion.CreateEllipse(m_szMagnifier.ToRect());
            oRegion.SetToWindow(m_oMagnifierWindow.GetHandle());
        }
    }
    m_nShape = nShape;
}

int CMagnifierWindow::GetShape() const {
    return m_nShape;
}

void CMagnifierWindow::Show() {
    boost::mutex::scoped_lock lock(mutex);
    if (m_oMagnifierWindow.Exists()) {
        m_oMagnifierWindow.SetSize(m_szMagnifier);
        m_oMagnifierWindow.SetPos(m_ptCursor - m_oMagnifierWindow.GetSize() / 2);
        m_oMagnifierWindow.Show();
        m_oMagnifierWindow.SetForeground();
    } else {
        m_ptCursor = CSystem::GetCursorPos();
        CWindowParameter oOpt;
        oOpt.SetParent(m_rWindow.GetHandle());
     // oOpt.GetStyle() = WS_BORDER | WS_CHILD;
     // oOpt.GetStyle() = WS_POPUP | WS_BORDER;
        oOpt.GetStyle() = WS_POPUP;
        oOpt.GetExStyle() = WS_EX_TOOLWINDOW | WS_EX_TOPMOST;
        oOpt.SetActiveOnCreate(false);
        m_oMagnifierWindow.CreateWithThread(oOpt);
    }
}

void CMagnifierWindow::Hide() {
    boost::mutex::scoped_lock lock(mutex);
    if (m_oMagnifierWindow.Exists()) {
        m_oMagnifierWindow.Hide();
     // m_rWindow.SetForeground();
    }
}

bool CMagnifierWindow::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetWnd() == m_rWindow.GetHandle()) {
        /// C̃bZ[W
        if (rPrms.GetMsg() == WM_ACTIVATEAPP) {
            if (m_oMagnifierWindow.Exists() && m_oMagnifierWindow.IsVisible()) {
                DWORD dwThreadID = static_cast<DWORD>(rPrms.GetLPrm());
                if (static_cast<BOOL>(rPrms.GetWPrm()) == FALSE && !CWindow::IsOwnWindowThread(dwThreadID)) {
                    Hide();
                }
            }
            return false;
        } else if (rPrms.GetMsg() == WM_MOUSEMOVE) {
            // ̈ړ
            if (m_oMagnifierWindow.Exists() && m_oMagnifierWindow.IsVisible()) {
                m_ptCursor = CSystem::GetCursorPos();
                m_oMagnifierWindow.SetPos(m_ptCursor - m_oMagnifierWindow.GetSize() / 2);
                m_oMagnifierWindow.PostUpdate();
            }
            return true;
        } else if (rPrms.GetMsg() == WM_DESTROY) {
            m_oMagnifierWindow.Close();
            return false;
        } else {
            return false;
        }
    } else if (rPrms.GetWnd() == m_oMagnifierWindow.GetHandle()) {
        /// g勾̃bZ[W
        if (rPrms.GetMsg() == WM_ACTIVATEAPP) {
            if (static_cast<BOOL>(rPrms.GetWPrm()) == FALSE) {
                m_oMagnifierWindow.Hide();
                return true;
            }
            return false;
        } else if (rPrms.GetMsg() == WM_PAINT) {
            m_oMagnifierWindow.HideCursor(); // ÔߌĂт܂
            CLocalExclusion::CLock oLock(*m_pExclusion);
            if (oLock.IsLocked()) {
                std::tr1::shared_ptr<PAINTSTRUCT> ps = m_oMagnifierWindow.BeginPaint();
                CSize szDst = m_oMagnifierWindow.GetClientSize(); // g勾̑傫
                CPoint ptSrcCenter = m_rWindow.ToClient(m_ptCursor); // g勾̒S
                m_pOwner.OnPaint(*ps, szDst, ptSrcCenter);
            }
            return false;
        } else if (rPrms.GetMsg() == WM_MOUSEMOVE) {
            // ̈ړ
            if (m_oMagnifierWindow.Exists() && m_oMagnifierWindow.IsVisible()) {   //ĂȂᗈȂƎvǁBB
                m_ptCursor = CSystem::GetCursorPos();
                m_oMagnifierWindow.SetPos(m_ptCursor - m_oMagnifierWindow.GetSize() / 2);
                m_oMagnifierWindow.PostUpdate();
            }
            return true;
        } else if (rPrms.GetMsg() == WM_LBUTTONUP || rPrms.GetMsg() == WM_RBUTTONUP || rPrms.GetMsg() == WM_MBUTTONUP) {
            m_oMagnifierWindow.Hide();
            return true;
        } else if (rPrms.GetMsg() == WM_CREATE) {
            // \̍XVƂ
         // m_oMagnifierWindow.SetParent(m_rWindow.GetHandle());
            m_oMagnifierWindow.HideCursor();
            m_oMagnifierWindow.SetEnableCloseButton(false); // Alt+F4ɂoO΍
            m_oMagnifierWindow.SetSize(m_szMagnifier);
            m_oMagnifierWindow.SetPos(m_ptCursor - m_oMagnifierWindow.GetSize() / 2);
            m_oMagnifierWindow.Show();
            m_oMagnifierWindow.SetForeground();
            // [Wݒ
            SetShape(m_nShape);
            return false;
        } else if (rPrms.GetMsg() == WM_CLOSE) {
            m_oMagnifierWindow.Delete();
            return true;
        } else {
            return false;
        }
    } else {
        return false;
    }
}

