#include "stdafx.h"
#include <Themis/TempFile.h>
#include <Themis/Http.h>
#include <Themis/BatchFile.h>
#include <Themis/Archive.h>
#include <Themis/File.h>
#include <Themis/Config.h>
#include <commctrl.h>
#include "resource.h"
#include "NetworkUpdater.h"

CNetworkUpdater::CNetworkUpdater(CWindow& w, CConfig& r)
: CResourceDialogBase(IDD_NETUPDATE), m_rWindow(w), m_rConfig(r), m_bUpdate(false) {
}

CNetworkUpdater::~CNetworkUpdater() {
}

bool CNetworkUpdater::HookMsg(CMsgHookPrms& rPrms) {
    if (rPrms.GetMsg() == WM_COMMAND) {
        switch (rPrms.GetWPrmLoWord()) {
        case IDCANCEL:
            CancelUpdate();
            GetDialog().Close();
            break;
        case IDOK: // s{^
            GetDialog().Button(IDOK).SetEnable(false);
            Update();
            break;
        }
        return false;
    } else if (rPrms.GetMsg() == WM_INITDIALOG) {
        CWnd oParentWindow(GetDialog().GetParent());
        GetDialog().SetPosClip(oParentWindow.ToScreen(oParentWindow.GetClientSize().GetCenterOf(GetDialog().GetSize())));  //
        CProgress(GetDialog().GetCtrl(IDC_PROGRESS1)).SetRange(0, 255);
        CProgress(GetDialog().GetCtrl(IDC_PROGRESS2)).SetRange(0, 255);
        GetDialog().Button(IDOK).SetEnable(false);
        m_bUpdate = false;
        // 擾[
        UpdateGetInfo();
        return true;
    } else if (rPrms.GetMsg() == WM_CLOSE) {
        StopUpdate();
        GetDialog().Delete();
        return true;
    } else {
        return false;
    }
}

CConfig& CNetworkUpdater::GetConfig() {
    return m_rConfig;
}

string CNetworkUpdater::GetConfigBaseName() {
    return L"Http";
}

string CNetworkUpdater::GetUpdateUrl() const {
    return L"http://hp.vector.co.jp/authors/VA039571/gv2/update.cfg";
}

void CNetworkUpdater::OnOldNotice(const vector<string>& vctOldNotice) {
    string strNotice;
    for (size_t i = 0, n = vctOldNotice.size(); i < n; i++) {
        strNotice += vctOldNotice[n - i - 1];
        strNotice += L"\r\n";
    }
    GetDialog().Editbox(IDC_EDIT1).SetText(strNotice);
}

void CNetworkUpdater::OnGetInfoProgress(unsigned nCurrent, unsigned nMax) {
    if (GetDialog().Exists()) {
        GetDialog().Static(IDC_STATIC1).SetText(string::Format(L"update.cfg %6d / %d", nCurrent, nMax));
        CProgress(GetDialog().GetCtrl(IDC_PROGRESS1)).Set(nCurrent * 256 / nMax);
        CProgress(GetDialog().GetCtrl(IDC_PROGRESS2)).Set(nCurrent * 256 / nMax);
    }
}

void CNetworkUpdater::OnGetInfoNoUpdate() {
    GetDialog().MsgBox(L"lbg[NXV", L"ݗpłXV͂܂łB");
    GetDialog().PostClose();
}

void CNetworkUpdater::OnGetInfo(const CNetworkUpdateInfo& oInfo) {
    GetDialog().Static(IDC_STATIC1).SetText(string());
    string strNotice;
    for (int i = 0, n = oInfo.GetNoticeCount(); i < n; i++) {
        strNotice += oInfo.GetNotice(n - i - 1).ToStr();
        strNotice += L"\r\n";
    }
    GetDialog().Editbox(IDC_EDIT1).SetText(strNotice);
    GetDialog().Button(IDOK).SetEnable(true);
}

void CNetworkUpdater::OnUpdateProgress(const string& strFileName, unsigned nCurrent, unsigned nMax, unsigned nCurrentAll, unsigned nMaxAll) {
    if (GetDialog().Exists()) {
     // GetDialog().Static(IDC_STATIC1).SetText(CFileName(strFileName).GetPureName() + string::Format(L" %6d / %d", nCurrent, nMax));
        GetDialog().Static(IDC_STATIC1).SetText(CFileName(strFileName).GetPureName() + string::Format(L" %6d / %d", nCurrentAll, nMaxAll));
        CProgress(GetDialog().GetCtrl(IDC_PROGRESS1)).Set(nCurrent * 256 / nMax);
        CProgress(GetDialog().GetCtrl(IDC_PROGRESS2)).Set(nCurrentAll * 256 / nMaxAll);
    }
}

bool CNetworkUpdater::ConfirmUpdate() {
    if (GetDialog().MsgBoxOkCancel(L"mF", L"XVs܂") != IDOK) {
        GetDialog().PostClose();
        return false;
    }
    GetDialog().Button(IDCANCEL).SetEnable(false); // LZsɂ
    return true;
}

void CNetworkUpdater::OnEndBackup() {
    // ÂobNAbv`̃t@C $backup-00$ փRsׂ
    for (int i = 0; i < 100; i++) {
        CFile oFile(CFileNameModuleDir(string::Format(L"т2.%02d.exe", i)));
        if (oFile.Exists()) {
            oFile.Move(CFileNameModuleDir(L"$backup-00$/"));
        } else if (0 < i) {
            break;
        }
    }
}

void CNetworkUpdater::OnError() {
 // GetDialog().PostClose();
}

void CNetworkUpdater::PostQuit() {
    m_bUpdate = true;
    GetDialog().PostClose();
}

